package cgl.axis.services.uddi_wscontext;

import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.Log;
import org.apache.axis.types.URI;

import cgl.axis.services.uddi.uddi_schema.*;
import cgl.axis.services.uddi.uddi_ext_schema.*;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.*;
import cgl.axis.services.uddi_wscontext.UDDI_WSContext_Helper;
import cgl.fthpis.datatype.publisher.PublisherID;
import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.HashMap;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class UDDI_WSContext_Helper {

    private static Log log = LogFactory.getLog(UDDI_WSContext_Helper.class);

    /**
     *
     */
    public UDDI_WSContext_Helper() {
    }

    /**
     * construct axis_contextBag object from a given vector stuffed with contexts
     */
    public ContextBag constructAXIS_ContextBag(java.util.Vector
                                               fthpis_contextVector) {
        ContextBag contextbag = null;
        if (fthpis_contextVector != null && fthpis_contextVector.size() != 0) {
            contextbag = new ContextBag();
            int size = fthpis_contextVector.size();
            log.info("fthpis_contextVector.size = " + size);
            Context[] contextArray =
                    new Context[size];

            int context_count = 0;
            for (java.util.Iterator iter = fthpis_contextVector.iterator();
                                           iter.hasNext(); ) {

                cgl.fthpis.datatype.attribute.Attribute fthpis_context =
                        (cgl.fthpis.datatype.attribute.Attribute) iter.next();
                URI context_key = null;
                if (fthpis_context != null) {
                   String fthpis_context_key = fthpis_context.getAttributeKey();
//                   String fthpis_context_key = "uuid:" +
//                                               fthpis_context.getAttributeKey();
                    log.info("context key = " + fthpis_context_key);
                    try {
                        context_key = new URI(fthpis_context_key);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }

                    if (fthpis_context.getNameVector() != null &&
                        fthpis_context.getNameVector().size() != 0) {
                        log.info("context user identifier key = " +
                                 fthpis_context.getNameVector().elementAt(0));
                    }

                    if (fthpis_context.getAttributeValue() != null) {
                        log.info("context value = " +
                                 fthpis_context.getAttributeValue().
                                 getAttributeValue());
                    }
                    log.info("context version = " +
                             fthpis_context.getAttributeVersion());
                }

                Context axis_context = new Context();

                //CONSTRUCT THE KEY
                axis_context.setContextKey(context_key);

                //CONSTRUCT AXIS METADATA FIELDS
                if (fthpis_context != null) {

                    if (fthpis_context.getNameVector() != null &&
                        fthpis_context.getNameVector().size() != 0) {
                        axis_context.setName(constructAXIS_Name(fthpis_context.
                                getNameVector()));
                    }

                    if (fthpis_context.geAttributeOwnerPublisherID() != null) {
                        //here we can set the owner of the context
                        //however, the user does not need to get this info
                        //it is ignored in this version.
                    }

                    if (fthpis_context.getAttributeValue() != null) {
                        byte[] byte_array = fthpis_context.getAttributeValue().
                                            getAttributeValue();
                        axis_context.setValue(byte_array);
                        //HashMap map = unmarshallHashMap(byte_array);
                        //printHashMap(map);
                    }
                }

                //CONSTRUCT AXIS METADATA ASSERTIONS
                java.util.Vector context_assertion_vector =
                        fthpis_context.getAttributeAssertionVector();
                if (context_assertion_vector != null) {
                    int assertion_size = context_assertion_vector.size();
                    ContextAssertion[] context_assertion_array =
                            new ContextAssertion[assertion_size];
                    int i = 0;
                    for (Iterator iterator = context_assertion_vector.iterator();
                                             iterator.hasNext(); ) {
                        cgl.fthpis.datatype.attribute.AttributeAssertion
                                assertion =
                                        (cgl.fthpis.datatype.attribute.
                                         AttributeAssertion)
                                        iterator.next();
                        ContextAssertion context_assertion =
                                new ContextAssertion();
                        org.apache.axis.types.URI fromkey = null;
                        org.apache.axis.types.URI propertykey = null;
                        org.apache.axis.types.URI tokey = null;
                        try {
                            fromkey = new org.apache.axis.types.URI(assertion.
                                    getAssertionSubjectKey());
                            propertykey = new org.apache.axis.types.URI(
                                    assertion.
                                    getAssertionSubjectKey());
                            tokey = new org.apache.axis.types.URI(assertion.
                                    getAssertionSubjectKey());
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                        context_assertion.setFromKey(fromkey);
                        context_assertion.setProperty_uri(propertykey);
                        context_assertion.setToKey(tokey);
                        context_assertion_array[i] = context_assertion;
                        i++;
                    }
                    axis_context.setContextAssertion(context_assertion_array);
                }

                //CONSTRUCT AXIS METADATA ACCESSRIGHTSINFO
                cgl.fthpis.datatype.attribute.AttributeAccessInfos infos =
                        fthpis_context.getAttributeAccessRightInfos();

                AccessRightInfo access_info = null;
                if (infos != null) {
                    access_info = new AccessRightInfo();
                    cgl.fthpis.datatype.attribute.AttributeAccessInfo
                            context_owner_access_info = infos.
                            getOwnerAttributeAccessInfo();
                    if (context_owner_access_info != null) {
                        Owner owner = new Owner();
                        owner.setReadAccess(context_owner_access_info.isReadOK());
                        owner.setWriteAccess(context_owner_access_info.
                                             isReadWriteOK());
                        access_info.setOwner(owner);
                        axis_context.setAccessRightInfo(access_info);
                    }

                    cgl.fthpis.datatype.attribute.AttributeAccessInfo
                            context_group_access_info = infos.
                            getGroupAttributeAccessInfo();
                    if (context_group_access_info != null) {
                        Group group = new Group();
                        group.setReadAccess(context_group_access_info.isReadOK());
                        group.setWriteAccess(context_group_access_info.
                                             isReadWriteOK());
                        if (context_group_access_info.getPublisherIDVector() != null) {
                            int publisher_size = context_group_access_info.
                                                 getPublisherIDVector().size();
                            String[] publisherIDarray = new String[
                                    publisher_size];
                            group.setPublisherID(publisherIDarray);
                        }
                        access_info.setGroup(group);
                        axis_context.setAccessRightInfo(access_info);

                    }

                    cgl.fthpis.datatype.attribute.AttributeAccessInfo
                            context_others_access_info =
                                    infos.getOthersAttributeAccessInfo();
                    if (context_others_access_info != null) {
                        Others others =
                                new Others();
                        others.setReadAccess(context_others_access_info.
                                             isReadOK());
                        others.setWriteAccess(context_others_access_info.
                                              isReadWriteOK());
                        access_info.setOthers(others);
                        axis_context.setAccessRightInfo(access_info);
                    }
                }

                //CONSTRUCT AXIS LEASE
                cgl.fthpis.datatype.lease.Lease juddiLease =
                        fthpis_context.getLease();
                cgl.axis.services.uddi.uddi_ext_schema.Lease lease = null;
                if (juddiLease != null) {
                    lease = constructAXIS_Lease(juddiLease);
                    axis_context.setLease(lease);
                }

                //set context key
                org.apache.axis.types.URI contextkey = null;
                try {
                    contextkey = new org.apache.axis.types.URI(
                            fthpis_context.getAttributeKey());
//                    contextkey = new org.apache.axis.types.URI("uuid:" +
//                            fthpis_context.getAttributeKey());
                } catch (Exception ex) {
                    ex.printStackTrace();
                }

                //set service key
                org.apache.axis.types.URI servicekey = null;
                try {
                    servicekey = new org.apache.axis.types.URI("uuid:" +
                            fthpis_context.getServiceKey());

                } catch (Exception ex) {
                    ex.printStackTrace();
                }

                //set session key
                org.apache.axis.types.URI sessionkey = null;
                try {
                    sessionkey = new org.apache.axis.types.URI(
                            fthpis_context.getSessionKey());
//                    sessionkey = new org.apache.axis.types.URI("uuid:" +
//                            fthpis_context.getSessionKey());
                } catch (Exception ex) {
                    ex.printStackTrace();
                }

                axis_context.setContextKey(contextkey);
                axis_context.setSessionKey(sessionkey);
                axis_context.setServiceKey(servicekey);

                contextArray[context_count] = axis_context;
                context_count++;

            }
            log.info("contextArray array size = " + contextArray.length);
            contextbag.setContext(contextArray);
        }
        return contextbag;
    }

    /**
     * construct axis_lease object from a given lease object
     */
    public cgl.axis.services.uddi.uddi_ext_schema.Lease constructAXIS_Lease(cgl.
            fthpis.datatype.lease.Lease fthpisLease) {
        cgl.axis.services.uddi.uddi_ext_schema.Lease lease = null;
        if (fthpisLease != null) {
            if (fthpisLease != null) {
                lease = new cgl.axis.services.uddi.uddi_ext_schema.Lease();

                if (fthpisLease.getLeaseStartDate() != null) {
                    lease.setLeaseStartTimeStamp(fthpisLease.getLeaseStartDate().
                                                 getLeaseStartDate());
                }
                if (fthpisLease.getLeaseExpirationDate() != null) {
                    lease.setLeaseExpirationTimeStamp(fthpisLease.
                            getLeaseExpirationDate().getLeaseExpirationDate());
                }
                if (fthpisLease.getIsInfinte() != null) {
                    boolean isinfinite = (new Boolean(fthpisLease.getIsInfinte())).
                                         booleanValue();
                    lease.setIsInfinite(isinfinite);
                }
            }
        }
        return lease;
    }

    /**
     * construct axis_name array from a given name vector
     */
    public cgl.axis.services.uddi.uddi_schema.Name[] constructAXIS_Name(java.
            util.Vector nameVec) {
        cgl.axis.services.uddi.uddi_schema.Name[] axis_name_array = null;
        if (nameVec != null) {
            int nameVecSize = nameVec.size();
            axis_name_array = new cgl.axis.services.uddi.uddi_schema.Name[
                              nameVecSize];
            int nameVec_count = 0;
            for (java.util.Iterator it = nameVec.iterator(); it.hasNext(); ) {
                org.apache.juddi.datatype.Name business_name = (org.apache.
                        juddi.
                        datatype.Name) it.next();
                cgl.axis.services.uddi.uddi_schema.Name name =
                        new cgl.axis.services.uddi.uddi_schema.Name(
                                business_name.
                                getValue());
                axis_name_array[nameVec_count] = name;
                nameVec_count++;
            }
        }
        return axis_name_array;
    }

    /**
     * construct axis_publisher array from a given vector stuffed with publisher objects
     */
    public cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherEntity[]
            constructAXIS_Publisher(java.util.Vector pubVec) {
        cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherEntity[]
                axis_publisher_array = null;
        if (pubVec != null) {
            int pubVecSize = pubVec.size();
            axis_publisher_array =
                    new cgl.axis.services.WSCTX.uddi_wsctx_schema.
                    PublisherEntity[pubVecSize];
            int pubVec_count = 0;
            for (java.util.Iterator it = pubVec.iterator(); it.hasNext(); ) {

                org.apache.juddi.datatype.publisher.Publisher publisher =
                        (org.apache.juddi.datatype.publisher.Publisher) it.next();
                cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherEntity
                        pub_entity = new cgl.axis.services.WSCTX.
                                     uddi_wsctx_schema.PublisherEntity();
                pub_entity.setPublisherID(publisher.getPublisherID());
                axis_publisher_array[pubVec_count] = pub_entity;
                pubVec_count++;
            }
        }
        return axis_publisher_array;
    }

    /**
     * construct axis_description array from a given vector stuffed with description objects
     */
    public cgl.axis.services.uddi.uddi_schema.Description[]
            constructAXIS_Description(java.util.Vector fthpis_description_vector) {
        //set description array
        cgl.axis.services.uddi.uddi_schema.Description[] axis_temp_desc_array = null;
        if (fthpis_description_vector != null) {
            int descVecSize = fthpis_description_vector.size();
            axis_temp_desc_array = new cgl.axis.services.uddi.uddi_schema.
                                   Description[descVecSize];
            int descVec_count = 0;
            for (java.util.Iterator it = fthpis_description_vector.iterator();
                                         it.hasNext(); ) {
                org.apache.juddi.datatype.Description business_description =
                        (org.apache.juddi.datatype.Description) it.next();
                cgl.axis.services.uddi.uddi_schema.Description description =
                        new cgl.axis.services.uddi.uddi_schema.Description(
                                business_description.
                                getValue());
                axis_temp_desc_array[descVec_count] = description;
                descVec_count++;
            }
        }
        return axis_temp_desc_array;
    }

    /**
     * construct axis_AccessRightInfo object from a given fthpis AttributeAccessInfos object
     */
    public AccessRightInfo
            constructAXIS_AccessRightInfo(cgl.fthpis.datatype.attribute.
                                          AttributeAccessInfos
                                          fthpis_accessrightinfos) {

        AccessRightInfo
                axis_temp_accessrightinfo = null;

        if (fthpis_accessrightinfos != null) {

            axis_temp_accessrightinfo =
                    new AccessRightInfo();

            cgl.fthpis.datatype.attribute.AttributeAccessInfo fthpis_owner_info =
                    fthpis_accessrightinfos.getOwnerAttributeAccessInfo();

            cgl.fthpis.datatype.attribute.AttributeAccessInfo fthpis_group_info =
                    fthpis_accessrightinfos.getOwnerAttributeAccessInfo();
            Vector publisherIDVector = fthpis_group_info.getPublisherIDVector();

            cgl.fthpis.datatype.attribute.AttributeAccessInfo fthpis_others_info =
                    fthpis_accessrightinfos.getOwnerAttributeAccessInfo();

            Owner axis_owner = new Owner();
            if (fthpis_others_info != null) {
                axis_owner.setReadAccess(fthpis_owner_info.isReadOK());
                axis_owner.setWriteAccess(fthpis_owner_info.isReadWriteOK());
            }

            Group axis_group = new Group();
            if (fthpis_group_info != null) {
                axis_group.setReadAccess(fthpis_group_info.isReadOK());
                axis_group.setWriteAccess(fthpis_group_info.isReadWriteOK());
                int publisher_number = publisherIDVector.size();
                String[] publisherIDArray = new String[publisher_number];
                int i = 0;
                for (Iterator iter = publisherIDVector.iterator(); iter.hasNext(); ) {
                    String publisherID = (String) iter.next();
                    publisherIDArray[i] = publisherID;
                    i++;
                }
                axis_group.setPublisherID(publisherIDArray);
            }

            Others axis_others = new Others();
            if (fthpis_others_info != null) {
                axis_others.setReadAccess(fthpis_others_info.isReadOK());
                axis_others.setWriteAccess(fthpis_others_info.isReadWriteOK());
            }

            axis_temp_accessrightinfo.setOwner(axis_owner);
            axis_temp_accessrightinfo.setGroup(axis_group);
            axis_temp_accessrightinfo.setOthers(axis_others);

        }
        return axis_temp_accessrightinfo;
    }

    /**
      * construct AttributeAccessInfos object from a given axis_AccessRightInfo object
      */
    public cgl.fthpis.datatype.attribute.AttributeAccessInfos
            constructFTHPIS_AccessRightInfo(AccessRightInfo access_right_info) {
        cgl.fthpis.datatype.attribute.AttributeAccessInfos
                attribute_access_infos = null;
        cgl.fthpis.datatype.attribute.AttributeAccessInfo info = null;

        if (access_right_info != null) {
            new cgl.fthpis.datatype.attribute.AttributeAccessInfos();

            Owner owner = access_right_info.getOwner();
            info = new cgl.fthpis.datatype.attribute.AttributeAccessInfo();
            info.setReadOK(true /*owner.isReadAccess()*/);
            info.setisReadWriteOK(true /*owner.isWriteAccess()*/);
            attribute_access_infos.setOwnerAttributeAccessInfo(info);

            Group group = access_right_info.getGroup();
            info = new cgl.fthpis.datatype.attribute.AttributeAccessInfo();
            info.setReadOK(group.isReadAccess());
            info.setisReadWriteOK(group.isWriteAccess());
            String[] publisherIDarray = group.getPublisherID();
            Vector publisherIDvector = new Vector();
            for (int i = 0; i < publisherIDarray.length; i++) {
                publisherIDvector.add(publisherIDarray[i]);
            }
            info.setPublisherIDVector(publisherIDvector);
            attribute_access_infos.setGroupAttributeAccessInfo(info);

            Others others = access_right_info.getOthers();
            info = new cgl.fthpis.datatype.attribute.AttributeAccessInfo();
            info.setReadOK(others.isReadAccess());
            info.setisReadWriteOK(others.isWriteAccess());
            attribute_access_infos.setOwnerAttributeAccessInfo(info);

        }
        return attribute_access_infos;
    }

    /**
     * construct fthpis AttributeBag object from a given array of axis generated Context
     * cgl.fthpis.datatype.attribute.Attribute is the Context object in FTHPIS implementation
     */
    public cgl.fthpis.datatype.attribute.AttributeBag
            constructFTHPIS_ContextBag(Context[]
                                      axis_context_array, String publisherID) {
        log.info("constructFTHPIS_ContextBag => I ");
        cgl.fthpis.datatype.attribute.AttributeBag attributeBag = null;
        if (axis_context_array != null) {

            if (axis_context_array != null) {

                java.util.Vector attributeBag_Vector = new java.util.Vector();
                for (int i = 0; i < axis_context_array.length; i++) {
                    Context axis_context = axis_context_array[i];

                    cgl.fthpis.datatype.attribute.Attribute fthpis_context =
                            new cgl.fthpis.datatype.attribute.Attribute();

                    //CONSTRUCT SESSIONKEY
                    org.apache.axis.types.URI axis_sessionkey =
                            axis_context_array[i].getSessionKey();
                    String session_key = "";
                    if (axis_sessionkey != null) {
                        session_key = axis_sessionkey.toString();
//                        java.util.StringTokenizer tokenizer =
//                                new java.util.StringTokenizer(session_key,
//                                "uuid:");
//                        session_key = tokenizer.nextToken();
                        //log.info("****** updated binding_key = " + binding_key);
                        fthpis_context.setSessionKey(session_key);
                    }

                    //CONSTRUCT SERVICEKEY
                    org.apache.axis.types.URI axis_servicekey =
                            axis_context_array[i].getServiceKey();
                    String service_key = "";
                    if (axis_servicekey != null) {
                        service_key = axis_servicekey.toString();
                        java.util.StringTokenizer tokenizer =
                                new java.util.StringTokenizer(service_key,
                                "uuid:");
                        service_key = tokenizer.nextToken();
                        //log.info("****** updated service_key = " + service_key);
                        fthpis_context.setServiceKey(service_key);
                    }

                    //CONSTRUCT CONTEXTKEY
                    org.apache.axis.types.URI axis_contextkey =
                            axis_context_array[i].getContextKey();
                    String context_key = "";
                    if (axis_contextkey != null) {
                        context_key = axis_contextkey.toString();
//                        java.util.StringTokenizer tokenizer =
//                                new java.util.StringTokenizer(context_key,
//                                "uuid:");
//                        context_key = tokenizer.nextToken();
                        //log.info("****** updated context_key = " + context_key);
                        fthpis_context.setAttributeKey(context_key);
                    }

                    fthpis_context.setNameVector(constructFTHPIS_NameVector(
                            axis_context.getName()));

                    PublisherID owner_publisherID = new PublisherID();
                    owner_publisherID.setValue(publisherID);
                    fthpis_context.setAttributeOwnerPublisherID(
                            owner_publisherID);

                    byte[] byte_array = null;
                    if (axis_context.getValue() == null) {
                        //String value = "";
                        //byte_array = value.getBytes();
                    } else {
                        byte_array = axis_context.getValue();
//                        HashMap m = unmarshallHashMap(byte_array);
//                        printHashMap(m);
                    }
                    fthpis_context.setAttributeValue(new cgl.fthpis.datatype.
                            attribute.AttributeValue(byte_array));

                    log.info(" axis_context.getValue() = " +  axis_context.getValue());

                    //if there is a attribute find qualifier I need to add this into attribute
                    if (axis_context.getFindQualifier() != null) {
                        String qualifier = axis_context.getFindQualifier();
                        fthpis_context.setAttributeFindQualifier(new cgl.fthpis.
                                datatype.attribute.AttributeFindQualifier(
                                        qualifier));
                    }

                    //lease
                    cgl.axis.services.uddi.uddi_ext_schema.Lease lease =
                            axis_context.getLease();
                    cgl.fthpis.datatype.lease.Lease fthpis_lease = null;
                    if (lease != null) {
                        fthpis_lease = constructFTHPIS_Lease(lease);
                    }
                    fthpis_context.setLease(fthpis_lease);

                    //version
                    fthpis_context.setAttributeVersion(1);

                    //add into attribute bag
                    attributeBag_Vector.add(fthpis_context);
                }

                attributeBag = new cgl.fthpis.datatype.attribute.AttributeBag();
                attributeBag.setAttributeVector(attributeBag_Vector);
            }
        }
        return attributeBag;
    }

//    //----------------------------------------------------------------------
//    public void printHashMap(HashMap map) {
//            if (map == null) {
//                    pr("\n -- HashMap is null --");
//                    return;
//            }
//            pr("\n -- print HashMap " + map + " --\n");
//    }
//    //----------------------------------------------------------------------
//    public HashMap unmarshallHashMap(byte[] by) {
//    HashMap map = null;
//    try {
//            // Deserialize from a byte array
//            ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(by));
//            map = (HashMap) in.readObject();
//            in.close();
//    } catch (ClassNotFoundException e) {
//                    pr("ClassNotFoundException");
//                    e.printStackTrace();
//    } catch (IOException e) {
//                    pr("IOException");
//                    e.printStackTrace();
//    }
//            return map;
//        }
//        //----------------------------------------------------------------------
        private void pr(String out) {
                System.out.println(out);
        }
        //----------------------------------------------------------------------

    /**
     * construct a Vector of Name object from given axis generated Name object array
     */
    public java.util.Vector constructFTHPIS_NameVector(cgl.axis.services.uddi.
            uddi_schema.
            Name[] name_array) {
        java.util.Vector nameVector = null;
        if (name_array != null) {
            nameVector = new java.util.Vector();
            int length = name_array.length;
            for (int t = 0; t < length; t++) {
                cgl.axis.services.uddi.uddi_schema.Name name =
                        name_array[t];
                nameVector.add(new org.apache.juddi.datatype.Name(
                        name.getValue().toString()));
            }
        }
        return nameVector;
    }

    /**
     * construct a Vector of Publisher objects from given publisher IDs
     */
    public java.util.Vector constructFTHPIS_PublisherVector(
            String[] pub_array) {
        java.util.Vector pubVector = null;
        if (pub_array != null && pub_array.length != 0) {
            pubVector = new java.util.Vector();
            int length = pub_array.length;
            for (int t = 0; t < length; t++) {
                cgl.fthpis.datatype.publisher.Publisher
                        pub = new cgl.fthpis.datatype.publisher.Publisher();
                log.info("UDDI_WSCONTEXT_HELPER = > pulisherID = " +
                         pub_array[t]);
                pub.setPublisherID(pub_array[t]);
                pubVector.add(pub);
            }
        }
        return pubVector;
    }

    /**
     * contruct a Vector of Publisher IDs from a given axis based Publisher Entity
     */
    public java.util.Vector constructFTHPIS_PublisherVector(
            PublisherEntity[] pub_entity_array) {
        java.util.Vector pubVector = null;
        if (pub_entity_array != null && pub_entity_array != null) {
            pubVector = new java.util.Vector();
            int length = pub_entity_array.length;
            for (int t = 0; t < length; t++) {
                cgl.fthpis.datatype.publisher.Publisher
                        pub = new cgl.fthpis.datatype.publisher.Publisher();
                PublisherEntity entity = pub_entity_array[t];
                log.info("UDDI_WSCONTEXT_HELPER = > entity.getPublisherID() = " +
                         entity.getPublisherID());
                log.info("UDDI_WSCONTEXT_HELPER = > entity.getPersonName() = " +
                         entity.getPersonName());
                PersonName[] person_name = entity.getPersonName();
                if (person_name != null && person_name.length != 0) {
                    String personname = person_name[0].getValue();
                    pub.setName(personname);
                }
                pub.setPublisherID(entity.getPublisherID());
                pubVector.add(pub);
            }
        }
        return pubVector;
    }

    /**
     * construct a Vector of Description objects from a given array of axis based Description objects
     */
    public java.util.Vector constructFTHPIS_DescVector(cgl.axis.services.uddi.
            uddi_schema.
            Description[] description_array) {
        java.util.Vector descVector = null;
        if (description_array != null) {
            descVector = new java.util.Vector();
            int length = description_array.length;
            for (int t = 0; t < length; t++) {
                cgl.axis.services.uddi.uddi_schema.Description description =
                        description_array[t];
                if (description != null) {
                    descVector.add(new org.apache.juddi.datatype.Description(
                            description.getValue().toString()));
                }
            }
        }
        return descVector;
    }

    /**
     * construct an array of FindQualifiers objects from a given axis based FindQualifiers object
     */
    public org.apache.juddi.datatype.request.FindQualifiers
            constructFTHPIS_FindQualifiers(cgl.axis.services.uddi.uddi_schema.FindQualifiers axis_findqualifiers) {
        org.apache.juddi.datatype.request.FindQualifiers fthpis_findQualifiers = null;
        //CONSTRUCT FINDQUALIFIERS
        if (axis_findqualifiers != null) {
            String[] findqualifier_array = axis_findqualifiers.getFindQualifier();
            fthpis_findQualifiers = new org.apache.juddi.datatype.request.
                                   FindQualifiers();
            java.util.Vector findQualifierVector = new java.util.Vector();
            if (findqualifier_array != null && findqualifier_array.length != 0) {
                int size = findqualifier_array.length;
                for (int i = 0; i < size; i++) {
                    String fieldname = findqualifier_array[i];
                    org.apache.juddi.datatype.request.FindQualifier
                            fthpis_findQualifier
                            = new org.apache.juddi.datatype.request.
                              FindQualifier(
                                      fieldname);
                    findQualifierVector.add(fthpis_findQualifier);
                }
                fthpis_findQualifiers.setFindQualifierVector(findQualifierVector);
            }
        }
        return fthpis_findQualifiers;
    }

    /**
     * construct a fthpis Lease object from a given axis based Lease object
     */
    public cgl.fthpis.datatype.lease.Lease constructFTHPIS_Lease(
            cgl.axis.services.uddi.uddi_ext_schema.Lease lease) {
        cgl.fthpis.datatype.lease.Lease fthpis_lease = null;
        if (lease != null) {
            fthpis_lease = new cgl.fthpis.datatype.lease.Lease();
            //this part is for insertion
            if (lease.isIsInfinite()) {
                fthpis_lease.setIsInfinite("true");
            } else {
                fthpis_lease.setIsInfinite("false");
                if (lease.getLeaseStartTimeStamp() != null) {
                    fthpis_lease.setLeaseTimeOut(lease.getLeaseStartTimeStamp());
                }
            }
            //this part for finding stuff
            if (lease.getLeaseExpirationTimeStamp() != null) {
                fthpis_lease.setLeaseExpirationDate(lease.
                        getLeaseExpirationTimeStamp());
            } else {
                long current_time = System.currentTimeMillis();
                fthpis_lease.setLeaseTimeOut((new Long(current_time)).toString());
            }
        }
        return fthpis_lease;
    }

}
