/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.axis.services.uddi_wscontext_client;

import java.util.Properties;

import cgl.axis.services.uddi_wscontext.stubs.*;
import cgl.axis.services.uddi.uddi_schema.*;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherDetail;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherEntity;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Save_publisher;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherInfo;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Find_publisher;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherList;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.PublisherInfos;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Delete_publisher;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Get_publisherDetail;
import cgl.util.BootStrap;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class AuthUserPublishExample {
    public AuthUserPublishExample() {
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // example values for session
    public static final String SESSION_NAME = "SESSION://C/D/E";
    public static final String SESSION_DESCRIPTION = "This is a X type Session";
    //Values for Find Qualifiers
    public static final String EXACT_NAME_MATCH = "exactNameMatch";
    public static final String CASE_SENSITIVE_MATCH = "caseSensitiveMatch";
    //Result limit
    public static final int RESULTS_MAX_ROWS_LIMIT = 100;



    UDDI_WSContextSoapBindingStub binding;

    /**
     * This function is to initilize binding object
    */
    public void setServiceBinding(Properties prop) {
        try {
            UDDI_WSContextServiceLocator locator = new
                    UDDI_WSContextServiceLocator();
            locator.setUDDI_WSContextEndpointAddress(prop.getProperty(
                    "UDDI_WSContext_WSDL"));
            System.out.println("wsdl = " + locator.getUDDI_WSContextAddress());
            binding = (UDDI_WSContextSoapBindingStub) locator.getUDDI_WSContext();
        } catch (javax.xml.rpc.ServiceException jre) {
            if (jre.getLinkedCause() != null) {
                jre.getLinkedCause().printStackTrace();
            }
            throw new junit.framework.AssertionFailedError(
                    "JAX-RPC ServiceException caught: " + jre);
        }
    }

    /**
     * This function is to get an authantication token from the WS-Context Store
     * number of service calls: 1 => get_authToken()
    */
    public AuthToken getAuthenticationToken(String userid, String cred) {
        AuthToken token = null;
        Get_authToken get_authToken = new Get_authToken();
        get_authToken.setUserID(userid);
        get_authToken.setCred(cred);
        try {
            token = binding.get_authToken(get_authToken);
        } catch (Exception ex) {
        }
        return token;
    }

    /**
     * This function is to save session to the WS-Context Store
     * number of service calls: 1 => save_session()
    */
    public PublisherDetail save_publisher(AuthToken token,
                                          String person_name,
                                          String publisherID,
                                          String publisherCred) {
        PublisherDetail detail = null;

        PublisherEntity[] pub_entity_array = new PublisherEntity[1];
        PublisherEntity pub_entity = new PublisherEntity();

        //username and credentials
        pub_entity.setPublisherID(publisherID);
        pub_entity.setPublisherCred(publisherCred);

        //set names
        PersonName [] personName_arr = new PersonName[1];
        PersonName personName = new PersonName();
        personName.setValue(person_name);
        personName_arr[0] = personName;
        pub_entity.setPersonName(personName_arr);

        pub_entity_array[0] = pub_entity;

        Save_publisher save_publisher = new Save_publisher();
        save_publisher.setAuthInfo(token.getAuthInfo());
        save_publisher.setPublisherEntity(pub_entity_array);

        try {
            detail = binding.save_publisher(save_publisher);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }

        return detail;
    }

    /**
     * This function is to find sessions in the WS-Context Store
     * number of service calls: 1 => find_session()
    */
    public PublisherInfo[] find_publisher(AuthToken token,
                                          String user_name,
                                          String qualifier) {

        PublisherInfo[] publisher_info_arr = null;

        //set names
        Name [] personName_arr = new Name[1];
        Name personName = new Name();
        personName.setValue(user_name);
        personName_arr[0] = personName;

        //set find qualifiers
        FindQualifiers find_qualifiers = new FindQualifiers();
        String[] qualifier_array = new String[2];
        qualifier_array[0] = qualifier;
        qualifier_array[1] = this.CASE_SENSITIVE_MATCH;
        find_qualifiers.setFindQualifier(qualifier_array);

        Find_publisher find_publisher = new Find_publisher();
        find_publisher.setAuthInfo(token.getAuthInfo());

        find_publisher.setName(personName_arr);

        find_publisher.setFindQualifiers(find_qualifiers);

        PublisherList list = null;
        try {
            list = binding.find_publisher(find_publisher);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }

        if (list != null) {
            PublisherInfos infos = list.getPublisherInfos();
            if (infos != null && infos.getPublisherInfo() != null) {
                publisher_info_arr = infos.getPublisherInfo();
            }
        }
        return publisher_info_arr;
    }

    /**
     * This function is to delete sessions from the WS-Context Store
     * number of service calls: 1 => delete_session()
    */
    public void deletePublisher(AuthToken token, String [] publisher_ID) {
        Delete_publisher delete_publisher = new Delete_publisher();
        delete_publisher.setAuthInfo(token.getAuthInfo());
        delete_publisher.setPublisherID(publisher_ID);

        try {
            binding.delete_publisher(delete_publisher);
        } catch (DispositionReport rep) {
            if (rep != null) {
                Result[] result_array = rep.getResult();
                if (result_array != null) {
                    if (result_array.length != 0) {
                        Result result = result_array[0];
                        ErrInfo err_info = result.getErrInfo();
                        if (err_info != null) {
                            System.out.println("error code = " +
                                               err_info.getErrCode());
                        }
                    }
                }
            }
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This function is to get session details from the WS-Context Store
     * number of service calls: 1 => get_sessionDetail()
    */
    public PublisherDetail getPublisherDetail(AuthToken token,
                                             String [] publisher_ID) {
        PublisherDetail detail = null;
        Get_publisherDetail get_publisherDetail = new Get_publisherDetail();
        get_publisherDetail.setAuthInfo(token.getAuthInfo());
        get_publisherDetail.setPublisherID(publisher_ID);
        try {
            detail = binding.get_publisherDetail(get_publisherDetail);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
        return detail;
    }

    /**
     * This function is to get session key from a given Session Detail object
    */
    public String getPublisherID(PublisherDetail detail) {
        String publisherID = null;
        if (detail != null) {
            PublisherEntity[] publisherEntity_arr = detail.getPublisherEntity();
            if (publisherEntity_arr != null && publisherEntity_arr.length != 0) {
                for (int i = 0; i < publisherEntity_arr.length; i++) {
                    PublisherEntity publisher = publisherEntity_arr[i];
                    if (publisher != null) {
                        publisherID = publisher.getPublisherID();
                    }
                }
            }
        }
        return publisherID;
    }

    /**
     * This function is to print AuthToken object to Console.
    */
    public void printToken(AuthToken token) {
        if (token != null) {
            AuthInfo authInfo = token.getAuthInfo();
            if (authInfo != null) {
                System.out.println("********* PRINT AuthToken ***********");
                System.out.println("authInfo = " + authInfo.getValue());
                System.out.println("publisherID = " + authInfo.getPublisherID());
            }
        }
    }

    /**
     * This function is to print session detail object to Console.
     */
    public void printPublisherDetail(PublisherDetail detail) {
        if (detail != null) {
            PublisherEntity[] publisher_arr = detail.getPublisherEntity();
            if (publisher_arr != null && publisher_arr.length != 0) {
                for (int i = 0; i < publisher_arr.length; i++) {
                    PublisherEntity publisher = publisher_arr[i];
                    if (publisher != null) {
                        System.out.println(
                                "********* PRINT SessionDetail ***********");
                        System.out.println("sessionKey = " +
                                           publisher.getPublisherID());
                        PersonName[] returned_name_array = publisher.getPersonName();
                        if (returned_name_array != null &&
                            returned_name_array.length != 0) {
                            System.out.println("user session identifer = " +
                                               returned_name_array[0].getValue());
                        }

                    }
                }
            }
        }
    }


    public void run() {
        Properties bootstrap = null;
        // Load bootsrap configuration
        try {
            bootstrap = BootStrap.loadProperties("properties.properties");
        } catch (Exception ex) {
        }

        //set binding
        setServiceBinding(bootstrap);

        //get authentication token
        AuthToken token = getAuthenticationToken(bootstrap.getProperty(
                "cgl.fthpis.jdbcUser"),
                                                 bootstrap.getProperty(
                "cgl.fthpis.jdbcPassword"));

        printToken(token);

        PublisherDetail detail = save_publisher(token,
                                                "John Doe",
                                                "jdoe",
                                                "changeIt");

        printPublisherDetail(detail);

        //find session
        PublisherInfo[] publisher_info_arr = find_publisher(token,
                                                            "John Doe",
                                                             this.EXACT_NAME_MATCH);

        String[] publisher_id_arr = null;
        if (publisher_info_arr != null && publisher_info_arr.length != 0) {
            //take a look at list as resulting list
            publisher_id_arr = new String[publisher_info_arr.length];
            for (int i = 0; i < publisher_info_arr.length; i++) {
                PublisherInfo info = publisher_info_arr[i];
                String publisherID = info.getPublisherID();
                System.out.println("publisherID = " + publisherID);
                publisher_id_arr[i] = publisherID;
            }

            //get the whole session with getsessiondetail
            PublisherDetail cdetail = getPublisherDetail(token, publisher_id_arr);
            printPublisherDetail(cdetail);
        }
//
////        //delete session
////        String publisherID = getPublisherID(detail);
////        String[] publisherid_arr = new String[1];
////        publisherid_arr[0] = publisherID;
////        deletePublisher(token, publisherid_arr);

    }

    public static void main(String args[]) {
        AuthUserPublishExample app = new AuthUserPublishExample();
        System.out.println(
                "\n*********** Running SessionPublishExample ***********");
        app.run();
        System.exit(0);
    }

    private void jbInit() throws Exception {
    }

}
