/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.axis.services.uddi_wscontext_client;

import cgl.axis.services.WSCTX.uddi_wsctx_schema.*;
import cgl.axis.services.uddi_wscontext.stubs.*;
import cgl.axis.services.uddi.uddi_schema.*;
import org.apache.axis.types.URI;
import java.util.Properties;
import cgl.util.BootStrap;
import cgl.axis.services.uddi.uddi_ext_schema.Lease;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class ContextFindExample {

    // Values for Session
    public static final String USER_DEFINED_IDENTIFIER = "context://user-defined-identifier";
    //Values for Find Qualifiers
    public static final String EXACT_NAME_MATCH = "exactNameMatch";
    public static final String CASE_SENSITIVE_MATCH = "caseSensitiveMatch";
    public static final String OR_ALL_KEYS = "orAllKeys";
    public static final String AND_ALL_KEYS = "andAllKeys";
    public static final String EQUAL_TO = "equalTo";
    public static final String NOT_EQUAL_TO = "notEqualTo";
    //Value Types
    public static final String NUMBER_VALUE_TYPE = "number";
    public static final String TEXT_VALUE_TYPE = "text";
    //Result limit
    public static final int RESULTS_MAX_ROWS_LIMIT = 100;

    Properties bootstrap = null;

    public static void main(String args[]) {
        ContextFindExample app = new ContextFindExample();
        System.out.println(
                "\n*********** Running SetContextExample ***********");
        app.run();
        System.exit(0);
    }

    public void run() {
        // Load bootsrap configuration
        try {
            bootstrap = BootStrap.loadProperties("properties.properties");
        } catch (Exception ex) {
        }
        try {

            UDDI_WSContextSoapBindingStub binding;
            try {
                UDDI_WSContextServiceLocator locator = new UDDI_WSContextServiceLocator();
                locator.setUDDI_WSContextEndpointAddress(bootstrap.getProperty("UDDI_WSContext_WSDL"));
                System.out.println("wsdl = " + locator.getUDDI_WSContextAddress());
                binding = (UDDI_WSContextSoapBindingStub) locator.getUDDI_WSContext();
            } catch (javax.xml.rpc.ServiceException jre) {
                if (jre.getLinkedCause() != null) {
                    jre.getLinkedCause().printStackTrace();
                }
                throw new junit.framework.AssertionFailedError(
                        "JAX-RPC ServiceException caught: " + jre);
            }
            Get_authToken get_authToken = new Get_authToken();
            get_authToken.setUserID(bootstrap.getProperty("cgl.fthpis.jdbcUser"));
            get_authToken.setCred(bootstrap.getProperty("cgl.fthpis.jdbcPassword"));
            AuthToken token = binding.get_authToken(get_authToken);
            AuthInfo authInfo = token.getAuthInfo();
            System.out.println("authInfo = " + authInfo.getValue());
            System.out.println("publisherID = " + authInfo.getPublisherID());

            System.out.println("\nSave '" + this.USER_DEFINED_IDENTIFIER + "'");

            //NAME
            Name[] namearray = new Name[1];
            Name name = new Name("%");
            namearray[0] = name;

            //TEST FIND SESSION
            FindQualifiers find_qualifiers = new FindQualifiers();
            String[] qualifier_array = new String[1];
            qualifier_array[0] = this.OR_ALL_KEYS;
            find_qualifiers.setFindQualifier(qualifier_array);
            URI s_key = null, c_key = null;
//            Find_session find_session = new Find_session();
//            find_session.setAuthInfo(authInfo);
//            find_session.setName(namearray);
//            find_session.setFindQualifiers(find_qualifiers);
//            SessionList list = null;
//            // Test operation
//            try {
//                list = binding.find_session(find_session);
//            } catch (DispositionReport e1) {
//                throw new junit.framework.AssertionFailedError(
//                        "DispositionReport Exception caught: " + e1);
//            }
//            if (list != null) {
//                SessionInfos infos = list.getSessionInfos();
//                if (infos != null || infos.getSessionInfo().length != 0) {
//                    SessionInfo info = infos.getSessionInfo(0);
//                    s_key = info.getSessionKey();
//                    System.out.println("Session key  = " + s_key);
//                }
//            }


            //TEST FIND CONTEXT
            FindQualifiers find_context_qualifiers = new FindQualifiers();
            String[] context_qualifier_array = new String[1];
            qualifier_array[0] = this.AND_ALL_KEYS;
            find_qualifiers.setFindQualifier(context_qualifier_array);

            Find_context find_context = new Find_context();

            find_context.setAuthInfo(authInfo);
            //find_context.setSessionKey(s_key);
            find_context.setName(namearray);
            find_context.setFindQualifiers(find_context_qualifiers);

            ContextList context_list = null;
            // Test operation
            try {
                context_list = binding.find_context(find_context);
            } catch (DispositionReport e1) {
                throw new junit.framework.AssertionFailedError(
                        "DispositionReport Exception caught: " + e1);
            }

            if (context_list != null) {
            ContextInfos infos = context_list.getContextInfos();
            if (infos != null && infos.getContextInfo().length != 0) {
                ContextInfo info = infos.getContextInfo(0);
                c_key = info.getContextKey();
                System.out.println("returned c_key = " + c_key);

                URI [] context_key_array = new URI[1];
                context_key_array[0] = c_key;

                Get_contextDetail get_contextDetail = new Get_contextDetail();
                get_contextDetail.setAuthInfo(authInfo);
                get_contextDetail.setContextKey(context_key_array);
                ContextDetail value = null;
                try {
                    value = binding.get_contextDetail(get_contextDetail);
                } catch (DispositionReport e1) {
                    throw new junit.framework.AssertionFailedError(
                            "DispositionReport Exception caught: " + e1);
                }
                if (value != null) {
                    ContextBag contextBag = value.getContextBag();

                    Context[] context_array = null;
                    if (contextBag != null && contextBag.getContext() != null) {
                        context_array = contextBag.getContext();
                        c_key = context_array[0].getContextKey();

                        System.out.println("saved c_key = " +
                                           context_array[0].getContextKey());
                        Name [] context_name_array = context_array[0].getName();
                        if (context_name_array != null) {
                            System.out.println("user identifer = " +
                                               context_name_array[0].getValue());
                        }
                        System.out.println("session key = " +
                                           context_array[0].getSessionKey());

                        Lease context_lease = context_array[0].getLease();
                        if (context_lease != null) {
                            System.out.println("start time = " +
                                               context_lease.getLeaseStartTimeStamp());
                            System.out.println("stop  time = " +
                                               context_lease.getLeaseExpirationTimeStamp());
                            System.out.println("timeout    = " +
                                               context_lease.getTimeout());
                            System.out.println("isinfinite = " +
                                               context_lease.isIsInfinite());
                        }
                        System.out.println("context version = " +
                                           context_array[0].getVersion());
                    }

//            //TEST DELETE BUSINESS
//
//            Delete_context delete_context = new Delete_context();
//
//            URI[] c_key_array = new URI[1];
//            c_key_array[0] = c_key;
//
//            delete_context.setAuthInfo(authInfo);
//            delete_context.setContextKey(c_key_array);
//
//            // Test operation
//            DispositionReport report = null;
//            try {
//                report = binding.delete_context(delete_context);
//            } catch (DispositionReport rep) {
//                //rep.printStackTrace();
//                if (rep != null) {
//                    Result[] result_array = rep.getResult();
//                    if (result_array != null) {
//                        if (result_array.length != 0) {
//                            Result result = result_array[0];
//                            ErrInfo err_info = result.getErrInfo();
//                            System.out.println("error code = " +
//                                               err_info.getErrCode());
//                        }
//                    }
//                }
//            }
                }
            }
        }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
