/*
 * Copyrights 2001-2004 CGL.
 *
 */

package cgl.axis.services.uddi_wscontext_client;

import cgl.axis.services.WSCTX.uddi_wsctx_schema.*;
import cgl.axis.services.uddi_wscontext.stubs.*;
import cgl.axis.services.uddi.uddi_schema.*;
import org.apache.axis.types.URI;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Save_session;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionEntity;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionDetail;
import cgl.axis.services.uddi.uddi_ext_schema.Lease;
import java.util.Properties;
import cgl.util.BootStrap;
import java.util.Vector;
import java.util.*;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class ContextPublishExample {

    // Values for Session
    public static final String USER_DEFINED_IDENTIFIER =
            "context://user-defined-identifier";
    public static final String CONTEXT_DESCRIPTION = "Context description here";
    //Values for Find Qualifiers
    public static final String EXACT_NAME_MATCH = "exactNameMatch";
    public static final String CASE_SENSITIVE_MATCH = "caseSensitiveMatch";

    //Value Types
    public static final String NUMBER_VALUE_TYPE = "number";
    public static final String TEXT_VALUE_TYPE = "text";
    //Result limit
    public static final int RESULTS_MAX_ROWS_LIMIT = 100;

    Properties bootstrap = null;

    UDDI_WSContextSoapBindingStub binding;

    /**
     * This function is to initilize binding object
    */
    public void setServiceBinding(Properties prop) {
        try {
            UDDI_WSContextServiceLocator locator = new
                    UDDI_WSContextServiceLocator();
            locator.setUDDI_WSContextEndpointAddress(bootstrap.getProperty(
                    "UDDI_WSContext_WSDL"));
            System.out.println("wsdl = " + locator.getUDDI_WSContextAddress());
            binding = (UDDI_WSContextSoapBindingStub) locator.getUDDI_WSContext();
        } catch (javax.xml.rpc.ServiceException jre) {
            if (jre.getLinkedCause() != null) {
                jre.getLinkedCause().printStackTrace();
            }
            throw new junit.framework.AssertionFailedError(
                    "JAX-RPC ServiceException caught: " + jre);
        }
    }

    /**
     * This function is to get an authantication token from the WS-Context Store
     * number of service calls: 1 => get_authToken()
    */
    public AuthToken getAuthenticationToken(String userid, String cred) {
        AuthToken token = null;
        Get_authToken get_authToken = new Get_authToken();
        get_authToken.setUserID(userid);
        get_authToken.setCred(cred);
        try {
            token = binding.get_authToken(get_authToken);
        } catch (Exception ex) {
        }
        return token;
    }

    /**
     * This function is to save session to the WS-Context Store
     * number of service calls: 1 => save_session()
    */
    public SessionDetail saveSession(AuthToken token,
                                      String user_session_identifier,
                                      String session_description,
                                      String[] child_node_user_key_arr) {
        SessionDetail detail = null;
        //set names
        Name[] namearray = new Name[1];
        namearray[0] = new Name(user_session_identifier);

        //set descriptions
        Description[] session_description_array = new Description[2];
        Description s_description = new Description(session_description);
        session_description_array[0] = s_description;

        //set lease criteria
        Lease lease = new Lease();
        lease.setIsInfinite(true);

        //construct session_directory metadata
        Context context = new Context();
        Name[] context_name_arr = new Name[1];
        context_name_arr[0] = new Name("session://session_directory_metadata");
        context.setName(context_name_arr);

        ////////////////////////////////////////////////////////////
        Vector vec = new Vector();
        if (!vec.contains(user_session_identifier))
        vec.add(user_session_identifier);
        if (child_node_user_key_arr != null) {
            for (int i = 0; i < child_node_user_key_arr.length; i++) {
                String key = child_node_user_key_arr[i];
                vec.add(key);
            }
        }
        byte[] array = marshallUserKeys(vec);
        ////////////////////////////////////////////////////////////
        context.setValue(array);

        //set session_metadata
        ContextBag contextBag = new ContextBag();
        Context[] context_arr = new Context[1];
        context_arr[0] = context;
        contextBag.setContext(context_arr);

        SessionEntity entity = new SessionEntity();
        entity.setName(namearray);
        entity.setDescription(session_description_array);
        entity.setLease(lease);
        entity.setContextBag(contextBag);

        SessionEntity[] entity_array = new SessionEntity[1];
        entity_array[0] = entity;

        Save_session save_session = new Save_session();
        save_session.setAuthInfo(token.getAuthInfo());
        save_session.setSessionEntity(entity_array);

        try {
            detail = binding.save_session(save_session);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }

        return detail;
    }

    /**
     * This function is to save Context to the WS-Context Store
     * number of service calls: 1 => saveContext()
    */
    public ContextDetail saveContext(AuthToken token,
                                     String user_identifier,
                                     URI session_key,
                                     URI context_key,
                                     byte [] context_byte_array) {
        ContextDetail value = null;

        //construct context entity
        Context context = new Context();

        //set authorization info
        AuthInfo authInfo = token.getAuthInfo();

        //set user identifier
        Name[] user_identifier_arr = new Name[1];
        user_identifier_arr[0] = new Name(user_identifier);

        //set context lease
        Lease lease = new Lease();
        lease.setIsInfinite(true);

        context.setContextKey(context_key);
        context.setName(user_identifier_arr);
        context.setValue(context_byte_array);
        context.setLease(lease);
        context.setSessionKey(session_key);

        ContextBag contextBag = new ContextBag();
        Context[] context_array = new Context[1];
        context_array[0] = context;
        contextBag.setContext(context_array);

        Save_context save_context = new Save_context();
        save_context.setAuthInfo(authInfo);
        save_context.setContextBag(contextBag);

        try {
            value = binding.save_context(save_context);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
        return value;
    }

    /**
     * This function is to delete Context from the WS-Context Store
     * number of service calls: 1 => delete_context()
    */
    public void deleteContext(AuthToken token,
                              URI[] context_key_arr) {
        Delete_context delete_context = new Delete_context();
        delete_context.setAuthInfo(token.getAuthInfo());
        delete_context.setContextKey(context_key_arr);

        try {
            binding.delete_context(delete_context);
        } catch (DispositionReport rep) {
            if (rep != null) {
                Result[] result_array = rep.getResult();
                if (result_array != null) {
                    if (result_array.length != 0) {
                        Result result = result_array[0];
                        ErrInfo err_info = result.getErrInfo();
                        if (err_info != null) {
                            System.out.println("error code = " +
                                               err_info.getErrCode());
                        }
                    }
                }
            }
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This function is to find Context in the WS-Context Store
     * number of service calls: 1 => find_context()
    */
    public ContextInfo[] find_context(AuthToken token,
                                      URI sessionkey,
                                      String user_identifier,
                                      String [] find_qualifier_arr) {
        ContextInfo[] context_info_arr = null;

        //set names
        Name[] namearray = new Name[1];
        namearray[0] = new Name(user_identifier);

        //set findqualifiers
        FindQualifiers find_qualifiers = new FindQualifiers();

        find_qualifiers.setFindQualifier(find_qualifier_arr);

        Find_context find_context = new Find_context();

        find_context.setAuthInfo(token.getAuthInfo());
        find_context.setSessionKey(sessionkey);
        find_context.setName(namearray);
        find_context.setFindQualifiers(find_qualifiers);

        ContextList list = null;
        try {
            list = binding.find_context(find_context);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }

        if (list != null) {
            ContextInfos infos = list.getContextInfos();
            if (infos != null && infos.getContextInfo() != null) {
                context_info_arr = infos.getContextInfo();
            }
        }
        return context_info_arr;
    }

    /**
     * This function is to retrieve user defined keys for the child sessions
     * of a given session.
     * number of service calls: 2 => find_context, get_contextDetail
    */
    public String[] getSessionChildUserIDs(AuthToken token, URI sessionKey) {
        String[] session_child_user_keys = null;
        //find context
        String[] qualifier_array = new String[2];
        qualifier_array[0] = this.EXACT_NAME_MATCH;
        qualifier_array[1] = this.CASE_SENSITIVE_MATCH;
        ContextInfo[] context_info_arr = find_context(token,
                sessionKey,
                "session://session_directory_metadata",
                qualifier_array);
        URI[] context_key_arr = null;
        if (context_info_arr != null && context_info_arr.length != 0) {
            context_key_arr = new URI[context_info_arr.length];
            for (int i = 0; i < context_info_arr.length; i++) {
                ContextInfo info = context_info_arr[i];
                URI context_key = info.getContextKey();
                context_key_arr[i] = context_key;
            }

            ContextDetail cdetail = getContextDetail(token, context_key_arr);
            ContextBag contextBag = cdetail.getContextBag();
            Context session_metadata_context = contextBag.getContext()[0];
            byte [] b_array = session_metadata_context.getValue();
            String payload = new String(b_array);

            Vector vec = unMarshallUserKeys(payload);
            //Vector vec = (Vector) deserializeObject(b_array);

            if (vec != null && vec.size() != 0) {
                session_child_user_keys = new String[vec.size()];
                int i = 0;
                for (Iterator iter = vec.iterator(); iter.hasNext(); ) {
                    String item = (String) iter.next();
                    session_child_user_keys[i] = item;
                    i++;
                }
            }
        }
        return session_child_user_keys;
    }

    /**
     * This function is to add a child node to a session within the
     * Session Metadata (coressponding to a AccessList in Linux file system).
     * number of service calls: 3 => find_context, get_contextDetail, saveContext
    */
    public void addSessionChildren(AuthToken token,
                                   URI sessionKey,
                                   String[] child_user_keys) {
        String[] qualifier_array = new String[2];
        qualifier_array[0] = this.EXACT_NAME_MATCH;
        qualifier_array[1] = this.CASE_SENSITIVE_MATCH;
        ContextInfo[] context_info_arr = find_context(token,
                sessionKey,
                "session://session_directory_metadata",
                qualifier_array);
        URI[] context_key_arr = null;
        if (context_info_arr != null && context_info_arr.length != 0) {
            context_key_arr = new URI[context_info_arr.length];
            for (int i = 0; i < context_info_arr.length; i++) {
                ContextInfo info = context_info_arr[i];
                URI context_key = info.getContextKey();
                context_key_arr[i] = context_key;
            }

            ContextDetail cdetail = getContextDetail(token, context_key_arr);
            ContextBag contextBag = cdetail.getContextBag();
            Context session_metadata_context = contextBag.getContext()[0];

            byte[] b_array = session_metadata_context.getValue();
            String keyname = new String(b_array);
            Vector tokenVector = unMarshallUserKeys(keyname);

            for (int i = 0; i < child_user_keys.length; i++) {
                String key = child_user_keys[i];
                if (!tokenVector.contains(key)) {
                    tokenVector.add(key);
                }
            }
            byte[] array = marshallUserKeys(tokenVector);
            session_metadata_context.setValue(array);
            ContextBag cBag = new ContextBag();
            Context[] context_array = new Context[1];
            context_array[0] = session_metadata_context;
            cBag.setContext(context_array);

            Save_context save_context = new Save_context();
            save_context.setAuthInfo(token.getAuthInfo());
            save_context.setContextBag(cBag);
            try {
                binding.save_context(save_context);
            } catch (DispositionReport e1) {
                throw new junit.framework.AssertionFailedError(
                        "DispositionReport Exception caught: " + e1);
            } catch (java.rmi.RemoteException e) {
                e.printStackTrace();
        }
        }
    }

    /**
     * This function is to remove child nodes from a session within the
     * Session Metadata (coressponding to a AccessList in Linux file system).
     * number of service calls: 3 => find_context, get_contextDetail, saveContext
    */
   public void removeSessionChildren(AuthToken token,
                                     URI sessionKey,
                                     String[] child_user_keys) {

       String directory_access_metadata = "session://session_directory_metadata";
       String[] qualifier_array = new String[2];
       qualifier_array[0] = this.EXACT_NAME_MATCH;
       qualifier_array[1] = this.CASE_SENSITIVE_MATCH;
       ContextInfo[] context_info_arr = find_context(token,
                                                     sessionKey,
                                                     directory_access_metadata,
                                                     qualifier_array);
       URI[] context_key_arr = null;
       if (context_info_arr != null && context_info_arr.length != 0) {
           context_key_arr = new URI[context_info_arr.length];
           for (int i = 0; i < context_info_arr.length; i++) {
               ContextInfo info = context_info_arr[i];
               URI context_key = info.getContextKey();
               context_key_arr[i] = context_key;
           }
           ContextDetail cdetail = getContextDetail(token, context_key_arr);
           ContextBag contextBag = cdetail.getContextBag();
           Context session_metadata_context = contextBag.getContext()[0];

           byte[] b_array = session_metadata_context.getValue();
           String payload = new String(b_array);

           Vector vec = unMarshallUserKeys(payload);
           for (int i = 0; i < child_user_keys.length; i++) {
               String key = child_user_keys[i];
               vec.remove(key);
           }
           byte[] array = marshallUserKeys(vec);
           session_metadata_context.setValue(array);
           saveContext(token,
                       directory_access_metadata,
                       sessionKey,
                       null,
                       array);
       }
    }

    /**
     * This function is to get Session Metadata (coressponding to a AccessList
     * in Linux file system) from WS-Context Service. From this context value
     * you can simply learn the child nodes of this given session.
     * number of service calls: find_context, get_contextDetail (2)
    */
    public Context getSessionDirectoryMetadata(AuthToken token, URI sessionKey) {
        Context context = null;
        //find context
        String[] qualifier_array = new String[2];
        qualifier_array[0] = this.EXACT_NAME_MATCH;
        qualifier_array[1] = this.CASE_SENSITIVE_MATCH;
        ContextInfo[] context_info_arr = find_context(token,
                                                      sessionKey,
                                                      "session://session_directory_metadata",
                                                      qualifier_array);
        URI[] context_key_arr = null;
        if (context_info_arr != null && context_info_arr.length != 0) {
            //take a look at list as resulting list
            context_key_arr = new URI[context_info_arr.length];
            for (int i = 0; i < context_info_arr.length; i++) {
                ContextInfo info = context_info_arr[i];
                URI context_key = info.getContextKey();
                context_key_arr[i] = context_key;
            }

            //get the whole context with getcontextdetail
            ContextDetail cdetail = getContextDetail(token, context_key_arr);
            ContextBag contextBag = cdetail.getContextBag();
            context = contextBag.getContext()[0];
        }
        return context;
    }

    /**
     * This function is to get Context Detail from WS-Context Service
     * number of service call: 1 => get_contextDetail()
    */
    public ContextDetail getContextDetail(AuthToken token,
                                          URI[] context_key_arr) {
        ContextDetail detail = null;
        Get_contextDetail get_contextDetail = new Get_contextDetail();
        get_contextDetail.setAuthInfo(token.getAuthInfo());
        get_contextDetail.setContextKey(context_key_arr);
        try {
            detail = binding.get_contextDetail(get_contextDetail);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
        return detail;
    }

    /**
     * This function is to retrieve "first" sessionKey from a given
     * session Detail object. In most cases you might store only
     * one session, so get only session detail in return. This is
     * to extract session key associated to that session detail
     */
    public URI getSessionKey(SessionDetail detail) {
        URI sessionkey = null;
        if (detail != null) {
            SessionEntity[] session_arr = detail.getSessionEntity();
            if (session_arr != null && session_arr.length != 0) {
                SessionEntity session = session_arr[0];
                sessionkey = session.getSessionKey();
//                for (int i = 0; i < session_arr.length; i++) {
//                    SessionEntity session = session_arr[i];
//                    if (session != null) {
//                        sessionkey = session.getSessionKey();
//                    }
//                }
            }
        }
        return sessionkey;
    }

    /**
     * This function is to retrieve "first" contextKey from a given
     * context Detail object. In most cases you might store only
     * one context, so get only context detail in return. This is
     * to extract context key associated to that context detail
     */
    public URI getContextKey(ContextDetail detail) {
        URI context_key = null;
        if (detail != null) {
            ContextBag contextBag = detail.getContextBag();
            if (contextBag != null &&
                contextBag.getContext() != null &&
                contextBag.getContext().length != 0) {
                Context[] context_arr = contextBag.getContext();
                Context context = context_arr[0];
                context_key = context.getContextKey();
//                for (int i = 0; i < context_arr.length; i++) {
//                    Context context = context_arr[i];
//                    if (context != null) {
//                        context_key = context.getContextKey();
//                    }
//                }
            }
        }
        return context_key;
    }

    /**
     * This function is to combine all userdefined keys in a string
     * and it returns a byte array corresponding to that string
     * e.g. String s = "vlb://session1 vlb://session2";
    */
    public byte [] marshallUserKeys(Vector vec) {
        String result = "";
        for (Iterator iter = vec.iterator(); iter.hasNext(); ) {
            String item = (String) iter.next();
            result = result + item + " ";
        }
        result = result.trim();
        return result.getBytes();
    }

    /**
     * This function is to unmarshall a string to retrieve the
     * user defined keys within a Vector
     * e.g. {vlb://session1, vlb://session2}
    */
   public Vector unMarshallUserKeys(String keyname) {
       Vector vec = null;
       StringTokenizer st;
       st = new StringTokenizer(keyname, " ");
       vec = new Vector();
       while (st.hasMoreTokens()) {
         String string = st.nextToken();
         vec.add(string);
      }
       return vec;
   }

   /**
    * This function is to print AuthToken object to Console.
    */
    public void printToken(AuthToken token) {
        if (token != null) {
            AuthInfo authInfo = token.getAuthInfo();
            if (authInfo != null) {
                System.out.println("********* PRINT AuthToken ***********");
                System.out.println("authInfo = " + authInfo.getValue());
                System.out.println("publisherID = " + authInfo.getPublisherID());
            }
        }
    }

    /**
     * This function is to print context object to Console.
     */
    public void printContext(Context context) {
        if (context != null) {
            System.out.println(
                    "********* PRINT Context ***********");
            System.out.println("saved context_key = " +
                               context.getContextKey());
            System.out.println("context operation type = " +
                               context.getValueType());
            Name[] returned_name_array = context.getName();
            if (returned_name_array != null &&
                returned_name_array.length != 0) {
                System.out.println("context user identifer = " +
                                   returned_name_array[0].getValue());
            }
            System.out.println("context session key = " +
                               context.getSessionKey());
            String data = new String(context.getValue());
            System.out.println("context DATA = " + data);
        }
    }

    /**
     * This function is to print context detail object to Console.
     */
    public void printContextDetail(ContextDetail detail) {
        if (detail != null) {
            ContextBag contextBag = detail.getContextBag();
            if (contextBag != null &&
                contextBag.getContext() != null &&
                contextBag.getContext().length != 0) {
                Context[] context_arr = contextBag.getContext();
                for (int i = 0; i < context_arr.length; i++) {
                    Context context = context_arr[i];
                    if (context != null) {
                        System.out.println(
                                "********* PRINT Context ***********");
                        System.out.println("saved context_key = " +
                                           context.getContextKey());
                        Name[] returned_name_array = context.getName();
                        if (returned_name_array != null &&
                            returned_name_array.length != 0) {
                            System.out.println("context user identifer = " +
                                               returned_name_array[0].getValue());
                        }
                        System.out.println("context session key = " +
                                           context.getSessionKey());
                        String data = new String(context.getValue());
                        System.out.println("context DATA = " + data);
                    }
                }
            } else {
                System.out.println("No corresponding context is retrieved!!!");

            }
        }
    }

    /**
     * This function is to print session detail object to Console.
     */
    public void printSessionDetail(SessionDetail detail) {
        if (detail != null) {
            SessionEntity[] session_arr = detail.getSessionEntity();
            if (session_arr != null && session_arr.length != 0) {
                for (int i = 0; i < session_arr.length; i++) {
                    SessionEntity session = session_arr[i];
                    if (session != null) {
                        System.out.println(
                                "********* PRINT SessionDetail ******");
                        System.out.println("sessionKey = " +
                                           session.getSessionKey());
                        Name[] returned_name_array = session.getName();
                        if (returned_name_array != null &&
                            returned_name_array.length != 0) {
                            System.out.println("user session identifer = " +
                                               returned_name_array[0].getValue());
                        }
                        Description[] returned_desc_array = session.
                                getDescription();
                        if (returned_desc_array != null &&
                            returned_desc_array.length != 0) {
                            System.out.println("session description = " +
                                               returned_desc_array[0].getValue());
                        }
                        ContextBag contextbag = session.getContextBag();
                        if (contextbag != null && contextbag.getContext() != null) {
                            int size = contextbag.getContext().length;
                            for (int j = 0; j < size; j++) {
                                Context context = contextbag.getContext()[j];
                                if (context != null) {
                                    System.out.println("saved context_key = " +
                                            context.getContextKey());
                                    if (context.getValueType() != null) {
                                        System.out.println(
                                                "context operation type = " +
                                                context.getValueType());
                                    }
                                    returned_name_array = context.getName();
                                    if (returned_name_array != null &&
                                        returned_name_array.length != 0) {
                                        System.out.println(
                                                "context user identifer = " +
                                                returned_name_array[0].getValue());
                                    }
                                    System.out.println("context session key = " +
                                            context.getSessionKey());
                                    String data = new String(context.getValue());
                                    System.out.println("context DATA = " + data);
                                }
                            }
                        }
                        System.out.println("************************************");
                    }
                }
            } else {
                System.out.println("No corresponding session is found!!!");
            }
        }
    }

    /**
     * This function is to find sessions in the WS-Context Store
     * number of service calls: 1 => find_session()
    */
    public SessionInfo[] find_session(AuthToken token,
                                      String user_identifier,
                                      String qualifier) {
        SessionInfo[] session_info_arr = null;

        //set names
        Name[] namearray = new Name[1];
        namearray[0] = new Name(user_identifier);

        //set find qualifiers
        FindQualifiers find_qualifiers = new FindQualifiers();
        String[] qualifier_array = new String[2];
        qualifier_array[0] = qualifier;
        qualifier_array[1] = this.CASE_SENSITIVE_MATCH;
        find_qualifiers.setFindQualifier(qualifier_array);

        Find_session find_session = new Find_session();
        find_session.setAuthInfo(token.getAuthInfo());
        find_session.setName(namearray);
        find_session.setFindQualifiers(find_qualifiers);

        SessionList list = null;
        try {
            list = binding.find_session(find_session);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }

        if (list != null) {
            SessionInfos infos = list.getSessionInfos();
            if (infos != null && infos.getSessionInfo() != null) {
                session_info_arr = infos.getSessionInfo();
            }
        }
        return session_info_arr;
    }

    /**
     * This function is to delete sessions from the WS-Context Store
     * number of service calls: 1 => delete_session()
    */
    public void deleteSession(AuthToken token, URI[] session_key_arr) {
        Delete_session delete_session = new Delete_session();
        delete_session.setAuthInfo(token.getAuthInfo());
        delete_session.setSessionKey(session_key_arr);

        try {
            binding.delete_session(delete_session);
        } catch (DispositionReport rep) {
            if (rep != null) {
                Result[] result_array = rep.getResult();
                if (result_array != null) {
                    if (result_array.length != 0) {
                        Result result = result_array[0];
                        ErrInfo err_info = result.getErrInfo();
                        System.out.println("error code = " +
                                           err_info.getErrCode());
                    }
                }
            }
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
    }

    /**
     * This function is to get session details from the WS-Context Store
     * number of service calls: 1 => get_sessionDetail()
    */
    public SessionDetail getSessionDetail(AuthToken token,
                                          URI[] session_key_arr) {
        SessionDetail detail = null;
        Get_sessionDetail get_sessionDetail = new Get_sessionDetail();
        get_sessionDetail.setAuthInfo(token.getAuthInfo());
        get_sessionDetail.setSessionKey(session_key_arr);
        try {
            detail = binding.get_sessionDetail(get_sessionDetail);
        } catch (DispositionReport e1) {
            throw new junit.framework.AssertionFailedError(
                    "DispositionReport Exception caught: " + e1);
        } catch (java.rmi.RemoteException e) {
            e.printStackTrace();
        }
        return detail;
    }

    public void wipeOut(AuthToken token, String session_id) {
        //find session
        SessionInfo[] session_info_arr = find_session(token,
                session_id, this.CASE_SENSITIVE_MATCH);

        URI[] session_key_arr = null;
        if (session_info_arr != null && session_info_arr.length != 0) {
            session_key_arr = new URI[session_info_arr.length];
            for (int i = 0; i < session_key_arr.length; i++) {
                SessionInfo info = session_info_arr[i];
                URI[] sessionkey_arr = new URI[1];
                sessionkey_arr[0] = info.getSessionKey();
                deleteSession(token, sessionkey_arr);
            }
        }
    }


    public void run() {
        // Load bootsrap configuration
        try {
            bootstrap = BootStrap.loadProperties("properties.properties");
        } catch (Exception ex) {
        }
        //set binding
        setServiceBinding(bootstrap);

        //get authentication token
        AuthToken token = getAuthenticationToken(bootstrap.getProperty(
                "cgl.fthpis.jdbcUser"),
                                                 bootstrap.getProperty(
                "cgl.fthpis.jdbcPassword"));
        //print token
        printToken(token);

        //let's wipe out all session with this name from database
        //at the startup //of this example.
        wipeOut(token, "vlab_test1://");

        String session_id = "vlab_test1://";
        String dec = "description here";

        //save session
        SessionDetail session_detail = saveSession(token,
                                                    session_id,
                                                    dec,
                                                    null);
        System.out.println("************************************");
        System.out.println("SNAPSHOT OF SessionDetail after save_Session");
        //print session detail
        printSessionDetail(session_detail);

        URI sessionkey = null;
//        try {
//            sessionkey = new URI(
//                    "uuid:EE91FBF0-6FFB-11DA-A022-C6453A8B4EE7-1134933879109");
//        } catch (Exception ex) {
//        }

        //get session key
        sessionkey = getSessionKey(session_detail);

        String[]  user_keys = getSessionChildUserIDs(token, sessionkey);

        System.out.println("************************************");
        System.out.println("SNAPSHOT OF THE CHILD NODE POINTER - I ");

        if (user_keys != null && user_keys.length != 0) {
            for (int i = 0; i < user_keys.length; i++) {
                System.out.println("user_keys[" + i + "]" + "=" + user_keys[i]);
            }
        }

        String [] additional_keys = new String[1];
        additional_keys[0] = "vlab_test1://a";
        addSessionChildren(token, sessionkey, additional_keys);
        user_keys = getSessionChildUserIDs(token, sessionkey);

        System.out.println("************************************");
        System.out.println("SNAPSHOT OF THE CHILD NODE POINTER - II ");
        if (user_keys != null && user_keys.length != 0) {
            for (int i = 0; i < user_keys.length; i++) {
                System.out.println("user_keys[" + i + "]" + "=" + user_keys[i]);
            }
        }

        // let's get updated session information
        URI[] sessionkey_arr = new URI[1];
        sessionkey_arr[0] = sessionkey;
        SessionDetail updated_session_detail = getSessionDetail(token, sessionkey_arr);

        System.out.println("************************************");
        System.out.println("SNAPSHOT OF SessionDetail after the updates on child session pointer");
        //print updated session detail
        printSessionDetail(updated_session_detail);

        //print session_directory to learn the child sessions.
        //Context session_metadata = getSessionDirectoryMetadata(token, sessionkey);
        //print session_directory to learn the child sessions.
        //printContext(session_metadata);


        //set context
        ContextDetail detail = saveContext(token,
                                           this.USER_DEFINED_IDENTIFIER,
                                           sessionkey,
                                           null,
                                           "data".getBytes());

        System.out.println("************************************");
        System.out.println("SNAPSHOT OF ContextDetail after the saveContext");
        printContextDetail(detail);

        URI ckey = getContextKey(detail);
        //set context
        detail = saveContext(token,
                             this.USER_DEFINED_IDENTIFIER,
                             sessionkey,
                             ckey,
                             "updated_data".getBytes());

        System.out.println("************************************");
        System.out.println("SNAPSHOT OF ContextDetail after the updating a Context with saveContext");
        printContextDetail(detail);
        //set qualifiers
        String [] qualifier_arr = new String[2];
        qualifier_arr[0] = this.EXACT_NAME_MATCH;
        qualifier_arr[1] = this.CASE_SENSITIVE_MATCH;

        //find context
        ContextInfo[] context_info_arr = find_context(token,
                                                      sessionkey,
                                                      this.USER_DEFINED_IDENTIFIER,
                                                      qualifier_arr);

        URI[] context_key_arr = null;
        if (context_info_arr != null && context_info_arr.length != 0) {
            //take a look at list as resulting list
            context_key_arr = new URI[context_info_arr.length];
            System.out.println("*******find_context results*********");
            for (int i = 0; i < context_info_arr.length; i++) {
                ContextInfo info = context_info_arr[i];
                URI context_key = info.getContextKey();
                System.out.println("context_key = " + context_key);
                Name[] user_identifier_arr = info.getName();
                Name user_identifier = user_identifier_arr[0];
                if (user_identifier != null) {
                    System.out.println("context user key = " +
                                       user_identifier.getValue());
                }
                context_key_arr[i] = context_key;
            }

            //get the whole context with getcontextdetail
            ContextDetail cdetail = getContextDetail(token, context_key_arr);
            printContextDetail(cdetail);
        }


        //delete context
        URI contextkey = getContextKey(detail);
        URI[] contextkey_arr = new URI[1];
        contextkey_arr[0] = contextkey;
        System.out.println("************************************");
        System.out.println("Let's delete the context with key of " + contextkey_arr[0]);
        deleteContext(token, contextkey_arr);

        System.out.println("************************************");
        System.out.println("Let' s check if the context with key of " + contextkey_arr[0] + " is deleted!");
        //get the whole context with getcontextdetail
        ContextDetail cdetail = getContextDetail(token, context_key_arr);
        printContextDetail(cdetail);

        //delete session
        System.out.println("************************************");
        System.out.println("Let's delete the session with key of " + sessionkey_arr[0]);
        deleteSession(token, sessionkey_arr);

        SessionDetail re_updated_session_detail = getSessionDetail(token, sessionkey_arr);
        System.out.println("************************************");
        System.out.println("Let' s check if the session with key of " + sessionkey_arr[0] + " is deleted!");
        //print session detail

        printSessionDetail(re_updated_session_detail);


    }

    public static void main(String args[]) {
        ContextPublishExample app = new ContextPublishExample();
        System.out.println(
                "\n*********** Running ContextPublishExample ***********");
        app.run();
        System.exit(0);
    }
}
