/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package cgl.axis.services.uddi_wscontext_client;

import cgl.axis.services.WSCTX.uddi_wsctx_schema.*;
import cgl.axis.services.uddi_wscontext.stubs.*;
import cgl.axis.services.uddi.uddi_schema.*;
import org.apache.axis.types.URI;
import java.util.Properties;
import cgl.util.BootStrap;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Save_session;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionEntity;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionDetail;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Find_session;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionList;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionInfos;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionInfo;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Delete_session;
import cgl.axis.services.uddi.uddi_ext_schema.Lease;
import cgl.axis.services.uddi.uddi_ext_schema.ServiceAttribute;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class GetContextExample {

    // Values for Session
    //public static final String USER_DEFINED_IDENTIFIER = "context://user-defined-identifier";
    //Values for Find Qualifiers
    public static final String EXACT_NAME_MATCH = "exactNameMatch";
    public static final String CASE_SENSITIVE_MATCH = "caseSensitiveMatch";
    public static final String OR_ALL_KEYS = "orAllKeys";
    public static final String AND_ALL_KEYS = "andAllKeys";
    public static final String EQUAL_TO = "equalTo";
    public static final String NOT_EQUAL_TO = "notEqualTo";
    //Value Types
    public static final String NUMBER_VALUE_TYPE = "number";
    public static final String TEXT_VALUE_TYPE = "text";
    //Result limit
    public static final int RESULTS_MAX_ROWS_LIMIT = 100;

    Properties bootstrap = null;

    public static void main(String args[]) {
        GetContextExample app = new GetContextExample();
        System.out.println(
                "\n*********** Running GetContextExample ***********");
        app.run();
        System.exit(0);
    }

    public void run() {
        // Load bootsrap configuration
        try {
            bootstrap = BootStrap.loadProperties("properties.properties");
        } catch (Exception ex) {
        }
        try {

            UDDI_WSContextSoapBindingStub binding;
            try {
                UDDI_WSContextServiceLocator locator = new UDDI_WSContextServiceLocator();
                locator.setUDDI_WSContextEndpointAddress(bootstrap.getProperty("UDDI_WSContext_WSDL"));
                System.out.println("wsdl = " + locator.getUDDI_WSContextAddress());
                binding = (UDDI_WSContextSoapBindingStub) locator.getUDDI_WSContext();
            } catch (javax.xml.rpc.ServiceException jre) {
                if (jre.getLinkedCause() != null) {
                    jre.getLinkedCause().printStackTrace();
                }
                throw new junit.framework.AssertionFailedError(
                        "JAX-RPC ServiceException caught: " + jre);
            }
            Get_authToken get_authToken = new Get_authToken();
            get_authToken.setUserID(bootstrap.getProperty("cgl.fthpis.jdbcUser"));
            get_authToken.setCred(bootstrap.getProperty("cgl.fthpis.jdbcPassword"));
            AuthToken token = binding.get_authToken(get_authToken);
            AuthInfo authInfo = token.getAuthInfo();

            System.out.println("authInfo = " + authInfo.getValue());
            System.out.println("publisherID = " + authInfo.getPublisherID());


            Get_contextDetail get_contextDetail = new Get_contextDetail();
            get_contextDetail.setAuthInfo(authInfo);
            URI [] context_uri_array = new URI[1];
            context_uri_array[0] = new URI("uuid:5C958F20-AB36-11DA-A4FC-8C2A73C812C1-1141446095603");
            get_contextDetail.setContextKey(context_uri_array);

            ContextDetail value = null;
            // Test operation
            try {
                value = binding.get_contextDetail(get_contextDetail);
            } catch (DispositionReport e1) {
                throw new junit.framework.AssertionFailedError(
                        "DispositionReport Exception caught: " + e1);
            }
            ContextBag contextBag = value.getContextBag();

            Context [] context_array = null;
            if (contextBag != null && contextBag.getContext().length != 0) {
                context_array = contextBag.getContext();
                if (context_array != null) {
                    Context context = context_array[0];
                    if (context != null) {
                        URI c_key = context.getContextKey();
                        System.out.println("saved c_key = " + c_key);
                        Name [] returned_name_array = context.getName();
                        if (returned_name_array != null && returned_name_array.length != 0) {
                           System.out.println("user identifer = " +
                                              returned_name_array[0].getValue());
                        }
                        System.out.println("session key = " + context.getSessionKey());
                        System.out.println("service key = " + context.getServiceKey());
                        System.out.println("context value type = " + context.getValueType());
                        System.out.println("context value = " + context.getValue());
                        System.out.println("context version = " + context.getVersion());
                        Lease context_lease = context.getLease();
                        if (context_lease != null) {
                           System.out.println("start time = " + context_lease.getLeaseStartTimeStamp());
                           System.out.println("stop  time = " + context_lease.getLeaseExpirationTimeStamp());
                           System.out.println("timeout    = " + context_lease.getTimeout());
                           System.out.println("isinfinite = " + context_lease.isIsInfinite());
                        }
                        ContextAssertion [] context_assertion_arr = context.getContextAssertion();
                        if ( context_assertion_arr != null) {
                            for (int i = 0; i < context_assertion_arr.length; i++) {
                                ContextAssertion assertion =
                                        context_assertion_arr[i];
                                System.out.println("assertion subject = " +
                                        assertion.getFromKey());
                                System.out.println("assertion prop = " +
                                        assertion.getProperty_uri());
                                System.out.println("assertion object = " +
                                        assertion.getToKey());
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
