/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.axis.services.uddi_wscontext_client;

import cgl.axis.services.uddi_wscontext.stubs.*;
import cgl.axis.services.uddi.uddi_schema.*;
import org.apache.axis.types.URI;
import java.util.Properties;
import cgl.util.BootStrap;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Save_session;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionEntity;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionDetail;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Find_session;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionList;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionInfos;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.SessionInfo;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Delete_session;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Get_sessionDetail;
import cgl.axis.services.uddi.uddi_ext_schema.Lease;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class SessionFindExample {

    // Values for Session
    public static final String SESSION_NAME = "SESSION://";//"SESSION://A/B/C";//"%";
    //Values for Find Qualifiers
    public static final String EXACT_NAME_MATCH = "exactNameMatch";
    public static final String CASE_SENSITIVE_MATCH = "caseSensitiveMatch";
    public static final String OR_ALL_KEYS = "orAllKeys";
    public static final String AND_ALL_KEYS = "andAllKeys";
    public static final String EQUAL_TO = "equalTo";
    public static final String NOT_EQUAL_TO = "notEqualTo";
    //Value Types
    public static final String NUMBER_VALUE_TYPE = "number";
    public static final String TEXT_VALUE_TYPE = "text";
    //Result limit
    public static final int RESULTS_MAX_ROWS_LIMIT = 100;

    Properties bootstrap = null;

    public static void main(String args[]) {
        SessionFindExample app = new SessionFindExample();
        System.out.println(
                "\n*********** Running SessionFindExample ***********");
        app.run();
        System.exit(0);
    }

    public void run() {
        // Load bootsrap configuration
        try {
            bootstrap = BootStrap.loadProperties("properties.properties");
        } catch (Exception ex) {
        }
        try {

            UDDI_WSContextSoapBindingStub binding;
            try {
                UDDI_WSContextServiceLocator locator = new
                        UDDI_WSContextServiceLocator();
                locator.setUDDI_WSContextEndpointAddress(bootstrap.getProperty(
                        "UDDI_WSContext_WSDL"));
                System.out.println("wsdl = " + locator.getUDDI_WSContextAddress());
                binding = (UDDI_WSContextSoapBindingStub) locator.
                          getUDDI_WSContext();
            } catch (javax.xml.rpc.ServiceException jre) {
                if (jre.getLinkedCause() != null) {
                    jre.getLinkedCause().printStackTrace();
                }
                throw new junit.framework.AssertionFailedError(
                        "JAX-RPC ServiceException caught: " + jre);
            }
            Get_authToken get_authToken = new Get_authToken();
            get_authToken.setUserID(bootstrap.getProperty("cgl.fthpis.jdbcUser"));
            get_authToken.setCred(bootstrap.getProperty(
                    "cgl.fthpis.jdbcPassword"));
            AuthToken token = binding.get_authToken(get_authToken);
            AuthInfo authInfo = token.getAuthInfo();
            System.out.println("authInfo = " + authInfo.getValue());
            System.out.println("publisherID = " + authInfo.getPublisherID());

            //Find the business where this search will take place
            System.out.println("\nFind '" + this.SESSION_NAME + "'");

//            //TEST FIND SESSION
//            FindQualifiers find_qualifiers = new FindQualifiers();
//            String[] qualifier_array = new String[1];
//            qualifier_array[0] = this.AND_ALL_KEYS;
//            find_qualifiers.setFindQualifier(qualifier_array);

            //NAME
            Name[] namearray = new Name[1];
            Name name = new Name(this.SESSION_NAME);
            namearray[0] = name;

            Find_session find_session = new Find_session();
            find_session.setAuthInfo(authInfo);
            find_session.setName(namearray);
//            find_session.setFindQualifiers(find_qualifiers);

            SessionList list = null;
            // Test operation
            try {
                list = binding.find_session(find_session);
            } catch (DispositionReport e1) {
                throw new junit.framework.AssertionFailedError(
                        "DispositionReport Exception caught: " + e1);
            }
            if (list != null) {
                SessionInfos infos = list.getSessionInfos();
                if (infos != null) {
                    SessionInfo[] session_info = infos.getSessionInfo();

                    if (session_info != null && session_info.length != 0) {
                        for (int i = 0; i < session_info.length; i++) {

                            SessionInfo info = session_info[i];
                            URI s_key = info.getSessionKey();
                            System.out.println("Session key  = " + s_key);

                            Get_sessionDetail get_sessionDetail = new Get_sessionDetail();
                            get_sessionDetail.setAuthInfo(authInfo);
                            URI[] s_key_array = new URI[1];
                            s_key_array[0] = s_key;
                            get_sessionDetail.setSessionKey(s_key_array);

                            SessionDetail sessionDetail = binding.get_sessionDetail(
                                    get_sessionDetail);
                            if (sessionDetail != null) {
                                SessionEntity[] session_entity_arr = sessionDetail.getSessionEntity();
                                if (session_entity_arr != null) {
                                    for (int k = 0; k < session_entity_arr.length; k++) {
                                        SessionEntity session = session_entity_arr[k];

                                        Name[] session_name_arr = session.getName();
                                        if (session_name_arr != null) {
                                            for (int j = 0;
                                                         j < session_name_arr.length; j++) {
                                                if (session_name_arr[j] != null) {
                                                    System.out.println(
                                                            "session name = " +
                                                            session_name_arr[j].
                                                            getValue());
                                                }
                                            }
                                        }

                                        Description[] session_desc_arr = session.getDescription();

                                        if (session_desc_arr != null) {
                                            for (int j = 0;
                                                         j < session_desc_arr.length; j++) {
                                                if (session_desc_arr[j] != null) {
                                                    System.out.println(
                                                            "session description = " +
                                                            session_desc_arr[j].
                                                            getValue());
                                                }
                                            }
                                        }

                                        Lease session_lease = session.getLease();
                                        if (session_lease != null) {
                                            System.out.println("start time = " +
                                                               session_lease.
                                                               getLeaseStartTimeStamp());
                                            System.out.println("stop  time = " +
                                                               session_lease.
                                                               getLeaseExpirationTimeStamp());
                                            System.out.println("isinfinite = " +
                                                               session_lease.isIsInfinite());
                                        }

                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
