/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis;

import java.util.Vector;

import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.datatype.response.DispositionReport;
import org.apache.juddi.error.RegistryException;

import cgl.fthpis.datatype.request.AuthInfo;
import cgl.fthpis.datatype.request.DeletePublisher;
import cgl.fthpis.datatype.request.DeleteService;
import cgl.fthpis.datatype.request.DiscardAuthToken;
import cgl.fthpis.datatype.request.FindPublisher;
import cgl.fthpis.datatype.request.FindService;
import cgl.fthpis.datatype.request.GetAuthToken;
import cgl.fthpis.datatype.request.GetPublisherDetail;
import cgl.fthpis.datatype.request.GetServiceDetail;
import cgl.fthpis.datatype.request.SavePublisher;
import cgl.fthpis.datatype.request.SaveService;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.datatype.response.PublisherDetail;
import cgl.fthpis.datatype.response.PublisherList;
import cgl.fthpis.datatype.response.ServiceDetail;
import cgl.fthpis.datatype.response.ServiceList;
import cgl.fthpis.datatype.request.SaveSession;
import cgl.fthpis.datatype.response.SessionDetail;
import cgl.fthpis.datatype.request.DeleteAttribute;
import cgl.fthpis.datatype.request.GetAttributeDetail;
import cgl.fthpis.datatype.request.SaveAttribute;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.request.FindServiceAttribute;
import cgl.fthpis.datatype.request.DeleteSession;
import cgl.fthpis.datatype.request.FindSession;
import cgl.fthpis.datatype.response.SessionList;
import cgl.fthpis.datatype.request.GetSessionDetail;
import cgl.fthpis.datatype.request.FindContext;
import cgl.fthpis.datatype.request.DeleteContext;
import cgl.fthpis.datatype.request.SaveContext;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.request.GetAuthTokenForWSContext;
import cgl.fthpis.datatype.response.AttributeList;

/**
 * Represents FTHPIS Abstract Registry and implements all
 * primary functionalities expected from both UDDI and WSContext Services.
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public abstract class AbstractRegistry
    implements IRegistry {

    /**
     *
     * @exception RegistryException;
     */
    public DispositionReport deleteFTHPISService(AuthInfo authInfo,
                                           Vector serviceKeyVector) throws
        RegistryException {
      DeleteService request = new DeleteService();
      request.setAuthInfo(authInfo);
      request.setServiceKeyVector(serviceKeyVector);

      return (DispositionReport) execute(request);
    }


    /**
     *
     * @exception RegistryException;
     */
    public ServiceList findFTHPISService(String businessKey,
                                   Vector nameVector,
                                   CategoryBag categoryBag,
                                   TModelBag tModelBag,
                                   FindQualifiers findQualifiers,
                                   int maxRows) throws
        RegistryException {
      FindService request = new FindService();
      request.setBusinessKey(businessKey);
      request.setNameVector(nameVector);
      request.setCategoryBag(categoryBag);
      request.setTModelBag(tModelBag);
      request.setFindQualifiers(findQualifiers);
      request.setMaxRows(maxRows);

      return (ServiceList) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public ServiceList findFTHPISService(String businessKey,
                                   Vector nameVector,
                                   CategoryBag categoryBag,
                                   TModelBag tModelBag,
                                   AttributeBag attributeBag,
                                   Lease lease,
                                   FindQualifiers findQualifiers,
                                   int maxRows) throws
        RegistryException {
      FindService request = new FindService();
      request.setBusinessKey(businessKey);
      request.setNameVector(nameVector);
      request.setCategoryBag(categoryBag);
      request.setTModelBag(tModelBag);
      request.setAttributeBag(attributeBag);
      request.setLease(lease);
      request.setFindQualifiers(findQualifiers);
      request.setMaxRows(maxRows);

      return (ServiceList) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public ServiceList findFTHPISService(AuthInfo authInfo,

                                   Vector sessionKeyVector,

                                   AttributeBag dynamicContextBag,

                                   FindQualifiers findQualifiers,
                                   int maxRows) throws
        RegistryException {
      FindService request = new FindService();
      request.setAuthInfo(authInfo);
      //
      request.setDynamicContextBag(dynamicContextBag);
      request.setSessionKeyVector(sessionKeyVector);
      //
      request.setFindQualifiers(findQualifiers);
      request.setMaxRows(maxRows);

      return (ServiceList) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public ServiceDetail getFTHPISServiceDetail(AuthInfo authInfo,
                                          String serviceKey) throws
        RegistryException {
      Vector keys = new Vector(1);
      keys.addElement(serviceKey);

      return getServiceDetail(authInfo, keys);
    }

    /**
     *
     * @exception RegistryException;
     */
    public ServiceDetail getFTHPISServiceDetail(AuthInfo authInfo,
                                          Vector serviceKeyVector) throws
        RegistryException {
      GetServiceDetail request = new GetServiceDetail();
      request.setAuthInfo(authInfo);
      request.setServiceKeyVector(serviceKeyVector);

      return (ServiceDetail) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public ServiceDetail saveFTHPISService(AuthInfo authInfo,
                                     Vector serviceVector) throws
        RegistryException {
      SaveService request = new SaveService();
      request.setAuthInfo(authInfo);
      request.setServiceVector(serviceVector);

      return (ServiceDetail) execute(request);
    }

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

  /**
   *
   * @exception RegistryException;
   */
  public DispositionReport deleteService(AuthInfo authInfo,
                                         Vector serviceKeyVector) throws
      RegistryException {
    DeleteService request = new DeleteService();
    request.setAuthInfo(authInfo);
    request.setServiceKeyVector(serviceKeyVector);

    return (DispositionReport) execute(request);
  }


  /**
   *
   * @exception RegistryException;
   */
  public ServiceList findService(String businessKey,
                                 Vector nameVector,
                                 CategoryBag categoryBag,
                                 TModelBag tModelBag,
                                 FindQualifiers findQualifiers,
                                 int maxRows) throws
      RegistryException {
    FindService request = new FindService();
    request.setBusinessKey(businessKey);
    request.setNameVector(nameVector);
    request.setCategoryBag(categoryBag);
    request.setTModelBag(tModelBag);
    request.setFindQualifiers(findQualifiers);
    request.setMaxRows(maxRows);

    return (ServiceList) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public ServiceList findService(String businessKey,
                                 Vector nameVector,
                                 CategoryBag categoryBag,
                                 TModelBag tModelBag,
                                 AttributeBag attributeBag,
                                 Lease lease,
                                 FindQualifiers findQualifiers,
                                 int maxRows) throws
      RegistryException {
    FindService request = new FindService();
    request.setBusinessKey(businessKey);
    request.setNameVector(nameVector);
    request.setCategoryBag(categoryBag);
    request.setTModelBag(tModelBag);
    request.setAttributeBag(attributeBag);
    request.setLease(lease);
    request.setFindQualifiers(findQualifiers);
    request.setMaxRows(maxRows);

    return (ServiceList) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public ServiceList findService(AuthInfo authInfo,
                                 String businessKey,
                                 Vector sessionKeyVector,
                                 Vector nameVector,
                                 CategoryBag categoryBag,
                                 TModelBag tModelBag,
                                 AttributeBag attributeBag,
                                 AttributeBag dynamicContextBag,
                                 Lease lease,
                                 FindQualifiers findQualifiers,
                                 int maxRows) throws
      RegistryException {
    FindService request = new FindService();
    request.setAuthInfo(authInfo);
    request.setBusinessKey(businessKey);
    request.setNameVector(nameVector);
    request.setCategoryBag(categoryBag);
    request.setTModelBag(tModelBag);
    request.setAttributeBag(attributeBag);
    //
    request.setDynamicContextBag(dynamicContextBag);
    request.setSessionKeyVector(sessionKeyVector);
    //
    request.setLease(lease);
    request.setFindQualifiers(findQualifiers);
    request.setMaxRows(maxRows);

    return (ServiceList) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public ServiceDetail getServiceDetail(AuthInfo authInfo,
                                        String serviceKey) throws
      RegistryException {
    Vector keys = new Vector(1);
    keys.addElement(serviceKey);

    return getServiceDetail(authInfo, keys);
  }

  /**
   *
   * @exception RegistryException;
   */
  public ServiceDetail getServiceDetail(AuthInfo authInfo,
                                        Vector serviceKeyVector) throws
      RegistryException {
    GetServiceDetail request = new GetServiceDetail();
    request.setAuthInfo(authInfo);
    request.setServiceKeyVector(serviceKeyVector);

    return (ServiceDetail) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public ServiceDetail saveService(AuthInfo authInfo,
                                   Vector serviceVector) throws
      RegistryException {
    SaveService request = new SaveService();
    request.setAuthInfo(authInfo);
    request.setServiceVector(serviceVector);

    return (ServiceDetail) execute(request);
  }

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
    /**
     *
     * @exception RegistryException
     */
    public AttributeDetail findServiceAttribute(String serviceKey,
                                                CategoryBag categoryBag,
                                                Lease lease,
                                                FindQualifiers findQualifiers,
                                                int maxRows) throws
        RegistryException {
      FindServiceAttribute request = new FindServiceAttribute();
      request.setServiceKey(serviceKey);
      request.setCategoryBag(categoryBag);
      request.setLease(lease);
      request.setFindQualifiers(findQualifiers);
      request.setMaxRows(maxRows);
      return (AttributeDetail) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public DispositionReport deleteAttribute(AuthInfo authInfo,
                                             Vector attributeKeyVector) throws
        RegistryException {
      DeleteAttribute request = new DeleteAttribute();
      request.setAuthInfo(authInfo);
      request.setAttributeKeyVector(attributeKeyVector);

      return (DispositionReport) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public AttributeDetail saveAttribute(AuthInfo authInfo,
                                         Vector attributeVector) throws
        RegistryException {
      SaveAttribute request = new SaveAttribute();
      request.setAuthInfo(authInfo);
      request.setAttributeVector(attributeVector);

      return (AttributeDetail) execute(request);
    }

    /**
     *
     * @exception RegistryException;
     */
    public AttributeDetail getAttributeDetail(Vector attributeKeys) throws
        RegistryException {
      GetAttributeDetail request = new GetAttributeDetail();
      request.setAttributeKeyVector(attributeKeys);

      return (AttributeDetail) execute(request);
    }

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

  /**
   *
   * @exception RegistryException
   */
  public AttributeList findContext(AuthInfo authInfo,
                                     String sessionKey,
                                     String serviceKey,
                                     Vector nameVector,
                                     Lease lease,
                                     FindQualifiers findQualifiers,
                                     int maxRows) throws
      RegistryException {
    FindContext request = new FindContext();
    request.setAuthInfo(authInfo);
    request.setSessionKey(sessionKey);
    request.setServiceKey(serviceKey);
    request.setNameVector(nameVector);
    request.setLease(lease);
    request.setFindQualifiers(findQualifiers);
    request.setMaxRows(maxRows);
    return (AttributeList) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public DispositionReport deleteContext(AuthInfo authInfo,
                                         Vector contextKeyVector) throws
      RegistryException {
    DeleteContext request = new DeleteContext();
    request.setAuthInfo(authInfo);
    request.setContextKeyVector(contextKeyVector);

    return (DispositionReport) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public AttributeDetail saveContext(AuthInfo authInfo,
                                     Vector contextVector) throws
      RegistryException {
    SaveContext request = new SaveContext();
    request.setAuthInfo(authInfo);
    request.setContextVector(contextVector);

    return (AttributeDetail) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public AttributeDetail getContextDetail(AuthInfo authInfo,
                                          Vector contextKeys) throws
      RegistryException {
    GetContextDetail request = new GetContextDetail();
    request.setAuthInfo(authInfo);
    request.setContextKeyVector(contextKeys);

    return (AttributeDetail) execute(request);
  }

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

  /**
   *
   * @exception RegistryException;
   */
  public AuthToken getAuthToken(String userID, String cred) throws
      RegistryException {
    GetAuthToken request = new GetAuthToken();
    request.setUserID(userID);
    request.setCredential(cred);

    return (AuthToken) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public AuthToken getAuthTokenForWSContext(String userID, String cred) throws
      RegistryException {
    GetAuthTokenForWSContext request = new GetAuthTokenForWSContext();
    request.setUserID(userID);
    request.setCredential(cred);

    return (AuthToken) execute(request);
  }
  /**
   *
   * @exception RegistryException;
   */
  public DispositionReport discardAuthToken(AuthInfo authInfo) throws
      RegistryException {
    DiscardAuthToken request = new DiscardAuthToken();
    request.setAuthInfo(authInfo);

    return (DispositionReport) execute(request);
  }

  /**
   * @exception RegistryException;
   */
  public PublisherList findPublisher(AuthInfo authInfo,
                                     String name,
                                     FindQualifiers findQualifiers,
                                     int maxRows) throws
      RegistryException {
    FindPublisher request = new FindPublisher();
    //THING_TO-DO
    //set the authinfo for findpublisher
    request.setName(name);
    request.setFindQualifiers(findQualifiers);
    request.setMaxRows(maxRows);

    return (PublisherList) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public DispositionReport deletePublisher(AuthInfo authInfo,
                                           Vector publisherIDVector) throws
      RegistryException {
    DeletePublisher request = new DeletePublisher();
    request.setAuthInfo(authInfo);
    request.setPublisherIDVector(publisherIDVector);

    return (DispositionReport) execute(request);
  }

  /**
   * @exception RegistryException;
   */
  public PublisherDetail getPublisherDetail(AuthInfo authInfo,
                                            Vector publisherIDVector) throws
      RegistryException {
    GetPublisherDetail request = new GetPublisherDetail();
    //THING_TO-DO
    //set the authinfo for publisherdetail
    request.setPublisherIDVector(publisherIDVector);

    return (PublisherDetail) execute(request);
  }

  /**
   * @exception RegistryException;
   */
  public PublisherDetail savePublisher(AuthInfo authInfo,
                                       Vector publisherVector) throws
      RegistryException {

    //System.out.println("AbstractRegistry => publisherVector = " + publisherVector);
    SavePublisher request = new SavePublisher();
    request.setAuthInfo(authInfo);
    request.setPublisherVector(publisherVector);

    return (PublisherDetail) execute(request);
  }

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

  /**
   *
   * @exception RegistryException;
   */
  public SessionDetail getSessionDetail(AuthInfo authInfo, String sessionKey) throws
      RegistryException {
    Vector keys = new Vector(1);
    keys.addElement(sessionKey);

    return getSessionDetail(authInfo, keys);
  }

  /**
   *
   * @exception RegistryException;
   */
  public SessionDetail getSessionDetail(AuthInfo authInfo,
                                        Vector sessionKeyVector) throws
      RegistryException {
    GetSessionDetail request = new GetSessionDetail();
    request.setAuthInfo(authInfo);
    request.setSessionKeyVector(sessionKeyVector);

    return (SessionDetail) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public SessionDetail saveSession(AuthInfo authInfo,
                                   Vector sessionVector) throws
      RegistryException {
    SaveSession request = new SaveSession();
    request.setAuthInfo(authInfo);
    request.setSessionEntityVector(sessionVector);

    return (SessionDetail) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public DispositionReport deleteSession(AuthInfo authInfo,
                                         Vector sessionKeyVector) throws
      RegistryException {
    DeleteSession request = new DeleteSession();
    request.setAuthInfo(authInfo);
    request.setSessionKeyVector(sessionKeyVector);

    return (DispositionReport) execute(request);
  }

  /**
   *
   * @exception RegistryException;
   */
  public SessionList findSession(AuthInfo authInfo,
                                 Vector nameVector,
                                 AttributeBag attributeBag,
                                 Vector serviceKeyVector,
                                 Lease lease,
                                 FindQualifiers findQualifiers,
                                 int maxRows) throws
      RegistryException {
    FindSession request = new FindSession();
    request.setAuthInfo(authInfo);
    request.setNameVector(nameVector);
    request.setAttributeBag(attributeBag);
    request.setServiceKeyVector(serviceKeyVector);
    request.setLease(lease);
    request.setFindQualifiers(findQualifiers);
    request.setMaxRows(maxRows);
    return (SessionList) execute(request);
  }

}
