/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis;

import java.util.Vector;

import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.datatype.response.DispositionReport;
import org.apache.juddi.error.RegistryException;

import cgl.fthpis.datatype.request.AuthInfo;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.datatype.response.PublisherDetail;
import cgl.fthpis.datatype.response.PublisherList;
import cgl.fthpis.datatype.response.ServiceDetail;
import cgl.fthpis.datatype.response.ServiceList;
import cgl.fthpis.datatype.response.SessionDetail;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.response.SessionList;
import cgl.fthpis.datatype.response.AttributeList;

/**
 * Represents FTHPIS Registry and implements all primary functionalities
 * expected from both UDDI and WSContext Services.
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public interface IRegistry {

    /**
     *
     * @exception RegistryException;
     */
    ServiceList findService(String businessKey, Vector nameVector,
                            CategoryBag categoryBag, TModelBag tModelBag,
                            FindQualifiers findQualifiers, int maxRows) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    ServiceList findService(String businessKey, Vector nameVector,
                            CategoryBag categoryBag, TModelBag tModelBag,
                            AttributeBag attributeBag, Lease lease,
                            FindQualifiers findQualifiers, int maxRows) throws
            RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    public ServiceList findService(AuthInfo info,
                                   String businessKey,
                                   Vector sessionKeyVector,
                                   Vector nameVector,
                                   CategoryBag categoryBag,
                                   TModelBag tModelBag,
                                   AttributeBag attributeBag,
                                   AttributeBag dynamicContextBag,
                                   Lease lease,
                                   FindQualifiers findQualifiers,
                                   int maxRows) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    ServiceDetail saveService(AuthInfo authInfo, Vector serviceVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    DispositionReport deleteService(AuthInfo authInfo, Vector serviceKeyVector) throws
            RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    ServiceDetail getServiceDetail(AuthInfo info, String serviceKey) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    ServiceDetail getServiceDetail(AuthInfo info, Vector serviceKeyVector) throws
            RegistryException;

////////////////////////////////////////////////////////////////////////////////

    /**
     *
     * @exception RegistryException;
     */
    ServiceList findFTHPISService(String businessKey, Vector nameVector,
                                  CategoryBag categoryBag, TModelBag tModelBag,
                                  FindQualifiers findQualifiers, int maxRows) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    ServiceList findFTHPISService(String businessKey, Vector nameVector,
                                  CategoryBag categoryBag, TModelBag tModelBag,
                                  AttributeBag attributeBag, Lease lease,
                                  FindQualifiers findQualifiers, int maxRows) throws
            RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    public ServiceList findFTHPISService(AuthInfo info,

                                         Vector sessionKeyVector,

                                         AttributeBag dynamicContextBag,

                                         FindQualifiers findQualifiers,
                                         int maxRows) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    ServiceDetail saveFTHPISService(AuthInfo authInfo, Vector serviceVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    DispositionReport deleteFTHPISService(AuthInfo authInfo,
                                          Vector serviceKeyVector) throws
            RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    ServiceDetail getFTHPISServiceDetail(AuthInfo info, String serviceKey) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    ServiceDetail getFTHPISServiceDetail(AuthInfo info, Vector serviceKeyVector) throws
            RegistryException;

////////////////////////////////////////////////////////////////////////////////

    /**
     *
     * @exception RegistryException
     */
    AttributeDetail findServiceAttribute(String serviceKey,
                                         CategoryBag categoryBag,
                                         Lease lease,
                                         FindQualifiers findQualifiers,
                                         int maxRows) throws
            RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    DispositionReport deleteAttribute(AuthInfo authInfo,
                                      Vector attributeKeyVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    AttributeDetail saveAttribute(AuthInfo authInfo, Vector attributeVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    AttributeDetail getAttributeDetail(Vector attributeKeys) throws
            RegistryException;

    /**
     *
     * @exception RegistryException
     */
    AttributeList findContext(AuthInfo info,
                              String sessionKey,
                              String serviceKey,
                              Vector nameVector,
                              Lease lease,
                              FindQualifiers findQualifiers,
                              int maxRows) throws
            RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    DispositionReport deleteContext(AuthInfo authInfo,
                                    Vector contextKeyVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    AttributeDetail saveContext(AuthInfo authInfo,
                                Vector contextVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    AttributeDetail getContextDetail(AuthInfo authInfo,
                                     Vector contextKeys) throws
            RegistryException;

    /**
     * "Used to delete an existing publisher accounts."
     *
     * @exception RegistryException;
     */
    DispositionReport deletePublisher(AuthInfo authInfo,
                                      Vector publisherIDVector) throws
            RegistryException;

    /**
     * @exception RegistryException;
     */
    PublisherDetail getPublisherDetail(AuthInfo authInfo,
                                       Vector publisherIDVector) throws
            RegistryException;

    /**
     * @exception RegistryException;
     */
    PublisherList findPublisher(AuthInfo authInfo,
                                String name,
                                FindQualifiers findQualifiers,
                                int maxRows) throws
            RegistryException;

    /**
     * @exception RegistryException;
     */
    PublisherDetail savePublisher(AuthInfo authInfo,
                                  Vector publisherVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    DispositionReport discardAuthToken(AuthInfo authInfo) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    AuthToken getAuthToken(String userID, String cred) throws RegistryException;


    /**
     *
     * @exception RegistryException;
     */
    AuthToken getAuthTokenForWSContext(String userID, String cred) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    SessionList findSession(AuthInfo authInfo,
                            Vector nameVector,
                            AttributeBag attributeBag,
                            Vector serviceKeyVector,
                            Lease lease,
                            FindQualifiers findQualifiers,
                            int maxRows) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    SessionDetail getSessionDetail(AuthInfo authInfo,
                                   String sessionKey) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    SessionDetail getSessionDetail(AuthInfo authInfo,
                                   Vector sessionKeyVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    DispositionReport deleteSession(AuthInfo authInfo,
                                    Vector sessionKeyVector) throws
            RegistryException;

    /**
     *
     * @exception RegistryException;
     */
    SessionDetail saveSession(AuthInfo authInfo,
                              Vector sessionVector) throws
            RegistryException;


    /**
     * @exception RegistryException
     */
    RegistryObject execute(RegistryObject request) throws RegistryException;
}
