/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore;

import java.util.Vector;

import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.RegistryException;

import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.response.PublisherInfo;
import cgl.fthpis.datatype.service.BusinessService;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datatype.AutharizationToken;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public interface FTHPISDataStore {
    /**
     * release this DataStore
     */
    void release();

    /**
     * begin transaction
     */
    void beginTrans() throws RegistryException;

    /**
     * commit transaction
     */
    void commit() throws RegistryException;

    /**
     * rollback transaction
     */
    void rollback() throws RegistryException;

    /**
     *
     * @param publisherID
     * @return publisher
     * @throws RegistryException
     */
    Publisher getPublisher(String publisherID) throws RegistryException;

    /**
     *
     * @param publisherID
     * @return boolean
     * @throws RegistryException
     */
    boolean isAdministrator(String publisherID) throws RegistryException;

    /**
     *
     * @param publisher
     * @return a secure Token
     * @throws RegistryException
     */
    String generateToken(Publisher publisher) throws RegistryException;

    /**
     *
     */
    void storeAuthToken(String token, Publisher publisher) throws
            RegistryException;

    /**
     *
     */
    void retireAuthToken(String token) throws RegistryException;

    /**
     *
     */
    Publisher getAuthTokenPublisher(String token) throws RegistryException;

    /**
     *
     */
    AutharizationToken getAuthToken(String token) throws RegistryException;

    /**
     *
     */
    boolean isAuthTokenExpired(String token) throws RegistryException;

    /**
     *
     */
    void touchAuthToken(String token) throws RegistryException;

    /**
     *
     */
    void saveSession(SessionEntity session, String publisherID) throws
            RegistryException;

    /**
     *
     */
    SessionEntity fetchSession(String sessionKey) throws RegistryException;

    /**
     *
     */
    void deleteSession(String sessionKey) throws RegistryException;

    /**
     *
     */
    void deleteContext(String contextKey) throws RegistryException;

    /**
     *
     */
    boolean isSessionAdmin(String sessionKey, String publisherID) throws
            RegistryException;

    /**
     *
     */
    boolean isSessionPublisher(String sessionKey, String publisherID) throws
            RegistryException;

    /**
     *
     */
    boolean isContextPublisher(String sessionKey, String publisherID) throws
            RegistryException;

    /**
     *
     */
    boolean isValidSessionKey(String sessionKey) throws RegistryException;

    /**
     *
     */
    boolean isValidContextKey(String contextKey) throws RegistryException;

    /**
     *
     */
    void saveService(BusinessService service) throws RegistryException;

    /**
     *
     */
    void saveSessionService(BusinessService service) throws RegistryException;

    /**
     *
     */
    void saveAttribute(Attribute attribute) throws RegistryException;

    /**
     *
     */
    void saveContext(Attribute attribute) throws RegistryException;

    /**
     *
     */
    BusinessService fetchService(String serviceKey) throws RegistryException;

    /**
     *
     */
    BusinessService fetchSessionService(String serviceKey) throws
            RegistryException;

    /**
     *
     */
    Attribute fetchAttribute(String attributeKey) throws RegistryException;

    /**
     *
     */
    Attribute fetchContext(String contextKey) throws RegistryException;

    /**
     *
     */
    void deleteService(String serviceKey) throws RegistryException;

    /**
     *
     */
    void deleteSessionService(String serviceKey) throws RegistryException;

    /**
     *
     */
    void deleteAttribute(String attributeKey) throws RegistryException;

    /**
     *
     */
    boolean isValidServiceKey(String serviceKey) throws RegistryException;

    /**
     *
     */
    boolean isValidAttributeKey(String bindingKey) throws RegistryException;


    /**
     *
     */
    boolean isServicePublisher(String serviceKey, String publisherID) throws
            RegistryException;

    /**
     *
     */
    Vector findSession(Vector serviceKeyVector,
                       Vector nameVector,
                       AttributeBag attributeBag,
                       Lease lease,
                       FindQualifiers findQualifiers) throws RegistryException;

    /**
     *
     */
    Vector findService(String businessKey,
                       Vector names,
                       CategoryBag categoryBag,
                       TModelBag tModelBag,
                       FindQualifiers findQualifiers) throws RegistryException;


    /**
     *
     */
    Vector findService(String businessKey,
                       Vector names,
                       CategoryBag categoryBag,
                       TModelBag tModelBag,
                       AttributeBag attributeBag,
                       FindQualifiers findQualifiers) throws RegistryException;

    /**
     *
     */
    Vector findService(String businessKey,
                       Vector names,
                       CategoryBag categoryBag,
                       TModelBag tModelBag,
                       AttributeBag attributeBag,
                       Lease lease,
                       FindQualifiers findQualifiers) throws RegistryException;

    /**
     *
     */
    Vector findSessionService(Vector sessionKeyVector,
                              Vector keyVector,
                              AttributeBag contextBag,
                              FindQualifiers findQualifiers) throws
            RegistryException;

    /**
     *
     */
    Vector findContext(String sessionKey,
                       String serviceKey,
                       Vector nameVector,
                       Lease lease,
                       FindQualifiers findQualifiers) throws RegistryException;

    /**
     *
     */
    Vector findServiceAttribute(String serviceKey,
                                CategoryBag categoryBag,
                                Lease lease,
                                FindQualifiers findQualifiers) throws
            RegistryException;

    /**
     *
     */
    void savePublisher(Publisher publisher) throws RegistryException;

    /**
     *
     */
    void deletePublisher(String publisherID) throws RegistryException;

    /**
     *
     */
    PublisherInfo fetchPublisherInfo(String publisherID) throws
            RegistryException;

    /**
     *
     */
    Vector findPublisher(String name, FindQualifiers findQualifiers) throws
            RegistryException;
}
