/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.AttributeAssertion;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class AttributeAssertionTable {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(AttributeAssertionTable.class);

    static String insertSQL = null;
    static String selectSQL = null;
    static String deleteByAttributeKeySQL = null;
    static String verifyOwnershipSQL = null;


    static {
        // buffer used to build SQL statements
        StringBuffer sql = null;

        // build insertSQL
        sql = new StringBuffer(150);
        sql.append("INSERT INTO CONTEXT_ASSERTION (");
        sql.append("PROPERTY_KEY,");
        sql.append("FROM_KEY,");
        sql.append("PROPERTY_URI,");
        sql.append("TO_KEY) ");
        sql.append("VALUES (?,?,?,?)");
        insertSQL = sql.toString();

        // build selectSQL
        sql = new StringBuffer(100);
        sql.append("SELECT ");
        sql.append("C.PROPERTY_KEY,");
        sql.append("C.FROM_KEY,");
        sql.append("C.PROPERTY_URI,");
        sql.append("C.TO_KEY ");
        sql.append("FROM CONTEXT_ASSERTION C ");
        sql.append("WHERE C.PROPERTY_KEY=? ");
        selectSQL = sql.toString();

        // build deleteByAttributeKeySQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM CONTEXT_ASSERTION ");
        sql.append("WHERE PROPERTY_KEY=?");
        deleteByAttributeKeySQL = sql.toString();

    }

    /**
     * Insert new row into the CONTEXT_ASSERTION table.
     *
     * @param  attribute_assertion AttributeAssertion object holding values to be inserted
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void insert(AttributeAssertion attribute_assertion,
                              Connection connection) throws
            java.sql.SQLException {
        log.info(
                "cgl.fthpis.datastore.jdbc.AttributeTable.insert() function called.");
        PreparedStatement statement = null;

        try {
            // prepare and execute the insert
            statement = connection.prepareStatement(insertSQL);
            statement.setString(1, attribute_assertion.getAttributeKey());
            statement.setString(2, attribute_assertion.getAssertionSubjectKey());
            statement.setString(3, attribute_assertion.getAssertionProperty());
            statement.setString(4, attribute_assertion.getAssertionObjectKey());
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) {
                e.printStackTrace();
                /* ignored */
            }

        }
    }

    /**
     * Delete multiple rows from the CONTEXT_ASSERTION table that are assigned to
     * the attributeKey specified.
     *
     * @param  attributeKey attributeKey
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void deleteByAttributeKey(String attributeKey,
                                            Connection connection) throws java.
            sql.
            SQLException {
        PreparedStatement statement = null;

        try {
            // prepare the delete
            statement = connection.prepareStatement(deleteByAttributeKeySQL);
            statement.setString(1, attributeKey);

            log.info("delete from CONTEXT_ASSERTION table:\n\n\t" +
                     deleteByAttributeKeySQL +
                     "\n\t PROPERTY_KEY=" + attributeKey + "\n");

            // execute
            int returnCode = statement.executeUpdate();

            log.info("delete was successful, rows deleted=" + returnCode);
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /**
     * Select one row from the CONTEXT_ASSERTION table.
     *
     * @param attributeKey primary key value
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select(String attributeKey,
                                            Connection connection) throws java.
            sql.SQLException {
        AttributeAssertion attribute_assertion = null;
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        Vector attribute_assertion_vector = new Vector();

        try {
            statement = connection.prepareStatement(selectSQL);
            statement.setString(1, attributeKey.toString());

            log.info("select from CONTEXT_ASSERTION table:\n\n\t" + selectSQL +
                     "\n\t PROPERTY_KEY=" + attributeKey.toString() + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                attribute_assertion = new AttributeAssertion();
                String attribute_Key = resultSet.getString(1);
                String attribute_FromKey = resultSet.getString(2);
                String attribute_PropertyURI = resultSet.getString(3);
                String attribute_ToKey = resultSet.getString(4);

                if (attribute_Key != null) {
                    attribute_assertion.setAttributeKey(attributeKey);
                    log.info("AttributeAssertionTable = > attributeKey " +
                             attributeKey);
                }
                if (attribute_FromKey != null) {
                    attribute_assertion.setAssertionSubjectKey(
                            attribute_FromKey);
                    log.info("AttributeAssertionTable = > attribute_FromKey " +
                             attribute_FromKey);
                }
                if (attribute_PropertyURI != null) {
                    attribute_assertion.setAssertionProperty(
                            attribute_PropertyURI);
                    log.info(
                            "AttributeAssertionTable = > attribute_PropertyURI " +
                            attribute_PropertyURI);
                }
                if (attribute_ToKey != null) {
                    attribute_assertion.setAssertionObjectKey(attribute_ToKey);
                    log.info("AttributeAssertionTable = > attribute_ToKey = " +
                             attribute_ToKey);
                }
                attribute_assertion_vector.add(attribute_assertion);
            }
            return attribute_assertion_vector;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/

    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        Transaction txn = new Transaction();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        if (connection != null) {
            try {
                // begin a new transaction
                txn.begin(connection);
                String[] attributeKey = uuidgen.uuidgen(4);
                //AttributeBag attributeBag = new AttributeBag();
                AttributeAssertion attribute_assertion = new AttributeAssertion();
                String attribute_key = attributeKey[0];
                attribute_assertion.setAttributeKey(attribute_key);
                attribute_assertion.setAssertionSubjectKey("subject");
                attribute_assertion.setAssertionProperty("property");
                attribute_assertion.setAssertionObjectKey("object");

                AttributeAssertionTable.insert(attribute_assertion, connection);
                Vector vector = AttributeAssertionTable.
                        select(attribute_assertion.getAttributeKey(),
                               connection);
                for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
                    AttributeAssertion result_attribute = (AttributeAssertion) iter.next();
                    log.info("result_attribute key = " +
                             result_attribute.getAttributeKey());
                    log.info("attribute key = " +
                             result_attribute.getAttributeKey());
                    log.info("attribute fromkey = " +
                             result_attribute.getAssertionSubjectKey());
                    log.info("attribute property = " +
                             result_attribute.getAssertionProperty());
                    log.info("attribute tokey = " +
                             result_attribute.getAssertionObjectKey());
                }
                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}
