/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.lease.Lease;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class AttributeCategoryTable
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(AttributeCategoryTable.class);

  static String insertSQL = null;
  static String selectSQL = null;
  static String deleteSQL = null;

  static {
    // buffer used to build SQL statements
    StringBuffer sql = null;

    // build insertSQL
    sql = new StringBuffer(150);
    sql.append("INSERT INTO PROPERTY_CATEGORY (");
    sql.append("PROPERTY_KEY,");
    sql.append("CATEGORY_ID,");
    sql.append("TMODEL_KEY_REF,");
    sql.append("KEY_NAME,");
    sql.append("KEY_VALUE) ");
    sql.append("VALUES (?,?,?,?,?)");
    insertSQL = sql.toString();

    // build selectSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("TMODEL_KEY_REF,");
    sql.append("KEY_NAME,");
    sql.append("KEY_VALUE, ");
    sql.append("CATEGORY_ID ");
    sql.append("FROM PROPERTY_CATEGORY ");
    sql.append("WHERE PROPERTY_KEY=? ");
    sql.append("ORDER BY CATEGORY_ID");
    selectSQL = sql.toString();

    // build deleteSQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM PROPERTY_CATEGORY ");
    sql.append("WHERE PROPERTY_KEY=?");
    deleteSQL = sql.toString();
  }

  /**
   * Insert new row into the PROPERTY_CATEGORY table.
   *
   * @param attributeKey String to the parent AttributeCategory object.
   * @param keyRefs A Vector of KeyedReference instances to insert.
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void insert(
    String attributeKey,
    Vector keyRefs,
    Connection connection)
    throws java.sql.SQLException
  {
    PreparedStatement statement = null;
    log.info("cgl.fthpis.datastore.jdbc.AttributeCategoryTable => insert => beginning");
    try
    {
      statement = connection.prepareStatement(insertSQL);
      statement.setString(1, attributeKey.toString());
      if (keyRefs != null) {
          int listSize = keyRefs.size();
          for (int categoryID = 0; categoryID < listSize; categoryID++) {
              KeyedReference keyRef = (KeyedReference) keyRefs.elementAt(
                      categoryID);

              // extract values to insert
              String tModelKeyValue = null;
              if (keyRef.getTModelKey() != null)
                  tModelKeyValue = keyRef.getTModelKey().toString();

              // set the values
              statement.setInt(2, categoryID);
              statement.setString(3, tModelKeyValue);
              statement.setString(4, keyRef.getKeyName());
              statement.setString(5, keyRef.getKeyValue());

              log.info(
                      "insert into PROPERTY_CATEGORY table:\n\n\t"
                      + insertSQL
                      + "\n\t PROPERTY_KEY="
                      + attributeKey.toString()
                      + "\n\t CATEGORY_ID="
                      + categoryID
                      + "\n\t TMODEL_KEY_REF="
                      + tModelKeyValue
                      + "\n\t KEY_NAME="
                      + keyRef.getKeyName()
                      + "\n\t KEY_VALUE="
                      + keyRef.getKeyValue()
                      + "\n");
              // insert
              statement.executeUpdate();
              log.info("cgl.fthpis.datastore.jdbc.AttributeCategoryTable => statement executed successfully...");
          }
      }
    }
    finally
    {
      try
      {
        statement.close();
      }
      catch (Exception e)
      { /* ignored */
        e.printStackTrace();
      }
    }
  }

  /**
   * Select all rows from the PROPERTY_CATEGORY table for a given attributeKey.
   *
   * @param  attributeKey String
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector select(String attributeKey, Connection connection)
    throws java.sql.SQLException
  {
    Vector keyRefList = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    try
    {
      // create a statement to query with
      statement = connection.prepareStatement(selectSQL);
      statement.setString(1, attributeKey.toString());

      log.info(
        "select from PROPERTY_CATEGORY table:\n\n\t"
          + selectSQL
          + "\n\t PROPERTY_KEY="
          + attributeKey.toString()
          + "\n");

      // execute the statement
      resultSet = statement.executeQuery();
      while (resultSet.next())
      {
        KeyedReference keyRef = new KeyedReference();
        keyRef.setTModelKey(resultSet.getString(1));
        keyRef.setKeyName(resultSet.getString(2));
        keyRef.setKeyValue(resultSet.getString(3));
        keyRefList.add(keyRef);
      }

      return keyRefList;
    }
    finally
    {
      try
      {
        resultSet.close();
        statement.close();
      }
      catch (Exception e)
      { /* ignored */
        e.printStackTrace();
      }
    }
  }

  /**
   * Delete multiple rows from the BINDING_CATEGORY table that are assigned to the
   * BindingKey specified.
   *
   * @param  bindingKey String
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void delete(String attributeKey, Connection connection)
    throws java.sql.SQLException
  {
    PreparedStatement statement = null;

    try
    {
      // prepare the delete
      statement = connection.prepareStatement(deleteSQL);
      statement.setString(1, attributeKey.toString());

      log.info(
        "delete from PROPERTY_CATEGORY table:\n\n\t"
          + deleteSQL
          + "\n\t PROPERTY_KEY="
          + attributeKey.toString()
          + "\n");

      // execute
      statement.executeUpdate();
    }
    finally
    {
      try
      {
        statement.close();
      }
      catch (Exception e)
      { /* ignored */
      }
    }
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("uddi");
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection) throws Exception
  {
    Transaction txn = new Transaction();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    if (connection != null)
    {
      try
      {
        String attributeKey = uuidgen.uuidgen_with_timestamp();
        Attribute attribute = new Attribute();
        attribute.setAttributeKey(attributeKey);
        attribute.setAttributeName(new AttributeName("cpu_deneme"));
        attribute.setAttributeValueType(new AttributeValueType("number"));
        attribute.setAttributeValue(new AttributeValue("5"));

        Vector keyRefs = new Vector();
        KeyedReference keyRef = new KeyedReference();
        keyRef.setKeyName("propertyType");
        keyRef.setKeyValue("QOS");
        keyRefs.add(keyRef);

        // begin a new transaction
        txn.begin(connection);

        // insert a new BindingTemplate
        AttributeTable.insert(attribute, connection);

        //generate lease
        Lease lease = new Lease();
        lease.setObjectKey(attributeKey);
        lease.setIsInfinite("false");
        lease.setLeaseTimeOut("1000");
        //Lease table for attribute
        LeaseTable.insertObjectLease(lease, connection);



        // insert a Collection of new Category KeyedReference objects
        AttributeCategoryTable.insert(attributeKey, keyRefs, connection);

        // select the Collection of KeyedReference objects
        keyRefs = AttributeCategoryTable.select(attributeKey, connection);

        for (int i = 0; i < keyRefs.size(); i++)
        {
          keyRef = (KeyedReference) keyRefs.elementAt(i);
          log.info("Key Name: " + keyRef.getKeyName());
          log.info("Key Value: " + keyRef.getKeyValue());
          log.info("TModelKey: " + keyRef.getTModelKey());
        }

//        // delete the Collection of KeyedReference objects
//        AttributeCategoryTable.delete(attributeKey, connection);
//
//        // select the Collection of KeyedReference objects
//        keyRefs = AttributeCategoryTable.select(attributeKey, connection);
//
//        for (int i = 0; i < keyRefs.size(); i++)
//        {
//          keyRef = (KeyedReference) keyRefs.elementAt(i);
//          log.info("Key Name: " + keyRef.getKeyName());
//          log.info("Key Value: " + keyRef.getKeyValue());
//          log.info("TModelKey: " + keyRef.getTModelKey());
//        }
        // commit the transaction
        txn.commit();
      }
      catch (Exception ex)
      {
        try
        {
          txn.rollback();
        }
        catch (java.sql.SQLException sqlex)
        {
          sqlex.printStackTrace();
        }
        throw ex;
      }
    }
  }
}
