/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.datatype.KeyedReference;

import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.util.MyByteBlob;
import cgl.fthpis.datatype.lease.Lease;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class AttributeTable {
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(AttributeTable.class);

  static String insertSQL1 = null, insertSQL2 = null, insertSQL3 = null;
  static String selectSQL = null;
  static String deleteByAttributeKeySQL = null;
  static String verifyOwnershipSQL = null;


  static {
    // buffer used to build SQL statements
    StringBuffer sql = null;

    // build insertSQL
    sql = new StringBuffer(150);
    sql.append("INSERT INTO PROPERTY_ENTITY (");
    sql.append("PROPERTY_KEY,");
    sql.append("PROPERTYTYPE,");
    sql.append("PROPERTYIDENTIFIER,");
    sql.append("PROPERTYVALUETYPE,");
    sql.append("PROPERTYVALUENUMBER,");
    sql.append("PROPERTYVERSION) ");
    sql.append("VALUES (?,?,?,?,?,?)");
    insertSQL1 = sql.toString();

    // build insertSQL
    sql = new StringBuffer(150);
    sql.append("INSERT INTO PROPERTY_ENTITY (");
    sql.append("PROPERTY_KEY,");
    sql.append("PROPERTYTYPE,");
    sql.append("PROPERTYIDENTIFIER,");
    sql.append("PROPERTYVALUETYPE,");
    sql.append("PROPERTYVALUESTRING,");
    sql.append("PROPERTYVERSION) ");
    sql.append("VALUES (?,?,?,?,?,?)");
    insertSQL2 = sql.toString();

    // build insertSQL
    sql = new StringBuffer(150);
    sql.append("INSERT INTO PROPERTY_ENTITY (");
    sql.append("PROPERTY_KEY,");
    sql.append("PROPERTYTYPE,");
    sql.append("PROPERTYIDENTIFIER,");
    sql.append("PROPERTYVALUETYPE,");
    sql.append("PROPERTYVALUEXML,");
    sql.append("PROPERTYVERSION) ");
    sql.append("VALUES (?,?,?,?,?,?)");
    insertSQL3 = sql.toString();

    // build selectSQL
    sql = new StringBuffer(100);
    sql.append("SELECT ");
    sql.append("C.PROPERTYTYPE,");
    sql.append("C.PROPERTYIDENTIFIER,");
    sql.append("C.PROPERTYVALUETYPE,");
    sql.append("C.PROPERTYVALUENUMBER,");
    sql.append("C.PROPERTYVALUESTRING,");
    sql.append("C.PROPERTYVALUEXML,");
    sql.append("C.PROPERTYVERSION ");
    sql.append("FROM PROPERTY_ENTITY C ");
    sql.append("WHERE C.PROPERTY_KEY=? ");
    sql.append("ORDER BY C.PROPERTYIDENTIFIER");
    selectSQL = sql.toString();

    // build deleteByAttributeKeySQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM PROPERTY_ENTITY ");
    sql.append("WHERE PROPERTY_KEY=?");
    deleteByAttributeKeySQL = sql.toString();
  }

  /**
   * Insert new row into the PROPERTY_ENTITY table.
   *
   * @param  attribute Attribute object holding values to be inserted
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void insert(Attribute attribute, Connection connection) throws
      java.sql.SQLException {
    log.info(
        "cgl.fthpis.datastore.jdbc.AttributeTable.insert() function called.");
    PreparedStatement statement = null;

    try {
      String identifier = null;
      String value_type = null;
      String value_string = null;
      String value_xml = null;
      java.sql.Blob newblob = null;
      float value_number = 0;
      int version = 0;



      AttributeName attributeName = attribute.getAttributeName();
      if (attributeName != null) {
        identifier = attributeName.getAttributeName();
      }
      AttributeValueType attributeValueType = attribute.getAttributeValueType();
      if (attributeValueType != null) {
        value_type = attributeValueType.getAttributeValueType();
      }

      log.info("debug point - 1");
      int attributeversion = attribute.getAttributeVersion();
      if (attributeversion != 0) {
        version = attributeversion;
      }

      log.info("debug point - 2");
      AttributeValue attributeValue = attribute.
            getAttributeValue();
      if (value_type.equals("text")) {
        if (attributeValue != null) {
          value_string = attributeValue.getAttributeValueAsString();
       }

        // prepare and execute the insert
        statement = connection.prepareStatement(insertSQL2);
        statement.setString(1, attribute.getAttributeKey()); //THIS WILL BE PROPERTY KEY
        statement.setString(2, null);
        statement.setString(3, identifier);
        statement.setString(4, value_type);
        statement.setString(5, value_string);
        statement.setInt(6, version);
      }
      else if (value_type.equals("xpath")) {
          if (attributeValue != null) {
              value_xml = attributeValue.getAttributeValueAsString();
              byte [] bytearray = value_xml.getBytes();
              newblob = new cgl.util.MyByteBlob(bytearray);
              log.info("length = " + newblob.length());
        }
        // prepare and execute the insert
        statement = connection.prepareStatement(insertSQL3);
        statement.setString(1, attribute.getAttributeKey()); //THIS WILL BE PROPERTY KEY
        statement.setString(2, null);
        statement.setString(3, identifier);
        statement.setString(4, value_type);
        statement.setBlob(5, newblob);
        statement.setInt(6, version);

      }
      else if (value_type.equals("number")) {
        if (attributeValue != null) {
          value_number = attributeValue.getAttributeValueAsNumber();
        }
        // prepare and execute the insert
        statement = connection.prepareStatement(insertSQL1);
        statement.setString(1, attribute.getAttributeKey()); //THIS WILL BE PROPERTY KEY
        statement.setString(2, null);
        statement.setString(3, identifier);
        statement.setString(4, value_type);
        statement.setFloat(5, value_number);
        statement.setInt(6, version);
      }
      statement.executeUpdate();
    }
    finally {
      try {
        statement.close();
      }
      catch (Exception e) {
        e.printStackTrace();
        /* ignored */}

    }
  }

  /**
   * Delete multiple rows from the PROPERTY_ENTITY table that are assigned to
   * the attributeKey specified.
   *
   * @param  attributeKey AttributeKey
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void deleteByAttributeKey(String attributeKey,
                                          Connection connection) throws java.sql.
      SQLException {
    PreparedStatement statement = null;

    try {
      // prepare the delete
      statement = connection.prepareStatement(deleteByAttributeKeySQL);
      statement.setString(1, attributeKey);

      log.info("delete from PROPERTY_ENTITY table:\n\n\t" +
                deleteByAttributeKeySQL +
                "\n\t PROPERTY_KEY=" + attributeKey + "\n");

      // execute
      int returnCode = statement.executeUpdate();

      log.info("delete was successful, rows deleted=" + returnCode);
    }
    finally {
      try {
        statement.close();
      }
      catch (Exception e) { /* ignored */}
    }
  }


  /**
   * Select one row from the PROPERTY_ENTITY table.
   *
   * @param attributeKey primary key value
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Attribute select(String attributeKey,Connection connection)
    throws java.sql.SQLException
  {
    Attribute attribute = null;
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    java.sql.Date sql_end_date = null, sql_begin_date = null;

    try
    {
      statement = connection.prepareStatement(selectSQL);
      statement.setString(1,attributeKey.toString());

      log.info("select from PROPERTY_ENTITY table:\n\n\t" + selectSQL +
        "\n\t PROPERTY_KEY=" + attributeKey.toString() + "\n");

      resultSet = statement.executeQuery();
      if (resultSet.next())
      {
        attribute = new Attribute();
        String attributeType = resultSet.getString(1); //("PROPERTY_SUBJECT");
        String attributeIdentifier = resultSet.getString(2); //("PROPERTY_NAME");
        String attributeValueType = resultSet.getString(3); //("PROPERTY_TYPE");
        float attributeValueNumber = 0;
        String attributeValueString = null;
        byte [] attributeValueXML = null;
        java.sql.Blob newblob = null;

        if (attributeValueType != null) {
          if (attributeValueType.equals("text")) {
            attributeValueString = resultSet.getString(5); //("PROPERTY_VALUE_STRING");
          }
          else if (attributeValueType.equals("xpath")) {

              newblob = resultSet.getBlob(6);


              java.io.InputStream is = null;
              byte [] bytes = null;
              try {
                  is = newblob.getBinaryStream();
                  bytes = new byte[(int) newblob.length()];
                  is.read(bytes);
              } catch (java.sql.SQLException e) {
                  bytes = null;
              } catch (java.io.IOException e) {
                  bytes = null;
              } finally {
                  try {
                      if (is != null) {
                          is.close();
                      }
                  } catch (Exception e) {}
          }
          //attributeValueXML = (new String(bytes)).toString();
          attributeValueXML = bytes;
          }
          else if (attributeValueType.equals("number")) {
            attributeValueNumber = resultSet.getFloat(4); //("PROPERTY_VALUE_NUMBER");
          }
        }


        int attributeVersion = resultSet.getInt(7); //("PROPERTYVERSION");

        if (attributeKey != null) {
          attribute.setAttributeKey(attributeKey);
          log.info("AttributeTable = > attributeKey " + attributeKey);
        }
        if (attributeIdentifier != null) {
          attribute.setAttributeName(new AttributeName(attributeIdentifier));
          log.info("AttributeTable = > attributeIdentifier " + attributeIdentifier);
        }
        if (attributeValueType != null) {
          attribute.setAttributeValueType(new AttributeValueType(attributeValueType));
          log.info("AttributeTable = > attributeValueType = " + attributeValueType);
        }
        if (attributeValueString != null) {
          attribute.setAttributeValue(new AttributeValue(attributeValueString));
          log.info("AttributeTable = > attributeValue = " + attributeValueString);
        }
        else if (attributeValueXML != null) {
          attribute.setAttributeValue(new AttributeValue(attributeValueXML));
          log.info("AttributeTable = > attributeValue = " + attributeValueXML);
        }
        else if (attributeValueNumber != 0) {
          attribute.setAttributeValue(new AttributeValue(attributeValueNumber));
          log.info("AttributeTable = > attributeValue = " + attributeValueNumber);
        }
        if (attributeVersion != 0) {
          attribute.setAttributeVersion(attributeVersion);
          log.info("AttributeTable => attributeVersion = " + attributeVersion);
        }
      }
      return attribute;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/

  public static void main(String[] args) throws Exception {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool", "true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("uddi");
      test(conn);
    }
    finally {
      if (conn != null) {
        conn.close();
      }
    }
  }

  public static void test(Connection connection) throws Exception {
    Transaction txn = new Transaction();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    if (connection != null) {
      try {
        // begin a new transaction
        txn.begin(connection);
        String[] attributeKey = uuidgen.uuidgen(4);
        //AttributeBag attributeBag = new AttributeBag();
        Attribute attribute = new Attribute();
        String attribute_key = attributeKey[0];
        attribute.setAttributeKey(attribute_key);
        attribute.setServiceKey("5FA9EF00-5229-11DA-AF00-A16090B517E0");
        attribute.setAttributeName(new AttributeName("cpu_deneme"));
        attribute.setAttributeValueType(new AttributeValueType("number"));
        attribute.setAttributeValue(new AttributeValue("3"));



        //attributeBag.addAttribute(attribute);
        // insert a new BusinessEntity
        //for (Iterator iter = attributeBag.getAttributeVector().iterator();
        //     iter.hasNext(); ) {
        //  Attribute item = (Attribute) iter.next();

          AttributeTable.insert(attribute, connection);
          Attribute result_attribute = AttributeTable.select(attribute.getAttributeKey(), connection);
          log.info("result_attribute key = " + result_attribute.getAttributeKey());
          log.info("name = " + result_attribute.getAttributeName().getAttributeName());
          log.info("type = " + result_attribute.getAttributeValueType().getAttributeValueType());
          log.info("value = " + result_attribute.getAttributeValue().getAttributeValue());
          log.info("version = " + result_attribute.getAttributeVersion());


          //generate lease
          Lease lease = new Lease();
          lease.setObjectKey(attribute_key);
          lease.setIsInfinite("false");
          lease.setLeaseTimeOut("1000");
          //Lease table for attribute
          LeaseTable.insertObjectLease(lease, connection);

          //generate category
          Vector keyRefs = new Vector();
          KeyedReference keyRef = new KeyedReference();
          keyRef.setKeyName("propertyType");
          keyRef.setKeyValue("QOS");
          keyRefs.add(keyRef);
          //AttributeCategoryTable
          AttributeCategoryTable.insert(attribute.getAttributeKey(), keyRefs, connection);

          //ServiceAttribute Table
          ServiceAttributeTable.insert(attribute, connection);
    //}
        // commit the transaction
        txn.commit();
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }
}
