/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.fthpis.datatype.publisher.Publisher;
import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.AutharizationToken;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class AuthTokenTable
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(AuthTokenTable.class);

  static String insertSQL = null;
  static String selectPublisherSQL = null;
  static String deleteSQL = null;
  static String touchSQL = null;
  static String selectLastUsedSQL = null;
  static String invalidateSQL = null;
  static String selectTokenStateSQL = null;
  static String selectAllTokenStateSQL = null;

  static
  {
    // buffer used to build SQL statements
    StringBuffer sql = null;

    // build insertSQL
    sql = new StringBuffer(350);
    sql.append("INSERT INTO AUTH_TOKEN (");
    sql.append("AUTH_TOKEN,");
    sql.append("PUBLISHER_ID,");
    sql.append("PUBLISHER_NAME,");
    sql.append("CREATED,");
    sql.append("LAST_USED,");
    sql.append("NUMBER_OF_USES,");
    sql.append("TOKEN_STATE) ");
    sql.append("VALUES (?,?,?,?,?,0,1)");
    insertSQL = sql.toString();

    // build selectPublisherSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("PUBLISHER_ID,");
    sql.append("PUBLISHER_NAME ");
    sql.append("FROM AUTH_TOKEN ");
    sql.append("WHERE AUTH_TOKEN=?");
    selectPublisherSQL = sql.toString();

    // build deleteSQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM AUTH_TOKEN ");
    sql.append("WHERE AUTH_TOKEN=?");
    deleteSQL = sql.toString();

    // build touchSQL
    sql = new StringBuffer(150);
    sql.append("UPDATE AUTH_TOKEN ");
    sql.append("SET LAST_USED=?,");
    sql.append("NUMBER_OF_USES=NUMBER_OF_USES+1 ");
    sql.append("WHERE AUTH_TOKEN=? ");
    touchSQL = sql.toString();

    // build selectLastUsedSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("LAST_USED ");
    sql.append("FROM AUTH_TOKEN ");
    sql.append("WHERE AUTH_TOKEN=?");
    selectLastUsedSQL = sql.toString();

    // build invalidateSQL
    sql = new StringBuffer(100);
    sql.append("UPDATE AUTH_TOKEN ");
    sql.append("SET LAST_USED=?,");
    sql.append("NUMBER_OF_USES=NUMBER_OF_USES+1,");
    sql.append("TOKEN_STATE=0 ");
    sql.append("WHERE AUTH_TOKEN=? ");
    invalidateSQL = sql.toString();

    // build selectTokenStateSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("TOKEN_STATE ");
    sql.append("FROM AUTH_TOKEN ");
    sql.append("WHERE AUTH_TOKEN=?");
    selectTokenStateSQL = sql.toString();

//    THING_TO_DO:
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("PUBLISHER_ID,");
    sql.append("PUBLISHER_NAME,");
    sql.append("CREATED,");
    sql.append("LAST_USED,");
    sql.append("NUMBER_OF_USES,");
    sql.append("TOKEN_STATE ");
    sql.append("FROM AUTH_TOKEN ");
    sql.append("WHERE AUTH_TOKEN=?");
    selectAllTokenStateSQL = sql.toString();

  }

  /**
   * Insert new row into the AUTH_TOKEN table.
   *
   * @param authToken
   * @param publisher
   * @param connection JDBC connection
   * @throws SQLException
   */
  public static void insert(String authToken,Publisher publisher,Connection connection)
    throws SQLException
  {
    PreparedStatement statement = null;
    Timestamp timestamp = new Timestamp(System.currentTimeMillis());

    try
    {
      statement = connection.prepareStatement(insertSQL);
      statement.setString(1,authToken);
      statement.setString(2,publisher.getPublisherID());
      statement.setString(3,publisher.getName());
      statement.setTimestamp(4,timestamp);
      statement.setTimestamp(5,timestamp);

      log.debug("insert into AUTH_TOKEN table:\n\n\t" + insertSQL +
        "\n\t AUTH_TOKEN=" + authToken +
        "\n\t PUBLISHER_ID=" + publisher.getPublisherID() +
        "\n\t PUBLISHER_NAME=" + publisher.getName() +
        "\n\t CREATED=" + timestamp.toString() +
        "\n\t LAST_USED=" + timestamp.toString() +
        "\n\t NUMBER_OF_USES=1" +
        "\n\t TOKEN_STATE=1\n");

      statement.executeUpdate();
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Select one row from the AUTH_TOKEN table.
   *
   * @param authToken
   * @param connection JDBC connection
   * @return Publisher The publisher's info
   * @throws SQLException
   */
  public static Vector selectALLAuthToken(String token,Connection connection)
    throws SQLException
  {
    Vector authTokenVector = new Vector();
    AutharizationToken authToken = null;
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("AuthToken.selectPublisher => A AUTH_TOKEN request is made to the UDDI Registry");
    try
    {
      statement = connection.prepareStatement(selectAllTokenStateSQL);
      statement.setString(1,token);

      log.debug("select from AUTH_TOKEN table:\n\n\t" + selectAllTokenStateSQL +
        "\n\t AUTH_TOKEN=" + authToken + "\n");

      resultSet = statement.executeQuery();
      if (resultSet.next())
      {
        authToken = new AutharizationToken();
        authToken.setAuthtoken(token);
        authToken.setPublisher_ID(resultSet.getString(1));//("PUBLISHER_ID"));
        authToken.setPublisher_Name(resultSet.getString(2));//("PUBLISHER_NAME"));
        authToken.set_created(resultSet.getTimestamp(3));//("CREATED");
        authToken.set_lastUsed(resultSet.getTimestamp(4));//("LAST_USED");
        authToken.set_numOfUses(resultSet.getInt(5));//("NUM_OF_USES");
        authToken.set_token_state(resultSet.getInt(6));//("TOKEN_STATE");
        authTokenVector.add(authToken);
      }

      return authTokenVector;
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Select one row from the AUTH_TOKEN table.
   *
   * @param authToken
   * @param connection JDBC connection
   * @return Publisher The publisher's info
   * @throws SQLException
   */
  public static AutharizationToken selectAuthToken(String token,Connection connection)
    throws SQLException
  {
    AutharizationToken authToken = null;
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("AuthToken.selectPublisher => A AUTH_TOKEN request is made to the UDDI Registry");
    try
    {
      statement = connection.prepareStatement(selectAllTokenStateSQL);
      statement.setString(1,token);

      log.debug("select from AUTH_TOKEN table:\n\n\t" + selectAllTokenStateSQL +
        "\n\t AUTH_TOKEN=" + token + "\n");

      resultSet = statement.executeQuery();
      if (resultSet.next())
      {
        authToken = new AutharizationToken();
        authToken.setAuthtoken(token);
        authToken.setPublisher_ID(resultSet.getString(1));//("PUBLISHER_ID"));
        authToken.setPublisher_Name(resultSet.getString(2));//("PUBLISHER_NAME"));
        authToken.set_created(resultSet.getTimestamp(3));//("CREATED");
        authToken.set_lastUsed(resultSet.getTimestamp(4));//("LAST_USED");
        authToken.set_numOfUses(resultSet.getInt(5));//("NUM_OF_USES");
        authToken.set_token_state(resultSet.getInt(6));//("TOKEN_STATE");
      }

      return authToken;
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Select one row from the AUTH_TOKEN table.
   *
   * @param authToken
   * @param connection JDBC connection
   * @return Publisher The publisher's info
   * @throws SQLException
   */
  public static Publisher selectPublisher(String authToken,Connection connection)
    throws SQLException
  {
    Publisher publisher = null;
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("AuthToken.selectPublisher => A AUTH_TOKEN request is made to the UDDI Registry");
    try
    {
      statement = connection.prepareStatement(selectPublisherSQL);
      statement.setString(1,authToken);

      log.debug("select from AUTH_TOKEN table:\n\n\t" + selectPublisherSQL +
        "\n\t AUTH_TOKEN=" + authToken + "\n");

      resultSet = statement.executeQuery();
      if (resultSet.next())
      {
        publisher = new Publisher();
        publisher.setPublisherID(resultSet.getString(1));//("PUBLISHER_ID"));
        publisher.setName(resultSet.getString(2));//("PUBLISHER_NAME"));
      }

      return publisher;
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { resultSet.close(); } catch (Exception e) { /* ignored */ }
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Delete row from the AUTH_TOKEN table.
   *
   * @param authToken
   * @param connection
   * @throws SQLException
   */
  public static void delete(String authToken,Connection connection)
    throws SQLException
  {
    log.info("AuthToken.delete => A AUTH_TOKEN deletion request is made to the UDDI Registry");
    PreparedStatement statement = null;

    try
    {
      // prepare the delete
      statement = connection.prepareStatement(deleteSQL);
      statement.setString(1,authToken);

      log.debug("delete from AUTH_TOKEN table:\n\n\t" + deleteSQL +
        "\n\t AUTH_TOKEN=" + authToken + "\n");

      // execute
      statement.executeUpdate();
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Update the PUBLISHER table for a particular PublisherID.
   *
   * @param connection JDBC connection
   * @throws SQLException
   */
  public static void touch(String authToken,Connection connection)
    throws SQLException
  {
    PreparedStatement statement = null;
    Timestamp timestamp = new Timestamp(System.currentTimeMillis());

    try
    {
      // create a statement to query with
      statement = connection.prepareStatement(touchSQL);
      statement.setTimestamp(1,timestamp);
      statement.setString(2,authToken);

      log.debug("update AUTH_TOKEN table:\n\n\t" + touchSQL +
       "\n\t AUTH_TOKEN=" + authToken +
       "\n\t LAST_USED=" + timestamp.toString() + "\n");

      // execute
      statement.executeUpdate();
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Update the PUBLISHER table for a particular PublisherID.
   *
   * @param connection JDBC connection
   * @throws SQLException
   */
  public static long selectLastUsed(String authToken,Connection connection)
    throws SQLException
  {
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    long lastUsed = -1;

    try
    {
      // create a statement to query with
      statement = connection.prepareStatement(selectLastUsedSQL);
      statement.setString(1,authToken);

      log.debug("select LAST_USED from AUTH_TOKEN table:\n\n\t" + selectLastUsedSQL +
       "\n\t AUTH_TOKEN=" + authToken + "\n");

      resultSet = statement.executeQuery();
      if (resultSet.next())
      {
        Timestamp timestamp = resultSet.getTimestamp("LAST_USED");
        if (timestamp != null)
          lastUsed = timestamp.getTime();
      }
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }

    return lastUsed;
  }

  /**
   * Update the AUTH_TOKEN table's LAST_USED property for a
   * particular authToken.
   *
   * @param connection JDBC connection
   * @throws SQLException
   */
  public static void invalidate(String authToken,Connection connection)
    throws SQLException
  {
    PreparedStatement statement = null;
    Timestamp timestamp = new Timestamp(System.currentTimeMillis());

    try
    {
      // create a statement to query with
      statement = connection.prepareStatement(invalidateSQL);
      statement.setTimestamp(1,timestamp);
      statement.setString(2,authToken);

      log.debug("update AUTH_TOKEN table:\n\n\t" + invalidateSQL +
        "\n\t LAST_USED=" + timestamp.toString() +
        "\n\t AUTH_TOKEN=" + authToken + "\n");

      // execute
      statement.executeUpdate();
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }
  }

  /**
   * Query the token's state. This is used to determine if the authToken
   * has been discarded, for example.
   *
   * @param authToken
   * @param connection
   * @throws SQLException
   */
  public static long selectTokenState(String authToken, Connection connection)
      throws SQLException
  {
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    long tokenState = -1;

    try
    {
      // create a statement to query with
      statement = connection.prepareStatement(selectTokenStateSQL);
      statement.setString(1,authToken);

      log.debug("SQL Statement: [" + selectTokenStateSQL + "], parameter authToken: [" + authToken + "]");

      resultSet = statement.executeQuery();
      if (resultSet.next())
      {
        tokenState = resultSet.getLong("TOKEN_STATE");
      }
    }
    catch(SQLException sqlex)
    {
      log.error(sqlex.getMessage());
      throw sqlex;
    }
    finally
    {
      try { statement.close(); } catch (Exception e) { /* ignored */ }
    }

    return tokenState;
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("wscontext");
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection)
    throws Exception
  {
    Transaction txn = new Transaction();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    if (connection != null)
    {
      try
      {
        // begin a new transaction
        txn.begin(connection);

        String authToken1 = "authToken:"+uuidgen.uuidgen();
        AuthTokenTable.insert(authToken1,new Publisher("uddi_user","Mehmet Aktas"),connection);

        AutharizationToken authtoken = AuthTokenTable.selectAuthToken(authToken1, connection);

        log.info("authoken = " + authtoken.getAuthtoken());
        log.info("publisherID = " + authtoken.getPublisher_ID());
        log.info("name = " + authtoken.getPublisher_Name());
        log.info("created = " + authtoken.get_created());
        log.info("last used = " + authtoken.get_lastUsed());
        log.info("number of uses = " + authtoken.get_numOfUses());
        log.info("token state = " + authtoken.get_token_state());


        // commit the transaction
        txn.commit();
      }
      catch(Exception ex)
      {
        try { txn.rollback(); }
        catch(java.sql.SQLException sqlex) { sqlex.printStackTrace(); }
        throw ex;
      }
    }
  }
}
