/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.Attribute;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class ContextNameTable
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(ContextNameTable.class);

  static String insertSQL = null;
  static String selectSQL = null;
  static String deleteSQL = null;

  static {
    // buffer used to build SQL statements
    StringBuffer sql = null;

    // build insertSQL
    sql = new StringBuffer(150);
    sql.append("INSERT INTO CONTEXT_NAME (");
    sql.append("CONTEXT_KEY,");
    sql.append("CONTEXT_NAME_ID,");
    sql.append("LANG_CODE,");
    sql.append("NAME) ");
    sql.append("VALUES (?,?,?,?)");
    insertSQL = sql.toString();

    // build selectSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("LANG_CODE,");
    sql.append("NAME, ");
    sql.append("CONTEXT_NAME_ID ");
    sql.append("FROM CONTEXT_NAME ");
    sql.append("WHERE CONTEXT_KEY=? ");
    sql.append("ORDER BY CONTEXT_NAME_ID");
    selectSQL = sql.toString();

    // build deleteSQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM CONTEXT_NAME ");
    sql.append("WHERE CONTEXT_KEY=?");
    deleteSQL = sql.toString();
  }

  /**
   * Insert new row into the CONTEXT_NAME table.
   *
   * @param  contextKey String to the ServiceAttribute (Context) object
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void insert(
    String contextKey,
    Vector nameList,
    Connection connection)
    throws java.sql.SQLException
  {
    if ((nameList == null) || (nameList.size() == 0))
      return; // everything is valid but no elements to insert

    PreparedStatement statement = null;
    try
    {
      statement = connection.prepareStatement(insertSQL);
      statement.setString(1, contextKey.toString());

      int listSize = nameList.size();
      for (int nameID = 0; nameID < listSize; nameID++)
      {
        Object sr = (Object) nameList.elementAt(nameID);
        Name name = (Name) nameList.elementAt(nameID);

        statement.setInt(2, nameID);
        statement.setString(3, name.getLanguageCode());
        statement.setString(4, name.getValue());

        log.info(
          "insert into CONTEXT_NAME table:\n\n\t"
            + insertSQL
            + "\n\t CONTEXT_KEY="
            + contextKey.toString()
            + "\n\t CONTEXT_NAME_ID="
            + nameID
            + "\n\t LANG_CODE="
            + name.getLanguageCode()
            + "\n\t NAME="
            + name.getValue()
            + "\n");

        statement.executeUpdate();
      }
    }
    finally
    {
      try
      {
        statement.close();
      }
      catch (Exception e)
      { /* ignored */
          e.printStackTrace();
      }
    }
  }

  /**
   * Select all rows from the CONTEXT_NAME table for a given contextKey.
   *
   * @param  contextKey String
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector select(String contextKey, Connection connection)
    throws java.sql.SQLException
  {
    Vector nameList = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    try
    {
      // create a statement to query with
      statement = connection.prepareStatement(selectSQL);
      statement.setString(1, contextKey.toString());

      log.info(
        "select from the CONTEXT_NAME table:\n\n\t"
          + selectSQL
          + "\n\t CONTEXT_KEY="
          + contextKey.toString()
          + "\n");

      // execute the statement
      resultSet = statement.executeQuery();
      while (resultSet.next())
      {
        Name name = new Name();
        //log.info("context name = " + resultSet.getString(2));
        name.setValue(resultSet.getString(2));//("NAME"));
        name.setLanguageCode(resultSet.getString(1));//("LANG_CODE"));
        nameList.add(name);
      }

      return nameList;
    }
    finally
    {
      try
      {
        resultSet.close();
        statement.close();
      }
      catch (Exception e)
      { /* ignored */
      }
    }
  }

  /**
   * Delete multiple rows from the CONTEXT_NAME table that are assigned
   * to the contextKey specified.
   *
   * @param  contextKey String
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void delete(String contextKey, Connection connection)
    throws java.sql.SQLException
  {
    PreparedStatement statement = null;

    try
    {
      // prepare the delete
      statement = connection.prepareStatement(deleteSQL);
      statement.setString(1, contextKey.toString());

      log.info(
        "delete from the CONTEXT_NAME table:\n\n\t"
          + deleteSQL
          + "\n\t CONTEXT_KEY="
          + contextKey.toString()
          + "\n");

      // execute
      statement.executeUpdate();
    }
    finally
    {
      try
      {
        statement.close();
      }
      catch (Exception e)
      { /* ignored */
      }
    }
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("wscontext");
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection) throws Exception
  {
    Transaction txn = new Transaction();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    if (connection != null)
    {
      try
      {
        String contextKey = uuidgen.uuidgen_with_timestamp();
        Attribute context = new Attribute();
        context.setAttributeKey(contextKey);

        Vector nameList = new Vector();
        nameList.add(new Name("Mehmet"));

        // begin a new transaction
        txn.begin(connection);

        // insert a Collection of Name objects
        ContextNameTable.insert(contextKey, nameList, connection);

        // select the Collection of Name objects
        nameList = ContextNameTable.select(contextKey, connection);

        // delete the Collection of Name objects
        //ContextNameTable.delete(contextKey,connection);

        // re-select the Collection of Name objects
        nameList = ContextNameTable.select(contextKey, connection);

        // commit the transaction
        txn.commit();
      }
      catch (Exception ex)
      {
        try
        {
          txn.rollback();
        }
        catch (java.sql.SQLException sqlex)
        {
          sqlex.printStackTrace();
        }
        throw ex;
      }
    }
  }
}
