/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.publisher.PublisherID;
import cgl.fthpis.datatype.attribute.Attribute;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class ContextPublisherTable {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(ContextPublisherTable.class);

    static String insertSQL = null;
    static String deleteSQL = null;
    static String selectSQL = null;
    static String selectByContextKeySQL = null;
    static String selectBySessionKeySQL = null;
    static String deleteByContextKeySQL = null;
    static String deleteByContextKeyAndPublisherIDSQL = null;
    static String selectPublishersBySessionKeySQL = null;
    static String verifyOwnershipSQL = null;

    static {
        // buffer used to build SQL statements
        StringBuffer sql = null;

        // build insertSQL
        sql = new StringBuffer(100);
        sql.append("INSERT INTO CONTEXT_PUBLISHER (");
        sql.append("CONTEXT_KEY,");
        sql.append("PUBLISHER_ID,");
        sql.append("LAST_UPDATE) ");
        sql.append("VALUES (?,?,?)");
        insertSQL = sql.toString();

        // build deleteSQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM CONTEXT_PUBLISHER ");
        sql.append("WHERE PUBLISHER_ID=?");
        deleteSQL = sql.toString();

        // build selectSQL
        sql = new StringBuffer(100);
        sql.append("SELECT ");
        sql.append("CONTEXT_KEY ");
        sql.append("FROM CONTEXT_PUBLISHER ");
        sql.append("WHERE PUBLISHER_ID=?");
        selectSQL = sql.toString();

        // build selectByBusinessKeySQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("PUBLISHER_ID ");
        sql.append("FROM CONTEXT_PUBLISHER ");
        sql.append("WHERE CONTEXT_KEY=?");
        selectByContextKeySQL = sql.toString();

        // build selectByBusinessKeySQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("CONTEXT_KEY ");
        sql.append("FROM SESSION_CONTEXT ");
        sql.append("WHERE SESSION_KEY=?");
        selectBySessionKeySQL = sql.toString();

        // build deleteByBusinessKeySQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM CONTEXT_PUBLISHER ");
        sql.append("WHERE CONTEXT_KEY=?");
        deleteByContextKeySQL = sql.toString();

        // build deleteByBusinessKeySQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM CONTEXT_PUBLISHER ");
        sql.append("WHERE CONTEXT_KEY=? ");
        sql.append("AND PUBLISHER_ID=?");
        deleteByContextKeyAndPublisherIDSQL = sql.toString();

        // build verifyOwnershipSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("* ");
        sql.append("FROM CONTEXT_ENTITY e, CONTEXT_PUBLISHER s ");
        sql.append("WHERE e.CONTEXT_KEY = s.CONTEXT_KEY ");
        sql.append("AND s.CONTEXT_KEY=? ");
        sql.append("AND e.PUBLISHER_ID=?");
        verifyOwnershipSQL = sql.toString();

        // build verifyOwnershipSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("s.PUBLISHER_ID ");
        sql.append("FROM SESSION_CONTEXT e, CONTEXT_PUBLISHER s ");
        sql.append("WHERE e.CONTEXT_KEY = s.CONTEXT_KEY ");
        sql.append("AND e.SESSION_KEY=?");
        selectPublishersBySessionKeySQL = sql.toString();

    }


    /**
     *
     * @throws java.sql.SQLException
     */
    public static void insert(String contextKey, Vector publisherIDVector,
                              Connection connection) throws java.sql.SQLException {
        log.info("cgl.fthpis.datastore.jdbc.SessionService.insert() function called.");
        if (publisherIDVector != null && publisherIDVector.size() != 0) {
            for (Iterator iter = publisherIDVector.iterator(); iter.hasNext(); ) {
                PublisherID id = (PublisherID) iter.next();
                ////////////////////////////////////////////////////////////////
                    PreparedStatement statement = null;
                    Timestamp timeStamp = new Timestamp(System.currentTimeMillis());
                    try {
                        statement = connection.prepareStatement(insertSQL);
                        statement.setString(1, contextKey);
                        statement.setString(2, id.getValue());
                        statement.setTimestamp(3, timeStamp);

                        log.debug("insert into CONTEXT_PUBLISHER table:\n\n\t" + insertSQL +
                                  "\n\t CONTEXT_KEY=" + contextKey +
                                  "\n\t PUBLISHER_ID=" + id.getValue() +
                                  "\n\t LAST_UPDATE=" + timeStamp.getTime() +
                                  "\n");

                        statement.executeUpdate();
                    } finally {
                        try {
                            statement.close();
                        } catch (Exception e) { /* ignored */}
                    }
                ////////////////////////////////////////////////////////////////
            }
        }
    }

    /**
     *
     * @throws java.sql.SQLException
     */
    public static void delete(String publisherID, Connection connection) throws
            java.sql.SQLException {
        PreparedStatement statement = null;

        try {
            // prepare
            statement = connection.prepareStatement(deleteSQL);
            statement.setString(1, publisherID.toString());

            log.debug("delete from CONTEXT_PUBLISHER table:\n\n\t" + deleteSQL +
                      "\n\t PUBLISHER_ID=" + publisherID.toString() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     *
     * @throws java.sql.SQLException
     */
    public static Vector select(String publisherID, Connection connection) throws
            java.sql.SQLException {
        Vector contextKeyVector = null;
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(selectSQL);
            statement.setString(1, publisherID.toString());

            log.info("select from CONTEXT_PUBLISHER table:\n\n\t" + selectSQL +
                     "\n\t PUBLISHER_ID=" + publisherID.toString() + "\n");

            resultSet = statement.executeQuery();
            contextKeyVector = new Vector();
            while (resultSet.next()) {
                contextKeyVector.add(resultSet.getString(1));
            }
            return contextKeyVector;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Delete multiple rows from the CONTEXT_ENTITY table that are assigned to
     * the ContextKey specified.
     *
     * @param contextKey contextKey
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void deleteByContextKey(String contextKey,
                                          Connection connection) throws
            java.sql.SQLException {
        PreparedStatement statement = null;

        try {
            // prepare the delete
            statement = connection.prepareStatement(deleteByContextKeySQL);
            statement.setString(1, contextKey.toString());

            log.debug("delet from the CONTEXT_PUBLISHER table:\n\n\t" +
                      deleteByContextKeySQL +
                      "\n\t CONTEXT_KEY=" + contextKey.toString() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Select all rows from the context_enitity table for
     * a given ContextKey.
     *
     * @param contextKey ContextKey
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector selectByContextKey(String contextKey,
                                            Connection connection) throws java.
            sql.
            SQLException {
        Vector publisherIDList = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            // create a statement to query with
            statement = connection.prepareStatement(selectByContextKeySQL);
            statement.setString(1, contextKey.toString());

            log.debug("select from CONTEXT_PUBLISHER table:\n\n\t" +
                      selectByContextKeySQL +
                      "\n\t CONTEXT_KEY=" + contextKey.toString() + "\n");

            // execute the statement
            resultSet = statement.executeQuery();
            while (resultSet.next()) {
                PublisherID publisherID = new PublisherID();
                publisherID.setValue(resultSet.getString(1));
                publisherIDList.add(publisherID);
            }

            return publisherIDList;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Verify that 'authorizedName' has the authority to update or delete
     * ContextEntity identified by the contextKey parameter
     *
     * @param contextKey
     * @param publisherID
     * @param connection
     * @throws java.sql.SQLException
     */
    public static boolean verifyOwnership(String contextKey, String publisherID,
                                          Connection connection) throws java.
            sql.
            SQLException {
        if ((contextKey == null) || (publisherID == null)) {
            return false;
        }

        boolean authorized = false;
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            statement = connection.prepareStatement(verifyOwnershipSQL);
            statement.setString(1, contextKey);
            statement.setString(2, publisherID);

            log.debug("checking ownership of CONTEXT_PUBLISHER:\n\n\t" +
                      verifyOwnershipSQL +
                      "\n\t CONTEXT_KEY=" + contextKey +
                      "\n\t PUBLISHER_ID=" + publisherID + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                authorized = true;
            }

            return authorized;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        Transaction txn = new Transaction();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        if (connection != null) {
            try {
                String contextKey = uuidgen.uuidgen_with_timestamp();

                Attribute context = new Attribute();
                context.setAttributeKey(contextKey);

                Vector publisherIDVector = new Vector();

                PublisherID publisherID = new PublisherID();
                publisherID.setValue("ali");
                publisherIDVector.add(publisherID);

                PublisherID publisher_ID = new PublisherID();
                publisher_ID.setValue("veli");
                publisherIDVector.add(publisher_ID);

                //context.setPublisherIDVector(publisherIDVector);

                // begin a new transaction
                txn.begin(connection);

                ContextPublisherTable.insert(contextKey, publisherIDVector, connection);

                // select a SessionService object
                Vector contextKeyVector = ContextPublisherTable.select("veli", connection);
                for (Iterator iter = contextKeyVector.iterator(); iter.hasNext(); ) {
                    String session_Key = (String) iter.next();
                    log.info("contextKey = " + session_Key);

                    Vector publisherID_Vector = ContextPublisherTable.
                                                selectByContextKey(session_Key,
                            connection);
                    for (Iterator iterator = publisherID_Vector.iterator();
                                             iterator.hasNext(); ) {
                        PublisherID publisher_id = (PublisherID) iterator.next();
                        log.info("publisher_id = " +
                                           publisher_id.getValue());
                    }
                }

//                // delete a SessionService object
//                ContextPublisherTable.delete(contextKey, connection);

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
