/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.util.MyByteBlob;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.AttributeAccessInfos;
import cgl.fthpis.datatype.attribute.AttributeAccessInfo;
import cgl.fthpis.datatype.publisher.PublisherID;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class ContextTable {

    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(ContextTable.class);

    static String insertSQL = null;
    static String selectSQL = null;
    static String deleteByContextKeySQL = null;
    static String verifyOwnershipSQL = null;


    static {
        // buffer used to build SQL statements
        StringBuffer sql = null;

        // build insertSQL
        sql = new StringBuffer(150);
        sql.append("INSERT INTO CONTEXT_ENTITY (");
        sql.append("CONTEXT_KEY,");
        sql.append("CONTEXTTYPE,");
        sql.append("OWNER_PUBLISHERID,");
        sql.append("OWNER_AUTH_INFO,");
        sql.append("GROUP_AUTH_INFO,");
        sql.append("OTHERS_AUTH_INFO,");
        sql.append("CONTEXTVALUEXML,");
        sql.append("CONTEXTVERSION) ");
        sql.append("VALUES (?,?,?,?,?,?,?,?)");
        insertSQL = sql.toString();

        // build selectSQL
        sql = new StringBuffer(100);
        sql.append("SELECT ");
        sql.append("CONTEXTTYPE,");
        sql.append("OWNER_PUBLISHERID,");
        sql.append("OWNER_AUTH_INFO,");
        sql.append("GROUP_AUTH_INFO,");
        sql.append("OTHERS_AUTH_INFO,");
        sql.append("CONTEXTVALUEXML,");
        sql.append("CONTEXTVERSION ");
        sql.append("FROM CONTEXT_ENTITY ");
        sql.append("WHERE CONTEXT_KEY = ? ");
        selectSQL = sql.toString();

        // build deleteByServiceKeySQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM CONTEXT_ENTITY ");
        sql.append("WHERE CONTEXT_KEY = ?");
        deleteByContextKeySQL = sql.toString();
    }

    /**
     *
     * @param  context Attribute holding values to be inserted
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void insert(Attribute context, Connection connection) throws
            java.sql.SQLException {
        log.info(
                "cgl.fthpis.datastore.jdbc.ContextTable.insert() function called.");
        PreparedStatement statement = null;

        try {
            String publisher_id = null;
            String owner_auth_info = "ReadWrite";
            String group_auth_info = null;
            String others_auth_info = null;

            String context_value_xml = null;
            java.sql.Blob context_value_blob = null;
            int context_version = 0;

            PublisherID publisherID = context.geAttributeOwnerPublisherID();
            if (publisherID != null) {
                publisher_id = publisherID.getValue();
            }

            AttributeAccessInfos access_infos = context.getAttributeAccessRightInfos();
            if (access_infos != null) {
                AttributeAccessInfo group_access_info = access_infos.
                        getGroupAttributeAccessInfo();
                if (group_access_info != null) {
                    if (group_access_info.isReadWriteOK()) {
                        group_auth_info = "ReadWrite";
                    } else if (group_access_info.isReadOK()) {
                        group_auth_info = "Read";
                    }
                }
                AttributeAccessInfo others_access_info = access_infos.
                        getOthersAttributeAccessInfo();
                if (others_access_info != null) {
                    if (others_access_info.isReadWriteOK()) {
                        others_auth_info = "ReadWrite";
                    } else if (others_access_info.isReadOK()) {
                        others_auth_info = "Read";
                    }
                }
            }

//            log.info("debug point - 1");
            int attributeversion = context.getAttributeVersion();
            if (attributeversion != 0) {
                context_version = attributeversion;
            }
//            log.info("debug point - 2");
            AttributeValue contextValue = context.getAttributeValue();

            if (contextValue != null) {
//                context_value_xml = contextValue.getAttributeValueAsString();
//                byte[] bytearray = context_value_xml.getBytes();
                byte[] bytearray = contextValue.getAttributeValue();
                context_value_blob = new cgl.util.MyByteBlob(bytearray);
//                log.info("length = " + context_value_blob.length());
            }
            log.info(
                    "insert into CONTEXT_ENTITY table:\n\n\t"
                    + insertSQL
                    + "\n\t CONTEXTKEY="
                    + context.getAttributeKey()
                    + "\n\t CONTEXTTYPE="
                    + null

                    +"\n\t OWNER_PUBLISHERID="
                    + publisher_id
                    +"\n\t OWNER_AUTH_INFO="
                    + owner_auth_info
                    + "\n\t GROUP_AUTH_INFO="
                    + group_auth_info
                    + "\n\t OTHERS_AUTH_INFO="
                    + others_auth_info

                    + "\n\t CONTEXTVALUEXML="
                    + context_value_blob
                    + "\n"); // prepare and execute the insert
            statement = connection.prepareStatement(insertSQL);
            statement.setString(1, context.getAttributeKey());
            statement.setString(2, null);
            statement.setString(3, publisher_id);
            statement.setString(4, owner_auth_info);
            statement.setString(5, group_auth_info);
            statement.setString(6, others_auth_info);
            statement.setBlob(7, context_value_blob);
            statement.setInt(8, context_version);

            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) {
                e.printStackTrace();
                /* ignored */
            }

        }
    }

    /**
     * Delete multiple rows from the CONTEXT_ENTITY table that are assigned to
     * the contextKey specified.
     *
     * @param  contextKey contextKey
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void deleteByContextKey(String contextKey,
                                          Connection connection) throws java.
            sql.
            SQLException {
        PreparedStatement statement = null;

        try {
            // prepare the delete
            statement = connection.prepareStatement(deleteByContextKeySQL);
            statement.setString(1, contextKey);

            log.info("delete from CONTEXT_ENTITY table:\n\n\t" +
                     deleteByContextKeySQL +
                     "\n\t CONTEXT_KEY=" + contextKey + "\n");

            // execute
            int returnCode = statement.executeUpdate();

            log.info("delete was successful, rows deleted=" + returnCode);
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /**
     * Select one row from the CONTEXT_ENTITY table.
     *
     * @param contextKey primary key value
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Attribute select(String contextKey, Connection connection) throws
            java.sql.SQLException {
        Attribute context = null;
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        try {
            statement = connection.prepareStatement(selectSQL);
            statement.setString(1, contextKey.toString());

            log.info("select from CONTEXT_ENTITY table:\n\n\t" + selectSQL +
                     "\n\t CONTEXT_KEY=" + contextKey.toString() + "\n");

            resultSet = statement.executeQuery();
            if (resultSet.next()) {
                context = new Attribute();

//                String context_type = resultSet.getString(1); //("CONTEXT_TYPE");

                String owner_publisher_id = resultSet.getString(2); //("CONTEXT_IDENTIFIER");

                String owner_auth_info = resultSet.getString(3); //("CONTEXTVALUETYPE");

                String group_auth_info = resultSet.getString(4); //("CONTEXTVALUETYPE");

//                String others_owner_info = resultSet.getString(5);


                java.sql.Blob contextValueBlob = resultSet.getBlob(6);

                java.io.InputStream is = null;
                byte[] bytes = null;
                try {
                    is = contextValueBlob.getBinaryStream();
                    bytes = new byte[(int) contextValueBlob.length()];
                    is.read(bytes);
                } catch (java.sql.SQLException e) {
                    bytes = null;
                } catch (java.io.IOException e) {
                    bytes = null;
                } finally {
                    try {
                        if (is != null) {
                            is.close();
                        }
                    } catch (Exception e) {}
                }
                byte [] context_value = bytes;
//                String context_value_string = null;
//                context_value_string = (new String(bytes)).toString();

                int contextVersion = resultSet.getInt(7); //("PROPERTYVERSION");

                if (contextKey != null) {
                    context.setAttributeKey(contextKey);
                    log.info("ContextTable = > contextKey " + contextKey);
                }

                if (owner_publisher_id != null) {
                    PublisherID publisherID = new PublisherID(owner_publisher_id);
                    context.setAttributeOwnerPublisherID(publisherID);
                }

                if (owner_auth_info != null) {
                    AttributeAccessInfos infos = new AttributeAccessInfos();
                    AttributeAccessInfo owner_info = new AttributeAccessInfo();
                    owner_info.setReadOK(true);
                    owner_info.setisReadWriteOK(true);

                    AttributeAccessInfo group_info = new AttributeAccessInfo();
                    if (group_auth_info != null) {
                        Vector vector = ContextPublisherTable.selectByContextKey(contextKey, connection);
                        group_info.setPublisherIDVector(vector);
                        if (group_auth_info.equals("ReadWrite")) {
                            group_info.setReadOK(true);
                            group_info.setisReadWriteOK(true);
                        } else if (group_auth_info.equals("Read")) {
                            group_info.setReadOK(true);
                            group_info.setisReadWriteOK(false);
                        }
                    }

                    AttributeAccessInfo others_info = new AttributeAccessInfo();
                    if (others_info.equals("ReadWrite")) {
                        others_info.setReadOK(true);
                        others_info.setisReadWriteOK(true);
                    } else if (others_info.equals("Read")) {
                        others_info.setReadOK(true);
                        others_info.setisReadWriteOK(false);
                    }

                    infos.setOwnerAttributeAccessInfo(owner_info);
                    infos.setGroupAttributeAccessInfo(group_info);
                    infos.setOthersAttributeAccessInfo(others_info);

                    context.setAttributeAccessRightInfos(infos);

                    log.info("ContextTable = > owner_auth_info = " + owner_auth_info);
                }

                if (context_value != null) {
                    context.setAttributeValue(new AttributeValue(
                            context_value));
                    log.info("ContextTable = > context_value_string = " +
                             context_value);
                }

                if (contextVersion != 0) {
                    context.setAttributeVersion(contextVersion);
                    log.info("ContextTable => version = " + contextVersion);
                }
            }
            return context;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/

    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        Transaction txn = new Transaction();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        if (connection != null) {
            try {
                String[] attributeKey = uuidgen.uuidgen(4);
                AttributeBag contextBag = new AttributeBag();
                Attribute context = new Attribute();
                context.setAttributeKey(attributeKey[0]);
                //context.setAttributeSource(new AttributeSource("session://"));
                //context.setAttributeName(new AttributeName("session_no"));
                //context.setAttributeValueType(new AttributeValueType("number"));
                AttributeAccessInfos infos = new AttributeAccessInfos();

                AttributeAccessInfo group_info = new AttributeAccessInfo();
                Vector publisherIDVector = new Vector();

                PublisherID publisherID = new PublisherID();
                publisherID.setValue("ali");
                publisherIDVector.add(publisherID);
                PublisherID publisher_ID = new PublisherID();
                publisher_ID.setValue("veli");
                publisherIDVector.add(publisher_ID);
                group_info.setPublisherIDVector(publisherIDVector);

                group_info.setReadOK(true);
                group_info.setisReadWriteOK(false);
                infos.setGroupAttributeAccessInfo(group_info);


                AttributeAccessInfo others_info = new AttributeAccessInfo();
                others_info.setReadOK(true);
                others_info.setisReadWriteOK(true);
                infos.setOthersAttributeAccessInfo(others_info);

                context.setAttributeAccessRightInfos(infos);

                context.setAttributeValue(new AttributeValue("3"));
                contextBag.addAttribute(context);

                // begin a new transaction
                txn.begin(connection);

                // insert a new BusinessEntity
                for (Iterator iter = contextBag.getAttributeVector().iterator();
                                     iter.hasNext(); ) {
                    Attribute item = (Attribute) iter.next();
                    ContextTable.insert(item, connection);
                }

//                ContextTable.deleteByContextKey(attributeKey[0], connection);
//
                context = ContextTable.select(attributeKey[0], connection);
                if (context != null) {
                    log.info("context key = " + context.getAttributeKey());

                    AttributeAccessInfos access_infos = context.
                            getAttributeAccessRightInfos();
                    if (access_infos != null) {
                        if (access_infos.getOwnerAttributeAccessInfo() != null) {
                            System.out.println("owner read = " +
                                               access_infos.getOwnerAttributeAccessInfo().
                                               isReadOK());
                            System.out.println("owner read/write = " +
                                               access_infos.
                                               getOwnerAttributeAccessInfo().
                                               isReadWriteOK());
                        }
                        if (access_infos.getGroupAttributeAccessInfo() != null) {
                            System.out.println("group read = " +
                                               access_infos.getGroupAttributeAccessInfo().
                                               isReadOK());
                            System.out.println("group read/write = " +
                                               access_infos.
                                               getGroupAttributeAccessInfo().
                                               isReadWriteOK());
                        }
                        if (access_infos.getOthersAttributeAccessInfo() != null) {
                            System.out.println("others read = " +
                                               access_infos.getOthersAttributeAccessInfo().
                                               isReadOK());
                            System.out.println("others read/write = " +
                                               access_infos.
                                               getOthersAttributeAccessInfo().
                                               isReadWriteOK());
                        }
                    }

                    log.info("context value = " +
                             context.getAttributeValue().getAttributeValue());
                    log.info("context version = " + context.getAttributeVersion());
                }
                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}
