/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import cgl.util.jdbc.ConnectionManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.datatype.request.FindQualifier;

/**
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindAttributeByCategoryQuery {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(FindAttributeByCategoryQuery.class);

    static String selectSQL;
    static {
        // build selectSQL
        StringBuffer sql = new StringBuffer(400);
        sql.append("SELECT P.PROPERTY_KEY ");
        sql.append(
                "FROM PROPERTY_CATEGORY C, SERVICE_PROPERTY P, PROPERTY_ENTITY E ");
        selectSQL = sql.toString();
    }

    /**
     * Select ...
     *
     * @param serviceKey
     * @param categoryBag
     * @param keysIn
     * @param qualifiers
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select_rec(String serviceKey,
                                    CategoryBag categoryBag,
                                    Vector keysIn,
                                    FindQualifiers qualifiers,
                                    Connection connection) throws java.sql.
            SQLException {
        // if there is a keysIn vector but it doesn't contain
        // any keys then the previous query has exhausted
        // all possibilities of a match so skip this call.
        if ((keysIn != null) && (keysIn.size() == 0)) {
            return keysIn;
        }

        Vector keysOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        // construct the SQL statement
        DynamicQuery sql = new DynamicQuery(selectSQL);
        appendWhere(sql, serviceKey, categoryBag, qualifiers);
        appendIn(sql, keysIn);
        //appendOrderBy(sql,qualifiers);

        try {
            log.info("sql statement = " + sql.toString());

            statement = sql.buildPreparedStatement(connection);
            resultSet = statement.executeQuery();

            while (resultSet.next()) {
                keysOut.addElement(resultSet.getString(1)); //("PROPERTY_KEY"));
            }

            return keysOut;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService ResultSet: " + e.getMessage());
            }

            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService Statement: " + e.getMessage());
            }
        }
    }

    /**
     *
     */
    public static Vector select(String serviceKey, CategoryBag categoryBag,
                                Vector keysIn, FindQualifiers qualifiers,
                                Connection connection) throws java.sql.
            SQLException {
        if (categoryBag != null) {
            log.info(
                    "cgl.fthpis.datastore.jdbc.FindAttributeByCategoryQuery.java => select() - 1");
            Vector keyrefencevector = categoryBag.getKeyedReferenceVector();
            log.info("cgl.fthpis.datastore.jdbc.FindAttributeByCategoryQuery.java => select() - 2 => keyrefencevector size = " +
                     keyrefencevector.size());
            for (Iterator iter = keyrefencevector.iterator(); iter.hasNext(); ) {
                KeyedReference item = (KeyedReference) iter.next();
                Vector vec = new Vector();
                vec.add(item);
                CategoryBag bag = new CategoryBag();
                bag.setKeyedReferenceVector(vec);
                keysIn = select_rec(serviceKey, bag, keysIn, qualifiers,
                                    connection);
            }
        }
        return keysIn;
    }


    /**
     *
     */
    private static void appendWhere(DynamicQuery sql, String serviceKey,
                                    CategoryBag categoryBag,
                                    FindQualifiers qualifiers) {
        sql.append(
                "WHERE (C.PROPERTY_KEY = P.PROPERTY_KEY) AND (C.PROPERTY_KEY = E.PROPERTY_KEY)");

        if (serviceKey != null) {
            sql.append(" AND P.SERVICE_KEY = ? ");
            sql.addValue(serviceKey);
        }

        if (categoryBag != null) {
            Vector keyedRefVector = categoryBag.getKeyedReferenceVector();

            if (keyedRefVector != null) {
                int vectorSize = keyedRefVector.size();
                if (vectorSize > 0) {
                    //sql.append("AND (");
                    sql.append("AND ");

                    for (int i = 0; i < vectorSize; i++) {
                        KeyedReference keyedRef = (KeyedReference)
                                                  keyedRefVector.elementAt(i);
                        String name = keyedRef.getKeyName();
                        String value = keyedRef.getKeyValue();

                        if ((name != null) && (value != null)) {
                            sql.append("(C.KEY_NAME = ? AND C.KEY_VALUE = ?)");
                            sql.addValue(name);
                            sql.addValue(value);

                            if (i + 1 < vectorSize) {
                                sql.append(" OR ");
                            }
//                sql.append(" AND ");
                        }
                    }
//          sql.append(") ");
                    sql.append(" ");
                }
            }
        }
    }

    /**
     * Utility method used to construct SQL "IN" statements such as
     * the following SQL example:
     *
     *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
     *
     * @param sql StringBuffer to append the final results to
     * @param keysIn Vector of Strings used to construct the "IN" clause
     */
    private static void appendIn(DynamicQuery sql, Vector keysIn) {
        if (keysIn == null) {
            return;
        }

        sql.append("AND P.PROPERTY_KEY IN (");

        int keyCount = keysIn.size();
        for (int i = 0; i < keyCount; i++) {
            String key = (String) keysIn.elementAt(i);
            sql.append("?");
            sql.addValue(key);

            if ((i + 1) < keyCount) {
                sql.append(",");
            }
        }

        sql.append(") ");
    }

    /**
     *
     */
    private static void appendOrderBy(DynamicQuery sql,
                                      FindQualifiers qualifiers) {
        sql.append("ORDER BY ");

        if (qualifiers == null) {
            sql.append("P.LAST_UPDATE DESC");
        } else if (qualifiers.sortByDateAsc) {
            sql.append("P.LAST_UPDATE ASC");
        } else {
            sql.append("P.LAST_UPDATE DESC");
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("uddi");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {

        Transaction txn = new Transaction();

        if (connection != null) {
            try {
                // begin a new transaction
                txn.begin(connection);

                String serviceKey = "5FA9EF00-5229-11DA-AF00-A16090B517E0";

                FindQualifiers findQualifiers = new FindQualifiers();
                String findq = "andAllKeys";
                FindQualifier find_qualifier = new FindQualifier();
                find_qualifier.setValue(findq);
                Vector vec = new Vector();
                vec.add(find_qualifier);
                findQualifiers.setFindQualifierVector(vec);

                org.apache.juddi.datatype.CategoryBag categoryBag = null;
                java.util.Vector keyedReferenceVector = new java.util.Vector();
                org.apache.juddi.datatype.KeyedReference keyedReference = new
                        org.apache.juddi.datatype.KeyedReference();
                keyedReference.setKeyName("propertyType");
                keyedReference.setKeyValue("QOS");
                keyedReferenceVector.add(keyedReference);
                categoryBag = new org.apache.juddi.datatype.CategoryBag();
                categoryBag.setKeyedReferenceVector(keyedReferenceVector);

                Vector vector = select(serviceKey, categoryBag, null,
                                       findQualifiers, connection);
                for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
                    String attributeKey = (String) iter.next();
                    log.info("attributeKey = " + attributeKey);
                }

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
