/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.TModelKey;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeFindQualifiers;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindServiceByAttributeCategoryQuery {

    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(
            FindServiceByAttributeCategoryQuery.class);

    static String selectSQL, selectByNameSQL;
    static {
        // build selectSQL
        StringBuffer sql = new StringBuffer(1200);
        sql.append("SELECT P.SERVICE_KEY ");
        sql.append(
                "FROM SERVICE_PROPERTY P, PROPERTY_CATEGORY C, PROPERTY_ENTITY E ");
        selectSQL = sql.toString();

        StringBuffer sql_string = new StringBuffer(1200);
        sql_string.append("SELECT E.PROPERTY_KEY ");
        sql_string.append(
                "FROM PROPERTY_ENTITY E, PROPERTY_CATEGORY C, SERVICE_PROPERTY P ");
        selectSQL = sql_string.toString();

    }

    /**
     * Select ...
     *
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select(AttributeBag attributeBag,
                                Vector keysIn,
                                FindQualifiers qualifiers,
                                Connection connection) throws java.sql.
            SQLException {
        log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeCategoryQuery.java => select() 0");
        try {

            Vector temp = new Vector();
            for (Iterator iterator = keysIn.iterator(); iterator.hasNext(); ) {
                String serviceKey = (String) iterator.next();
                attributeBag = update_AttributeBag(serviceKey, attributeBag,
                        qualifiers, connection);
                Vector attributeVector = attributeBag.getAttributeVector();

                log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeCategoryQuery.java => select() I + attributeVector = " +
                         attributeVector);

                boolean flag = false;

                for (Iterator iter = attributeVector.iterator(); iter.hasNext(); ) {
                    Attribute attribute = (Attribute) iter.next();
                    CategoryBag bag = attribute.getCategoryBag();
                    String propertyKey = attribute.getAttributeKey();

                    log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeCategoryQuery.java => select() II + bag = " +
                             bag);
                    if (bag != null && bag.size() != 0) {
                        Vector flag_vec = new Vector();
                        Vector keyrefencevector = bag.getKeyedReferenceVector();
                        log.info("keyreferencevector size = " +
                                 keyrefencevector.size());
                        for (Iterator it = keyrefencevector.iterator();
                                           it.hasNext(); ) {
                            KeyedReference kr = (KeyedReference) it.next();
                            Vector vec = new Vector();
                            vec.add(kr);
                            CategoryBag cbag = new CategoryBag();
                            cbag.setKeyedReferenceVector(vec);

                            Vector keys_in = select_rec(propertyKey, serviceKey,
                                    cbag, qualifiers, connection);
                            log.info("KEYS_IN VECTOR AFTER SELECT_REC SIZE = " +
                                     keys_in.size());
                            if (keys_in == null || keys_in.size() == 0) {
                                String attribute_findqualifier = "";
                                if (attribute.getAttributeFindQualifier() == null ||
                                    attribute.getAttributeFindQualifier().
                                    getQualifierValue() == null ||
                                    attribute.getAttributeFindQualifier().
                                    getQualifierValue() == "") {
                                    attribute_findqualifier = "equalTo";
                                } else {
                                    attribute_findqualifier = attribute.
                                            getAttributeFindQualifier().
                                            getQualifierValue();
                                }
                                if (attribute_findqualifier.equals(
                                        AttributeFindQualifiers.
                                        EQUAL_TO)) {
                                    //flag_vec.add("true");
                                    flag = true; //it is supposed to be equal category, since no match, remove it
                                } else {
                                    //flag_vec.
                                    flag = false; //it is supposed to be not equal, so don't remove it even, no match category
                                }
                            }
                        }
                    }
                }
                log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeCategoryQuery.java => select() III");

                if (flag) {
                    temp.add(serviceKey);
                    log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeCategoryQuery.java => select() IV");
                }
            }
            if (temp != null && temp.size() != 0) {
                for (Iterator iter = temp.iterator(); iter.hasNext(); ) {
                    String item = (String) iter.next();
                    keysIn.remove(item);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        return keysIn;
    }

    /**
     * Select ...
     *
     * @param businessKey
     * @param categoryBag
     * @param keysIn
     * @param qualifiers
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static AttributeBag update_AttributeBag(String serviceKey,
                                                   AttributeBag query_attributebag,
                                                   FindQualifiers qualifiers,
                                                   Connection connection) throws java.sql.SQLException {
        Vector keysOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        Vector vec = query_attributebag.getAttributeVector();
        Vector new_attribute_vector = new Vector();

        for (Iterator iter = vec.iterator(); iter.hasNext(); ) {
            Attribute item = (Attribute) iter.next();
            item = set_propertykey(serviceKey, item, qualifiers, connection);
            new_attribute_vector.add(item);
        }
        AttributeBag bag = new AttributeBag();
        bag.setAttributeVector(new_attribute_vector);
        return bag;
    }


    /**
     * Select ...
     *
     * @param businessKey
     * @param categoryBag
     * @param keysIn
     * @param qualifiers
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Attribute set_propertykey(String serviceKey,
                                            Attribute attribute,
                                            FindQualifiers qualifiers,
                                            Connection connection) throws java.
            sql.SQLException {

        Vector keysOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        String key = null;

        // construct the SQL statement
        DynamicQuery sql = new DynamicQuery(selectSQL);
        appendWhere2(sql, serviceKey, attribute, qualifiers);

        try {
            log.info("sql statement = " + sql.toString());

            statement = sql.buildPreparedStatement(connection);
            resultSet = statement.executeQuery();

            while (resultSet.next()) {
                key = resultSet.getString(1); //("PROPERTY_KEY"));
            }
            attribute.setAttributeKey(key);
            return attribute;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService ResultSet: " + e.getMessage());
            }

            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService Statement: " + e.getMessage());
            }
        }
    }


    /**
     * Select ...
     *
     * @param businessKey
     * @param categoryBag
     * @param keysIn
     * @param qualifiers
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector select_rec(String propertyKey,
                                    String serviceKey,
                                    CategoryBag categoryBag,
                                    FindQualifiers qualifiers,
                                    Connection connection) throws java.sql.
            SQLException {
        // if there is a keysIn vector but it doesn't contain
        // any keys then the previous query has exhausted
        // all possibilities of a match so skip this call.
//    if ((keysIn != null) && (keysIn.size() == 0))
//      return keysIn;

        Vector keysOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        // construct the SQL statement
        DynamicQuery sql = new DynamicQuery(selectSQL);
        appendWhere(sql, propertyKey, serviceKey, categoryBag, qualifiers);
        //appendIn(sql,keysIn);
        //appendOrderBy(sql,qualifiers);

        try {
            log.info("sql statement = " + sql.toString());

            statement = sql.buildPreparedStatement(connection);
            resultSet = statement.executeQuery();

            while (resultSet.next()) {
                keysOut.addElement(resultSet.getString(1)); //("PROPERTY_KEY"));
            }

            return keysOut;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService ResultSet: " + e.getMessage());
            }

            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService Statement: " + e.getMessage());
            }
        }
    }

    /**
     *
     */
    private static void appendWhere2(DynamicQuery sql,
                                     String serviceKey,
                                     Attribute attribute,
                                     FindQualifiers qualifiers) {
        sql.append(
                "WHERE (C.PROPERTY_KEY = P.PROPERTY_KEY) AND (C.PROPERTY_KEY = E.PROPERTY_KEY) ");

        if (serviceKey != null) {
            sql.append("AND (P.SERVICE_KEY = ? ");
            sql.addValue(serviceKey);
        }
        sql.append(") ");

        sql.append("AND (");

        String text = attribute.getAttributeValue().getAttributeValueAsString();

        if ((text != null) && (text.length() > 0)) {
            sql.append("E.PROPERTYVALUESTRING = ?");
            sql.addValue(text);
        }

        sql.append(") ");
    }


    /**
     *
     */
    private static void appendWhere(DynamicQuery sql,
                                    String propertyKey,
                                    String serviceKey,
                                    CategoryBag categoryBag,
                                    FindQualifiers qualifiers) {
        if (propertyKey != null) {
            sql.append("WHERE C.PROPERTY_KEY = ? ");
            sql.addValue(propertyKey);
        }

        sql.append(
                "AND (C.PROPERTY_KEY = P.PROPERTY_KEY) AND (C.PROPERTY_KEY = E.PROPERTY_KEY) ");

        if (serviceKey != null) {
            sql.append("AND P.SERVICE_KEY = ? ");
            sql.addValue(serviceKey);
        }

        if (categoryBag != null) {
            Vector keyedRefVector = categoryBag.getKeyedReferenceVector();

            if (keyedRefVector != null) {
                int vectorSize = keyedRefVector.size();
                if (vectorSize > 0) {
                    //sql.append("AND (");
                    sql.append("AND ");

                    for (int i = 0; i < vectorSize; i++) {
                        KeyedReference keyedRef = (KeyedReference)
                                                  keyedRefVector.elementAt(i);
                        String name = keyedRef.getKeyName();
                        String value = keyedRef.getKeyValue();

                        if ((name != null) && (value != null)) {
                            sql.append("(C.KEY_NAME = ? AND C.KEY_VALUE = ?)");
                            sql.addValue(name);
                            sql.addValue(value);

                            if (i + 1 < vectorSize) {
                                //sql.append(" OR ");
                                sql.append(" AND ");
                            }
                        }
                    }
//          sql.append(") ");
                    sql.append(" ");
                }
            }
        }
    }


    /**
     * Utility method used to construct SQL "IN" statements such as
     * the following SQL example:
     *
     *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
     *
     * @param sql StringBuffer to append the final results to
     * @param keysIn Vector of Strings used to construct the "IN" clause
     */
    private static void appendIn(DynamicQuery sql,
                                 Vector keysIn) {
        if (keysIn == null) {
            return;
        }

        sql.append("AND P.SERVICE_KEY IN (");

        int keyCount = keysIn.size();
        for (int i = 0; i < keyCount; i++) {
            String key = (String) keysIn.elementAt(i);
            sql.append("?");
            sql.addValue(key);

            if ((i + 1) < keyCount) {
                sql.append(",");
            }
        }

        sql.append(") ");
    }

    /**
     *
     */
//  private static void appendOrderBy(DynamicQuery sql,FindQualifiers qualifiers)
//  {
//    sql.append("ORDER BY ");
//
//    if (qualifiers == null)
//      sql.append("E.LAST_UPDATE DESC");
//    else if (qualifiers.sortByDateAsc)
//      sql.append("E.LAST_UPDATE ASC");
//    else
//      sql.append("E.LAST_UPDATE DESC");
//  }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection();
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        String businessKey = "5D2AFE60-1285-11DA-BE60-CCA2ED977765";

        TModelBag tModelBag = new TModelBag();
        Vector tModelKeyVector = new Vector();
        tModelKeyVector.addElement(new TModelKey(""));
        tModelKeyVector.addElement(new TModelKey(""));
        tModelKeyVector.addElement(new TModelKey(
                "2a33d7d7-2b73-4de9-99cd-d4c51c186bce"));
        tModelKeyVector.addElement(new TModelKey(
                "2a33d7d7-2b73-4de9-99cd-d4c51c186bce"));
        tModelBag.setTModelKeyVector(tModelKeyVector);

        Vector keysIn = new Vector();
        keysIn.add("13411e97-24cf-43d1-bee0-455e7ec5e9fc");
        keysIn.add("3f244f19-7ba7-4c3e-a93e-ae33e530794b");
        keysIn.add("3009f336-98c1-4193-a22f-fea73e79c909");
        keysIn.add("3ef4772f-e04b-46ed-8065-c5a4e167b5ba");

        Transaction txn = new Transaction();

        if (connection != null) {
            try {
                // begin a new transaction
                txn.begin(connection);

                //select(businessKey,keysIn,null,null,connection);
                //select(businessKey,null,null,null,connection);

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
