/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.Name;

import cgl.util.jdbc.ConnectionManager;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeFindQualifier;
import cgl.fthpis.datatype.attribute.AttributeFindQualifiers;



/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindServiceByContextBagQuery
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(FindServiceByContextBagQuery.class);

  static String selectSQL;
  static String selectByServiceKeySQL;
  static
  {
    // build selectSQL
    StringBuffer sql = new StringBuffer(200);
    sql.append("SELECT S.SERVICE_KEY, S.LAST_UPDATE ");
    sql.append("FROM SERVICE_CONTEXT S, CONTEXT_ENTITY C, CONTEXT_NAME N ");
    selectSQL = sql.toString();


    // build selectByServiceKeySQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("C.CONTEXT_KEY,");
    sql.append("C.CONTEXTTYPE,");
    sql.append("C.OWNER_PUBLISHERID,");
    sql.append("C.OWNER_AUTH_INFO,");
    sql.append("C.GROUP_AUTH_INFO,");
    sql.append("C.OTHERS_AUTH_INFO,");
    sql.append("C.CONTEXTVALUEXML,");
    sql.append("C.CONTEXTVERSION ");
    sql.append("FROM SERVICE_CONTEXT S, CONTEXT_ENTITY C ");
    sql.append("WHERE S.SERVICE_KEY=? ");
    selectByServiceKeySQL = sql.toString();
  }

  /**
   * Select ...
   *
   * @param businessKey
   * @param categoryBag
   * @param keysIn
   * @param qualifiers
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */

  public static Vector select_rec(AttributeBag context_bag,
                                  Vector keysIn,
                                  FindQualifiers qualifiers,
                                  Connection connection)
    throws java.sql.SQLException
  {
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByContextBagQuery.select_rec() function called. I ");
    // if there is a keysIn vector but it doesn't contain
    // any keys then the previous query has exhausted
    // all possibilities of a match so skip this call.
    if ((keysIn != null) && (keysIn.size() == 0))
      return keysIn;

    log.info("cgl.fthpis.datastore.jdbc.FindServiceByContextBagQuery.select_rec() function called. II ");

    Vector keysOut = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    // construct the SQL statement
    DynamicQuery sql = new DynamicQuery(selectSQL);
    appendWhere(sql,context_bag);
    appendIn(sql,keysIn);
    //appendOrderBy(sql,qualifiers);

    log.info("cgl.fthpis.datastore.jdbc.FindServiceByContextBagQuery.select_rec() function called. III ");

    try
    {
      log.info("sql statement = " + sql.toString());

      statement = sql.buildPreparedStatement(connection);
      resultSet = statement.executeQuery();
      log.info("debug point after satement is executed...");

      while (resultSet.next()) {
        if (!keysOut.contains(resultSet.getString(1))) {
          keysOut.addElement(resultSet.getString(1)); //("SERVICE_KEY"));
          //log.info("service key = " + resultSet.getString(1));
        }
      }
      return keysOut;
    }
    finally
    {
      try {
        resultSet.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService ResultSet: "+e.getMessage());
      }

      try {
        statement.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService Statement: "+e.getMessage());
      }


    }
  }


  public static Vector select(AttributeBag context_bag,
                              Vector keysIn,
                              FindQualifiers qualifiers,
                              Connection connection)
    throws java.sql.SQLException
  {

    try {
      //run sql queries
      if (qualifiers.orAllKeys) {
        Vector v = context_bag.getAttributeVector();
        if (v != null) {
            AttributeBag b = new AttributeBag();
            b.setAttributeVector(v);
            keysIn = select_rec( b, keysIn, qualifiers, connection);
        }
      } else {
        int size = context_bag.size();
        for (int i = 0; i < size; i++) {
            Vector attributevector = context_bag.getAttributeVector();
            if (attributevector != null) {
                for (Iterator iter = attributevector.iterator(); iter.hasNext(); ) {
                    Attribute item = (Attribute) iter.next();
                    if (item.getAttributeValueType() != null) {
                        Vector vec = new Vector();
                        vec.add(item);
                        AttributeBag bag = new AttributeBag();
                        bag.setAttributeVector(vec);
                        keysIn = select_rec(bag, keysIn,
                                            qualifiers,
                                            connection);
                    }
                }
            }
        }
      }


    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    return keysIn;
  }

  /**
   * Select all rows from the business_service table for a given
   * BusinessKey.
   *
   * @param  serviceKey ServiceKey
   * @param  connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector selectByServiceKey(String serviceKey,
                                          Connection connection) throws java.
      sql.SQLException {
    Vector contextVector = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    try {
      // create a statement to query with
      statement = connection.prepareStatement(selectByServiceKeySQL);
      statement.setString(1, serviceKey.toString());


      // execute the statement
      resultSet = statement.executeQuery();

      Attribute context = null;
      while (resultSet.next()) {

            context = new Attribute();
            context.setServiceKey(serviceKey);
            String contextKey = resultSet.getString(1); //("CONTEXT_KEY");
            String context_type = resultSet.getString(2); //("CONTEXTTYPE");
            String context_identifier = resultSet.getString(3); //("CONTEXTIDENTIFIER");
            String context_value_type = resultSet.getString(4); //("CONTEXTVALUETYPE");
            float context_value_number = 0;
            String context_value_string = null;
            byte [] context_value_XML = null;
            java.sql.Blob newblob = null;
            if (context_value_type != null) {
              if (context_value_type.equals("text")) {
                context_value_string = resultSet.getString(6); //("CONTEXTVALUESTRING");
              }
              else if (context_value_type.equals("xpath")) {
                newblob = resultSet.getBlob(7); //("CONTEXTVALUENUMBER");
                java.io.InputStream is = null;
                byte [] bytes = null;
                try {
                    is = newblob.getBinaryStream();
                    bytes = new byte[(int) newblob.length()];
                    is.read(bytes);
                } catch (java.sql.SQLException e) {
                    bytes = null;
                } catch (java.io.IOException e) {
                    bytes = null;
                } finally {
                    try {
                        if (is != null) {
                            is.close();
                        }
                    } catch (Exception e) {}
            }
              //context_value_XML = (new String(bytes)).toString();
              context_value_XML = bytes;
              }
              else if (context_value_type.equals("number")) {
                context_value_number = resultSet.getFloat(5); //("CONTEXTVALUENUMBER");
              }
            }
            int contextVersion = resultSet.getInt(8); //("CONTEXTVERSION");

            if (serviceKey != null) {
              context.setServiceKey(serviceKey);
              log.info("AttributeTable = > serviceKey " + serviceKey);
            }
            if (contextKey != null) {
              context.setAttributeKey(contextKey);
              log.info("AttributeTable = > contextKey " + contextKey);
            }
            if (context_type != null) {
              context.setAttributeName(new AttributeName(context_type));
              log.info("AttributeTable = > context_type " + context_type);

            }
            if (context_identifier != null) {
              context.setAttributeName(new AttributeName(context_identifier));
              log.info("AttributeTable = > context_identifier " + context_identifier);

            }
            if (context_value_type != null) {
              context.setAttributeValueType(new AttributeValueType(context_value_type));
              log.info("AttributeTable = > context_value_type = " + context_value_type);
            }
            if (context_value_string != null) {
              context.setAttributeValue(new AttributeValue(
                  context_value_string));
              log.info("AttributeTable = > value = " + context_value_string);
            }
            else if (context_value_XML != null) {
              context.setAttributeValue(new AttributeValue(context_value_XML));
              log.info("AttributeTable = > context_value_XML = " + context_value_XML);
            }
            else if (context_value_number != 0) {
              context.setAttributeValue(new AttributeValue(context_value_number));
              log.info("AttributeTable = > context_value_number = " + context_value_number);
            }
            if (contextVersion != 0) {
              context.setAttributeVersion(contextVersion);
              log.info("AttributeTable => version = " + contextVersion);
            }
          if (context_identifier.equals("data_file")) {
            contextVector.add(context);
            //log.info("data file = " + context);
          }
            context = null;
      }
      return contextVector;
    }
    finally {
      try {
        resultSet.close();
      }
      catch (Exception e) { /* ignored */}
      try {
        statement.close();
      }
      catch (Exception e) { /* ignored */}
    }
  }



  /**
   *
   */
  private static void appendWhere(DynamicQuery sql,
                                  AttributeBag contextBag)
  {
    try {
      sql.append("WHERE (C.CONTEXT_KEY=S.CONTEXT_KEY) AND (S.CONTEXT_KEY=N.CONTEXT_KEY) ");

      if (contextBag != null) {
        Vector contextVector = contextBag.getAttributeVector();

        if (contextVector != null) {
          int vectorSize =contextVector.size();
          if (vectorSize > 0) {
            sql.append("AND ");

            for (int i = 0; i < vectorSize; i++) {
              Attribute context = (Attribute) contextVector.elementAt(i);
              String context_name = null;
              if (context.getNameVector() != null) {
                  Vector nameVector = context.getNameVector();
                  Name name = (Name) nameVector.elementAt(0);
                  context_name = name.getValue();
              }

              log.info("debug point - 1");
                if (context_name != null) {
                    sql.append("(N.NAME = ?)");
                    //sql.addValue(context_key);
                    sql.addValue(context_name);
                }
            }

            sql.append(" ");
          }
        }
        log.info("debug point - 5");
      }
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Utility method used to construct SQL "IN" statements such as
   * the following SQL example:
   *
   *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
   *
   * @param sql StringBuffer to append the final results to
   * @param keysIn Vector of Strings used to construct the "IN" clause
   */
  private static void appendIn(DynamicQuery sql,Vector keysIn)
  {
    if (keysIn == null)
      return;

    sql.append("AND S.SERVICE_KEY IN (");

    int keyCount = keysIn.size();
    for (int i=0; i<keyCount; i++)
    {
      String key = (String)keysIn.elementAt(i);
      sql.append("?");
      sql.addValue(key);

      if ((i+1) < keyCount)
        sql.append(",");
    }

    sql.append(") ");
  }

  /**
   *
   */
  private static void appendOrderBy(DynamicQuery sql,FindQualifiers qualifiers)
  {
    sql.append("ORDER BY ");

    if (qualifiers == null)
      sql.append("S.LAST_UPDATE DESC");
    else if (qualifiers.sortByDateAsc)
      sql.append("S.LAST_UPDATE ASC");
    else
      sql.append("S.LAST_UPDATE DESC");
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("wscontext");
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection)
    throws Exception
  {
    log.info("debug point - 0");
    FindQualifiers findQualifiers = new FindQualifiers();
    String findq = "andAllKeys";
    FindQualifier find_qualifier = new FindQualifier();
    find_qualifier.setValue(findq);
    Vector vec = new Vector();
    vec.add(find_qualifier);
    findQualifiers.setFindQualifierVector(vec);

    AttributeBag contextBag = new AttributeBag();
    Attribute context =  new Attribute();
    Vector nameVector = new Vector();
    nameVector.add(new Name("session_no"));
    context.setNameVector(nameVector);

    context.setAttributeName(new AttributeName("cpu_number"));

    context.setAttributeValueType(new AttributeValueType("number"));
    context.setAttributeValue(new AttributeValue("2"));
    context.setAttributeFindQualifier(new AttributeFindQualifier(AttributeFindQualifiers.EQUAL_TO));
    contextBag.addAttribute(context);

    Transaction txn = new Transaction();

    if (connection != null)
    {
      try
      {
        // begin a new transaction
        txn.begin(connection);
        log.info("debug point in main - 1");

        //select(businessKey,attributeBag,keysIn,null,connection);
        Vector vector = select(contextBag, null, findQualifiers, connection);

        log.info("debug point in main - 2");

        for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
          Object item = (Object)iter.next();
          log.info("serviceKey = " + item.toString());

        }

        log.info("debug point in main - 3");

        // commit the transaction
        txn.commit();
        log.info("FindServiceByAttributeBagQuery is completed fine.");
      }
      catch(Exception ex)
      {
        try { txn.rollback(); }
        catch(java.sql.SQLException sqlex) { sqlex.printStackTrace(); }
        throw ex;
      }
    }
  }
}
