/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;

import cgl.util.jdbc.ConnectionManager;
import org.apache.juddi.util.jdbc.DynamicQuery;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindServiceByNameQuery
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(FindServiceByNameQuery.class);

  static String selectSQL;
  static
  {
    // build selectSQL
    StringBuffer sql = new StringBuffer(200);
    sql.append("SELECT S.SERVICE_KEY,S.LAST_UPDATE,N.NAME ");
    sql.append("FROM BUSINESS_SERVICE S,SERVICE_NAME N ");
    selectSQL = sql.toString();
  }

  static String selectSQL2;
  static
  {
    // build selectSQL
    StringBuffer sql = new StringBuffer(200);
    sql.append("SELECT S.SERVICE_KEY,S.LAST_UPDATE,N.NAME ");
    sql.append("FROM SESSION_SERVICE S,SERVICE_NAME N ");
    selectSQL2 = sql.toString();
  }

  /**
   * Select ...
   *
   * @param businessKey primary key value
   * @param names
   * @param keysIn
   * @param qualifiers
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector select(Vector names,
                              Vector keysIn,
                              FindQualifiers qualifiers,
                              Connection connection)
    throws java.sql.SQLException
  {
    // if there is a keysIn vector but it doesn't contain
    // any keys then the previous query has exhausted
    // all possibilities of a match so skip this call.
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 1");
    if (keysIn != null) {
      if (keysIn.size() == 0)
        return keysIn;
    }
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 2");

    Vector keysOut = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 3");

    // construct the SQL statement
    DynamicQuery sql = new DynamicQuery(selectSQL);
    appendWhere(sql, names, qualifiers);
    if (keysIn != null)
      appendIn(sql,keysIn);
    appendOrderBy(sql,qualifiers);
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 4");

    try
    {
      log.debug(sql.toString());
      log.info(sql.toString());
      statement = sql.buildPreparedStatement(connection);
      resultSet = statement.executeQuery();

      while (resultSet.next())
        keysOut.addElement(resultSet.getString(1));//("SERVICE_KEY"));

      return keysOut;
    }
    finally
    {
      try {
        resultSet.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService ResultSet: "+e.getMessage(),e);
      }

      try {
        statement.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService Statement: "+e.getMessage(),e);
      }
    }
  }

  /**
   * Select ...
   *
   * @param businessKey primary key value
   * @param names
   * @param keysIn
   * @param qualifiers
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector select2(Vector names,
                               Vector keysIn,
                               FindQualifiers qualifiers,
                               Connection connection)
    throws java.sql.SQLException
  {
    // if there is a keysIn vector but it doesn't contain
    // any keys then the previous query has exhausted
    // all possibilities of a match so skip this call.
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 1");
    if (keysIn != null) {
      if (keysIn.size() == 0)
        return keysIn;
    }
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 2");

    Vector keysOut = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 3");

    // construct the SQL statement
    DynamicQuery sql = new DynamicQuery(selectSQL2);
    appendWhere(sql,names,qualifiers);
    if (keysIn != null)
      appendIn(sql,keysIn);
    appendOrderBy(sql,qualifiers);
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 4");

    try
    {
      log.debug(sql.toString());
      log.info(sql.toString());
      statement = sql.buildPreparedStatement(connection);
      resultSet = statement.executeQuery();

      while (resultSet.next())
        keysOut.addElement(resultSet.getString(1));//("SERVICE_KEY"));

      return keysOut;
    }
    finally
    {
      try {
        resultSet.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService ResultSet: "+e.getMessage(),e);
      }

      try {
        statement.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService Statement: "+e.getMessage(),e);
      }
    }
  }

  /**
   * Select ...
   *
   * @param businessKey primary key value
   * @param names
   * @param keysIn
   * @param qualifiers
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector select(String businessKey,
                              Vector names,
                              Vector keysIn,
                              FindQualifiers qualifiers,
                              Connection connection)
    throws java.sql.SQLException
  {
    // if there is a keysIn vector but it doesn't contain
    // any keys then the previous query has exhausted
    // all possibilities of a match so skip this call.
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 1");
    if (keysIn != null) {
      if (keysIn.size() == 0)
        return keysIn;
    }
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 2");

    Vector keysOut = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 3");

    // construct the SQL statement
    DynamicQuery sql = new DynamicQuery(selectSQL);
    appendWhere(sql,businessKey,names,qualifiers);
    if (keysIn != null)
      appendIn(sql,keysIn);
    appendOrderBy(sql,qualifiers);
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 4");

    try
    {
      log.debug(sql.toString());
      log.info(sql.toString());
      statement = sql.buildPreparedStatement(connection);
      resultSet = statement.executeQuery();

      while (resultSet.next()) {
          keysOut.addElement(resultSet.getString(1)); //("SERVICE_KEY"));
          log.info("serviceKey = " + resultSet.getString(1));
      }
      return keysOut;
    }
    finally
    {
      try {
        resultSet.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService ResultSet: "+e.getMessage(),e);
      }

      try {
        statement.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService Statement: "+e.getMessage(),e);
      }
    }
  }

  /**
   * Select ...
   *
   * @param businessKey primary key value
   * @param names
   * @param keysIn
   * @param qualifiers
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static Vector select2(String businessKey,Vector names,Vector keysIn,FindQualifiers qualifiers,Connection connection)
    throws java.sql.SQLException
  {
    // if there is a keysIn vector but it doesn't contain
    // any keys then the previous query has exhausted
    // all possibilities of a match so skip this call.
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 1");
    if (keysIn != null) {
      if (keysIn.size() == 0)
        return keysIn;
    }
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 2");

    Vector keysOut = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 3");

    // construct the SQL statement
    DynamicQuery sql = new DynamicQuery(selectSQL2);
    appendWhere(sql,businessKey,names,qualifiers);
    if (keysIn != null)
      appendIn(sql,keysIn);
    appendOrderBy(sql,qualifiers);
    log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.select() 4");

    try
    {
      log.debug(sql.toString());
      log.info(sql.toString());
      statement = sql.buildPreparedStatement(connection);
      resultSet = statement.executeQuery();

      while (resultSet.next()) {
          keysOut.addElement(resultSet.getString(1)); //("SERVICE_KEY"));
          log.info("serviceKey = " + resultSet.getString(1));
      }
      return keysOut;
    }
    finally
    {
      try {
        resultSet.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService ResultSet: "+e.getMessage(),e);
      }

      try {
        statement.close();
      }
      catch (Exception e)
      {
        log.warn("An Exception was encountered while attempting to close " +
          "the Find BusinessService Statement: "+e.getMessage(),e);
      }
    }
  }


  /**
   *
   */
  private static void appendWhere(DynamicQuery sql,Vector names,FindQualifiers qualifiers)
  {
    sql.append("WHERE N.SERVICE_KEY = S.SERVICE_KEY ");

    if ((names != null) && (names.size() > 0))
    {
      sql.append("AND (");

      int nameSize = names.size();
      for (int i=0; i<nameSize; i++)
      {
        Name name = (Name)names.elementAt(i);
        log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.appendWhere()=> name = " + name.getValue());

        String text = name.getValue();
        String lang = name.getLanguageCode();

        if ((text != null) && (text.length() > 0))
        {
          if ((qualifiers != null) && (qualifiers.exactNameMatch))
          {
            sql.append("(NAME = ?");
            sql.addValue(text);
          }
          else
          {
            sql.append("(NAME LIKE ?");
            sql.addValue(text.endsWith("%") ? text : text+"%");
          }

          if ((lang != null) && (lang.length() > 0))
          {
            sql.append(" AND LANG_CODE = ?");
            sql.addValue(lang);
          }

          sql.append(")");

          if (i+1 < nameSize)
            sql.append(" OR ");
        }
      }

      sql.append(") ");
    }
  }

  /**
   *
   */
  private static void appendWhere(DynamicQuery sql,String businessKey,Vector names,FindQualifiers qualifiers)
  {
    sql.append("WHERE N.SERVICE_KEY = S.SERVICE_KEY ");

    // per UDDI v2.0 Programmers API Errata (pg. 14), businessKey is
    // is no longer a required attribute of the find_service server.

    if((businessKey != null) && (businessKey.length() > 0))
    {
      sql.append("AND S.BUSINESS_KEY = ? ");
      sql.addValue(businessKey);
    }


    if ((names != null) && (names.size() > 0))
    {
      sql.append("AND (");

      int nameSize = names.size();
      for (int i=0; i<nameSize; i++)
      {
        Name name = (Name)names.elementAt(i);
        log.info("cgl.fthpis.datastore.jdbc.FindServiceByNameQuery.appendWhere()=> name = " + name.getValue());

        String text = name.getValue();
        String lang = name.getLanguageCode();

        if ((text != null) && (text.length() > 0))
        {
          if ((qualifiers != null) && (qualifiers.exactNameMatch))
          {
            sql.append("(NAME = ?");
            sql.addValue(text);
          }
          else
          {
            sql.append("(NAME LIKE ?");
            sql.addValue(text.endsWith("%") ? text : text+"%");
          }

          if ((lang != null) && (lang.length() > 0))
          {
            sql.append(" AND LANG_CODE = ?");
            sql.addValue(lang);
          }

          sql.append(")");

          if (i+1 < nameSize)
            sql.append(" OR ");
        }
      }

      sql.append(") ");
    }
  }

  /**
   * Utility method used to construct SQL "IN" statements such as
   * the following SQL example:
   *
   *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
   *
   * @param sql StringBuffer to append the final results to
   * @param keysIn Vector of Strings used to construct the "IN" clause
   */
  private static void appendIn(DynamicQuery sql,Vector keysIn)
  {
    if (keysIn == null)
      return;

    sql.append("AND S.SERVICE_KEY IN (");

    int keyCount = keysIn.size();
    for (int i=0; i<keyCount; i++)
    {
      String key = (String)keysIn.elementAt(i);
      sql.append("?");
      sql.addValue(key);

      if ((i+1) < keyCount)
        sql.append(",");
    }
    sql.append(") ");
  }

  /**
   *
   */
  private static void appendOrderBy(DynamicQuery sql,FindQualifiers qualifiers)
  {
    sql.append("ORDER BY ");

    if ((qualifiers == null) ||
       ((!qualifiers.sortByNameAsc) && (!qualifiers.sortByNameDesc) &&
        (!qualifiers.sortByDateAsc) && (!qualifiers.sortByDateDesc)))
    {
      sql.append("N.NAME ASC,S.LAST_UPDATE DESC");
    }
    else if (qualifiers.sortByNameAsc || qualifiers.sortByNameDesc)
    {
      if (qualifiers.sortByNameAsc && qualifiers.sortByDateDesc)
        sql.append("N.NAME ASC,S.LAST_UPDATE DESC");
      else if (qualifiers.sortByNameAsc && qualifiers.sortByDateAsc)
        sql.append("N.NAME ASC,S.LAST_UPDATE ASC");
      else if (qualifiers.sortByNameDesc && qualifiers.sortByDateDesc)
        sql.append("N.NAME DESC,S.LAST_UPDATE DESC");
      else
        sql.append("N.NAME DESC,S.LAST_UPDATE ASC");
    }
    else if (qualifiers.sortByDateAsc || qualifiers.sortByDateDesc)
    {
      if (qualifiers.sortByDateDesc)
        sql.append("S.LAST_UPDATE ASC,N.NAME ASC");
      else
        sql.append("S.LAST_UPDATE DESC,N.NAME ASC");
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection();
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection)
    throws Exception
  {

//    Vector names_ = new Vector();
//    names_.add(new Name("S"));
//    FindBusinessByNameQuery find = new FindBusinessByNameQuery();
//
//
//
////    String businessKey = "0e70128c-f7c6-4854-b292-d2f13b638acf";
//
//    Vector names = new Vector();
//    names.add(new Name("S"));
//    names.add(new Name("w"));
//    names.add(new Name("W"));
//    names.add(new Name("Web Feature Service"));
//    //names.add(new Name("X"));
//    //names.add(new Name("Select","en"));
//    //names.add(new Name("Inflex","en"));
//
//    Vector keysIn = null;
////    keysIn = new Vector();
////    keysIn.add("0e70128c-f7c6-4854-b292-d2f13b638acf");
////    keysIn.add("b405450a-64f5-4f95-8131-450429d0ae8c");
////    keysIn.add("3009f336-98c1-4193-a22f-fea73e79c909");
////    keysIn.add("45994713-d3c3-40d6-87b5-6ce51f36001c");
////    keysIn.add("901b15c5-799c-4387-8337-a1a35fceb791");
////    keysIn.add("80fdae14-0e5d-4ea6-8eb8-50fde422056d");
////    keysIn.add("e1996c33-c436-4004-9e3e-14de191bcc6b");
////    keysIn.add("3ef4772f-e04b-46ed-8065-c5a4e167b5ba");
//
//    Transaction txn = new Transaction();
//
//    if (connection != null)
//    {
//      try
//      {
//        // begin a new transaction
//        txn.begin(connection);
//        Vector vec = find.select(names_, null, null, connection);
//        for (Iterator iter_ = vec.iterator(); iter_.hasNext(); ) {
//          Object item_ = (Object)iter_.next();
//          log.info("business key = " + item_.toString());
//          select(item_.toString(),names,keysIn,null,connection);
//          Vector vector = select(item_.toString(),names,null,null,connection);
//
//        for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
//          Object item = (Object)iter.next();
//          log.info("service key = " + item.toString());
//
//        }
//      }
//
//        // commit the transaction
//        txn.commit();
//      }
//      catch(Exception ex)
//      {
//        try { txn.rollback(); }
//        catch(java.sql.SQLException sqlex) { sqlex.printStackTrace(); }
//        throw ex;
//      }
//    }
  }
}
