/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.datatype.request.FindQualifier;

import cgl.util.jdbc.ConnectionManager;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeFindQualifier;
import cgl.fthpis.datatype.attribute.AttributeFindQualifiers;

import java.io.StringReader;
import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.dom4j.DocumentHelper;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindServiceByXPATHQuery {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(FindServiceByXPATHQuery.class);

    static String selectSQL;
    static String selectByServiceKeySQL;
    static {
        // build selectSQL
        StringBuffer sql = new StringBuffer(200);
        sql.append("SELECT S.SERVICE_KEY, S.LAST_UPDATE ");
        sql.append(
                "FROM BUSINESS_SERVICE S, SERVICE_PROPERTY C, PROPERTY_ENTITY T ");
        selectSQL = sql.toString();

        // build selectByServiceKeySQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("T.PROPERTY_KEY,");
        sql.append("T.PROPERTYTYPE,");
        sql.append("T.PROPERTYIDENTIFIER,");
        sql.append("T.PROPERTYVALUETYPE,");
        sql.append("T.PROPERTYVALUENUMBER,");
        sql.append("T.PROPERTYVALUESTRING,");
        sql.append("T.PROPERTYVALUEXML,");
        sql.append("T.PROPERTYVERSION ");
        sql.append("FROM PROPERTY_ENTITY T, SERVICE_PROPERTY C ");
        sql.append("WHERE T.PROPERTY_KEY=C.PROPERTY_KEY AND C.SERVICE_KEY=? ");
        sql.append("ORDER BY T.PROPERTYIDENTIFIER");
        selectByServiceKeySQL = sql.toString();

    }

    /**
     * Find node in a given XML String
     *
     * @param String xmlInstanceString
     * @param String xpath_query
     */
    public static boolean findNode(String xmlInstanceString, String xpath_query) {
        boolean result = false;
        try {
            org.dom4j.io.SAXReader reader = new org.dom4j.io.SAXReader();
            org.dom4j.Document document = reader.read(new java.io.StringReader(
                    xmlInstanceString));

            org.dom4j.XPath xpathSelector = DocumentHelper.createXPath(
                    xpath_query);
            result = xpathSelector.booleanValueOf(document);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return result;
    }


    /**
     * Select ...
     *
     * @param businessKey
     * @param categoryBag
     * @param keysIn
     * @param qualifiers
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */

    public static Vector select_xpath(String businessKey,
                                      AttributeBag xpath_attributeBag,
                                      Vector keysIn,
                                      FindQualifiers qualifiers,
                                      Connection connection) throws java.sql.
            SQLException {
        log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeBagQuery.select_xpath() function was called. I ");

        if ((keysIn != null) && (keysIn.size() == 0)) {
            return keysIn;
        }

        Vector xpath_AttributeBagVector = xpath_attributeBag.getAttributeVector();

        Vector keysOut = new Vector();

        try {
            for (Iterator iter = keysIn.iterator(); iter.hasNext(); ) {
                boolean r = false;
                String serviceKey = (String) iter.next();
                Vector attributeBagVector = selectByServiceKey(serviceKey,
                        connection);
                for (Iterator it = attributeBagVector.iterator(); it.hasNext(); ) {

                    Attribute item = (Attribute) it.next();
                    if (item.getAttributeName() != null) {
                        if (item.getAttributeName().getAttributeName().equals(
                                "data_file")) {
                            if (item.getAttributeValue() != null) {

                                for (java.util.Iterator iterator =
                                        xpath_AttributeBagVector.iterator();
                                        iterator.hasNext(); ) {
                                    Attribute query_attribute = (Attribute)
                                            iterator.next();
                                    String xpath_query = query_attribute.
                                            getAttributeValue().getAttributeValueAsString();

                                    String data_file = item.getAttributeValue().getAttributeValueAsString();
                                    String decode = data_file;
//                  String decode = encoder.decode(data_file);

                                    log.info("xpath_query = " + xpath_query);
//                  log.info("data_file = " + decode);

                                    boolean result = findNode(decode,
                                            xpath_query);
                                    if (result) {
                                        r = true;
                                    }

                                }
                            }
                        }
                    }

                }
                if (r) {
                    log.info("result = " + r + " serviceKey = " + serviceKey);
                    keysOut.add(serviceKey);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeBagQuery.select_xpath() function was called. II ");
        return keysOut;
    }


    /**
     * Select ...
     *
     * @param businessKey
     * @param categoryBag
     * @param keysIn
     * @param qualifiers
     * @param connection JDBC connection
     * @throws java.sql.SQLException
     */

    public static Vector select_rec(AttributeBag attributeBag, Vector keysIn,
                                    FindQualifiers qualifiers,
                                    Connection connection) throws java.sql.
            SQLException {
        log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeBagQuery.select_rec() function was called. I ");
        // if there is a keysIn vector but it doesn't contain
        // any keys then the previous query has exhausted
        // all possibilities of a match so skip this call.
        if ((keysIn != null) && (keysIn.size() == 0)) {
            return keysIn;
        }

        log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeBagQuery.select_rec() function was called. II ");

        Vector keysOut = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        // construct the SQL statement
        DynamicQuery sql = new DynamicQuery(selectSQL);
        appendWhere(sql, /*businessKey,*/ attributeBag, qualifiers);
        appendIn(sql, keysIn);
        appendOrderBy(sql, qualifiers);

        log.info("cgl.fthpis.datastore.jdbc.FindServiceByAttributeBagQuery.select_rec() function was called. III ");

        try {
            log.info("sql statement = " + sql.toString());

            statement = sql.buildPreparedStatement(connection);
            resultSet = statement.executeQuery();
            log.info("debug point after satement is executed...");

            while (resultSet.next()) {
                if (!keysOut.contains(resultSet.getString(1))) {
                    keysOut.addElement(resultSet.getString(1)); //("SERVICE_KEY"));
                    log.info("service key = " + resultSet.getString(1));
                }
            }
            return keysOut;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService ResultSet: " + e.getMessage());
            }

            try {
                statement.close();
            } catch (Exception e) {
                log.warn(
                        "An Exception was encountered while attempting to close " +
                        "the Find BusinessService Statement: " + e.getMessage());
            }

        }
    }


    public static Vector select(String businessKey,
                                AttributeBag xpath_attribute_bag, Vector keysIn,
                                FindQualifiers qualifiers,
                                Connection connection) throws java.sql.
            SQLException {
        try {
//      Vector initial_keysIn = keysIn;
            log.info("xpath debug 1 ");
            //run xpath queries first on existing metadata files
            if (qualifiers.orAllKeys) {
                log.info("xpath debug 11 ");
                Vector v = xpath_attribute_bag.getAttributeVector();
                if (v != null) {
                    AttributeBag b = new AttributeBag();
                    b.setAttributeVector(v);
                    keysIn = select_xpath(businessKey, b, keysIn, qualifiers,
                                          connection);
                }
            } else {
                log.info("xpath debug 111 ");
                int size = xpath_attribute_bag.size();
                for (int i = 0; i < size; i++) {
                    log.info("xpath debug 1111 xpath_vector" +
                             xpath_attribute_bag.getAttributeVector());
                    Vector attributevector = xpath_attribute_bag.
                                             getAttributeVector();
                    if (attributevector != null) {
                        if (attributevector.size() != 0) {
                            for (Iterator iter = attributevector.iterator();
                                                 iter.hasNext(); ) {
                                Attribute item = (Attribute) iter.next();
                                if (item.getAttributeValueType() != null) {
                                    if (item.getAttributeValueType().
                                        getAttributeValueType().equals("xpath")) {
                                        Vector vec = new Vector();
                                        vec.add(item);
                                        AttributeBag bag = new AttributeBag();
                                        bag.setAttributeVector(vec);
                                        log.info("xpath debug 11111 ");
                                        //String xpath_query = item.getAttributeValue().getAttributeValue();
                                        keysIn = select_xpath(businessKey, bag,
                                                keysIn,
                                                qualifiers, connection);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            log.info("keysIn size = " + keysIn.size());

        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return keysIn;
    }

    /**
     * Select all rows from the business_service table for a given
     * BusinessKey.
     *
     * @param  serviceKey ServiceKey
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector selectByServiceKey(String serviceKey,
                                            Connection connection) throws java.
            sql.SQLException {
        Vector attributeVector = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            // create a statement to query with
            statement = connection.prepareStatement(selectByServiceKeySQL);
            statement.setString(1, serviceKey.toString());

            // execute the statement
            resultSet = statement.executeQuery();

            Attribute attribute = null;
            while (resultSet.next()) {

                attribute = new Attribute();
                //attribute.setServiceKey(serviceKey);
                //String bindingKey = resultSet.getString(1); //("BINDING_KEY");
                String attributeKey = resultSet.getString(1); //("PROPERTY_KEY");
                String attribute_type = resultSet.getString(2); //("PROPERTY_SUBJECT");
                String attributeIdentifier = resultSet.getString(3); //("PROPERTY_NAME");
                String attributeValueType = resultSet.getString(4); //("PROPERTY_TYPE");
                float attributeValueNumber = 0;
                String attributeValueString = null;
                byte [] attributeValueXML = null;
                java.sql.Blob newblob = null;
                if (attributeValueType != null) {
                    if (attributeValueType.equals("text")) {
                        attributeValueString = resultSet.getString(6); //("PROPERTY_VALUE_STRING");
                    } else if (attributeValueType.equals("xpath")) {
                        newblob = resultSet.getBlob(7); //("PROPERTY_VALUE_NUMBER");
                        java.io.InputStream is = null;
                        byte[] bytes = null;
                        try {
                            is = newblob.getBinaryStream();
                            bytes = new byte[(int) newblob.length()];
                            is.read(bytes);
                        } catch (java.sql.SQLException e) {
                            bytes = null;
                        } catch (java.io.IOException e) {
                            bytes = null;
                        } finally {
                            try {
                                if (is != null) {
                                    is.close();
                                }
                            } catch (Exception e) {}
                        }
                        //attributeValueXML = (new String(bytes)).toString();
                        attributeValueXML = bytes;
                    } else if (attributeValueType.equals("number")) {
                        attributeValueNumber = resultSet.getFloat(5); //("PROPERTY_VALUE_NUMBER");
                    }
                }
                int attributeVersion = resultSet.getInt(8); //("PROPERTYVERSION");

//            if (serviceKey != null) {
//              attribute.setServiceKey(serviceKey);
//              log.info("AttributeTable = > serviceKey " + serviceKey);
//            }
//            if (bindingKey != null) {
//              attribute.setBindingKey(bindingKey);
//              log.info("AttributeTable = > bindingKey " + bindingKey);
//            }
                if (attributeKey != null) {
                    attribute.setAttributeKey(attributeKey);
                    log.info("AttributeTable = > attributeKey " +
                             attributeKey);

                }
//            if (attribute_type != null) {
//              attribute.setAttributeSource(new AttributeSource());
//              log.info("AttributeTable = > attributeKey " +
//                                 attributeKey);
//
//            }
                if (attributeIdentifier != null) {
                    attribute.setAttributeName(new AttributeName(
                            attributeIdentifier));
                    log.info("AttributeTable = > attributeIdentifier " +
                             attributeIdentifier);

                }
                if (attributeValueType != null) {
                    attribute.setAttributeValueType(new AttributeValueType(
                            attributeValueType));
                    log.info("AttributeTable = > type = " + attributeValueType);
                }
                if (attributeValueString != null) {
                    attribute.setAttributeValue(new AttributeValue(
                            attributeValueString));
                    log.info("AttributeTable = > value = " +
                             attributeValueString);
                } else if (attributeValueXML != null) {
                    attribute.setAttributeValue(new AttributeValue(
                            attributeValueXML));
                    //log.info("AttributeTable = > value = " +
                    //                   attributeValueXML);
                } else if (attributeValueNumber != 0) {
                    attribute.setAttributeValue(new AttributeValue(
                            attributeValueNumber));
                    log.info("AttributeTable = > value = " +
                             attributeValueNumber);
                }
                if (attributeVersion != 0) {
                    attribute.setAttributeVersion(attributeVersion);
                    log.info("AttributeTable => version = " +
                             attributeVersion);
                }
                if (attributeIdentifier.equals("data_file")) {
                    attributeVector.add(attribute);
                    //log.info("data file = " + attribute);
                }
                attribute = null;
            }

            return attributeVector;
        } finally {
            try {
                resultSet.close();
            } catch (Exception e) { /* ignored */}
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     *
     */
    private static void appendWhere(DynamicQuery sql,
                                    AttributeBag attributeBag,
                                    FindQualifiers qualifiers) {
        try {

            sql.append("WHERE C.SERVICE_KEY = S.SERVICE_KEY ");

            if (attributeBag != null) {
                Vector attributeVector = attributeBag.getAttributeVector();

                if (attributeVector != null) {
                    int vectorSize = attributeVector.size();
                    if (vectorSize > 0) {
                        sql.append("AND ");

                        for (int i = 0; i < vectorSize; i++) {
                            Attribute attribute = (Attribute) attributeVector.
                                                  elementAt(i);

                            String attribute_name = "";
                            if (attribute.getAttributeName() != null) {
                                attribute_name = attribute.getAttributeName().
                                                 getAttributeName();
                            }

                            String attribute_type = "";
                            if (attribute.getAttributeValueType() != null) {
                                attribute_type = attribute.
                                                 getAttributeValueType().
                                                 getAttributeValueType();
                            }

                            String attribute_findqualifier = "";
                            if (attribute.getAttributeFindQualifier() == null ||
                                attribute.getAttributeFindQualifier().
                                getQualifierValue() == null ||
                                attribute.getAttributeFindQualifier().
                                getQualifierValue() == "") {
                                log.info(
                                        "~~~~~~~~~~~~~~~~ equalTo ~~~~~~~~~~~~~" +
                                        attribute.getAttributeFindQualifier());
                                attribute_findqualifier = "equalTo";
                            } else {
                                log.info("~~~~~~~~~~~~~~~~ else ~~~~~~~~~~~~~" +
                                         attribute.getAttributeFindQualifier().
                                         getQualifierValue());
                                attribute_findqualifier = attribute.
                                        getAttributeFindQualifier().
                                        getQualifierValue();
                            }

                            Float number = null;
                            String attribute_value = "";
                            if (attribute_type.equals("number")) {
                                number = new Float(attribute.getAttributeValue().getAttributeValueAsNumber());
                            } else if (attribute_type.equals("text")) {
                                attribute_value = attribute.getAttributeValue().getAttributeValueAsString();
                            }
                            log.info("debug point - 1");

                            if ((attribute_name != null)) {
                                log.info("debug point -2");
                                if (attribute_type != null) {
                                    if (attribute_type.equals("text")) {

                                        if (attribute_findqualifier.equals(
                                                AttributeFindQualifiers.
                                                EQUAL_TO)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUESTRING = ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(attribute_value);

                                        } else if (attribute_findqualifier.
                                                equals(
                                                AttributeFindQualifiers.
                                                NOT_EQUAL_TO)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUESTRING != ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(attribute_value);

                                        } else {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUESTRING = ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        }

                                    } else if (attribute_type.equals("number")) {
                                        if (attribute_findqualifier.equals(
                                                AttributeFindQualifiers.
                                                GREATER_THAN)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUENUMBER > ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        } else if (attribute_findqualifier.
                                                equals(
                                                AttributeFindQualifiers.
                                                LESS_THAN)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUENUMBER < ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        } else if (attribute_findqualifier.
                                                equals(
                                                AttributeFindQualifiers.
                                                GREATER_THAN_OR_EQUAL)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUENUMBER >= ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        } else if (attribute_findqualifier.
                                                equals(
                                                AttributeFindQualifiers.
                                                LESS_THAN_OR_EQUAL)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUENUMBER <= ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        } else if (attribute_findqualifier.
                                                equals(
                                                AttributeFindQualifiers.
                                                EQUAL_TO)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUENUMBER = ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        } else if (attribute_findqualifier.
                                                equals(
                                                AttributeFindQualifiers.
                                                NOT_EQUAL_TO)) {
                                            sql.append(
                                                    "(T.PROPERTYIDENTIFIER = ? AND T.PROPERTYVALUENUMBER != ?)");
                                            sql.addValue(attribute_name);
                                            sql.addValue(number);
                                        }
                                    } else {
                                        sql.append("(T.PROPERTYIDENTIFIER = ?)");
                                        sql.addValue(attribute_name);
                                    }

                                }
                                if (i + 1 < vectorSize) {
                                    if (qualifiers.andAllKeys) {
                                        sql.append(" AND ");
                                        log.info(
                                                "~~~~~~[1]~~~~~~~~~~ HERE ANDALLKEYS ~~~~~~~~~~~~~" +
                                                qualifiers.toString());
                                    } else if (qualifiers.orAllKeys) {
                                        sql.append(" OR ");
                                        log.info(
                                                "~~~~~~[2]~~~~~~~~~~ HERE OR_ALL_KEYS ~~~~~~~~~~~~~" +
                                                qualifiers.toString());
                                    } else {
                                        log.info(
                                                "~~~~~~[3]~~~~~~~~~~ HERE AND_ALL_KEYS ~~~~~~~~~~~~~" +
                                                qualifiers.toString());
                                        sql.append(" AND ");
                                    }
//                  sql.append(" AND ");
                                }
                            }
                        }

                        sql.append(" ");
                    }
                }
                log.info("debug point - 5");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Utility method used to construct SQL "IN" statements such as
     * the following SQL example:
     *
     *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
     *
     * @param sql StringBuffer to append the final results to
     * @param keysIn Vector of Strings used to construct the "IN" clause
     */
    private static void appendIn(DynamicQuery sql, Vector keysIn) {
        if (keysIn == null) {
            return;
        }

        sql.append("AND S.SERVICE_KEY IN (");

        int keyCount = keysIn.size();
        for (int i = 0; i < keyCount; i++) {
            String key = (String) keysIn.elementAt(i);
            sql.append("?");
            sql.addValue(key);

            if ((i + 1) < keyCount) {
                sql.append(",");
            }
        }

        sql.append(") ");
    }

    /**
     *
     */
    private static void appendOrderBy(DynamicQuery sql,
                                      FindQualifiers qualifiers) {
        sql.append("ORDER BY ");

        if (qualifiers == null) {
            sql.append("S.LAST_UPDATE DESC");
        } else if (qualifiers.sortByDateAsc) {
            sql.append("S.LAST_UPDATE ASC");
        } else {
            sql.append("S.LAST_UPDATE DESC");
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection();
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        String businessKey = "ABA3FC70-1B43-11D9-BC70-AE751C3C76BE";
        log.info("debug point - 0");
        FindQualifiers findQualifiers = new FindQualifiers();
        String findq = "andAllKeys";
        FindQualifier find_qualifier = new FindQualifier();
        find_qualifier.setValue(findq);
        Vector vec = new Vector();
        vec.add(find_qualifier);
        findQualifiers.setFindQualifierVector(vec);

        AttributeBag attributeBag = new AttributeBag();
        Attribute attribute = new Attribute();
        attribute.setAttributeName(new AttributeName("cpu_number"));
        attribute.setAttributeValueType(new AttributeValueType("number"));
        attribute.setAttributeValue(new AttributeValue("2"));
        attribute.setAttributeFindQualifier(new AttributeFindQualifier(
                AttributeFindQualifiers.GREATER_THAN));
        attributeBag.addAttribute(attribute);

        attribute = new Attribute();
        attribute.setAttributeName(new AttributeName("bandwith"));
        attribute.setAttributeValueType(new AttributeValueType("number"));
        attribute.setAttributeValue(new AttributeValue("110"));
        attribute.setAttributeFindQualifier(new AttributeFindQualifier(
                AttributeFindQualifiers.GREATER_THAN));
        attributeBag.addAttribute(attribute);

        Transaction txn = new Transaction();

        if (connection != null) {
            try {
                // begin a new transaction
                txn.begin(connection);
                log.info("debug point in main - 1");

                //select(businessKey,attributeBag,keysIn,null,connection);
                Vector vector = select(businessKey, attributeBag, null,
                                       findQualifiers, connection);

                log.info("debug point in main - 2");

                for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
                    Object item = (Object) iter.next();
                    log.info("serviceKey = " + item.toString());

                }

                log.info("debug point in main - 3");

                // commit the transaction
                txn.commit();
                log.info("FindServiceByAttributeBagQuery is completed fine.");
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }
}
