/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.DynamicQuery;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.util.jdbc.ConnectionManager;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.lease.LeaseExpirationDate;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindSessionByLeaseQuery {
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(FindSessionByLeaseQuery.class);

  static String selectSQL;
  static {
    // build selectSQL
    StringBuffer sql = new StringBuffer(200);
    sql.append("SELECT S.SESSION_KEY, S.LAST_UPDATE ");
    sql.append("FROM SESSION_ENTITY S, LEASE C ");
    selectSQL = sql.toString();
  }

  /**
   * Select ...
   *
   * @param lease
   * @param keysIn
   * @param qualifiers
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */

  public static Vector select(Lease lease,
                              Vector keysIn,
                              FindQualifiers qualifiers,
                              Connection connection) throws
      java.sql.SQLException {
    log.info("cgl.fthpis.datastore.jdbc.FindSessionByLeaseQuery.select() function was called.");
    // if there is a keysIn vector but it doesn't contain
    // any keys then the previous query has exhausted
    // all possibilities of a match so skip this call.
    if ( (keysIn != null) && (keysIn.size() == 0)) {
      return keysIn;
    }

    Vector keysOut = new Vector();
    PreparedStatement statement = null;
    ResultSet resultSet = null;

    // construct the SQL statement
    DynamicQuery sql = new DynamicQuery(selectSQL);
    appendWhere(sql, lease, qualifiers);
    appendIn(sql, keysIn);
    appendOrderBy(sql, qualifiers);

    try {
      log.info("sql statement = " + sql.toString());

      statement = sql.buildPreparedStatement(connection);
      resultSet = statement.executeQuery();
      log.info("debug boint after satement is executed... resultSet = " + resultSet);

      while (resultSet.next()) {
        if (!keysOut.contains(resultSet.getString(1))) {
          keysOut.addElement(resultSet.getString(1)); //("SESSION_KEY"));
          log.info("session key = " + resultSet.getString(1));
        }
      }
      return keysOut;
    }
    finally {
      try {
        resultSet.close();
      }
      catch (Exception e) {
        log.warn("An Exception was encountered while attempting to close " +
                 "the ResultSet: " + e.getMessage());
      }

      try {
        statement.close();
      }
      catch (Exception e) {
        log.warn("An Exception was encountered while attempting to close " +
                 "the Statement: " + e.getMessage());
      }
    }
  }

  /**
   *
   */
  private static void appendWhere(DynamicQuery sql, Lease lease, FindQualifiers qualifiers) {
    sql.append("WHERE C.OBJECT_KEY = S.SESSION_KEY ");
    if (lease != null) {
      sql.append("AND (");
      java.sql.Date sql_end_date = null;
      LeaseExpirationDate lease_expiration_timestamp = lease.getLeaseExpirationDate();
      String leaseEndDate = "";
      if (lease_expiration_timestamp != null) {
          leaseEndDate = lease_expiration_timestamp.getLeaseExpirationDate();
      }
      //Timestamp stop_timeStamp = new Timestamp(System.currentTimeMillis());
        try {
          if (leaseEndDate != null) {
              long stop_time = (new Long(leaseEndDate)).longValue();
              sql_end_date = org.apache.juddi.util.ISO_8601_UTC.convert(stop_time);
              //stop_timeStamp = new Timestamp(stop_time);
              //log.info(stop_timeStamp.toString());
          }
        }
        catch (Exception ex) {
          ex.printStackTrace();
        }
      sql.append(("C.LEASEEXPIRATIONDATE >= ?"));
      sql.addValue(sql_end_date);
      //sql.addValue(stop_timeStamp);
      sql.append(") ");
    }
  }

  /**
   * Utility method used to construct SQL "IN" statements such as
   * the following SQL example:
   *
   *   SELECT * FROM TABLE WHERE MONTH IN ('jan','feb','mar')
   *
   * @param sql StringBuffer to append the final results to
   * @param keysIn Vector of Strings used to construct the "IN" clause
   */
  private static void appendIn(DynamicQuery sql, Vector keysIn) {
    if (keysIn == null) {
      return;
    }
    sql.append("AND S.SESSION_KEY IN (");

    int keyCount = keysIn.size();
    for (int i = 0; i < keyCount; i++) {
      String key = (String) keysIn.elementAt(i);
      sql.append("?");
      sql.addValue(key);

      if ( (i + 1) < keyCount) {
        sql.append(",");
      }
    }

    sql.append(") ");
  }

  /**
   *
   */
  private static void appendOrderBy(DynamicQuery sql, FindQualifiers qualifiers) {
    sql.append("ORDER BY ");

    if (qualifiers == null) {
      sql.append("S.LAST_UPDATE DESC");
    }
    else if (qualifiers.sortByDateAsc) {
      sql.append("S.LAST_UPDATE ASC");
    }
    else {
      sql.append("S.LAST_UPDATE DESC");
    }
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/

  public static void main(String[] args) throws Exception {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool", "true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection("wscontext");
      test(conn);
    }
    finally {
      if (conn != null) {
        conn.close();
      }
    }
  }

  public static void test(Connection connection) throws Exception {
    Lease lease = new Lease();
    lease.setLeaseExpirationDate("1169336085000");

    Transaction txn = new Transaction();

    if (connection != null) {
      try {
        // begin a new transaction
        txn.begin(connection);
        log.info("debug point in main - 1");

        Vector vector = select(lease, null, null, connection);

        log.info("debug point in main - 2");

        for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
          Object item = (Object) iter.next();
          log.info("sessionKey = " + item.toString());

        }
        log.info("debug point in main - 3");

        // commit the transaction
        txn.commit();
        log.info("FindServiceByAttributeBagQuery is completed fine.");
      }
      catch (Exception ex) {
        try {
          txn.rollback();
        }
        catch (java.sql.SQLException sqlex) {
          sqlex.printStackTrace();
        }
        throw ex;
      }
    }
  }

}
