/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Vector;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.binding.BindingTemplate;
import org.apache.juddi.datatype.binding.BindingTemplates;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UnknownUserException;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.datastore.DataStore;
import org.apache.juddi.datastore.DataStoreFactory;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.UserMismatchException;
import org.apache.juddi.datatype.request.FindQualifier;

import cgl.fthpis.datatype.AutharizationToken;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datastore.jdbc.ServiceNameTable;
import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.response.PublisherInfo;
import cgl.fthpis.datatype.service.BusinessService;
import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class JDBCDataStore implements FTHPISDataStore {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(JDBCDataStore.class);

    // private db connection associated with this datastore
    private Connection connection = null;

    // private XA transaxtion object
    private Transaction transaction = null;

    /**
     * Create a new JDBCDataStore and aquire a JDBC
     * connection from the connection pool.
     */
    public JDBCDataStore() {
        try {
            this.connection = ConnectionManager.aquireConnection("uddi");
        } catch (SQLException sqlex) {
            log.error("Exception occured while attempting to " +
                      "aquire a JDBC connection: " + sqlex.getMessage());
            log.info("Exception occured while attempting to " +
                     "aquire a JDBC connection: " + sqlex.getMessage());
        }
    }

    /**
     * Release all JDBC connections used by this
     * JDBCDataStore back into the connection pool.
     */
    public void release() {
        try {
            if (connection != null) {
                this.connection.close();
                this.connection = null;
            }
        } catch (SQLException sqlex) {
            log.error("Exception occured while attempting to " +
                      "close a JDBC connection: " + sqlex.getMessage());
        }
    }

    /**
     *
     */
    public Connection getConnection() {
        return this.connection;
    }

    /**
     * begin a new transaction
     */
    public void beginTrans() throws org.apache.juddi.error.RegistryException {
        try {
            this.transaction = new Transaction();
            this.transaction.begin(connection);
        } catch (SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     * commit on all connections.
     */
    public void commit() throws org.apache.juddi.error.RegistryException {
        try {
            this.transaction.commit();
        } catch (SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     * rollback on all connections.
     */
    public void rollback() throws org.apache.juddi.error.RegistryException {
        try {
            this.transaction.rollback();
        } catch (SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void savePublisher(Publisher publisher) throws org.apache.juddi.
            error.RegistryException {
        log.info("JDBCDataSTORE => savePublisher I => Publisher = " + publisher);

        try {
            if ((publisher != null) && (connection != null)) {
                // create the Publisher account
                PublisherTable.insert(publisher, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void deletePublisher(String publisherID) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((publisherID != null) && (connection != null)) {
                // delete the Publisher account
                PublisherTable.delete(publisherID, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public PublisherInfo fetchPublisherInfo(String publisherID) throws org.
            apache.juddi.error.RegistryException {
        PublisherInfo info = null;

        if ((publisherID != null) && (connection != null)) {
            try {
                Publisher publisher = PublisherTable.select(publisherID,
                        connection);
                info = new PublisherInfo();
                info.setPublisherID(publisherID);
                info.setNameValue(publisher.getName());
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }

        return info;
    }

    /**
     *
     */
    public Vector findPublisher(String name, FindQualifiers findQualifiers) throws
            org.apache.juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            // always perform this query - even when not searching by Name!!!
            keyVector = FindPublisherByNameQuery.select(name, keyVector,
                    findQualifiers, connection);
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }

    /**
     * verify that the individual or system identified by
     * the 'publisherID' is a valid jUDDI user (aka publisher).
     *
     * @param publisherID
     * @return publisher
     * @throws UDDIException
     */
    public Publisher getPublisher(String publisherID) throws RegistryException {
        // a publisherID must be specified.
        if (publisherID == null) {
            return null;
        }

        Publisher publisher = null;

        try {
            log.info("cgl.fthpis.datastore.jdbc.JDBCDataStore => connection = " +
                     connection);
            publisher = PublisherTable.select(publisherID, connection);
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return publisher;
    }


    /**
     *
     */
    public AutharizationToken getAuthToken(String token) throws org.apache.
            juddi.error.RegistryException {
        AutharizationToken authtoken = null;

        if (token != null) {
            try {
                authtoken = AuthTokenTable.selectAuthToken(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
        return authtoken;
    }

    /**
     * Will return true if the publisherID parameter specified has been tagged
     * with registry administrator priviledges otherwise returns false. An
     * Unknown user exception is thrown if the publisherID specified is null,
     * blank or does not exist (can't be found in the PUBLISHER table).
     *
     * @param publisherID
     * @return Returns true if publisherID parameter specified has jUDDI
     *   administrator priv's otherwise returns false.
     * @throws RegistryException
     */
    public boolean isAdministrator(String publisherID) throws org.apache.juddi.
            error.RegistryException {
        if ((publisherID == null) || (publisherID.length() == 0)) {
            throw new UnknownUserException("publisherID = " + publisherID);
        }

        try {
            Publisher publisher = PublisherTable.select(publisherID, connection);
            if (publisher == null) {
                throw new UnknownUserException("publisherID = " + publisherID);
            } else {
                return publisher.isAdmin();
            }
        } catch (java.sql.SQLException sqlex) {
            log.error(sqlex.getMessage());
            throw new RegistryException(sqlex);
        }
    }

    /**
     * Will return true if the publisherID parameter specified is
     * currently enabled otherwise returns false. An UnknownUserException
     * is thrown if the publisherID specified is null, blank or does not
     * exist (can't be found in the PUBLISHER table).
     *
     * @param publisherID
     * @return Returns true if publisherID parameter specified has jUDDI
     *   administrator priv's otherwise returns false.
     * @throws RegistryException
     */
    public boolean isEnabled(String publisherID) throws org.apache.juddi.error.
            RegistryException {
        if ((publisherID == null) || (publisherID.length() == 0)) {
            throw new UnknownUserException("publisherID = " + publisherID);
        }

        try {
            Publisher publisher = PublisherTable.select(publisherID, connection);
            if (publisher == null) {
                throw new UnknownUserException("publisherID = " + publisherID);
            } else {
                return publisher.isEnabled();
            }
        } catch (java.sql.SQLException sqlex) {
            log.error(sqlex.getMessage());
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     * @param publisher
     * @return String
     * @throws UDDIException
     */
    public String generateToken(Publisher publisher) throws RegistryException {
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        String token = "authToken:" + uuidgen.uuidgen_with_timestamp();

        log.info("Generated token '" + token + "' for user: '" +
                 publisher.getPublisherID() + "/" + publisher.getName() + "'");

        return token;
    }

    /**
     *
     */
    public void storeAuthToken(String token, Publisher publisher) throws org.
            apache.juddi.error.RegistryException {
        if ((token != null) && (publisher != null)) {
            try {
                AuthTokenTable.insert(token, publisher, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
    }

    /**
     *
     */
    public void retireAuthToken(String token) throws org.apache.juddi.error.
            RegistryException {
        if (token != null) {
            try {
                AuthTokenTable.invalidate(token, connection);
                AuthTokenTable.touch(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
    }

    /**
     *
     */
    public Publisher getAuthTokenPublisher(String token) throws org.apache.
            juddi.error.RegistryException {
        Publisher publisher = null;

        if (token != null) {
            try {
                publisher = AuthTokenTable.selectPublisher(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }

        return publisher;
    }

    /**
     *
     */
    public boolean isAuthTokenExpired(String token) throws org.apache.juddi.
            error.RegistryException {
        boolean expired = false;

        if (token != null) {
            try {
                long tokenState = AuthTokenTable.selectTokenState(token,
                        connection);
                if (tokenState <= 0) {
                    return expired = true;
                }

                long lastUsed = AuthTokenTable.selectLastUsed(token, connection);
                if (lastUsed > 0) { // -1 is returned when token isn't found
                    long timeOut = Config.getLongProperty(
                            "juddi.authTokenTimeout", 3600) * 1000L; // convert from seconds to milliseconds
                    long currTime = System.currentTimeMillis();
                    if ((currTime - lastUsed) >= timeOut) {
                        expired = true;
                    }
                }
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }

        return expired;
    }

    /**
     *
     */
    public void touchAuthToken(String token) throws org.apache.juddi.error.
            RegistryException {
        if (token != null) {
            try {
                AuthTokenTable.touch(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
    }

    ////////////////////STATIC CONTEXT FUNCTIONS/////////////////////////////////

    /**
     *
     */
    public void saveService(cgl.fthpis.datatype.service.BusinessService service) throws
            org.apache.juddi.error.RegistryException {
        try {
            if ((service != null) && (connection != null)) {
                String serviceKey = service.getServiceKey();

                // insert the BusinessService object
                BusinessServiceTable.insert(service, connection);

                // insert all of the BusinessService's Name objects
                if (service.getNameVector() != null) {
                    ServiceNameTable.insert(serviceKey, service.getNameVector(),
                                            connection);
                }

                // insert all of the BusinessService's Description objects
                if (service.getDescriptionVector() != null) {
                    ServiceDescTable.insert(serviceKey,
                                            service.getDescriptionVector(),
                                            connection);
                }

                // insert the BusinessService's CategoryBag KeyedReferences
                CategoryBag catBag = service.getCategoryBag();
                if ((catBag != null) && (catBag.getKeyedReferenceVector() != null)) {
                    ServiceCategoryTable.insert(serviceKey,
                                                catBag.getKeyedReferenceVector(),
                                                connection);
                }

                log.info("Extracts BindingTemplates from the service.. ");

                // extract the binding template objects
                BindingTemplates bindings = service.getBindingTemplates();
                if (bindings == null) {
                    return; // no binding templates were present
                }
                // convert the binding templates to a vector of templates
                Vector bindingList = bindings.getBindingTemplateVector();
                if (bindingList == null || bindingList.size() == 0) {
                    return; // a binding template vector wasn't found
                }

                // aquire a reference to the shared UUIDGen instance
                UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

                //      burada juddi JDBC' ye ulasip bu function' i cagir
                DataStore juddiStore = DataStoreFactory.getDataStore();
                log.info("Got the DataStore I ..");
                try {
                    juddiStore.beginTrans();
                    // save all of the binding templates that were found
                    int listSize = bindingList.size();
                    for (int i = 0; i < listSize; i++) {
                        log.info(
                                "HEY: Getting the binding from binding vector II..");

                        BindingTemplate binding = (BindingTemplate) bindingList.
                                                  elementAt(i);
                        String bindingkey = uuidgen.uuidgen();
                        log.info("serviceKey = " + serviceKey +
                                 "  binding key = " + bindingkey);
                        binding.setServiceKey(serviceKey);
                        binding.setBindingKey(bindingkey);
                        juddiStore.saveBinding(binding);
                        log.info("HEY Saved the binding III..");

                    }
                    juddiStore.commit();
                    log.info("After saving the binding III - 1..");
                } catch (InvalidKeyPassedException ikpex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) ikpex;
                } catch (UserMismatchException umex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) umex;
                } catch (RegistryException regex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) regex;
                } catch (Exception ex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    ex.printStackTrace();
                    throw new RegistryException(ex);
                } finally {
                    if (juddiStore != null) {
                        juddiStore.release();
                    }
                }
                log.info("Before processing the attributeBag IV..");
                //String defaultbinding = ((BindingTemplate) bindingList.get(0)).getBindingKey();

                // extract the binding template objects
                AttributeBag attributeBag = service.getAttributeBag();

                if (attributeBag != null) {
                    log.info("Before processing the attributeBag V..");
                    java.util.Vector attributeVector = attributeBag.
                            getAttributeVector();
                    if (attributeVector != null) {
                        if (attributeVector.size() != 0) {
                            //    return; // no binding templates were present

                            // aquire a reference to the shared UUIDGen instance
                            UUIDGen prop_uuidgen = UUIDGenFactory.getUUIDGen();

                            // save all of the binding templates that were found
                            int list_Size = attributeVector.size();
                            for (int i = 0; i < list_Size; i++) {
                                Attribute attribute = (Attribute)
                                        attributeVector.
                                        elementAt(
                                                i);
                                attribute.setServiceKey(serviceKey);
                                //associate this with default binding
                                //attribute.setBindingKey(defaultbinding);
                                attribute.setAttributeKey(prop_uuidgen.uuidgen());
                                saveAttribute(attribute);
                            }
                        }
                    }
                }
                Lease lease = service.getLease();
                log.info(
                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveService() lease = " +
                        lease);

                if (lease == null) {
                    return; //no lease exists
                }

                lease.setObjectKey(serviceKey);
                saveServiceLease(lease);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public Attribute fetchAttribute(String attributeKey) throws org.apache.
            juddi.error.RegistryException {
        Attribute attribute = null;

        log.info("cgl.fthpis.datastore.jdbc.JDBCDataStore.java => fetchpAttribute => attributeKey = " +
                 attributeKey);
        try {
            if ((attributeKey != null) && (connection != null)) {
                // fetch the Attribute itself
                attribute = AttributeTable.select(attributeKey, connection);

                // fetch the service associated to this attribute
                Vector service_vector = ServiceAttributeTable.select(
                        attributeKey, connection);
                if (service_vector != null) {
                    String serviceKey = (String) service_vector.elementAt(0);
                    attribute.setServiceKey(serviceKey);
                }

                // fetch the attribute's CategoryBag
                CategoryBag bag = new CategoryBag();
                bag.setKeyedReferenceVector(AttributeCategoryTable.select(
                        attributeKey, connection));
                attribute.setCategoryBag(bag);

                //fetch the lease associated to this attribute
                Lease lease = new Lease();
                Vector v = LeaseTable.selectObjectLease(attributeKey,
                        connection);
                if (v != null) {
                    if (v.size() != 0) {
                        lease = (Lease) v.elementAt(0);
                        attribute.setLease(lease);
                    }
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return attribute;
    }


    /**
     *
     */
    public void deleteAttribute(String attributeKey) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((attributeKey != null) && (connection != null)) {
                // delete all the categories associated to this attribute
                AttributeCategoryTable.delete(attributeKey, connection); // UDDI v3.0

                // delete the link between attribute and the service
                ServiceAttributeTable.delete(attributeKey, connection);

                // delete the lease associated to attribute
                LeaseTable.deleteObjectLease(attributeKey, connection);

                // finally, delete the Attribute itself.
                AttributeTable.deleteByAttributeKey(attributeKey, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }


    /**
     *
     */
    public BusinessService fetchService(String serviceKey) throws org.apache.
            juddi.error.RegistryException {
        BusinessService service = null;

        try {
            if ((serviceKey != null) && (connection != null)) {
                service = BusinessServiceTable.select(serviceKey, connection);
                service.setNameVector(ServiceNameTable.select(serviceKey,
                        connection));
                service.setDescriptionVector(ServiceDescTable.select(serviceKey,
                        connection));

                CategoryBag bag = new CategoryBag();
                bag.setKeyedReferenceVector(ServiceCategoryTable.select(
                        serviceKey, connection));
                service.setCategoryBag(bag);

                // 'fetch' the BusinessService's BindingTemplate objects
                DataStore juddiStore = DataStoreFactory.getDataStore();
                Vector bindingVector = new Vector();
                try {
                    juddiStore.beginTrans();
                    Vector qualifierVector = new Vector();
                    qualifierVector.add(new FindQualifier(FindQualifier.
                            OR_ALL_KEYS));
                    FindQualifiers qualifiers = new FindQualifiers();
                    qualifiers.setFindQualifierVector(qualifierVector);
                    Vector vec = juddiStore.findBinding(serviceKey, null, null,
                            qualifiers);
                    if (vec != null) {
                        for (Iterator iter = vec.iterator(); iter.hasNext(); ) {
                            String bindingKey = (String) iter.next();
                            BindingTemplate binding = juddiStore.fetchBinding(
                                    bindingKey);
                            vec.add(binding);
                        }
                    }
                    juddiStore.commit();
                } catch (InvalidKeyPassedException ikpex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) ikpex;
                } catch (UserMismatchException umex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) umex;
                } catch (RegistryException regex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) regex;
                } catch (Exception ex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    ex.printStackTrace();
                    throw new RegistryException(ex);
                } finally {
                    if (juddiStore != null) {
                        juddiStore.release();
                    }
                }

                BindingTemplates bindings = new BindingTemplates();
                bindings.setBindingTemplateVector(bindingVector);
                service.setBindingTemplates(bindings);

                Vector temp = fetchAttributeByServiceKey(serviceKey);
                Vector attributeVector = null;
                if (temp != null) {
                    attributeVector = new Vector();
                    for (Iterator iter = temp.iterator(); iter.hasNext(); ) {
                        Attribute attribute = (Attribute) iter.next();
                        String attributeKey = attribute.getAttributeKey();
                        log.info("attributeKey = " + attributeKey);
                        // fetch the Attribute's CategoryBag (UDDI v3.0)
                        CategoryBag cbag = new CategoryBag();
                        cbag.setKeyedReferenceVector(AttributeCategoryTable.
                                select(
                                        attributeKey, connection));
                        attribute.setCategoryBag(cbag);
                        Lease lease = new Lease();
                        Vector v = LeaseTable.selectObjectLease(attributeKey,
                                connection);
                        if (v != null) {
                            if (v.size() != 0) {
                                lease = (Lease) v.elementAt(0);
                                attribute.setLease(lease);
                            }
                        }
                        attributeVector.add(attribute);
                    }
                }
                AttributeBag attributeBag = new AttributeBag();
                //attributeBag.setAttributeVector(AttributeTable.selectByServiceKey(serviceKey, connection));
                attributeBag.setAttributeVector(attributeVector);

                service.setAttributeBag(attributeBag);
                Lease lease = new Lease();
                Vector v = LeaseTable.selectObjectLease(serviceKey, connection);
                if (v != null) {
                    if (v.size() != 0) {
                        lease = (Lease) v.elementAt(0);
                        service.setLease(lease);
                    }
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return service;
    }

    /**
     *
     */
    public void deleteService(String serviceKey) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((serviceKey != null) && (connection != null)) {

                DataStore juddiStore = DataStoreFactory.getDataStore();
                try {
                    juddiStore.beginTrans();
                    Vector qualifierVector = new Vector();
                    qualifierVector.add(new FindQualifier(FindQualifier.
                            OR_ALL_KEYS));
                    FindQualifiers qualifiers = new FindQualifiers();
                    qualifiers.setFindQualifierVector(qualifierVector);
                    Vector vec = juddiStore.findBinding(serviceKey, null, null,
                            qualifiers);
                    if (vec != null) {
                        for (Iterator iter = vec.iterator(); iter.hasNext(); ) {
                            String bindingKey = (String) iter.next();
                            juddiStore.deleteBinding(bindingKey);
                        }
                    }
                    juddiStore.commit();
                } catch (InvalidKeyPassedException ikpex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) ikpex;
                } catch (UserMismatchException umex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) umex;
                } catch (RegistryException regex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {}
                    throw (RegistryException) regex;
                } catch (Exception ex) {
                    try {
                        juddiStore.rollback();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    ex.printStackTrace();
                    throw new RegistryException(ex);
                } finally {
                    if (juddiStore != null) {
                        juddiStore.release();
                    }
                }

                // delete the immediate dependents of BusinessService
                ServiceNameTable.delete(serviceKey, connection);
                ServiceDescTable.delete(serviceKey, connection);
                ServiceCategoryTable.delete(serviceKey, connection);

//        AttributeTable.deleteByServiceKey(serviceKey, connection);
                deleteAttributeByServiceKey(serviceKey);

                AttributeCategoryTable.delete(serviceKey, connection);
                LeaseTable.deleteObjectLease(serviceKey, connection);

                // finally, delete the BusinessService itself.
                BusinessServiceTable.delete(serviceKey, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }


    /**
     *
     */
    public boolean isValidAttributeKey(String attributeKey) throws org.apache.
            juddi.error.RegistryException {
        try {
            if ((attributeKey != null) && (connection != null) &&
                (AttributeTable.select(attributeKey, connection) != null)) {
                return true;
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
        // default to false
        return false;
    }


    /**
     *
     */
    public boolean isValidServiceKey(String serviceKey) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((serviceKey != null) && (connection != null) &&
                (BusinessServiceTable.select(serviceKey, connection) != null)) {
                return true;
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }

    /**
     *
     */
    public boolean isServicePublisher(String serviceKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        try {
            if ((publisherID != null) && (serviceKey != null) && (connection != null)) {
                return BusinessServiceTable.verifyOwnership(serviceKey,
                        publisherID, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }

    /**
     *
     */
    public void saveAttribute(Attribute attribute) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((attribute != null) && (connection != null)) {
                String attributeKey = attribute.getAttributeKey();

                // insert the attribute object
                AttributeTable.insert(attribute, connection);

                // insert the BusinessService's CategoryBag KeyedReferences
                CategoryBag catBag = attribute.getCategoryBag();
                log.info(
                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveAttribute() categorybag = " +
                        catBag);
                if ((catBag != null) && (catBag.getKeyedReferenceVector() != null)) {
                    AttributeCategoryTable.insert(attributeKey,
                                                  catBag.
                                                  getKeyedReferenceVector(),
                                                  connection);
                }

                // insert the link between attribute and the service
                ServiceAttributeTable.insert(attribute, connection);

                // insert the lease associated to this attribute
                log.info(
                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveAttribute() before lease ");
                Lease lease = attribute.getLease();
                log.info(
                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveAttribute() after lease = " +
                        lease);
                if (lease == null) {
                    return;
                }
                lease.setObjectKey(attributeKey);
                saveAttributeLease(lease);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void saveAttributeLease(Lease lease) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((lease != null) && (connection != null)) {
                // insert the BindingTemplate object
                LeaseTable.insertObjectLease(lease, connection);

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void saveServiceLease(Lease lease) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((lease != null) && (connection != null)) {
                // insert the BindingTemplate object
                LeaseTable.insertObjectLease(lease, connection);

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    private Vector fetchAttributeByServiceKey(String serviceKey) throws org.
            apache.juddi.error.RegistryException {
        Vector attributeVector = new Vector();

        try {
            if ((serviceKey != null) && (connection != null)) {
                Vector tempList = ServiceAttributeTable.selectByServiceKey(
                        serviceKey, connection);
                for (int i = 0; i < tempList.size(); i++) {
                    Attribute attribute = (Attribute) tempList.elementAt(i);
                    attributeVector.add(attribute);
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return attributeVector;
    }


    /**
     *
     */
    public Vector findService(String businessKey,
                              Vector nameVector,
                              CategoryBag categoryBag,
                              TModelBag tModelBag,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        Vector keyVector = null;

        log.info(
                "inside => cgl.fthpis.datastore.jdbc.JDBCDataStore = findService I ");
        try {
            if ((tModelBag != null) && (tModelBag.size() > 0)) {
                keyVector = FindServiceByTModelKeyQuery.select(businessKey,
                        tModelBag, keyVector, findQualifiers, connection);
            }

            if ((categoryBag != null) && (categoryBag.size() > 0)) {
                keyVector = FindServiceByCategoryQuery.select(businessKey,
                        categoryBag, keyVector, findQualifiers, connection);
            }

            log.info(
                    "inside => cgl.fthpis.datastore.jdbc.JDBCDataStore = findService II");
            // always perform this query - even when not searching by Name!!!
            keyVector = FindServiceByNameQuery.select(businessKey, nameVector,
                    keyVector, findQualifiers, connection);
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }


    /**
     *
     */
    public Vector findService(String businessKey,
                              Vector nameVector,
                              CategoryBag categoryBag,
                              TModelBag tModelBag,
                              AttributeBag attributeBag,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            if ((tModelBag != null) && (tModelBag.size() > 0)) {
                keyVector = FindServiceByTModelKeyQuery.select(businessKey,
                        tModelBag, keyVector, findQualifiers, connection);
            }

            if ((categoryBag != null) && (categoryBag.size() > 0)) {
                keyVector = FindServiceByCategoryQuery.select(businessKey,
                        categoryBag, keyVector, findQualifiers, connection);
            }

            if ((attributeBag != null) && (attributeBag.size() > 0)) {
                keyVector = FindServiceByAttributeBagQuery.select(
                        /*businessKey,*/attributeBag, keyVector,
                        findQualifiers, connection);
            }

            // always perform this query - even when not searching by Name!!!
            keyVector = FindServiceByNameQuery.select(businessKey, nameVector,
                    keyVector, findQualifiers, connection);
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }


    /**
     *
     */
    public Vector findService(String businessKey,
                              Vector nameVector,
                              CategoryBag categoryBag,
                              TModelBag tModelBag,
                              AttributeBag attributeBag,
                              Lease lease,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        Vector keyVector = null;
        log.info("SERVICE_KEY VECTOR size after FindServiceByTModelKeyQuery  ");

        try {
            if ((tModelBag != null) && (tModelBag.size() > 0)) {
                keyVector = FindServiceByTModelKeyQuery.select(businessKey,
                        tModelBag, keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SERVICE_KEY VECTOR size after FindServiceByTModelKeyQuery  = " +
                        keyVector.size());
            }

            if ((categoryBag != null) && (categoryBag.size() > 0)) {
                keyVector = FindServiceByCategoryQuery.select(businessKey,
                        categoryBag, keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SERVICE_KEY VECTOR size after FindServiceByCategoryQuery  = " +
                        keyVector.size());
            }

            if ((attributeBag != null) && (attributeBag.size() > 0) &&
                attributeBag.getAttributeVector() != null) {

                Vector xpath_attributeVector = null; //new Vector();
                Vector sql_attributeVector = null; //new Vector();
                AttributeBag xpath_attribute_bag = null; //new AttributeBag();
                AttributeBag sql_attribute_bag = null; //new AttributeBag();
                //separate Xpath queries from SQL queries
                Vector attribute_vector = attributeBag.getAttributeVector();
                for (Iterator iter = attribute_vector.iterator(); iter.hasNext(); ) {
                    Attribute item = (Attribute) iter.next();
                    if (item.getAttributeValueType() != null) {
                        if (item.getAttributeValueType().getAttributeValueType().
                            equals("xpath")) {
                            xpath_attribute_bag = new AttributeBag();
                            xpath_attributeVector = new Vector();
                            //log.info("xpath data type for this attribute" + item.getAttributeName());
                            xpath_attributeVector.add(item);
                            xpath_attribute_bag.setAttributeVector(
                                    xpath_attributeVector);
                        } else {
                            sql_attribute_bag = new AttributeBag();
                            sql_attributeVector = new Vector();
                            //log.info("sql data type for this attribute" + item.getAttributeName());
                            sql_attributeVector.add(item);
                            sql_attribute_bag.setAttributeVector(
                                    sql_attributeVector);
                        }
                    }
                }

                if ((sql_attribute_bag != null) &&
                    (sql_attribute_bag.size() > 0)
                        /*&& sql_attribute_bag.getAttributeVector() != null*/) {
                    keyVector = FindServiceByAttributeBagQuery.select(
                            /*businessKey,*/sql_attribute_bag, keyVector,
                            findQualifiers, connection);
                }

                if (keyVector != null) {
                    log.info(
                            "SERVICE_KEY VECTOR size after FindServiceByAttributeBagQuery  = " +
                            keyVector.size());
                }

                if ((sql_attribute_bag != null) &&
                    (sql_attribute_bag.size() > 0)
                        /*&& sql_attribute_bag.getAttributeVector() != null*/) {
                    keyVector = FindServiceByAttributeCategoryQuery.select(
                            sql_attribute_bag, keyVector, findQualifiers,
                            connection);
                }

                if (keyVector != null) {
                    log.info(
                            "SERVICE_KEY VECTOR size after FindServiceByAttributeCategoryQuery  = " +
                            keyVector.size());
                }

                if ((xpath_attribute_bag != null) &&
                    (xpath_attribute_bag.size() > 0)
                        /*&& xpath_attribute_bag.getAttributeVector() != null*/) {
                    keyVector = FindServiceByXPATHQuery.select(businessKey,
                            xpath_attribute_bag, keyVector, findQualifiers,
                            connection);
                }

                if (keyVector != null) {
                    log.info(
                            "SERVICE_KEY VECTOR size after FindServiceByXPATHQuery  = " +
                            keyVector.size());
                }

            }

            if ((lease != null)) {
                keyVector = FindServiceByLeaseQuery.select( /*businessKey,*/
                                    lease, keyVector, findQualifiers,
                                    connection);
            }

            if (keyVector != null) {
                log.info(
                        "SERVICE_KEY VECTOR size after FindServiceByLeaseQuery  = " +
                        keyVector.size());
            }

            // always perform this query - even when not searching by Name!!!
            keyVector = FindServiceByNameQuery.select(businessKey, nameVector,
                    keyVector, findQualifiers, connection);

            if (keyVector != null) {
                log.info(
                        "SERVICE_KEY VECTOR size after FindServiceByNameQuery  = " +
                        keyVector.size());
            }

        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }

    /**
     *
     */
    public Vector findServiceAttribute(String serviceKey,
                                       CategoryBag categoryBag,
                                       Lease lease,
                                       FindQualifiers findQualifiers) throws
            org.apache.juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            if (serviceKey != null) {
                keyVector = FindAttributeByServiceKeyQuery.select(serviceKey,
                        keyVector, findQualifiers, connection);
            }

            log.info("DEBUG POINT...");
            if (keyVector != null) {
                log.info("keyVector size 1 = " + keyVector.size());
            }

            if ((categoryBag != null) && (categoryBag.size() > 0)) {
                keyVector = FindAttributeByCategoryQuery.select(serviceKey,
                        categoryBag, keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 2 = " + keyVector.size());
                log.info("lease = " + lease);
            }
            if (lease != null) {
                keyVector = FindAttributeByLeaseQuery.select(lease, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 3 = " + keyVector.size());
            }

        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }


    /**
     *
     */
    private void deleteAttributeByServiceKey(String serviceKey) throws org.
            apache.juddi.error.RegistryException {
        try {
            if ((serviceKey != null) && (connection != null)) {
                // obtain a vector of BusinessServices associated with the BusinessKey
                Vector contexts = ServiceAttributeTable.selectByServiceKey(
                        serviceKey, connection);

                // loop through the vector deleting each server in turn
                int listSize = contexts.size();
                for (int i = 0; i < listSize; i++) {
                    Attribute attribute = (Attribute) contexts.elementAt(i);
                    deleteAttribute(attribute.getAttributeKey());
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

    }

    ////////////////////DYNAMIC CONTEXT FUNCTIONS/////////////////////////////////
    /**
     *
     */
    public Vector findSessionService(Vector sessionKeyVector,
                                     Vector keyVector,
                                     AttributeBag contextBag,
                                     FindQualifiers findQualifiers) throws org.
            apache.juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public void saveSessionService(BusinessService service) throws org.apache.
            juddi.error.RegistryException {
    }


    /**
     *
     */
    public Vector findSession(Vector serviceKeyVector,
                              Vector nameVector,
                              AttributeBag attributeBag,
                              Lease lease,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }


    /**
     *
     */
    public void deleteSession(String sessionKey) throws org.apache.juddi.error.
            RegistryException {
    }

    /**
     *
     */
    public void deleteSessionService(String serviceKey) throws org.apache.juddi.
            error.RegistryException {
    }

    /**
     *
     */
    public Attribute fetchContext(String contextKey) throws org.apache.juddi.
            error.RegistryException {
        return null;
    }

    /**
     *
     */
    public SessionEntity fetchSession(String sessionKey) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public void deleteContext(String contextKey) throws org.apache.juddi.error.
            RegistryException {
    }

    /**
     *
     */
    public BusinessService fetchSessionService(String serviceKey) throws org.
            apache.juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public Vector findContext(String sessionKey,
                              String serviceKey,
                              Vector nameVector,
                              Lease lease,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public boolean isSessionAdmin(String sessionKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        return false;
    }

    /**
     *
     */
    public boolean isSessionPublisher(String sessionKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        return false;
    }

    /**
     *
     */
    public boolean isContextPublisher(String contextKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        return false;
    }

    /**
     *
     */
    public boolean isValidSessionKey(String sessionKey) throws org.apache.juddi.
            error.RegistryException {
        return false;
    }

    /**
     *
     */
    public boolean isValidContextKey(String contextKey) throws org.apache.juddi.
            error.RegistryException {
        return false;
    }

    /**
     *
     */
    public void saveContext(Attribute context) throws org.apache.juddi.error.
            RegistryException {
    }

    /**
     *
     */
    public void saveSession(SessionEntity session, String publisherID) throws
            org.apache.juddi.error.RegistryException {
    }

    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection();
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection conn) throws Exception {
    }
}
