/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Iterator;
import java.util.Date;
import java.sql.Timestamp;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.util.Config;

import cgl.util.jdbc.ConnectionManager;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.lease.LeaseTimeOut;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class LeaseTable {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(LeaseTable.class);

    static String insertObjectLeaseSQL = null;
    static String selectObjectLeaseSQL = null;
    static String deleteObjectLeaseSQL = null;

    static {
        // buffer used to build SQL statements
        StringBuffer sql = null;

        // build insertSQL
        sql = new StringBuffer(200);
        sql.append("INSERT INTO LEASE (");
        sql.append("OBJECT_KEY,");
        sql.append("LEASEAPPLIEDONDATE,");
        sql.append("LEASESTARTFROMDATE,");
        sql.append("LEASEEXPIRATIONDATE,");
        sql.append("LEASELASTRENEWALDATE,");
        sql.append("ISINFINITE) ");
        sql.append("VALUES (?,?,?,?,?,?)");
        insertObjectLeaseSQL = sql.toString();

        // build selectSQL
        sql = new StringBuffer(200);
        sql.append("SELECT ");
        sql.append("LEASEAPPLIEDONDATE,");
        sql.append("LEASESTARTFROMDATE,");
        sql.append("LEASEEXPIRATIONDATE,");
        sql.append("LEASELASTRENEWALDATE,");
        sql.append("ISINFINITE ");
        sql.append("FROM LEASE ");
        sql.append("WHERE OBJECT_KEY=? ");
        selectObjectLeaseSQL = sql.toString();

        // build deleteSQL
        sql = new StringBuffer(100);
        sql.append("DELETE FROM LEASE ");
        sql.append("WHERE OBJECT_KEY=?");
        deleteObjectLeaseSQL = sql.toString();
    }

    /**
     *
     */
    public static void insertObjectLease(Lease lease, Connection connection) throws
            java.sql.
            SQLException {

        log.info("LeaseTable.insertObjectLease => a new record is inserted.");
        PreparedStatement statement = null;
        String service_key = lease.getObjectKey();
        java.sql.Date lease_sql_end_date = null, lease_sql_begin_date = null,
                lease_applied_on_date = null, lease_last_renewal_date = null;
        java.sql.Timestamp start_timeStamp = null, stop_timeStamp = null;

        String isInfinite = lease.getIsInfinte();
        LeaseTimeOut lease_time_out = lease.getLeaseTimeOut();
        String leaseTimeOut = "";
        if (lease_time_out != null) {
            leaseTimeOut = lease_time_out.getLeaseTimeOut();
        }
        if (isInfinite != null) {
            if (isInfinite.equals("true")) {
                stop_timeStamp = null;
            } else {
                long timeout = (new Long(leaseTimeOut)).longValue();
                log.info("inserted lease end date = " + leaseTimeOut);
                long currenttime = System.currentTimeMillis();
                start_timeStamp = new Timestamp(currenttime);
                stop_timeStamp = new Timestamp(currenttime + timeout);

                lease_sql_begin_date = org.apache.juddi.util.ISO_8601_UTC.
                                       convert(start_timeStamp.getTime());
                lease_sql_end_date = org.apache.juddi.util.ISO_8601_UTC.convert(
                        stop_timeStamp.getTime());
                lease_applied_on_date = lease_sql_begin_date;
                lease_last_renewal_date = lease_sql_begin_date;
                log.info("sql_begin_date = " + lease_sql_begin_date.toGMTString());
                log.info("sql_end_date = " + lease_sql_end_date.toGMTString());
            }
        } else {
            long timeout = (new Long(leaseTimeOut)).longValue();
            log.info("inserted lease end date = " + leaseTimeOut);
            long currenttime = System.currentTimeMillis();
            start_timeStamp = new Timestamp(currenttime);
            stop_timeStamp = new Timestamp(currenttime + timeout);

            lease_sql_begin_date = org.apache.juddi.util.ISO_8601_UTC.convert(
                    start_timeStamp.getTime());
            lease_sql_end_date = org.apache.juddi.util.ISO_8601_UTC.convert(
                    stop_timeStamp.getTime());
            lease_applied_on_date = lease_sql_begin_date;
            lease_last_renewal_date = lease_sql_begin_date;
            log.info("sql_begin_date = " + lease_sql_begin_date.toGMTString());
            log.info("sql_end_date = " + lease_sql_end_date.toGMTString());
        }
        try {
            try {
                deleteObjectLease(service_key, connection);
            } catch (Exception ex) {}

        } catch (Exception ex) {
            ex.printStackTrace();
        }

        try {
            statement = connection.prepareStatement(insertObjectLeaseSQL);
            statement.setString(1, service_key);
            statement.setDate(2, lease_applied_on_date);
            statement.setDate(3, lease_sql_begin_date);
            statement.setDate(4, lease_sql_end_date);
            statement.setDate(5, lease_last_renewal_date);
            statement.setString(6, isInfinite);

            log.info("inserSQL = " + insertObjectLeaseSQL);

            log.info("insert into LEASE table:\n\n\t" + insertObjectLeaseSQL +
                     "\n\t OBJECT_KEY=" + service_key +
                     "\n\t LEASEAPPLIEDONDATE=" + lease_applied_on_date +
                     "\n\t LEASESTARTFROMDATE=" + lease_sql_begin_date +
                     "\n\t LEASEEXPIRATIONDATE=" + lease_sql_end_date +
                     "\n\t LEASELASTRENEWALDATE=" + lease_last_renewal_date +
                     "\n\t ISINFINITE=" + isInfinite +
                     "\n");

            statement.executeUpdate();

        } finally {
            try {
                statement.close();
            } catch (Exception e) {
                e.printStackTrace();
                /* ignored */
            }
        }
    }

    /**
     * Select all rows from the LEASE table for a given object_key.
     *
     * @param  object_key String
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static Vector selectObjectLease(String object_key,
                                           Connection connection) throws
            java.sql.SQLException {
        log.info(
                "LeaseTable.selectServiceLease => a record selection requested.");
        Vector results = new Vector();
        PreparedStatement statement = null;
        ResultSet resultSet = null;

        try {
            // create a statement to query with
            statement = connection.prepareStatement(selectObjectLeaseSQL);
            statement.setString(1, object_key);

            log.info("select from LEASE table:\n\n\t" + selectObjectLeaseSQL +
                     "\n\t OBJECT_KEY=" + object_key.toString() + "\n");

            // execute the statement
            resultSet = statement.executeQuery();
            Lease lease = null;
            while (resultSet.next()) {
                String isinfinite = resultSet.getString("ISINFINITE");

                String sd = null;
                java.sql.Date s_d = resultSet.getDate("LEASESTARTFROMDATE");
                if (s_d != null) {
                    java.util.Date startdate = new java.util.Date(s_d.getTime());
                    Timestamp startDate = new Timestamp(startdate.getTime());
                    sd = startDate.toString();
                }
                String ed = null;
                java.sql.Date e_d = resultSet.getDate("LEASEEXPIRATIONDATE");
                if (e_d != null) {
                    java.util.Date enddate = new java.util.Date(e_d.getTime());
                    Timestamp stopDate = new Timestamp(enddate.getTime());
                    ed = stopDate.toString();
                }
                //String timeOut = (new Long(stopDate.getTime() - startDate.getTime())).toString();

                lease = new Lease();
                //lease.setServiceKey(serviceKey);
                lease.setLeaseStartDate(sd);
                lease.setLeaseExpirationDate(ed);
                //lease.setLeaseTimeOut(timeOut);
                lease.setIsInfinite(isinfinite);
                results.add(lease);
            }
            return results;
        } finally {
            try {
                resultSet.close();
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /**
     * Delete multiple rows from the LEASE table that are assigned to the
     * uddiKey specified.
     *
     * @param  object_key String
     * @param  connection JDBC connection
     * @throws java.sql.SQLException
     */
    public static void deleteObjectLease(String object_key,
                                         Connection connection) throws
            java.sql.SQLException {
        log.info(
                "LeaseTable.deleteServiceLease => a record deletion requested.");
        PreparedStatement statement = null;

        try {
            // prepare the delete
            statement = connection.prepareStatement(deleteObjectLeaseSQL);
            statement.setString(1, object_key.toString());

            log.info("delete from LEASE table:\n\n\t" + deleteObjectLeaseSQL +
                     "\n\t OBJECT_KEY=" + object_key.toString() + "\n");

            // execute
            statement.executeUpdate();
        } finally {
            try {
                statement.close();
            } catch (Exception e) { /* ignored */}
        }
    }

    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/

    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection("wscontext");
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection connection) throws Exception {
        Transaction txn = new Transaction();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        if (connection != null) {
            try {
                String authorizedUserID = "juddi";

                // begin a new transaction
                txn.begin(connection);

                String service_key = "key03";

//                java.util.Date currentTime = new java.util.Date();
//                String startDateString = org.apache.juddi.util.ISO_8601_UTC.format(currentTime, 2);
//                String endDateString = org.apache.juddi.util.ISO_8601_UTC.format(currentTime, 2);
//                String endDateString = (new Long((currentTime.getTime() + 10000))).toString();

                Lease lease = new Lease();
                //lease.setLeaseStartDate(startDateString);
                //lease.setLeaseExpirationDate(endDateString);
                lease.setIsInfinite("false");
                lease.setLeaseTimeOut("90000000000");
                lease.setObjectKey(service_key);

                LeaseTable.insertObjectLease(lease, connection);
                // re-select the Collection of AddressLine objects
                Vector results = LeaseTable.selectObjectLease(service_key,
                        connection);
                //Iterator iterator = results.iterator();

                for (Iterator iter = results.iterator(); iter.hasNext(); ) {
                    Lease item = (Lease) iter.next();
                    log.info("lease start date = " + item.getLeaseStartDate());
                    log.info("lease expire date = " +
                             item.getLeaseExpirationDate());
                    //log.info("service key = " + item.getServiceKey());
                }

                // delete the Collection of AddressLine objects
                //LeaseTable.delete(service_key,connection);

                // commit the transaction
                txn.commit();
            } catch (Exception ex) {
                try {
                    txn.rollback();
                } catch (java.sql.SQLException sqlex) {
                    sqlex.printStackTrace();
                }
                throw ex;
            }
        }
    }

}
