/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datastore.jdbc;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Vector;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UnknownUserException;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.Transaction;

import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.util.jdbc.ConnectionManager;
import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.jdbc.FindSessionByNameQuery;
import cgl.fthpis.datastore.jdbc.SessionEntityTable;
import cgl.fthpis.datastore.jdbc.SessionDescTable;
import cgl.fthpis.datastore.jdbc.SessionNameTable;
import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datatype.response.PublisherInfo;
import cgl.fthpis.datatype.service.BusinessService;
import cgl.fthpis.datatype.service.BusinessServices;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.SessionKey;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.response.AttributeInfo;
import cgl.fthpis.datatype.attribute.AttributeAssertion;
import cgl.fthpis.datatype.AutharizationToken;
import cgl.fthpis.datatype.attribute.AttributeAccessInfos;
import cgl.fthpis.datatype.attribute.AttributeAccessInfo;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class WSContextJDBCDataStore implements FTHPISDataStore {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(WSContextJDBCDataStore.class);

    // private db connection associated with this datastore
    private Connection connection = null;

    // private transaxtion object
    private Transaction transaction = null;

    /**
     * Create a new JDBCDataStore and aquire a JDBC
     * connection from the connection pool.
     */
    public WSContextJDBCDataStore() {
        try {
            this.connection = ConnectionManager.aquireConnection("wscontext");
        } catch (SQLException sqlex) {
            log.error("Exception occured while attempting to " +
                      "aquire a JDBC connection: " + sqlex.getMessage());
            log.info("Exception occured while attempting to " +
                     "aquire a JDBC connection: " + sqlex.getMessage());
        }
    }

    /**
     * Release all JDBC connections used by this
     * JDBCDataStore back into the connection pool.
     */
    public void release() {
        try {
            if (connection != null) {
                this.connection.close();
                this.connection = null;
            }
        } catch (SQLException sqlex) {
            log.error("Exception occured while attempting to " +
                      "close a JDBC connection: " + sqlex.getMessage());
        }
    }

    /**
     *
     */
    public Connection getConnection() {
        return this.connection;
    }

    /**
     * begin a new transaction
     */
    public void beginTrans() throws org.apache.juddi.error.RegistryException {
        try {
            this.transaction = new Transaction();
            this.transaction.begin(connection);
        } catch (SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     * commit on all connections.
     */
    public void commit() throws org.apache.juddi.error.RegistryException {
        try {
            this.transaction.commit();
        } catch (SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     * rollback on all connections.
     */
    public void rollback() throws org.apache.juddi.error.RegistryException {
        try {
            this.transaction.rollback();
        } catch (SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    ////////////////////////////////////////////////////////////
    /**
     *
     */
    public void savePublisher(Publisher publisher) throws org.apache.juddi.
            error.RegistryException {
        try {
            log.info(
                    "WSContextJDBCDataSTORE => savePublisher I => Publisher = " +
                    publisher);
            if ((publisher != null) && (connection != null)) {
                log.info(
                        "WSContextJDBCDataSTORE => savePublisher II => Publisher = " +
                        publisher);
                // create the Publisher account
                PublisherTable.insert(publisher, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void deletePublisher(String publisherID) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((publisherID != null) && (connection != null)) {
                // delete the Publisher account
                PublisherTable.delete(publisherID, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public PublisherInfo fetchPublisherInfo(String publisherID) throws org.
            apache.juddi.error.RegistryException {
        PublisherInfo info = null;

        if ((publisherID != null) && (connection != null)) {
            try {
                Publisher publisher = PublisherTable.select(publisherID,
                        connection);
                info = new PublisherInfo();
                info.setPublisherID(publisherID);
                info.setNameValue(publisher.getName());
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }

        return info;
    }

    /**
     *
     */
    public Vector findPublisher(String name, FindQualifiers findQualifiers) throws
            org.apache.juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            // always perform this query - even when not searching by Name!!!
            keyVector = FindPublisherByNameQuery.select(name, keyVector,
                    findQualifiers, connection);
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }

    /**
     * verify that the individual or system identified by
     * the 'publisherID' is a valid jUDDI user (aka publisher).
     *
     * @param publisherID
     * @return publisher
     * @throws UDDIException
     */
    public Publisher getPublisher(String publisherID) throws RegistryException {
        // a publisherID must be specified.
        if (publisherID == null) {
            return null;
        }

        Publisher publisher = null;

        try {
            log.info("cgl.fthpis.datastore.jdbc.JDBCDataStore => connection = " +
                     connection);
            publisher = PublisherTable.select(publisherID, connection);
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return publisher;
    }

    ////////////////////////////////////////////////////////////
    /**
     * Will return true if the publisherID parameter specified has been tagged
     * with registry administrator priviledges otherwise returns false. An
     * Unknown user exception is thrown if the publisherID specified is null,
     * blank or does not exist (can't be found in the PUBLISHER table).
     *
     * @param publisherID
     * @return Returns true if publisherID parameter specified has jUDDI
     *   administrator priv's otherwise returns false.
     * @throws RegistryException
     */
    public boolean isAdministrator(String publisherID) throws org.apache.juddi.
            error.RegistryException {
        if ((publisherID == null) || (publisherID.length() == 0)) {
            throw new UnknownUserException("publisherID = " + publisherID);
        }

        try {
            Publisher publisher = PublisherTable.select(publisherID, connection);
            if (publisher == null) {
                throw new UnknownUserException("publisherID = " + publisherID);
            } else {
                return publisher.isAdmin();
            }
        } catch (java.sql.SQLException sqlex) {
            log.error(sqlex.getMessage());
            throw new RegistryException(sqlex);
        }
    }

    /**
     * Will return true if the publisherID parameter specified is
     * currently enabled otherwise returns false. An UnknownUserException
     * is thrown if the publisherID specified is null, blank or does not
     * exist (can't be found in the PUBLISHER table).
     *
     * @param publisherID
     * @return Returns true if publisherID parameter specified has jUDDI
     *   administrator priv's otherwise returns false.
     * @throws RegistryException
     */
    public boolean isEnabled(String publisherID) throws org.apache.juddi.error.
            RegistryException {
        if ((publisherID == null) || (publisherID.length() == 0)) {
            throw new UnknownUserException("publisherID = " + publisherID);
        }

        try {
            Publisher publisher = PublisherTable.select(publisherID, connection);
            if (publisher == null) {
                throw new UnknownUserException("publisherID = " + publisherID);
            } else {
                return publisher.isEnabled();
            }
        } catch (java.sql.SQLException sqlex) {
            log.error(sqlex.getMessage());
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     * @param publisher
     * @return String
     * @throws UDDIException
     */
    public String generateToken(Publisher publisher) throws RegistryException {
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        String token = "authToken:" + uuidgen.uuidgen_with_timestamp();

        log.info("Generated token '" + token + "' for user: '" +
                 publisher.getPublisherID() + "/" + publisher.getName() + "'");

        return token;
    }

    /**
     *
     */
    public void storeAuthToken(String token, Publisher publisher) throws org.
            apache.juddi.error.RegistryException {
        if ((token != null) && (publisher != null)) {
            try {
                AuthTokenTable.insert(token, publisher, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
    }

    /**
     *
     */
    public void retireAuthToken(String token) throws org.apache.juddi.error.
            RegistryException {
        if (token != null) {
            try {
                AuthTokenTable.invalidate(token, connection);
                AuthTokenTable.touch(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
    }

    /**
     *
     */
    public Publisher getAuthTokenPublisher(String token) throws org.apache.
            juddi.error.RegistryException {
        Publisher publisher = null;

        if (token != null) {
            try {
                publisher = AuthTokenTable.selectPublisher(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }

        return publisher;
    }

    /**
     *
     */
    public AutharizationToken getAuthToken(String token) throws org.apache.
            juddi.error.RegistryException {
        AutharizationToken authtoken = null;

        if (token != null) {
            try {
                authtoken = AuthTokenTable.selectAuthToken(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
        return authtoken;
    }

    /**
     *
     */
    public boolean isAuthTokenExpired(String token) throws org.apache.juddi.
            error.RegistryException {
        boolean expired = false;

        if (token != null) {
            try {
                long tokenState = AuthTokenTable.selectTokenState(token,
                        connection);
                if (tokenState <= 0) {
                    return expired = true;
                }

                long lastUsed = AuthTokenTable.selectLastUsed(token, connection);
                if (lastUsed > 0) { // -1 is returned when token isn't found
                    long timeOut = Config.getLongProperty(
                            "juddi.authTokenTimeout", 3600) * 1000L; // convert from seconds to milliseconds
                    long currTime = System.currentTimeMillis();
                    if ((currTime - lastUsed) >= timeOut) {
                        expired = true;
                    }
                }
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }

        return expired;
    }

    /**
     *
     */
    public void touchAuthToken(String token) throws org.apache.juddi.error.
            RegistryException {
        if (token != null) {
            try {
                AuthTokenTable.touch(token, connection);
            } catch (java.sql.SQLException sqlex) {
                throw new RegistryException(sqlex);
            }
        }
    }

    ////////////////////DYNAMIC CONTEXT FUNCTIONS/////////////////////////////////

    /**
     *
     */
    public void saveSession(SessionEntity session, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        try {
            if ((session != null) && (connection != null)) {
                String sessionKey = session.getSessionKey();

//                log.info("sessionKey = " + sessionKey + " publisherID = " + publisherID);

                // insert the SessionEntity object
                SessionEntityTable.insert(session, publisherID, connection);

//                log.info("saveSession I ");

                // insert all of the SessionEntity Name objects
                if (session.getNameVector() != null &&
                    session.getNameVector().size() != 0) {
                    SessionNameTable.insert(sessionKey, session.getNameVector(),
                                            connection);
                }

//                log.info("saveSession II ");

                // insert all of the SessionEntity Description objects
                if (session.getDescriptionVector() != null) {
                    SessionDescTable.insert(sessionKey,
                                            session.getDescriptionVector(),
                                            connection);
                }

//                log.info("saveSession III ");

                // extract the contextbag object
                if (session.getContextBag() != null) {
                    AttributeBag contextBag = session.getContextBag();

                    java.util.Vector contextVector = contextBag.
                            getAttributeVector();
                    if (contextBag == null || contextVector == null ||
                        contextVector.size() == 0) {
                        return;
                    }

                    // aquire a reference to the shared UUIDGen instance
                    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

                    // save all of the contexts that are found
                    int listSize = contextVector.size();
                    for (int i = 0; i < listSize; i++) {
                        Attribute context = (Attribute) contextVector.
                                            elementAt(i);
                        context.setSessionKey(sessionKey);
                        context.setAttributeKey("uuid:" + uuidgen.uuidgen_with_timestamp());
                        saveContext(context);
                    }
                }

//                log.info("saveSession IV ");

                if (session.getLease() != null) {
                    Lease lease = session.getLease();
                    log.info(
                            "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveSession() lease = " +
                            lease);

                    if (lease == null) {
                        return; //no lease exists
                    }

                    lease.setObjectKey(sessionKey);
                    //log.info("org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveService() lease start date = " +
                    //         lease.getLeaseStartDate());
                    //log.info("org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveService() lease expiration date = " +
                    //         lease.getLeaseExpirationDate());
                    saveSessionLease(lease);
                }

//                log.info("saveSession V ");

                // 'save' the SessionEntity's BusinessService objects
                BusinessServices services = session.getSessionServices();
                if ((services != null) && (services.getBusinessServiceVector() != null)) {
                    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();
                    Vector serviceVector = services.getBusinessServiceVector();
                    int serviceListSize = serviceVector.size();
                    for (int j = 0; j < serviceListSize; j++) {
                        BusinessService service = (BusinessService)
                                                  serviceVector.elementAt(j);
                        service.addSessionKey(new SessionKey(sessionKey));
                        service.setServiceKey(uuidgen.uuidgen());
                        saveSessionService(service);
                    }
                }
//               log.info("saveSession VI ");

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public SessionEntity fetchSession(String sessionKey) throws org.apache.
            juddi.error.RegistryException {
        SessionEntity session = null;

        try {
            if ((sessionKey != null) && (connection != null)) {
                session = SessionEntityTable.select(sessionKey, connection);

                //fetch session name vector
                Vector sessionnamevector = SessionNameTable.select(sessionKey,
                        connection);
                log.info("fetchSession => sessionnamevector = " +
                         sessionnamevector);
                session.setNameVector(sessionnamevector);

                //fetch session description vector
                Vector sessiondescriptionvector = SessionDescTable.select(
                        sessionKey, connection);
                log.info("fetchSession =>  sessiondescriptionvector = " +
                         sessiondescriptionvector);
                session.setDescriptionVector(sessiondescriptionvector);

                //fetch session context vector
                Vector temp = fetchContextBySessionKey(sessionKey);
                Vector contextVector = null;
                if (temp != null) {
                    contextVector = new Vector();
                    for (Iterator iter = temp.iterator(); iter.hasNext(); ) {
                        Attribute context = (Attribute) iter.next();
                        String contextKey = context.getAttributeKey();
                        log.info("fetchSession =>  contextKey = " + contextKey);
                        contextVector.add(context);
                    }
                }
                AttributeBag contextBag = new AttributeBag();
                contextBag.setAttributeVector(contextVector);
                session.setContextBag(contextBag);

                // 'fetch' the SessionEntity's BusinessService objects
                Vector serviceVector = fetchServiceBySessionKey(sessionKey);
                BusinessServices services = new BusinessServices();
                services.setBusinessServiceVector(serviceVector);

                session.setSessionServices(services);

                Lease lease = new Lease();
                Vector v = LeaseTable.selectObjectLease(sessionKey, connection);
                if (v != null) {
                    if (v.size() != 0) {
                        lease = (Lease) v.elementAt(0);
                    }
                }
                session.setLease(lease);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return session;
    }

    /**
     *
     */
    public void deleteSession(String sessionKey) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((sessionKey != null) && (connection != null)) {

                SessionServiceTable.deleteBySessionKey(sessionKey, connection);

                SessionNameTable.delete(sessionKey, connection);

                SessionDescTable.delete(sessionKey, connection);

                LeaseTable.deleteObjectLease(sessionKey, connection);

                deleteContextBySessionKey(sessionKey);

                // finally, delete the BusinessEntity itself.
                SessionEntityTable.delete(sessionKey, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            log.error(sqlex.getMessage(), sqlex);
            throw new RegistryException(sqlex);
        }
    }


    /**
     *
     */
    public boolean isSessionAdmin(String sessionKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        try {
            if ((publisherID != null) && (sessionKey != null) && (connection != null)) {
                return SessionEntityTable.verifyOwnership(sessionKey,
                        publisherID, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }


    /**
     *
     */
    public boolean isSessionPublisher(String sessionKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        try {
            if ((publisherID != null) && (sessionKey != null) && (connection != null)) {
                return SessionEntityTable.verifyOwnership(sessionKey,
                        publisherID, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }

    /**
     *
     */
    public boolean isContextPublisher(String contextKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        try {
            if ((publisherID != null) && (contextKey != null) && (connection != null)) {
                return ContextPublisherTable.verifyOwnership(contextKey,
                        publisherID, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }

    /**
     *
     */
    public boolean isValidSessionKey(String sessionKey) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((sessionKey != null) && (connection != null) &&
                (SessionEntityTable.select(sessionKey, connection) != null)) {
                return true;
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }

    /**
     *
     */
    public boolean isValidContextKey(String contextKey) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((contextKey != null) && (connection != null) &&
                (ContextTable.select(contextKey, connection) != null)) {
                return true;
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        // default to false
        return false;
    }


    /**
     *
     */
    public void saveSessionService(BusinessService service) throws org.apache.
            juddi.error.RegistryException {
        try {
            if ((service != null) && (connection != null)) {
                String serviceKey = service.getServiceKey();

                // insert the BusinessService object
                SessionServiceTable.insert(service, connection);

                // extract the  objects
                AttributeBag dynamicContextBag = service.getDynamicContextBag();

                if (dynamicContextBag != null) {
                    java.util.Vector dynamicContextVector = dynamicContextBag.
                            getAttributeVector();
                    if (dynamicContextBag == null || dynamicContextVector == null ||
                        dynamicContextVector.size() == 0) {
                        return; // no objects were present
                    }

                    // aquire a reference to the shared UUIDGen instance
                    UUIDGen prop_uuidgen = UUIDGenFactory.getUUIDGen();

                    // save all of the objects that were found
                    int listSize = dynamicContextVector.size();
                    for (int i = 0; i < listSize; i++) {
                        Attribute context = (Attribute) dynamicContextVector.
                                            elementAt(i);
                        context.setServiceKey(serviceKey);
                        //associate this with default binding
                        context.setAttributeKey(prop_uuidgen.uuidgen());
                        saveContext(context);
                    }
                }

//                Lease lease = service.getLease();
//                log.info(
//                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveService() lease = " +
//                        lease);
//
//                if (lease == null) {
//                    return; //no lease exists
//                }
//
//                lease.setObjectKey(serviceKey);
//                saveServiceLease(lease);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public Attribute fetchContext(String contextKey) throws org.apache.juddi.
            error.RegistryException {
        Attribute context = null;

        log.info(
                "cgl.fthpis.datastore.jdbc.JDBCDataStore.java => fetchContext => contextKey = " +
                contextKey);
        try {
            if ((contextKey != null) && (connection != null)) {
                // fetch the BindingTempate and it's Description Vector
                context = ContextTable.select(contextKey, connection);

                //fetch context name vector
                Vector contextnamevector = ContextNameTable.select(contextKey,
                        connection);
                log.info("fetchContext => contextnamevector = " +
                         contextnamevector);
                context.setNameVector(contextnamevector);

                Attribute c1 = SessionContextTable.select(contextKey,
                        connection);
                if (c1 != null) {
                    context.setSessionKey(c1.getSessionKey());
                }

                Attribute c2 = ServiceContextTable.select(contextKey,
                        connection);
                if (c2 != null) {
                    context.setServiceKey(c2.getServiceKey());
                }

                Lease lease = new Lease();
                Vector v = LeaseTable.selectObjectLease(contextKey, connection);
                if (v != null) {
                    if (v.size() != 0) {
                        lease = (Lease) v.elementAt(0);
                        context.setLease(lease);
                    }
                }

                Vector vec = AttributeAssertionTable.select(contextKey,
                        connection);
                if (vec != null) {
                    if (vec.size() != 0) {
                        context.setAttributeAssertionVector(vec);
                    }
                }

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return context;
    }

    /**
     *
     */
    private Vector fetchContextByServiceKey(String serviceKey) throws org.
            apache.juddi.error.RegistryException {
        Vector contextVector = new Vector();

        try {
            if ((serviceKey != null) && (connection != null)) {
                Vector tempList = ServiceContextTable.selectByServiceKey(
                        serviceKey, connection);
                for (int i = 0; i < tempList.size(); i++) {
                    Attribute context = (Attribute) tempList.elementAt(i);
                    contextVector.add(fetchContext(context.getAttributeKey()));
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return contextVector;
    }

    /**
     *
     */
    private Vector fetchContextBySessionKey(String sessionKey) throws org.
            apache.juddi.error.RegistryException {
        Vector contextVector = new Vector();

        try {
            if ((sessionKey != null) && (connection != null)) {
                Vector tempList = SessionContextTable.selectBySessionKey(
                        sessionKey, connection);
                for (int i = 0; i < tempList.size(); i++) {
                    Attribute context = (Attribute) tempList.elementAt(i);
                    contextVector.add(fetchContext(context.getAttributeKey()));
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return contextVector;
    }

    /**
     *
     */
    public void deleteContext(String contextKey) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((contextKey != null) && (connection != null)) {
                //delete the dependents of SessionEntity
                ContextTable.deleteByContextKey(contextKey, connection);

                //delete the publishers from this context
                ContextPublisherTable.deleteByContextKey(contextKey, connection);

                //delete the related entry from Service_Context
                ServiceContextTable.delete(contextKey, connection);

                //delete the related entry from Session_Context
                SessionContextTable.delete(contextKey, connection);

                //delete the related entry from Lease table
                LeaseTable.deleteObjectLease(contextKey, connection);

                //delete context assertions
                AttributeAssertionTable.deleteByAttributeKey(contextKey,
                        connection);

                //delete context name
                ContextNameTable.delete(contextKey, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            log.error(sqlex.getMessage(), sqlex);
            throw new RegistryException(sqlex);
        }
    }


    /**
     *
     */
    public BusinessService fetchSessionService(String serviceKey) throws org.
            apache.juddi.error.RegistryException {
        BusinessService service = null;

        try {
            if ((serviceKey != null) && (connection != null)) {
                service = SessionServiceTable.select(serviceKey, connection);

                Vector temp = fetchContextByServiceKey(serviceKey);
                Vector contextVector = null;
                if (temp != null) {
                    contextVector = new Vector();
                    for (Iterator iter = temp.iterator(); iter.hasNext(); ) {
                        Attribute context = (Attribute) iter.next();
                        String contextKey = context.getAttributeKey();
                        log.info("contextKey = " + contextKey);
                        contextVector.add(context);
                    }
                }
                AttributeBag contextBag = new AttributeBag();
                contextBag.setAttributeVector(contextVector);
                service.setDynamicContextBag(contextBag);

            }

            //Burda dynamic context ve lease bilgilerini database den cekmek gerekli
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return service;
    }


    /**
     *
     */
    public void deleteSessionService(String serviceKey) throws org.apache.juddi.
            error.RegistryException {
        try {
            if ((serviceKey != null) && (connection != null)) {
                // delete the immediate dependents of BusinessService
                deleteContextByServiceKey(serviceKey);

                LeaseTable.deleteObjectLease(serviceKey, connection);

                // finally, delete the BusinessService itself.
                SessionServiceTable.delete(serviceKey, connection);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    private Vector fetchServiceBySessionKey(String sessionKey) throws org.
            apache.juddi.error.RegistryException {
        Vector serviceList = new Vector();

        try {
            if ((sessionKey != null) && (connection != null)) {
                Vector tempList = SessionServiceTable.selectBySessionKey(
                        sessionKey, connection);
                for (int i = 0; i < tempList.size(); i++) {
                    BusinessService service = (BusinessService) tempList.
                                              elementAt(i);
                    log.info("service = " + service);
                    serviceList.add(fetchSessionService(service.getServiceKey()));
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return serviceList;
    }

    /**
     *
     */
    private void deleteContextByServiceKey(String serviceKey) throws org.apache.
            juddi.error.RegistryException {
        try {
            if ((serviceKey != null) && (connection != null)) {
                // obtain a vector of BusinessServices associated with the BusinessKey
                Vector contexts = ServiceContextTable.selectByServiceKey(
                        serviceKey, connection);

                // loop through the vector deleting each server in turn
                int listSize = contexts.size();
                for (int i = 0; i < listSize; i++) {
                    Attribute context = (Attribute) contexts.elementAt(i);
                    deleteContext(context.getAttributeKey());
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

    }

    /**
     *
     */
    private void deleteContextBySessionKey(String sessionKey) throws org.apache.
            juddi.error.RegistryException {
        try {
            if ((sessionKey != null) && (connection != null)) {
                // obtain a vector of BusinessServices associated with the BusinessKey
                Vector contexts = SessionContextTable.selectBySessionKey(
                        sessionKey, connection);

                // loop through the vector deleting each server in turn
                int listSize = contexts.size();
                for (int i = 0; i < listSize; i++) {
                    Attribute context = (Attribute) contexts.elementAt(i);
                    deleteContext(context.getAttributeKey());
                }
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

    }

    /**
     *
     */
    public void saveContext(Attribute context) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((context != null) && (connection != null)) {
                String contextKey = context.getAttributeKey();
                String sessionKey = context.getSessionKey();
                String serviceKey = context.getServiceKey();

                Vector attributeAssertionVector = context.
                                                  getAttributeAssertionVector();

                log.info("WSContextJDBCDataStore.saveContext I ");
                // insert the attribute object
                ContextTable.insert(context, connection);
                log.info("WSContextJDBCDataStore.saveContext II");

                // insert all of the SessionEntity Name objects
                if (context.getNameVector() != null &&
                    context.getNameVector().size() != 0) {
                    ContextNameTable.insert(contextKey, context.getNameVector(),
                                            connection);
                }
                log.info("WSContextJDBCDataStore.saveContext III");

                if ((serviceKey != null) && (serviceKey.length() != 0)) {
                    //insert service context
                    ServiceContextTable.insert(context, connection);
                }

                if ((sessionKey != null) && (sessionKey.length() != 0)) {
                    //insert service context
                    SessionContextTable.insert(context, connection);
                }

                //insert publishers for this context
                if ((contextKey != null) && (contextKey.length() != 0)) {
                    AttributeAccessInfos access_infos = context.
                            getAttributeAccessRightInfos();
                    if (access_infos != null) {
                        AttributeAccessInfo access_info = access_infos.
                                getGroupAttributeAccessInfo();
                        Vector publisherIDVector = access_info.
                                getPublisherIDVector();
                        if (publisherIDVector != null &&
                            publisherIDVector.size() != 0) {
                            ContextPublisherTable.insert(contextKey,
                                    publisherIDVector, connection);
                        }
                    }
                }

                // insert assertions
                if ((attributeAssertionVector != null) &&
                    (attributeAssertionVector.size() != 0)) {
                    for (Iterator iter = attributeAssertionVector.iterator();
                                         iter.hasNext(); ) {
                        AttributeAssertion context_assertion = (
                                AttributeAssertion) iter.next();
                        context_assertion.setAttributeKey(contextKey);
                        context_assertion.setAssertionSubjectKey(contextKey);
                        AttributeAssertionTable.insert(context_assertion,
                                connection);
                    }
                }

                // insert lease
                log.info(
                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveAttribute() before lease ");
                Lease lease = context.getLease();
                log.info(
                        "org.apachche.juddi.datastore.jdbc.JDBCDataStore.saveAttribute() after lease = " +
                        lease);

                if (lease == null) {
                    return; //no lease exists
                }

                lease.setObjectKey(contextKey);
                saveContextLease(lease);
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void saveContextLease(Lease lease) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((lease != null) && (connection != null)) {
                // insert the BindingTemplate object
                LeaseTable.insertObjectLease(lease, connection);

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }


    /**
     *
     */
    public void saveServiceLease(Lease lease) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((lease != null) && (connection != null)) {
                // insert the BindingTemplate object
                LeaseTable.insertObjectLease(lease, connection);

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public void saveSessionLease(Lease lease) throws org.apache.juddi.error.
            RegistryException {
        try {
            if ((lease != null) && (connection != null)) {
                // insert the BindingTemplate object
                LeaseTable.insertObjectLease(lease, connection);

            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }
    }

    /**
     *
     */
    public Vector findSession(Vector serviceKeyVector,
                              Vector nameVector,
                              AttributeBag contextBag,
                              Lease lease,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            if ((serviceKeyVector != null) && (serviceKeyVector.size() > 0)) {
                keyVector = FindSessionByServiceKeyBagQuery.select(
                        serviceKeyVector, keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SESSION_KEY VECTOR size after FindSessionByServiceKeyQuery  = " +
                        keyVector.size());
            }

            if ((contextBag != null) && (contextBag.size() > 0) &&
                contextBag.getAttributeVector() != null) {
                keyVector = FindSessionByContextBagQuery.select(contextBag,
                        keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SESSION_KEY VECTOR size after FindSessionByContextBagQuery  = " +
                        keyVector.size());
            }

            if ((lease != null)) {
                keyVector = FindSessionByLeaseQuery.select(lease, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SESSION_KEY VECTOR size after FindSessionByLeaseQuery  = " +
                        keyVector.size());
            }

            // always perform this query - even when not searching by Name!!!
            if (nameVector != null) {
                keyVector = FindSessionByNameQuery.select(nameVector, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SESSION_KEY VECTOR size after FindSessionByNameQuery  = " +
                        keyVector.size());
            }
        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }

    /**
     *
     */
    public Vector findSessionService(Vector sessionKeyVector,
                                     Vector keyVector,
                                     AttributeBag contextBag,
                                     FindQualifiers findQualifiers) throws org.
            apache.juddi.error.RegistryException {
//    Vector keyVector = null;

        try {
            if ((sessionKeyVector != null) && (sessionKeyVector.size() > 0)) {
                keyVector = FindServiceBySessionKeyBagQuery.select(
                        sessionKeyVector, keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "SERVICE_KEY VECTOR size after FindServiceBySessionKeyBagQuery  = " +
                        keyVector.size());
            }

            if ((contextBag != null) && (contextBag.size() > 0) &&
                contextBag.getAttributeVector() != null) {

                keyVector = FindServiceByContextBagQuery.select(contextBag,
                        keyVector,
                        findQualifiers,
                        connection);
            }

            if (keyVector != null) {
                log.info(
                        "SERVICE_KEY VECTOR size after FindServiceByContextBagQuery  = " +
                        keyVector.size());
            }

        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }


    /**
     *
     */
    public Vector findContext(String sessionKey,
                              String serviceKey,
                              Vector nameVector,
                              Lease lease,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        Vector keyVector = null;

        try {

            if (serviceKey != null && sessionKey != null) {
                keyVector = FindContextBySessionAndServiceKeyQuery.select(
                        sessionKey, serviceKey, keyVector, findQualifiers,
                        connection);
            } else if (serviceKey != null) {
                keyVector = FindContextByServiceKeyQuery.select(serviceKey,
                        keyVector, findQualifiers, connection);
            } else if (sessionKey != null) {
                keyVector = FindContextBySessionKeyQuery.select(sessionKey,
                        keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 1 = " + keyVector.size());
            }

            if (lease != null) {
                keyVector = FindContextByLeaseQuery.select(lease, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 2 = " + keyVector.size());
            }

            // always perform this query - even when not searching by Name!!!
            if (nameVector != null) {
                log.info("nameVector = " + nameVector.toString());
                keyVector = FindContextByNameQuery.select(nameVector, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info(
                        "CONTEXT_KEY VECTOR size after FindContextByNameQuery  = " +
                        keyVector.size());
            }

        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }

    /**
     *
     */
    public Vector findServiceContext(String serviceKey,
                                     Lease lease,
                                     FindQualifiers findQualifiers) throws org.
            apache.juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            if (serviceKey != null) {
                keyVector = FindContextByServiceKeyQuery.select(serviceKey,
                        keyVector, findQualifiers, connection);
            }

            log.info("DEBUG POINT...");
            if (keyVector != null) {
                log.info("keyVector size 1 = " + keyVector.size());
            }

            if (lease != null) {
                keyVector = FindContextByLeaseQuery.select(lease, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 3 = " + keyVector.size());
            }

        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }


    /**
     *
     */
    public Vector findSessionContext(String sessionKey,
                                     Lease lease,
                                     FindQualifiers findQualifiers) throws org.
            apache.juddi.error.RegistryException {
        Vector keyVector = null;

        try {
            if (sessionKey != null) {
                keyVector = FindContextBySessionKeyQuery.select(sessionKey,
                        keyVector, findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 1 = " + keyVector.size());
            }

            if (lease != null) {
                keyVector = FindContextByLeaseQuery.select(lease, keyVector,
                        findQualifiers, connection);
            }

            if (keyVector != null) {
                log.info("keyVector size 3 = " + keyVector.size());
            }

        } catch (java.sql.SQLException sqlex) {
            throw new RegistryException(sqlex);
        }

        return keyVector;
    }

    ////////////////////STATIC CONTEXT FUNCTIONS/////////////////////////////////
    /**
     *
     */
    public void saveService(cgl.fthpis.datatype.service.BusinessService service) throws
            org.apache.juddi.error.RegistryException {
    }

    /**
     *
     */
    public Attribute fetchAttribute(String attributeKey) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public AttributeInfo fetchAttributeInfo(String attributeKey) throws org.
            apache.juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public void deleteAttribute(String attributeKey) throws org.apache.juddi.
            error.RegistryException {
    }

    /**
     *
     */
    public BusinessService fetchService(String serviceKey) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public void deleteService(String serviceKey) throws org.apache.juddi.error.
            RegistryException {
    }

    /**
     *
     */
    public boolean isServicePublisher(String serviceKey, String publisherID) throws
            org.apache.juddi.error.RegistryException {
        return false;
    }

    /**
     *
     */
    public void saveAttribute(Attribute attribute) throws org.apache.juddi.
            error.RegistryException {
    }


    /**
     *
     */
    public boolean isValidAttributeKey(String attributeKey) throws org.apache.
            juddi.error.RegistryException {
        return false;
    }


    /**
     *
     */
    public boolean isValidServiceKey(String serviceKey) throws org.apache.juddi.
            error.RegistryException {
        return false;
    }

    /**
     *
     */
    public Vector findService(String businessKey,
                              Vector nameVector,
                              CategoryBag categoryBag,
                              TModelBag tModelBag,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public Vector findService(String businessKey,
                              Vector nameVector,
                              CategoryBag categoryBag,
                              TModelBag tModelBag,
                              AttributeBag attributeBag,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public Vector findService(String businessKey,
                              Vector nameVector,
                              CategoryBag categoryBag,
                              TModelBag tModelBag,
                              AttributeBag attributeBag,
                              Lease lease,
                              FindQualifiers findQualifiers) throws org.apache.
            juddi.error.RegistryException {
        return null;
    }

    /**
     *
     */
    public Vector findServiceAttribute(String serviceKey,
                                       CategoryBag categoryBag,
                                       Lease lease,
                                       FindQualifiers findQualifiers) throws
            org.apache.juddi.error.RegistryException {
        return null;
    }

    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) throws Exception {
        // make sure we're using a DBCP DataSource and
        // not trying to use JNDI to aquire one.
        Config.setStringProperty("juddi.useConnectionPool", "true");

        Connection conn = null;
        try {
            conn = ConnectionManager.aquireConnection();
            test(conn);
        } finally {
            if (conn != null) {
                conn.close();
            }
        }
    }

    public static void test(Connection conn) throws Exception {
        log.info("Starting JDBCDataStore Test");

//    BusinessEntityTable.test(conn);
//    BusinessDescTable.test(conn);
//    BusinessCategoryTable.test(conn);
//    BusinessIdentifierTable.test(conn);
//    BusinessNameTable.test(conn);
//    DiscoveryURLTable.test(conn);
//    ContactTable.test(conn);
//    ContactDescTable.test(conn);
//    PhoneTable.test(conn);
//    EmailTable.test(conn);
//    AddressTable.test(conn);
//    AddressLineTable.test(conn);
        BusinessServiceTable.test(conn);
//    ServiceDescTable.test(conn);
//    ServiceCategoryTable.test(conn);
//    ServiceNameTable.test(conn);
//    BindingTemplateTable.test(conn);
        AttributeTable.test(conn);
//    BindingDescTable.test(conn);
//    TModelInstanceInfoTable.test(conn);
//    TModelInstanceInfoDescTable.test(conn);
//    InstanceDetailsDescTable.test(conn);
//    InstanceDetailsDocDescTable.test(conn);
//    TModelTable.test(conn);
//    TModelDescTable.test(conn);
//    TModelCategoryTable.test(conn);
//    TModelIdentifierTable.test(conn);
//    TModelDocDescTable.test(conn);
//    PublisherAssertionTable.test(conn);
//    AuthTokenTable.test(conn);
//    PublisherTable.test(conn);

        log.info("Concluding JDBCDataStore Test");
    }
}
