/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datatype.attribute;

import java.util.Vector;
import java.util.Hashtable;

import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Name;

import space.*;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.publisher.PublisherID;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class Attribute
    implements RegistryObject, Entry {

  private static Log log = LogFactory.getLog(Attribute.class);

  // the unique key for metadata
  public String attributeKey;

  //replication stuff
  int rcount = 0;
  int affinity = 1;
  Hashtable hostingServerTable;

  // the unique key for sessionKey
  // this is only used for dynamic metadata
  String sessionKey;

  // the unique key of the enclosing service
  String serviceKey;

  // the unique key for metadata defined by user
  // user-context-identifier
  Vector nameVector;

  // the source for the metadata. session or service
  // this is only used for dynamic metadata
  PublisherID attribute_owner_publisherID;

  //this is only used by static metadata world
  AttributeName attribute_name;

  //context - all context is in String format
  AttributeValue attribute_value;

  //context - value type
  AttributeValueType attribute_value_type;

  AttributeFindQualifier attribute_find_qualifier;

  Lease lease;

  //These stuff has only been used for UDDI-Extended
  CategoryBag categoryBag;

  //ASSERTIONs
  Vector attributeAssertionVector;

  //SECURITY POLICY
  AttributeAccessInfos attribute_access_infos;

  int property_version;

  /**
   * Constructs a new empty Attribute.
   */
  public Attribute() {
      log.info("cgl.fthpis.datatype.attribute.Attribute() || ...Context() constructor called.");
  }


  /**
   * Sets the sessionKey of this attribute.
   *
   * @param key The new sessionKey.
   */
  public void setSessionKey(String key) {
    this.sessionKey = key;
  }

  /**
   * Returns the sessionKey of this attribute.
   *
   * @return The sessionKey of this attribute.
   */
  public String getSessionKey() {
    return this.sessionKey;
  }

  /**
   * Sets the serviceKey of this attribute.
   *
   * @param key The new serviceKey key.
   */
  public void setServiceKey(String key) {
    this.serviceKey = key;
  }

  /**
   * Returns the serviceKey of this attribute.
   *
   * @return The serviceKey of this attribute.
   */
  public String getServiceKey() {
    return this.serviceKey;
  }


  /**
   * Sets the attributeKey of this attribute.
   *
   * @param key The new attributeKey.
   */
  public void setAttributeKey(String key) {
    this.attributeKey = key;
  }

  /**
   * Returns the attributeKey of this attribute.
   *
   * @return The attributeKey of this attribute.
   */
  public String getAttributeKey() {
    return this.attributeKey;
  }

  /**
   *
   */
  public void setAttributeOwnerPublisherID(PublisherID id) {
    this.attribute_owner_publisherID = id;
  }

  /**
   *
   */
  public PublisherID geAttributeOwnerPublisherID() {
    return this.attribute_owner_publisherID;
  }


  /**
   *
   */
  public void setAttributeName(AttributeName propertyname) {
    this.attribute_name = propertyname;
  }

  /**
   *
   */
  public AttributeName getAttributeName() {
    return this.attribute_name;
  }

  /**
   *
   */
  public void setAttributeAccessRightInfos(AttributeAccessInfos access_infos) {
    this.attribute_access_infos = access_infos;
  }

  /**
   *
   */
  public AttributeAccessInfos getAttributeAccessRightInfos() {
    return this.attribute_access_infos;
  }

  /**
   *
   */
  public void setAttributeValueType(AttributeValueType type) {
    this.attribute_value_type = type;
  }

  /**
   *
   */
  public AttributeValueType getAttributeValueType() {
    return this.attribute_value_type;
  }

  /**
   *
   */
  public AttributeValue getAttributeValue() {
    return this.attribute_value;
  }

  /**
   *
   */
  public void setAttributeValue(AttributeValue value) {
    this.attribute_value = value;
  }

  /**
   *
   */
  public AttributeFindQualifier getAttributeFindQualifier() {
    return this.attribute_find_qualifier;
  }

  /**
   *
   */
  public void setAttributeFindQualifier(AttributeFindQualifier value) {
    this.attribute_find_qualifier = value;
  }

  /**
   *
   */
  public void addCategory(KeyedReference ref) {
    // just return if the KeyedReference parameter is null (nothing to add)
    if (ref == null) {
      return;
    }

    // make sure the CategoryBag has been initialized
    if (this.categoryBag == null) {
      this.categoryBag = new CategoryBag();
    }

    this.categoryBag.addKeyedReference(ref);
  }

  /**
   *
   */
  public CategoryBag getCategoryBag() {
    return this.categoryBag;
  }

  /**
   *
   */
  public void setCategoryBag(CategoryBag bag) {
    this.categoryBag = bag;
  }

  /**
   *
   */
  public void addAttributeAssertion(AttributeAssertion ref) {
    // just return if the KeyedReference parameter is null (nothing to add)
    if (ref == null) {
      return;
    }

    // make sure the CategoryBag has been initialized
    if (this.attributeAssertionVector == null) {
      this.attributeAssertionVector = new Vector();
    }

    this.attributeAssertionVector.add(ref);
  }

  /**
   *
   */
  public Vector getAttributeAssertionVector() {
    return this.attributeAssertionVector;
  }

  /**
   *
   */
  public void setAttributeAssertionVector(Vector vec) {
    this.attributeAssertionVector = vec;
  }


  /**
   *
   */
  public Lease getLease() {
    return this.lease;
  }

  /**
   *
   */
  public void setLease(Lease lease) {
    this.lease = lease;
  }

  /**
   *
   */
  public void addName(Name name)
  {
    // just return if name is null (nothing to add)
    if (name == null)
      return;

    if (this.nameVector == null)
      this.nameVector = new Vector();
    this.nameVector.add(name);
  }

  /**
   *
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }

  /**
   *
   */
  public Vector getNameVector()
  {
    return this.nameVector;
  }

  /**
   *
   */
  public void setAttributeVersion(int value) {
    this.property_version = value;
  }

  /**
   *
   */
  public int getAttributeVersion() {
    return this.property_version;
  }

  /**
   *
   */
  public void setAffinityNumber(int num) {
    this.affinity = num;
  }

  /**
   *
   */
  public int getAffinityNumber() {
    return this.affinity;
  }

  /**
   *
   */
  public void setRequestCount(int value) {
    this.rcount = value;
  }

  /**
   *
   */
  public int getRequestCount() {
    return this.rcount;
  }

  /**
   *
   */
  public void addHostingServer(String host, Integer rcount)
  {
    if (host == null)
      return;

    if (this.hostingServerTable == null)
      this.hostingServerTable = new Hashtable();
    this.hostingServerTable.put(host, rcount);
  }

  /**
   *
   */
  public void setHostingServerTable(Hashtable table)
  {
    this.hostingServerTable = table;
  }

  /**
   *
   */
  public Hashtable getHostingServerTable()
  {
    return this.hostingServerTable;
  }

}
