/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datatype.lease;

import org.apache.juddi.datatype.RegistryObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class Lease  implements RegistryObject{

  private static Log log = LogFactory.getLog(Lease.class);
  String objectKey =  null;
  LeaseStartDate leaseStartDate = null;
  LeaseExpirationDate leaseExpirationDate = null;
  LeaseTimeOut leaseTimeOut = null;
  String isInfinite = null;


  /**
   * Default constructor.
   * Avoid using the default constructor for validation. It does not validate
   * required fields. Instead, use the required fields constructor to perform
   * validation.
   */
  public Lease() {
      log.info("cgl.fthpis.datatype.lease.Lease() constructor called.");
  }

  /**
   * Construct the object with required fields.
   *
   * @param leaseStartDate  String
   */
  public Lease(String start_date)
  {
        this.leaseStartDate = new LeaseStartDate(start_date);
  }

  /**
   *
   */
  public void setObjectKey(String object_key) {
    objectKey = object_key;
  }

  /**
   *
   */
  public String getObjectKey() {
    return this.objectKey;
  }

  /**
   *
   */
  public LeaseStartDate getLeaseStartDate()
  {
     return leaseStartDate;
  }

  /**
   *
   */
  public LeaseTimeOut getLeaseTimeOut()
  {
     return leaseTimeOut;
  }

  /**
   *
   */
  public LeaseExpirationDate getLeaseExpirationDate()
  {
     return leaseExpirationDate;
  }

  /**
   *
   */
  public void setLeaseStartDate(String start_date)
  {
     if (this.leaseStartDate == null) {
       this.leaseStartDate = new LeaseStartDate(start_date);
     } else{
       this.leaseStartDate.setLeaseStartDate(start_date);
     }

  }

  /**
   *
   */
  public void setLeaseExpirationDate(String ex_date)
  {
    if (this.leaseExpirationDate == null) {
      this.leaseExpirationDate = new LeaseExpirationDate(ex_date);
    } else{
      this.leaseExpirationDate.setLeaseExpirationDate(ex_date);
    }
  }

  /**
   *
   */
  public void setLeaseTimeOut(String ex_date)
  {
    if (this.leaseTimeOut == null) {
      this.leaseTimeOut = new LeaseTimeOut(ex_date);
    } else{
      this.leaseTimeOut.setLeaseTimeOut(ex_date);
    }
  }

  /**
   *
   */
  public void setIsInfinite(String value) {
    isInfinite = value;
  }

  /**
   *
   */
  public String getIsInfinte() {
    return this.isInfinite;
  }
}
