/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datatype.request;



import org.apache.juddi.datatype.RegistryObject;

import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.datatype.request.FindQualifier;

import cgl.fthpis.datatype.lease.Lease;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import java.util.Vector;
import org.apache.juddi.datatype.Name;
/**
 * "Used to locate specific attribute/context. Returns a contextDetail message."
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindContext implements RegistryObject,Inquiry
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(FindContext.class);
  AuthInfo authInfo;
  String contextKey;
  String serviceKey;
  String sessionKey;
  Vector nameVector;
  Lease lease;
  FindQualifiers findQualifiers;
  String generic;
  int maxRows;

  /**
   *
   */
  public FindContext()
  {
      log.info("cgl.fthpis.datatype.request.FindContext() constructor called.");
  }


  /**
   * Construct a new find_context request.
   *
   * @param serviceKey The key of the businessService to search in.
   * @param serviceKey The key of the session to search in.   *
   * @throws NullPointerException If the given key is null.
   */
  public FindContext(String serviceKey, String sessionKey)
  {
    log.info("org.apache.juddi.datatype.request.FindContext() constructor called.");
    setServiceKey(serviceKey);
    setSessionKey(sessionKey);
  }

  /**
   *
   * @param key A reference to the contextEntity.
   */
  public void setContextKey(String key)
  {
    contextKey = key;
  }

  /**
   *
   * @return The contextkey of the referenced Context.
   */
  public String getContextKey()
  {
    return contextKey;
  }

  /**
   *
   * @param key A reference to the businessService.
   */
  public void setServiceKey(String key)
  {
    serviceKey = key;
  }

  /**
   *
   * @return The servicekey of the referenced businessService.
   */
  public String getServiceKey()
  {
    return serviceKey;
  }

  /**
   *
   * @param key A reference to the session entity.
   */
  public void setSessionKey(String key)
  {
    sessionKey = key;
  }

  /**
   *
   * @return The sessionKey of the referenced session entity.
   */
  public String getSessionKey()
  {
    return sessionKey;
  }

  /**
   *
   * @param genericValue
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   * @return String UDDI request's generic value.
   */
  public String getGeneric()
  {
    return this.generic;
  }

  /**
   * Sets the Lease value
   *
   * @param bag The new Lease
   */
  public void setLease(Lease lease)
  {
    this.lease = lease;
  }

  /**
   * Returns the Lease value
   *
   * @return The Lease value
   */
  public Lease getLease()
  {
    return this.lease;
  }

  /**
   *
   */
  public int getMaxRows()
  {
    return maxRows;
  }

  /**
   *
   */
  public void setMaxRows(int maxRows)
  {
    this.maxRows = maxRows;
  }

  /**
   *
   */
  public void setMaxRows(String maxRows)
  {
    setMaxRows(Integer.parseInt(maxRows));
  }

  /**
   *
   */
  public void addFindQualifier(FindQualifier findQualifier)
  {
    if (this.findQualifiers == null)
      this.findQualifiers = new FindQualifiers();
    this.findQualifiers.addFindQualifier(findQualifier);
  }

  /**
   *
   */
  public void setFindQualifiers(FindQualifiers findQualifiers)
  {
    this.findQualifiers = findQualifiers;
  }

  /**
   *
   */
  public FindQualifiers getFindQualifiers()
  {
    return this.findQualifiers;
  }

  /**
   *
   */
  public void setAuthInfo(AuthInfo authInfo)
  {
    this.authInfo = authInfo;
  }

  /**
   *
   */
  public AuthInfo getAuthInfo()
  {
    return this.authInfo;
  }

  /**
   *
   * @param nameValue The name argument of the search.
   */
  public void addName(Name nameValue)
  {
    if (this.nameVector == null)
      this.nameVector = new Vector();
    this.nameVector.add(nameValue);
  }

  /**
   *
   * @return The name argument of the search, or null if the argument has not been specified.
   */
  public Vector getNameVector()
  {
    return this.nameVector;
  }

  /**
   *
   * @param names The name argument of the search.
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }
}
