/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.request;

import java.util.Vector;

import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.TModelBag;
import org.apache.commons.logging.Log;

import cgl.fthpis.datatype.SessionKey;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.lease.Lease;

import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindFTHPISService implements RegistryObject,Inquiry
{
  private static Log log = LogFactory.getLog(FindFTHPISService.class);
  AuthInfo authInfo;
  String generic;
  String businessKey;
  Vector nameVector;
  CategoryBag categoryBag;
  AttributeBag staticContextBag;
  Lease lease;
  TModelBag tModelBag;

  //String sessionKey;
  Vector sessionKeyVector;
  AttributeBag dynamicContextBag;

  FindQualifiers findQualifiers;
  int maxRows;

  /**
   * Constructs a new empty find_service request.
   */
  public FindFTHPISService()
  {
    log.info("cgl.fthpis.datatype.request.FindService() constructor called.");
  }

  /**
   * Construct a new find_service request. The services searched for must be part of the
   * businessEntity with the given businessKey.
   *
   * @param businessKey The key of the businessEntity to search in.
   * @throws NullPointerException If the given key is null.
   */
  public FindFTHPISService(String businessKey)
  {
    setBusinessKey(businessKey);
  }

  /**
   * Construct a new find_service request. The services searched for must be part of the
   * businessEntity with the given businessKey.
   *
   * @param businessKey The key of the businessEntity to search in.
   * @throws NullPointerException If the given key is null.
   */
  public FindFTHPISService(String businessKey, Vector sessionKeyVector)
  {
    setBusinessKey(businessKey);
    setSessionKeyVector(sessionKeyVector);
  }

  /**
   * Sets the businesskey of this find_service request to the given key. Only services of the
   * referenced businessEntity are searched.
   *
   * @param key A reference to the businessEntity.
   * @throws NullPointerException If the given key is null.
   */
  public void setBusinessKey(String key)
  {
    businessKey = key;
  }

  /**
  * Returns the businesskey of this find_service request. Only services of the referenced
  * businessEntity are searched.
  *
  * @return The businesskey of the referenced businessEntity.
  */
  public String getBusinessKey()
  {
    return businessKey;
  }

  /**
   *
   * @param genericValue
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   * @return String UDDI request's generic value.
   */
  public String getGeneric()
  {
    return this.generic;
  }

  /**
   * Sets the name argument of the search to the given name. This value is a partial
   * name. The serviceList return contains ServiceInfo objects for services whose
   * name matches the value passed (leftmost match).
   *
   * @param nameValue The name argument of the search.
   */
  public void addSessionKey(SessionKey sessionKey)
  {
    if (this.sessionKeyVector == null)
      this.sessionKeyVector = new Vector();
    this.sessionKeyVector.add(sessionKey);
  }

  /**
   * Returns the name argument of the search. Null is returned if the name
   * argument for this search has not been specified.
   *
   * @return The name argument of the search, or null if the argument has not been specified.
   */
  public Vector getSessionKeyVector()
  {
    return this.sessionKeyVector;
  }

  /**
   * Sets the name argument of the search to the given name. This value is a partial
   * name. The serviceList return contains ServiceInfo objects for services whose
   * name matches the value passed (leftmost match).
   *
   * @param names The name argument of the search.
   */
  public void setSessionKeyVector(Vector vec)
  {
    this.sessionKeyVector = vec;
  }

  /**
   * Sets the name argument of the search to the given name. This value is a partial
   * name. The serviceList return contains ServiceInfo objects for services whose
   * name matches the value passed (leftmost match).
   *
   * @param nameValue The name argument of the search.
   */
  public void addName(Name nameValue)
  {
    if (this.nameVector == null)
      this.nameVector = new Vector();
    this.nameVector.add(nameValue);
  }

  /**
   * Returns the name argument of the search. Null is returned if the name
   * argument for this search has not been specified.
   *
   * @return The name argument of the search, or null if the argument has not been specified.
   */
  public Vector getNameVector()
  {
    return this.nameVector;
  }

  /**
   * Sets the name argument of the search to the given name. This value is a partial
   * name. The serviceList return contains ServiceInfo objects for services whose
   * name matches the value passed (leftmost match).
   *
   * @param names The name argument of the search.
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }

  /**
   * Adds a category reference to the categoryBag argument of this search.
   *
   * @param ref The category reference to add.
   */
  public void addCategory(KeyedReference ref)
  {
    // just return if the KeyedReference parameter is null (nothing to add)
    if (ref == null)
      return;

    // make sure the CategoryBag has been initialized
    if (this.categoryBag == null)
      this.categoryBag = new CategoryBag();

    this.categoryBag.addKeyedReference(ref);
  }

  /**
   * Sets the CategoryBag value
   *
   * @param bag The new CategoryBag
   */
  public void setCategoryBag(CategoryBag bag)
  {
    categoryBag = bag;
  }

  /**
   * Returns the CategoryBag value
   *
   * @return The CategoryBag value
   */
  public CategoryBag getCategoryBag()
  {
    return this.categoryBag;
  }

  /**
   *
   */
  public void setLease(Lease lease_)
  {
    lease = lease_;
  }

  /**
   *
   */
  public Lease getLease()
  {
    return this.lease;
  }


  /**
   *
   */
  public void addAttribute(Attribute attribute)
  {
    if (attribute == null)
      return;

    if (this.staticContextBag == null)
      this.staticContextBag = new AttributeBag();

    this.staticContextBag.addAttribute(attribute);
  }

  /**
   *
   */
  public void setAttributeBag(AttributeBag bag)
  {
    //log.info("org.apache.juddi.datatype.request.setAttributeBag = setAttributeBag");
    staticContextBag = bag;
  }

  /**
   *
   */
  public AttributeBag getAttributeBag()
  {
    return this.staticContextBag;
  }


  /**
   *
   */
  public void addDynamicContext(Attribute dynamicContext)
  {
    // just return if the KeyedReference parameter is null (nothing to add)
    if (dynamicContext == null)
      return;

    // make sure the CategoryBag has been initialized
    if (this.dynamicContextBag == null)
      this.dynamicContextBag = new AttributeBag();

    this.dynamicContextBag.addAttribute(dynamicContext);
  }

  /**
   *
   */
  public void setDynamicContextBag(AttributeBag bag)
  {
    dynamicContextBag = bag;
  }

  /**
   *
   */
  public AttributeBag getDynamicContextBag()
  {
    return this.dynamicContextBag;
  }

  /**
   * Adds a tModel reference to the tModelBag argument of this search. This tModelBag argument
   * lets you search for businesses that have bindings that are compatible with a specific
   * tModel pattern.
   *
   * @param key The key of the tModel to add to the tModelBag argument.
   */
  public void addTModelKey(String key)
  {
    // just return if the TModel key parameter is null (nothing to add)
    if (key == null)
      return;

    // make sure the TModelBag has been initialized
    if (this.tModelBag == null)
      this.tModelBag = new TModelBag();

    this.tModelBag.addTModelKey(key);
  }

  /**
   * Sets the TModelBag value.
   *
   * @param bag the new TModelBag.
   */
  public void setTModelBag(TModelBag bag)
  {
    this.tModelBag = bag;
  }

  /**
   * Returns the list of tModel references as an enumeration. If the tModelBag has not
   * been specified, an empty list is returned.
   *
   * @return The list of the tModel references.
   */
  public TModelBag getTModelBag()
  {
    return this.tModelBag;
  }

  /**
   *
   */
  public int getMaxRows()
  {
    return maxRows;
  }

  /**
   *
   */
  public void setMaxRows(int maxRows)
  {
    this.maxRows = maxRows;
  }

  /**
   *
   */
  public void setMaxRows(String maxRows)
  {
    setMaxRows(Integer.parseInt(maxRows));
  }

  /**
   *
   */
  public void addFindQualifier(FindQualifier findQualifier)
  {
    if (this.findQualifiers == null)
      this.findQualifiers = new FindQualifiers();
    this.findQualifiers.addFindQualifier(findQualifier);
  }

  /**
   *
   */
  public void setFindQualifiers(FindQualifiers findQualifiers)
  {
    this.findQualifiers = findQualifiers;
  }

  /**
   *
   */
  public FindQualifiers getFindQualifiers()
  {
    return this.findQualifiers;
  }

  /**
   *
   */
  public void setAuthInfo(AuthInfo authInfo)
  {
    this.authInfo = authInfo;
  }

  /**
   *
   */
  public AuthInfo getAuthInfo()
  {
    return this.authInfo;
  }
}
