/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datatype.request;


import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.datatype.RegistryObject;

import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.datatype.request.FindQualifier;

import cgl.fthpis.datatype.lease.Lease;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

/**
 * "Used to locate specific attribute within a registered
 *  businessService. Returns a attributeDetail message."
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindServiceAttribute implements RegistryObject,Inquiry
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(FindServiceAttribute.class);
  String serviceKey;
  String sessionKey;
  String generic;
  CategoryBag categoryBag;
  Lease lease;
  FindQualifiers findQualifiers;
  int maxRows;

  /**
   *
   */
  public FindServiceAttribute()
  {
      log.info("cgl.fthpis.datatype.request.FindServiceAttribute() constructor called.");
  }


  /**
   * Construct a new find_serviceAttribute request.
   *
   * @param serviceKey The key of the businessService to search in.
   * @throws NullPointerException If the given key is null.
   */
  public FindServiceAttribute(String serviceKey)
  {
    log.info("cgl.fthpis.datatype.request.FindServiceAttribute(String serviceKey) constructor called.");
    setServiceKey(serviceKey);
  }

  /**
   * Sets the servicekey of this find_serviceAttribute request to the given key.
   *
   * @param key A reference to the businessService.
   */
  public void setServiceKey(String key)
  {
    serviceKey = key;
  }

  /**
   * Returns the servicekey of this find_serviceAttribute request.
   *
   * @return The servicekey of the referenced businessService.
   */
  public String getServiceKey()
  {
    return serviceKey;
  }

  /**
   *
   */
  public void setSessionKey(String key)
  {
    sessionKey = key;
  }

  /**
   *
   */
  public String getSessionKey()
  {
    return sessionKey;
  }

  /**
   *
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   */
  public String getGeneric()
  {
    return this.generic;
  }

  /**
   *
   */
  public void addCategory(KeyedReference ref)
  {
    // just return if the KeyedReference parameter is null (nothing to add)
    if (ref == null)
      return;

    // make sure the CategoryBag has been initialized
    if (this.categoryBag == null)
      this.categoryBag = new CategoryBag();

    this.categoryBag.addKeyedReference(ref);
  }

  /**
   *
   */
  public void setCategoryBag(CategoryBag bag)
  {
    categoryBag = bag;
  }

  /**
   *
   */
  public CategoryBag getCategoryBag()
  {
    return this.categoryBag;
  }

  /**
   * Sets the Lease value
   *
   * @param The new Lease
   */
  public void setLease(Lease lease)
  {
    this.lease = lease;
  }

  /**
   * Returns the Lease value
   *
   * @return The Lease value
   */
  public Lease getLease()
  {
    return this.lease;
  }

  /**
   *
   */
  public int getMaxRows()
  {
    return maxRows;
  }

  /**
   *
   */
  public void setMaxRows(int maxRows)
  {
    this.maxRows = maxRows;
  }

  /**
   *
   */
  public void setMaxRows(String maxRows)
  {
    setMaxRows(Integer.parseInt(maxRows));
  }

  /**
   *
   */
  public void addFindQualifier(FindQualifier findQualifier)
  {
    if (this.findQualifiers == null)
      this.findQualifiers = new FindQualifiers();
    this.findQualifiers.addFindQualifier(findQualifier);
  }

  /**
   *
   */
  public void setFindQualifiers(FindQualifiers findQualifiers)
  {
    this.findQualifiers = findQualifiers;
  }

  /**
   *
   */
  public FindQualifiers getFindQualifiers()
  {
    return this.findQualifiers;
  }

}
