/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.request;

import java.util.Vector;

import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.ServiceKey;

import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;

/**
 * Used to locate information about one or more sessions. Returns a
 * sessionList message that matches the conditions specified.
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindSession implements RegistryObject,Inquiry
{
  AuthInfo authInfo;
  String generic;
  Vector serviceKeyVector;
  Vector nameVector;
  AttributeBag attributeBag;
  Lease lease;
  FindQualifiers findQualifiers;
  int maxRows;


  /**
   * Constructs a new empty find_session request.
   */
  public FindSession()
  {
  }

  /**
   *
   * @param genericValue
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   * @return String request's generic value.
   */
  public String getGeneric()
  {
    return this.generic;
  }


  /**
   *
   * @param serviceKey The serviceKey argument of the search.
   */
  public void addServiceKey(ServiceKey serviceKey)
  {
    if (this.serviceKeyVector == null)
      this.serviceKeyVector = new Vector();
    this.serviceKeyVector.add(serviceKey);
  }

  /**
   *
   * @param names The name argument of the search.
   */
  public void setServiceKeyVector(Vector serviceKeyVector)
  {
    this.serviceKeyVector = serviceKeyVector;
  }

  /**
   *
   * @return The vector of serviceKey of the search, or null if the argument has not
   *  been specified.
   */
  public Vector getServiceKeyVector()
  {
    return serviceKeyVector;
  }

  /**
   *
   * @param nameValue The name argument of the search.
   */
  public void addName(Name nameValue)
  {
    if (this.nameVector == null)
      this.nameVector = new Vector();
    this.nameVector.add(nameValue);
  }

  /**
   *
   * @param names The name argument of the search.
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }

  /**
   * Returns the name argument of the search. Null is returned if the name
   * argument for this search has not been specified.
   *
   * @return The vector of Name argument of the search, or null if the argument has not
   *  been specified.
   */
  public Vector getNameVector()
  {
    return nameVector;
  }

  /**
    * Sets the Lease value
    *
    * @param The new Lease
    */
   public void setLease(Lease lease_)
   {
     lease = lease_;
   }

   /**
    * Returns the Lease value
    *
    * @return The Lease value
    */
   public Lease getLease()
   {
     return this.lease;
   }


   /**
    * Adds an attribute to the attributeBag
    *
    * @param ref attribute to add.
    */
   public void addAttribute(Attribute attribute)
   {
     if (attribute == null)
       return;

     if (this.attributeBag == null)
       this.attributeBag = new AttributeBag();

     this.attributeBag.addAttribute(attribute);
   }

   /**
    * Sets the AttributeBag value
    *
    * @param bag The new AttributeBag
    */
   public void setAttributeBag(AttributeBag bag)
   {
     attributeBag = bag;
   }

   /**
    * Returns the AttributeBag value
    *
    * @return The AttributeBag value
    */
   public AttributeBag getAttributeBag()
   {
     return this.attributeBag;
  }

  /**
   *
   */
  public int getMaxRows()
  {
    return maxRows;
  }

  /**
   *
   */
  public void setMaxRows(int maxRows)
  {
    this.maxRows = maxRows;
  }

  /**
   *
   */
  public void setMaxRows(String maxRows)
  {
    setMaxRows(Integer.parseInt(maxRows));
  }

  /**
   *
   */
  public void addFindQualifier(FindQualifier findQualifier)
  {
    if (this.findQualifiers == null)
      this.findQualifiers = new FindQualifiers();
    this.findQualifiers.addFindQualifier(findQualifier);
  }

  /**
   *
   */
  public void setFindQualifiers(FindQualifiers findQualifiers)
  {
    this.findQualifiers = findQualifiers;
  }

  /**
   *
   */
  public FindQualifiers getFindQualifiers()
  {
    return findQualifiers;
  }


  /**
   *
   */
  public void setAuthInfo(AuthInfo authInfo)
  {
    this.authInfo = authInfo;
  }

  /**
   *
   */
  public AuthInfo getAuthInfo()
  {
    return this.authInfo;
  }
}
