/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.datatype.request;

import java.util.Vector;

import org.apache.juddi.datatype.RegistryObject;

import cgl.fthpis.datatype.AttributeKey;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

/**
 * "Used to get full context information suitable for make one
 *  or more attribute requests. Returns a contextDetail message."
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class GetContextDetail implements RegistryObject,Inquiry
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(GetContextDetail.class);  //private static Log log = LogFactory.getLog(GetAttributeDetail.class);
  AuthInfo authInfo;
  String generic;
  Vector contextKeyVector;

  /**
   * Construct a new empty get_contextDetail request
   */
  public GetContextDetail()
  {
      log.info("cgl.fthpis.datatype.request.GetContextDetail() constructor called.");
  }

  /**
   *
   * @param genericValue
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   * @return String request's generic value.
   */
  public String getGeneric()
  {
    return this.generic;
  }

  /**
   * Add a reference to a context
   *    *
   * @param key The contextkey of the referenced Context.
   */
  public void addContextKey(AttributeKey key)
  {
    if ((key != null) && (key.getValue() != null))
      addContextKey(key.getValue());
  }

  /**
   * Adds a ContextKey to the collection of binding keys used by this query.
   *
   * @param key A ContextKey
   */
  public void addContextKey(String key)
  {
    if (contextKeyVector == null)
      contextKeyVector = new Vector();
    contextKeyVector.add(key);
  }

  /**
   * Add a collection of ContextKey references to this search.
   *
   * @param keys The collection of contextKeys.
   */
  public void setContextKeyVector(Vector keys)
  {
    this.contextKeyVector = keys;
  }

  /**
   *
   */
  public Vector getContextKeyVector()
  {
    return this.contextKeyVector;
  }

  /**
   *
   */
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();

    if (this.contextKeyVector != null)
      for (int i=0; i<this.contextKeyVector.size(); i++)
        buffer.append(this.contextKeyVector.elementAt(i));

    return buffer.toString();
  }

  /**
   *
   */
  public void setAuthInfo(AuthInfo authInfo)
  {
    this.authInfo = authInfo;
  }

  /**
   *
   */
  public AuthInfo getAuthInfo()
  {
    return this.authInfo;
  }
}
