/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.request;

import java.util.Vector;

import cgl.fthpis.datatype.SessionKey;
import org.apache.juddi.datatype.RegistryObject;

/**
 * "Used to get the full sessionEntity information for one or more
 *  sessions. Returns a sessionDetail message."
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class GetSessionDetail implements RegistryObject,Inquiry
{
  AuthInfo authInfo;
  String generic;
  Vector sessionKeyVector;

  /**
   * Construct a new empty get_sessionDetail request.
   */
  public GetSessionDetail()
  {
  }

  /**
   *
   * @param genericValue
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   * @return String request's generic value.
   */
  public String getGeneric()
  {
    return this.generic;
  }

  /**
   * Add a SessionKey to the collection of SessionKeys
   *
   * @param key The new SessionKey to add
   */
  public void addSessionKey(SessionKey key)
  {
    if ((key != null) && (key.getValue() != null))
      addSessionKey(key.getValue());
  }

  /**
   * Add a reference to an extended session entity.
   *
   * @param key The key of the referenced extended session entity.
   */
  public void addSessionKey(String key)
  {
    if (sessionKeyVector == null)
      sessionKeyVector = new Vector();
    sessionKeyVector.add(key);
  }

  /**
   * Returns the SessionKey Vector
   */
  public Vector getSessionKeyVector()
  {
    return this.sessionKeyVector;
  }

  /**
   * Sets the SessionKey Vector
   *
   * @param keys The new collection of SessionKeys
   */
  public void setSessionKeyVector(Vector keys)
  {
    this.sessionKeyVector = keys;
  }

  /**
   *
   */
  public void setAuthInfo(AuthInfo authInfo)
  {
    this.authInfo = authInfo;
  }

  /**
   *
   */
  public AuthInfo getAuthInfo()
  {
    return this.authInfo;
  }
}
