/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.request;

import java.util.Vector;

import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.UploadRegister;
import cgl.fthpis.datatype.session.SessionEntity;

/**
 * "Used to register new sessionEntity information or update existing
 *  sessionEntity information.  Use this to control the overall
 *  information about the entire session.  "
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class SaveSession implements RegistryObject,Publish
{
  String generic;
  AuthInfo authInfo;
  Vector sessionVector;
  Vector uploadRegisterVector;

  /**
   *
   */
  public SaveSession()
  {
  }

  /**
   *
   */
  public SaveSession(AuthInfo info, SessionEntity session)
  {
    this.authInfo = info;
    addSessionEntity(session);
  }

  /**
   *
   */
  public SaveSession(AuthInfo info,Vector sessions)
  {
    this.authInfo = info;
    this.sessionVector = sessions;
  }

  /**
   *
   * @param genericValue
   */
  public void setGeneric(String genericValue)
  {
    this.generic = genericValue;
  }

  /**
   *
   * @return String request's generic value.
   */
  public String getGeneric()
  {
    return this.generic;
  }

  /**
   *
   */
  public void addSessionEntity(SessionEntity sessionEntity)
  {
    if (this.sessionVector == null)
      this.sessionVector = new Vector();
    this.sessionVector.add(sessionEntity);
  }

  /**
   *
   */
  public void setSessionEntityVector(Vector sessions)
  {
    this.sessionVector = sessions;
  }

  /**
   *
   */
  public Vector getSessionEntityVector()
  {
    return sessionVector;
  }

  /**
   *
   */
  public void addUploadRegister(UploadRegister register)
  {
    if (this.uploadRegisterVector == null)
      this.uploadRegisterVector = new Vector();
    this.uploadRegisterVector.add(register);
  }

  /**
   *
   */
  public void setUploadRegisterVector(Vector registers)
  {
    this.uploadRegisterVector = registers;
  }

  /**
   *
   */
  public Vector getUploadRegisterVector()
  {
    return this.uploadRegisterVector;
  }

  /**
   *
   */
  public void setAuthInfo(AuthInfo authInfo)
  {
    this.authInfo = authInfo;
  }

  /**
   *
   */
  public AuthInfo getAuthInfo()
  {
    return authInfo;
  }
}
