/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.response;

import java.util.Vector;

import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.SessionKey;
import cgl.fthpis.datatype.attribute.AttributeName;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class AttributeInfo implements RegistryObject
{
  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(AttributeInfo.class);
  //bare minimum information
  String attributeKey;
  Vector nameVector;

//  String serviceKey;
//  String sessionKey;
//  AttributeName attributeIdentifier;
//  Vector descVector;
//  SessionInfos infos;
//  Lease lease;


  /**
   * default constructor
   */
  public AttributeInfo()
  {
  }

  /**
   * copy constructor
   */
  public AttributeInfo(AttributeInfo attributeInfo)
  {
    attributeKey = attributeInfo.getAttributeKey();
  }

  /**
   *
   */
  public AttributeInfo(String key)
  {
    this.attributeKey = key;
  }

  /**
   *
   */
  public void setAttributeKey(String key)
  {
    this.attributeKey = key;
  }

  /**
   *
   */
  public String getAttributeKey()
  {
    return this.attributeKey;
  }

  /**
   * Add a name to this session entity.
   *
   * @param name The Name to add.
   */
  public void addName(Name name)
  {
    // just return if name is null (nothing to add)
    if (name == null)
      return;

    if (this.nameVector == null)
      this.nameVector = new Vector();
    this.nameVector.add(name);
  }

  /**
   * Set the set of names of this session entity to the given set.
   *
   * @param names The new set of contacts.
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }

  /**
   * Returns the names of this session entity.
   *
   * @return The names of thes session entity. If this session entity
   *  doesn't have any names, an empty set is returned.
   */
  public Vector getNameVector()
  {
    return this.nameVector;
  }

//  /**
//   *
//   */
//  public void setSessionKey(String key)
//  {
//    this.sessionKey = key;
//  }
//
//  /**
//   *
//   */
//  public String getSessionKey()
//  {
//    return this.sessionKey;
//  }
//
//  /**
//   *
//   */
//  public void setAttributeName(AttributeName name)
//  {
//    this.attributeIdentifier = name;
//  }
//
//  /**
//   *
//   */
//  public AttributeName getAttributeName()
//  {
//    return this.attributeIdentifier;
//  }
//
//  /**
//   *
//   */
//  public void setServiceKey(String key)
//  {
//    this.serviceKey = key;
//  }
//
//  /**
//   *
//   */
//  public String getServiceKey()
//  {
//    return this.serviceKey;
//  }


//  /**
//   *
//   */
//  public void addDescription(Description desc)
//  {
//    if (this.descVector == null)
//      this.descVector = new Vector();
//    this.descVector.add(desc);
//  }
//
//  /**
//   *
//   */
//  public void setDescriptionVector(Vector descriptions)
//  {
//    this.descVector = descriptions;
//  }
//
//  /**
//   *
//   */
//  public Vector getDescriptionVector()
//  {
//    return this.descVector;
//  }
//
//  /**
//   *
//   */
//  public void addServiceInfo(ServiceInfo info)
//  {
//    if (this.infos == null)
//      this.infos = new ServiceInfos();
//    this.infos.addServiceInfo(info);
//  }
//
//  /**
//   *
//   */
//  public void setServiceInfos(ServiceInfos infos)
//  {
//    this.infos = infos;
//  }
//
//  /**
//   *
//   */
//  public ServiceInfos getServiceInfos()
//  {
//    return this.infos;
//  }
//
//  public void setLease(Lease lease) {
//    this.lease = lease;
//  }
//
//  public Lease getLease() {
//    return this.lease;
//  }

//  /**
//   * Sets the name argument of the search to the given name. This value is a partial
//   * name. The serviceList return contains ServiceInfo objects for services whose
//   * name matches the value passed (leftmost match).
//   *
//   * @param nameValue The name argument of the search.
//   */
//  public void addSessionKey(SessionKey sessionKey)
//  {
//    if (this.sessionKeyVector == null)
//      this.sessionKeyVector = new Vector();
//    this.sessionKeyVector.add(sessionKey);
//  }
//
//  /**
//   * Returns the name argument of the search. Null is returned if the name
//   * argument for this search has not been specified.
//   *
//   * @return The name argument of the search, or null if the argument has not been specified.
//   */
//  public Vector getSessionKeyVector()
//  {
//    return this.sessionKeyVector;
//  }
//
//  /**
//   * Sets the name argument of the search to the given name. This value is a partial
//   * name. The serviceList return contains ServiceInfo objects for services whose
//   * name matches the value passed (leftmost match).
//   *
//   * @param names The name argument of the search.
//   */
//  public void setSessionKeyVector(Vector vec)
//  {
//    this.sessionKeyVector = vec;
//  }

}
