/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.datatype.service;

import java.util.Vector;

import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.KeyedReference;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.binding.BindingTemplate;
import org.apache.juddi.datatype.binding.BindingTemplates;

import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.SessionKey;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class BusinessService implements RegistryObject
{
  Vector sessionKeyVector;
  String businessKey;
  String serviceKey;
  Vector nameVector;
  Vector descrVector;
  BindingTemplates bindingTemplates;
  CategoryBag categoryBag;
  AttributeBag attributeBag;
  AttributeBag dynamicContextBag;
  Lease lease;

  /**
   * Constructs a new initialized BusinessService instance.
   */
  public BusinessService()
  {
  }

  /**
   * Sets the String of this BusinessService. If the BusinessEntity for
   * this BusinessService has been set, this new String must be null or
   * must be the same as the BusinessEntity. If the BusinessEntity for this
   * BindingTemplate has not been set, this new String may not be null.
   *
   * @param key The new business key.
   */
  public void setBusinessKey(String key)
  {
    this.businessKey = key;
  }

  /**
   * Returns the business key of this business service. This method can return
   * null if the business entity for this business service has been set and if
   * the business key for this business service has not been set.
   *
   * @return The business key of this business service.
   */
  public String getBusinessKey()
  {
    return this.businessKey;
  }

  /**
   * Sets the service key of this business service to the given key.
   *
   * @param key The service key of this business service.
   */
  public void setServiceKey(String key)
  {
    serviceKey = key;
  }

  /**
   * Returns the service key of this business service.
   *
   * @return The service key of this business service.
   */
  public String getServiceKey()
  {
    return serviceKey;
  }

  /**
   * Add a name to this BusinessService.
   *
   * @param name The Name to add.
   */
  public void addName(Name name)
  {
    if (nameVector == null)
      nameVector = new Vector();
    nameVector.add(name);
  }

  /**
   * Adds the given description.
   *
   * @param desc The description to add.
   */
  public void addDescription(Description desc)
  {
    if (descrVector == null)
      descrVector = new Vector();
    descrVector.add(desc);
  }

  /**
   * Returns the names of this BusinessService.
   *
   * @return The names of thes BusinessService. If this BusinessService
   *  doesn't have any names, an empty set is returned.
   */
  public Vector getNameVector()
  {
    return nameVector;
  }

  /**
   * Sets the name list to the current one. Ignores any object in the
   * collection that is not an "instanceof" the Name class.
   *
   * @param names the Names object to set
   */
  public void setNameVector(Vector names)
  {
    this.nameVector = names;
  }

  /**
   * Sets the name argument of the search to the given name. This value is a partial
   * name. The serviceList return contains ServiceInfo objects for services whose
   * name matches the value passed (leftmost match).
   *
   * @param nameValue The name argument of the search.
   */
  public void addSessionKey(SessionKey sessionKey)
  {
    if (this.sessionKeyVector == null)
      this.sessionKeyVector = new Vector();
    this.sessionKeyVector.add(sessionKey);
  }

  /**
   * Returns the name argument of the search. Null is returned if the name
   * argument for this search has not been specified.
   *
   * @return The name argument of the search, or null if the argument has not been specified.
   */
  public Vector getSessionKeyVector()
  {
    return this.sessionKeyVector;
  }

  /**
   * Sets the name argument of the search to the given name. This value is a partial
   * name. The serviceList return contains ServiceInfo objects for services whose
   * name matches the value passed (leftmost match).
   *
   * @param names The name argument of the search.
   */
  public void setSessionKeyVector(Vector vec)
  {
    this.sessionKeyVector = vec;
  }

  /**
   * Sets the description list to the current one. Ignores any object in the
   * collection that is not an "instanceof" the Description class.
   *
   * @param descs Descriptions object to set
   */
  public void setDescriptionVector(Vector descs)
  {
    this.descrVector = descs;
  }

  /**
   * Returns the descriptions.
   *
   * @return the descriptions object
   */
  public Vector getDescriptionVector()
  {
    return this.descrVector;
  }

  /**
   * Add a binding template to this business service.
   *
   * @param binding The binding template to add.
   */
  public void addBindingTemplate(BindingTemplate binding)
  {
    if (this.bindingTemplates == null)
      this.bindingTemplates = new BindingTemplates();
    this.bindingTemplates.addBindingTemplate(binding);
  }

  /**
   * Returns the binding templates of this business service.
   *
   * @return The binding templates of this business service.
   */
  public BindingTemplates getBindingTemplates()
  {
    return this.bindingTemplates;
  }

  /**
   * Sets the binding templates of this business service.
   */
  public void setBindingTemplates(BindingTemplates bindings)
  {
    this.bindingTemplates = bindings;
  }

  /**
   * Add a attribute to this business service.
   *
   * @param The attribute to add.
   */
  public void addAttribute(Attribute attribute)
  {
    if (this.attributeBag == null)
      this.attributeBag = new AttributeBag();
    this.attributeBag.addAttribute(attribute);
  }

  /**
   * Returns the staticAttributebag of this business service.
   *
   * @return The staticAttributebag of this business service.
   */
  public AttributeBag getAttributeBag()
  {
    return this.attributeBag;
  }

  /**
   * Sets the staticAttributebag of this business service.
   */
  public void setAttributeBag(AttributeBag attributebag)
  {
    this.attributeBag = attributebag;
  }

  /**
   * Add a dynamicContextBag to this business service.
   *
   * @param The dynamicContextBag to add.
   */
  public void addDynamicContext(Attribute dynamicContext)
  {
    if (this.dynamicContextBag == null)
      this.dynamicContextBag = new AttributeBag();
    this.dynamicContextBag.addAttribute(dynamicContext);
  }

  /**
   * Returns the dynamicContextBag of this business service.
   *
   * @return The dynamicContextBag of this business service.
   */
  public AttributeBag getDynamicContextBag()
  {
    return this.dynamicContextBag;
  }

  /**
   * Sets the dynamicContextBag of this business service.
   */
  public void setDynamicContextBag(AttributeBag dynamicContextBag)
  {
    this.dynamicContextBag = dynamicContextBag;
  }

  /**
   * Returns the lease of this business service.
   *
   * @return the lease of this business service.
   */
  public Lease getLease()
  {
    return this.lease;
  }

  /**
   * Sets the lease of this business service.
   */
  public void setLease(Lease lease)
  {
    this.lease = lease;
  }


  /**
   * Add a category to the categorybag of this business service.
   * @param keyref The category to add.
   */
  public void addCategory(KeyedReference ref)
  {
    // just return if the KeyedReference parameter is null (nothing to add)
    if (ref == null)
      return;

    // make sure the CategoryBag has been initialized
    if (this.categoryBag == null)
      this.categoryBag = new CategoryBag();

    this.categoryBag.addKeyedReference(ref);
  }

  /**
   * Returns the categorybag of this business service. If this business
   * service doesn't contain any categories, an empty enumeration is
   * returned.
   *
   * @return The categorybag of this business service.
   */
  public CategoryBag getCategoryBag()
  {
    return this.categoryBag;
  }

  /**
   * Set the categorybag of this business service to the given one.
   *
   * @param bag The new categorybag.
   */
  public void setCategoryBag(CategoryBag bag)
  {
    this.categoryBag = bag;
  }
}
