/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import org.apache.juddi.function.IFunction;
import org.apache.juddi.error.AuthTokenExpiredException;
import org.apache.juddi.error.AuthTokenRequiredException;
import org.apache.juddi.error.RegistryException;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.request.AuthInfo;
import cgl.fthpis.registry.FTHPISRegistryEngine;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public abstract class AbstractFunction implements IFunction
{
  // protected reference to parent Registry instance
  protected FTHPISRegistryEngine registry = null;

  /**
   * @param registry
   */
  protected AbstractFunction(FTHPISRegistryEngine registry)
  {
    this.registry = registry;
  }

  /**
   * @return RegistryEngine
   */
  public FTHPISRegistryEngine getRegistry()
  {
    return this.registry;
  }

  /**
   * @param registry
   */
  public void setRegistry(FTHPISRegistryEngine registry)
  {
    this.registry = registry;
  }

  /**
   * Verify the authentication parameters.
   */
  protected Publisher getPublisher(AuthInfo authInfo, FTHPISDataStore dataStore)
    throws RegistryException
  {
    Publisher publisher = null;

    if ((authInfo == null) || (authInfo.getValue() == null))
      throw new AuthTokenRequiredException("authToken: null");

    String authToken = authInfo.getValue();
    if (authToken.trim().length() == 0)
      throw new AuthTokenRequiredException("authToken: "+authToken);

    publisher = dataStore.getAuthTokenPublisher(authToken);
    if (publisher == null)
      throw new AuthTokenRequiredException("authToken: "+authToken);

    if (dataStore.isAuthTokenExpired(authToken))
      throw new AuthTokenExpiredException("authToken: "+authToken);

    // Token is valid so 'touch' so that it's
    // internal 'expiration clock' is reset.
    dataStore.touchAuthToken(authToken);

    return publisher;
  }
}
