/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.function;

import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.response.DispositionReport;
import org.apache.juddi.datatype.response.Result;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UserMismatchException;
import org.apache.juddi.util.Config;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.request.AuthInfo;
import cgl.fthpis.datatype.request.GetAuthToken;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.request.DeleteContext;
import cgl.fthpis.function.GetAuthTokenForWSContextFunction;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class DeleteContextFunction extends AbstractFunction
{
  // private reference to FTHPIS Logger
  private static Log log = LogFactory.getLog(DeleteContextFunction.class);

  /**
   *
   */
  public DeleteContextFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    // extract individual parameters
    DeleteContext request = (DeleteContext)regObject;
    String generic = request.getGeneric();
    AuthInfo authInfo = request.getAuthInfo();
    Vector contextKeyVector = request.getContextKeyVector();

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();

    try
    {
      dataStore.beginTrans();

      // validate authentication parameters
      Publisher publisher = getPublisher(authInfo,dataStore);
      String publisherID = publisher.getPublisherID();
      log.info("org.apache.juddi.function.DeleteAttributeFunction 0");

      // validate request parameters
      for (int i=0; i<contextKeyVector.size(); i++)
      {
        // grab the next key from the vector
        String contextKey = (String)contextKeyVector.elementAt(i);
        log.info("org.apache.juddi.function.DeleteContextFunction 1 => contextKey = " + contextKey);
        // check that this binding template really exists.
        // If not then throw an InvalidKeyPassedException.
        if ((contextKey == null) || (contextKey.length() == 0)
        //    ||
        //    (!dataStore.isValidContextKey(contextKey))
                )
          throw new InvalidKeyPassedException(contextKey);

//        // check to make sure that 'authorizedName' controls the
//        // context entity if not then throw a UserMismatchException.
//        if (!dataStore.isContextPublisher(contextKey,publisherID))
//          throw new UserMismatchException(contextKey);
      }
      log.info("org.apache.juddi.function.DeleteContextFunction 2");

      // delete the contexts
      for (int i=0; i<contextKeyVector.size(); i++)
      {
        String contextKey = (String)contextKeyVector.elementAt(i);
        dataStore.deleteContext(contextKey);

        log.info("Publisher '"+publisherID+ "' deleted Context with key: "+ contextKey);
        log.info("Publisher '"+ publisherID+"' deleted Context with key: " + contextKey);
      }

      dataStore.commit();
    }
    catch(InvalidKeyPassedException keyex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(keyex.getMessage());
      throw (RegistryException)keyex;
    }
    catch(UserMismatchException umex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(umex.getMessage());
      throw (RegistryException)umex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(regex.toString());
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ex.toString().toString());
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
      	dataStore.release();
    }

    // didn't encounter an exception so let's create
    // and return a successfull DispositionReport
    DispositionReport dispRpt = new DispositionReport();
    dispRpt.setGeneric(generic);
    dispRpt.setOperator(Config.getOperator());
    dispRpt.addResult(new Result(Result.E_SUCCESS));
    return dispRpt;
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
     GetAuthToken authTokenRequest = new GetAuthToken("maktas","changeIt");
     AuthToken authToken = (AuthToken)(new GetAuthTokenForWSContextFunction(reg).execute(authTokenRequest));
     log.info(" AuthInfo: "+ authToken.getAuthInfo());
     AuthInfo authInfo = authToken.getAuthInfo();

      Vector contextKeyVector = new Vector();
      String key = "D6891060-4509-11DA-AB9A-BBF6BD889468";
      contextKeyVector.add(key);

      // create & execute the DeleteBinding request
      DeleteContext request = new DeleteContext();
      request.setAuthInfo(authInfo);


      request.setContextKeyVector(contextKeyVector);

      DispositionReport response = (DispositionReport)reg.execute(request);
      log.info("errno: "+response.toString());
    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }

}
