/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.NameTooLongException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.TooManyOptionsException;
import org.apache.juddi.error.UnsupportedException;
import org.apache.juddi.util.Config;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.datatype.response.ServiceInfos;
import cgl.fthpis.datatype.response.ServiceInfo;
import cgl.fthpis.datatype.response.ServiceList;
import cgl.fthpis.datatype.SessionKey;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.*;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.request.FindFTHPISService;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class FindFTHPISServiceFunction extends AbstractFunction {
    // private reference to FTHPIS logger
    private static Log log = LogFactory.getLog(FindFTHPISServiceFunction.class);
    Properties bootstrap = null;
    /**
     *
     */
    public FindFTHPISServiceFunction(FTHPISRegistryEngine registry) {
        super(registry);
    }

    /**
     *
     */
    public RegistryObject execute(RegistryObject regObject) throws
            RegistryException {
        log.info(
                "**** FindServiceFunction.execute(RegistryObject regObject) function called.");
        FindFTHPISService request = (FindFTHPISService) regObject;
        String generic = request.getGeneric();
        Vector sessionKeyVector = request.getSessionKeyVector();
        AttributeBag dynamicContextBag = request.getDynamicContextBag();
        FindQualifiers qualifiers = request.getFindQualifiers();
        int maxRows = request.getMaxRows();


        //log.info("**** FindServiceFunction.execute(RegistryObject regObject) I");
        // first make sure we need to continue with this request. If
        // no arguments were passed in then we'll simply return
        // an empty ServiceList (aka "a zero match result set").
        if (((dynamicContextBag == null) || (dynamicContextBag.size() == 0)) &&
            ((sessionKeyVector == null) || (sessionKeyVector.size() == 0))) {
            //log.info("org.apache.juddi.function.FindServiceFunction => return empty list");
            ServiceList list = new ServiceList();
            list.setServiceInfos(new ServiceInfos());
            list.setGeneric(generic);
            list.setOperator(Config.getOperator());
            list.setTruncated(false);
            return list;
        }

        log.info("**** FindServiceFunction.execute(RegistryObject regObject) IV");
        // validate the 'qualifiers' parameter as much as possible up-front before
        // calling into the data layer for relational validation.
        if (qualifiers != null) {
            Vector qVector = qualifiers.getFindQualifierVector();
            if ((qVector != null) && (qVector.size() > 0)) {
                for (int i = 0; i < qVector.size(); i++) {
                    FindQualifier qualifier = (FindQualifier) qVector.elementAt(i);
                    String qValue = qualifier.getValue();

                    if ((!qValue.equals(FindQualifier.EXACT_NAME_MATCH)) &&
                        (!qValue.equals(FindQualifier.CASE_SENSITIVE_MATCH)) &&
                        (!qValue.equals(FindQualifier.OR_ALL_KEYS)) &&
                        (!qValue.equals(FindQualifier.OR_LIKE_KEYS)) &&
                        (!qValue.equals(FindQualifier.AND_ALL_KEYS)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_NAME_ASC)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_NAME_DESC)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_DATE_ASC)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_DATE_DESC)) &&
                        (!qValue.equals(FindQualifier.SERVICE_SUBSET)) &&
                        (!qValue.equals(FindQualifier.COMBINE_CATEGORY_BAGS))
                            ) {
                        throw new UnsupportedException("findQualifier=" +
                                qValue);
                    }
                }
            }
        }
        Vector infoVector = null;
        boolean truncatedResults = false;

        log.info("**** FindServiceFunction.execute(RegistryObject regObject) V");

        Vector keyVector = null;
        FTHPISDataStore wscontextDataStore = WSContextDataStoreFactory.getDataStore();
        try {
            wscontextDataStore.beginTrans();

            if (((dynamicContextBag != null) && (dynamicContextBag.size() != 0))
                || (sessionKeyVector != null) && (sessionKeyVector.size() != 0))
            {
                log.info("**** FindServiceFunction.execute(RegistryObject regObject) VI");
                //keyVector = dataStore.findService(businessKey,nameVector,categoryBag,tModelBag, qualifiers);
                //keyVector = dataStore.findService(businessKey,nameVector,categoryBag,tModelBag, attributeBag, qualifiers);
                keyVector = wscontextDataStore.findSessionService(
                        sessionKeyVector,
                        keyVector,
                        dynamicContextBag,
                        qualifiers);
            }
            log.info("**** FindServiceFunction.execute(RegistryObject regObject) VIII + keyVector size " + keyVector.size());
            if ((keyVector != null) && (keyVector.size() > 0)) {
                log.info("org.apache.juddi.function.FindServiceFunction.execute() => keyVector size = " + keyVector.size());
                // if a maxRows value has been specified and it's less than
                // the number of rows we are about to return then only return
                // maxRows specified.
                int rowCount = keyVector.size();
                if ((maxRows > 0) && (maxRows < rowCount)) {
                    rowCount = maxRows;
                    truncatedResults = true;
                }

                // iterate through the business server keys fetching
                // each associated ServiceInfo in sequence.
                infoVector = new Vector(rowCount);
                for (int i = 0; i < rowCount; i++) {
                    ServiceInfo combinedServiceInfo = new ServiceInfo();
                    combinedServiceInfo.setServiceKey((String) keyVector.elementAt(i));
                    infoVector.addElement(combinedServiceInfo);
//                    ServiceInfo combinedInfo =  (ServiceInfo)infotable.get((String) keyVector.elementAt(i));
//                    ServiceInfo sessionServiceInfo = wscontextDataStore.fetchSessionServiceInfo((String) keyVector.elementAt(i));
//                    if (sessionServiceInfo != null &&
//                        sessionServiceInfo.getSessionKeyVector() != null &&
//                        sessionServiceInfo.getSessionKeyVector().size() != 0) {
//                        combinedInfo.setSessionKeyVector(
//                                sessionServiceInfo.getSessionKeyVector());
//                    }
//                    infoVector.addElement(combinedInfo);
                }
            }

            wscontextDataStore.commit();

            // create a new ServiceInfos instance and stuff
            // the new Vector of ServiceInfos into it.
            ServiceInfos infos = new ServiceInfos();
            infos.setServiceInfoVector(infoVector);

            // create a new ServiceList instance and stuff
            // the new serviceInfos instance into it.
            ServiceList list = new ServiceList();
            list.setGeneric(generic);
            list.setServiceInfos(infos);
            list.setOperator(Config.getOperator());
            list.setTruncated(truncatedResults);
            return list;
        } catch (TooManyOptionsException tmoex) {
            try {
                wscontextDataStore.rollback();
            } catch (Exception e) {}
            log.info(tmoex.getMessage());
            throw (RegistryException) tmoex;
        } catch (NameTooLongException ntlex) {
            try {
                wscontextDataStore.rollback();
            } catch (Exception e) {}
            log.info(ntlex.getMessage());
            throw (RegistryException) ntlex;
        } catch (UnsupportedException suppex) {
            try {
                wscontextDataStore.rollback();
            } catch (Exception e) {}
            log.info(suppex.getMessage());
            throw (RegistryException) suppex;
        } catch (RegistryException regex) {
            try {
                wscontextDataStore.rollback();
            } catch (Exception e) {}
            log.error(regex);
            throw (RegistryException) regex;
        } catch (Exception ex) {
            try {
                wscontextDataStore.rollback();
            } catch (Exception e) {}
            log.error(ex);
            throw new RegistryException(ex);
        } finally {
            if (wscontextDataStore != null) {
                wscontextDataStore.release();
            }
        }
    }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) {
        // initialize the registry
        FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
        reg.init();

        try {
            FindFTHPISService request = new FindFTHPISService();

            // generate a find qualifier
            Vector qualifierVector = new Vector();
            qualifierVector.add(new FindQualifier(FindQualifier.OR_ALL_KEYS));
            FindQualifiers qualifiers = new FindQualifiers();
            qualifiers.setFindQualifierVector(qualifierVector);
            request.setFindQualifiers(qualifiers);

            // session key
            Vector sessionKeyVector = new Vector();
            SessionKey sessionKey = new SessionKey();
            sessionKey.setValue("656A4850-4508-11DA-BDFA-C3B7320D6066");
            sessionKeyVector.add(sessionKey);
            request.setSessionKeyVector(sessionKeyVector);

            // generate a dynamic context
            AttributeBag contextBag = new AttributeBag();
            Vector contextVector = new Vector();
            Attribute context = new Attribute();

            Vector nameVector = new Vector();
            nameVector.add(new Name("session_no"));
            context.setNameVector(nameVector);

            context.setAttributeFindQualifier(new AttributeFindQualifier(AttributeFindQualifiers.EQUAL_TO));
            contextVector.add(context);
            contextBag.setAttributeVector(contextVector);
            request.setDynamicContextBag(contextBag);

            // invoke the server
            ServiceList response = (ServiceList) (new FindFTHPISServiceFunction(reg).execute(request));

            ServiceInfos serviceInfos = response.getServiceInfos();
            Vector serviceInfoVector = serviceInfos.getServiceInfoVector();
            if (serviceInfoVector != null) {
                for (Iterator iter = serviceInfoVector.iterator(); iter.hasNext(); ) {
                    ServiceInfo serviceInfo = (ServiceInfo) iter.next();
                    log.info("serviceKey = " + serviceInfo.getServiceKey());
                }
            }

            log.info(response);
        } catch (Exception ex) {
            // write execption to the console
            ex.printStackTrace();
        } finally {
            // destroy the registry
            reg.dispose();
        }
    }
}

