
/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;
import java.util.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.CategoryBag;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.datatype.TModelBag;
import org.apache.juddi.datatype.request.FindQualifier;
import org.apache.juddi.datatype.request.FindQualifiers;
import org.apache.juddi.error.NameTooLongException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.TooManyOptionsException;
import org.apache.juddi.error.UnsupportedException;
import org.apache.juddi.util.Config;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.ExtendedUDDIDataStoreFactory;
import cgl.fthpis.datatype.request.FindService;
import cgl.fthpis.datatype.response.ServiceInfos;
import cgl.fthpis.datatype.response.ServiceInfo;
import cgl.fthpis.datatype.response.ServiceList;
import cgl.fthpis.datatype.SessionKey;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.attribute.*;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FindServiceFunction extends AbstractFunction {
    // private reference to FTHPIS logger
    private static Log log = LogFactory.getLog(FindServiceFunction.class);
    Properties bootstrap = null;
    /**
     *
     */
    public FindServiceFunction(FTHPISRegistryEngine registry) {
        super(registry);
    }

    /**
     *
     */
    public RegistryObject execute(RegistryObject regObject) throws
            RegistryException {
        log.info(
                "**** FindServiceFunction.execute(RegistryObject regObject) function called.");
        FindService request = (FindService) regObject;
        String generic = request.getGeneric();
        String businessKey = request.getBusinessKey();
        Vector nameVector = request.getNameVector();
        CategoryBag categoryBag = request.getCategoryBag();
        AttributeBag attributeBag = request.getAttributeBag();
        Lease lease = request.getLease();
        TModelBag tModelBag = request.getTModelBag();
        FindQualifiers qualifiers = request.getFindQualifiers();
        int maxRows = request.getMaxRows();

        //log.info("**** FindServiceFunction.execute(RegistryObject regObject) I");
        // first make sure we need to continue with this request. If
        // no arguments were passed in then we'll simply return
        // an empty ServiceList (aka "a zero match result set").
        if (((nameVector == null) || (nameVector.size() == 0)) &&
            ((categoryBag == null) || (categoryBag.size() == 0)) &&
            ((attributeBag == null) || (attributeBag.size() == 0)) &&
            ((lease == null)) &&
            ((tModelBag == null) || (tModelBag.size() == 0))) {
            //log.info("org.apache.juddi.function.FindServiceFunction => return empty list");
            ServiceList list = new ServiceList();
            list.setServiceInfos(new ServiceInfos());
            list.setGeneric(generic);
            list.setOperator(Config.getOperator());
            list.setTruncated(false);
            return list;
        }

        //log.info("**** FindServiceFunction.execute(RegistryObject regObject) III");
        // validate the 'name' parameters as much as possible up-front before
        // calling into the data layer for relational validation.
        if (nameVector != null) {
            // only allowed to specify a maximum of 5 names (implementation
            // dependent).  This value is configurable.
            int maxNames = Config.getMaxNameElementsAllowed();
            if ((nameVector != null) && (nameVector.size() > maxNames)) {
                throw new TooManyOptionsException("max_names= " + maxNames);
            }

            // names can not exceed the maximum character length of 255).
            // This value is configurable.
            int maxNameLength = Config.getMaxNameLengthAllowed();
            for (int i = 0; i < nameVector.size(); i++) {
                String name = ((Name) nameVector.elementAt(i)).getValue();
                if (name.length() > maxNameLength) {
                    throw new NameTooLongException(name + " (max=" +
                            maxNameLength + ")");
                }
            }
        }

        //log.info("**** FindServiceFunction.execute(RegistryObject regObject) IV");
        // validate the 'qualifiers' parameter as much as possible up-front before
        // calling into the data layer for relational validation.
        if (qualifiers != null) {
            Vector qVector = qualifiers.getFindQualifierVector();
            if ((qVector != null) && (qVector.size() > 0)) {
                for (int i = 0; i < qVector.size(); i++) {
                    FindQualifier qualifier = (FindQualifier) qVector.elementAt(
                            i);
                    String qValue = qualifier.getValue();

                    if ((!qValue.equals(FindQualifier.EXACT_NAME_MATCH)) &&
                        (!qValue.equals(FindQualifier.CASE_SENSITIVE_MATCH)) &&
                        (!qValue.equals(FindQualifier.OR_ALL_KEYS)) &&
                        (!qValue.equals(FindQualifier.OR_LIKE_KEYS)) &&
                        (!qValue.equals(FindQualifier.AND_ALL_KEYS)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_NAME_ASC)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_NAME_DESC)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_DATE_ASC)) &&
                        (!qValue.equals(FindQualifier.SORT_BY_DATE_DESC)) &&
                        (!qValue.equals(FindQualifier.SERVICE_SUBSET)) &&
                        (!qValue.equals(FindQualifier.COMBINE_CATEGORY_BAGS))
                            ) {
                        throw new UnsupportedException("findQualifier=" +
                                qValue);
                    }
                }
            }
        }
        boolean truncatedResults = false;
//        Vector infoVector = null;
//        //log.info("org.apache.juddi.function.FindServiceFunction.execute() => nameVector = " + nameVector);
//        if (nameVector != null) {
//            org.apache.juddi.datatype.Name name = (org.apache.juddi.datatype.
//                    Name) nameVector.elementAt(0);
//            log.info("name = " + name.getValue());
//        }
        //log.info("org.apache.juddi.function.FindServiceFunction.execute() => categoryBag = " + categoryBag);
        //log.info("org.apache.juddi.function.FindServiceFunction.execute() => attributeBag = " + attributeBag);
        //log.info("org.apache.juddi.function.FindServiceFunction.execute() => businessKey = " + businessKey);
        //log.info("org.apache.juddi.function.FindServiceFunction.execute() => qualifiers = " + qualifiers);
        //perform the search for matching business services (return only keys in requested order)
        //log.info("org.apache.juddi.function.FindServiceFunction.execute() => dataStore = " + dataStore);
        //log.info("**** FindServiceFunction.execute(RegistryObject regObject) V");

        Vector keyVector = null;
        FTHPISDataStore uddidataStore = ExtendedUDDIDataStoreFactory.getDataStore();
        log.info("**** FindServiceFunction.execute(RegistryObject regObject) VI");
        try {
            uddidataStore.beginTrans();
            log.info("**** FindServiceFunction.execute(RegistryObject regObject) VI-1");
            log.info("**** FindServiceFunction.execute(RegistryObject regObject) VI-2");
            keyVector = uddidataStore.findService(businessKey,
                                                  nameVector,
                                                  categoryBag,
                                                  tModelBag,
                                                  attributeBag,
                                                  lease,
                                                  qualifiers);

//            log.info("**** FindServiceFunction.execute(RegistryObject regObject) VIII + keyVector size " +
//                     keyVector.size());
//            if ((keyVector != null) && (keyVector.size() > 0)) {
//                //log.info("org.apache.juddi.function.FindServiceFunction.execute() => keyVector size = " + keyVector.size());
//                // if a maxRows value has been specified and it's less than
//                // the number of rows we are about to return then only return
//                // maxRows specified.
//                int rowCount = keyVector.size();
//                if ((maxRows > 0) && (maxRows < rowCount)) {
//                    rowCount = maxRows;
//                    truncatedResults = true;
//                }
//
//                // iterate through the business server keys fetching
//                // each associated ServiceInfo in sequence.
//                infoVector = new Vector(rowCount);
//                for (int i = 0; i < rowCount; i++) {
//                log.info("**** FindServiceFunction.execute(RegistryObject regObject) VIII-1");
//                    ServiceInfo fthpisServiceInfo =
//                            uddidataStore.fetchServiceInfo((String) keyVector.elementAt(i));
//                    log.info("**** FindServiceFunction.execute(RegistryObject regObject) VIII-2 " + " (String) keyVector.elementAt(i) " + (String) keyVector.elementAt(i) + "  => fthpisServiceInfo " + fthpisServiceInfo);
//                    infotable.put((String) keyVector.elementAt(i), fthpisServiceInfo);
//                log.info("**** FindServiceFunction.execute(RegistryObject regObject) VIII-3");
//
//                }
//            }
//
//            log.info("**** FindServiceFunction.execute(RegistryObject regObject) VII ");

            uddidataStore.commit();

            // create a new ServiceInfos instance and stuff
            // the new Vector of ServiceInfos into it.
            ServiceInfos infos = new ServiceInfos();
            Vector service_infos_vector = new Vector();
            for (Iterator iter = keyVector.iterator(); iter.hasNext(); ) {
                String item = (String) iter.next();
                ServiceInfo info = new ServiceInfo();
                info.setServiceKey(item);
                service_infos_vector.add(info);
            }


            infos.setServiceInfoVector(service_infos_vector);

            // create a new ServiceList instance and stuff
            // the new serviceInfos instance into it.
            ServiceList list = new ServiceList();
            list.setGeneric(generic);
            list.setServiceInfos(infos);
            list.setOperator(Config.getOperator());
            list.setTruncated(truncatedResults);
            return list;
        } catch (TooManyOptionsException tmoex) {
            try {
                uddidataStore.rollback();
            } catch (Exception e) {}
            log.info(tmoex.getMessage());
            throw (RegistryException) tmoex;
        } catch (NameTooLongException ntlex) {
            try {
                uddidataStore.rollback();
            } catch (Exception e) {}
            log.info(ntlex.getMessage());
            throw (RegistryException) ntlex;
        } catch (UnsupportedException suppex) {
            try {
                uddidataStore.rollback();
            } catch (Exception e) {}
            log.info(suppex.getMessage());
            throw (RegistryException) suppex;
        } catch (RegistryException regex) {
            try {
                uddidataStore.rollback();
            } catch (Exception e) {}
            log.error(regex);
            throw (RegistryException) regex;
        } catch (Exception ex) {
            try {
                uddidataStore.rollback();
            } catch (Exception e) {}
            log.error(ex);
            throw new RegistryException(ex);
        } finally {
            if (uddidataStore != null) {
                uddidataStore.release();
            }
        }

     }


    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) {
        // initialize the registry
        FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
        reg.init();

        try {
            FindService request = new FindService();

            request.setBusinessKey("5A647250-16CD-11DA-A9BC-D9EA77EFF5F9");

            Vector sessionKeyVector = new Vector();
            SessionKey sessionKey = new SessionKey();
            sessionKey.setValue("656A4850-4508-11DA-BDFA-C3B7320D6066");
            sessionKeyVector.add(sessionKey);
            request.setSessionKeyVector(sessionKeyVector);

            // generate a find qualifier
            Vector qualifierVector = new Vector();
            qualifierVector.add(new FindQualifier(FindQualifier.OR_ALL_KEYS));
            FindQualifiers qualifiers = new FindQualifiers();
            qualifiers.setFindQualifierVector(qualifierVector);
            request.setFindQualifiers(qualifiers);

            // generate a Name Vector
            Vector nameVector = new Vector();
            Name name = new Name();
            name.setValue("GIS Service");
            nameVector.add(name);
            request.setNameVector(nameVector);

            // generate a static context
            AttributeBag attributeBag = new AttributeBag();
            Vector attributeVector = new Vector();
            Attribute attribute = new Attribute();
            attribute.setAttributeName(new AttributeName("cpu_number"));
            attribute.setAttributeValueType(new AttributeValueType("number"));
            attribute.setAttributeValue(new AttributeValue(3));
            attribute.setAttributeFindQualifier(new AttributeFindQualifier(
                    AttributeFindQualifiers.EQUAL_TO));
            attributeVector.add(attribute);
            attributeBag.setAttributeVector(attributeVector);
            request.setAttributeBag(attributeBag);

            // generate a dynamic context
            AttributeBag contextBag = new AttributeBag();
            Vector contextVector = new Vector();
            Attribute context = new Attribute();
            context.setAttributeName(new AttributeName("session_no"));
            context.setAttributeValueType(new AttributeValueType("number"));
            context.setAttributeValue(new AttributeValue(6));
            context.setAttributeFindQualifier(new AttributeFindQualifier(
                    AttributeFindQualifiers.EQUAL_TO));
            contextVector.add(context);
            contextBag.setAttributeVector(contextVector);
            request.setDynamicContextBag(contextBag);

            // generate a Lease
            Lease lease = new Lease();
            lease.setLeaseExpirationDate("1169336085000");
            request.setLease(lease);

            // invoke the server
            ServiceList response =
                  (ServiceList) (new FindServiceFunction(reg).execute(request));

            ServiceInfos serviceInfos = response.getServiceInfos();
            Vector serviceInfoVector = serviceInfos.getServiceInfoVector();
            if (serviceInfoVector != null) {
                for (Iterator iter = serviceInfoVector.iterator(); iter.hasNext(); ) {
                    ServiceInfo serviceInfo = (ServiceInfo) iter.next();
                    log.info("serviceKey = " + serviceInfo.getServiceKey());
                }
            }

            log.info(response);
        } catch (Exception ex) {
            // write execption to the console
            ex.printStackTrace();
        } finally {
            // destroy the registry
            reg.dispose();
        }
    }
}

