/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.HashMap;

import org.apache.juddi.function.IFunction;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import cgl.fthpis.datatype.request.DeletePublisher;
import cgl.fthpis.datatype.request.DeleteService;
import cgl.fthpis.datatype.request.FindPublisher;
import cgl.fthpis.datatype.request.FindService;
import cgl.fthpis.datatype.request.GetAuthToken;
import cgl.fthpis.datatype.request.GetPublisherDetail;
import cgl.fthpis.datatype.request.GetServiceDetail;
import cgl.fthpis.datatype.request.SavePublisher;
import cgl.fthpis.datatype.request.SaveService;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.datatype.request.SaveSession;
import cgl.fthpis.datatype.request.FindSession;
import cgl.fthpis.datatype.request.FindServiceAttribute;
import cgl.fthpis.datatype.request.DeleteAttribute;
import cgl.fthpis.datatype.request.DeleteSession;
import cgl.fthpis.datatype.request.DeleteContext;
import cgl.fthpis.datatype.request.SaveAttribute;
import cgl.fthpis.datatype.request.GetAttributeDetail;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.request.GetSessionDetail;
import cgl.fthpis.datatype.request.FindContext;
import cgl.fthpis.datatype.request.SaveContext;
import cgl.fthpis.datatype.request.GetAuthTokenForWSContext;
import cgl.fthpis.datatype.request.SaveFTHPISService;
import cgl.fthpis.datatype.request.DeleteFTHPISService;
import cgl.fthpis.datatype.request.FindFTHPISService;
import cgl.fthpis.datatype.request.GetFTHPISServiceDetail;

/**
 * Holds a static HashMap linking the string representation of operations to
 * instantances of the appropriate maker class. Returns a reference to an instance
 * of a maker object. FunctionMaker follows the Singleton pattern.  Use getInstance
 * instead of the 'new' operator to get an instance of this class.
 *
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class FunctionMaker
{
  // private reference to the logger
  private static Log log = LogFactory.getLog(FunctionMaker.class);

  // table of shared Function instances
  private HashMap functions = null;

  /**
   *
   */
  public FunctionMaker(FTHPISRegistryEngine registry)
  {

    functions = new HashMap();
    IFunction function = null;


    //updated functions
    function = new DeleteFTHPISServiceFunction(registry);
    functions.put(DeleteFTHPISService.class.getName(),function);

    function = new FindFTHPISServiceFunction(registry);
    functions.put(FindFTHPISService.class.getName(),function);

    function = new SaveFTHPISServiceFunction(registry);
    functions.put(SaveFTHPISService.class.getName(),function);

    function = new GetFTHPISServiceDetailFunction(registry);
    functions.put(GetFTHPISServiceDetail.class.getName(),function);

    ///////////////////
    function = new DeleteServiceFunction(registry);
    functions.put(DeleteService.class.getName(),function);

    function = new FindServiceFunction(registry);
    functions.put(FindService.class.getName(),function);

    function = new SaveServiceFunction(registry);
    functions.put(SaveService.class.getName(),function);

    function = new GetServiceDetailFunction(registry);
    functions.put(GetServiceDetail.class.getName(),function);

    function = new DeletePublisherFunction(registry);
    functions.put(DeletePublisher.class.getName(),function);

    function = new FindPublisherFunction(registry);
    functions.put(FindPublisher.class.getName(),function);

    function = new SavePublisherFunction(registry);
    functions.put(SavePublisher.class.getName(),function);

    function = new GetPublisherDetailFunction(registry);
    functions.put(GetPublisherDetail.class.getName(),function);
    //newly added functions

    function = new GetContextDetailFunction(registry);
    functions.put(GetContextDetail.class.getName(),function);

    function = new FindContextFunction(registry);
    functions.put(FindContext.class.getName(),function);

    function = new SaveContextFunction(registry);
    functions.put(SaveContext.class.getName(),function);

    function = new DeleteContextFunction(registry);
    functions.put(DeleteContext.class.getName(),function);

    function = new GetAuthTokenForWSContextFunction(registry);
    functions.put(GetAuthTokenForWSContext.class.getName(),function);

    function = new GetSessionDetailFunction(registry);
    functions.put(GetSessionDetail.class.getName(),function);

    function = new FindSessionFunction(registry);
    functions.put(FindSession.class.getName(),function);

    function = new SaveSessionFunction(registry);
    functions.put(SaveSession.class.getName(),function);

    function = new DeleteSessionFunction(registry);
    functions.put(DeleteSession.class.getName(),function);

    function = new FindServiceAttributeFunction(registry);
    functions.put(FindServiceAttribute.class.getName(),function);

    function = new DeleteAttributeFunction(registry);
    functions.put(DeleteAttribute.class.getName(),function);

    function = new SaveAttributeFunction(registry);
    functions.put(SaveAttribute.class.getName(),function);

    function = new GetAttributeDetailFunction(registry);
    functions.put(GetAttributeDetail.class.getName(),function);

    function = new GetAuthTokenFunction(registry);
    functions.put(GetAuthToken.class.getName(),function);

  }

  /**
   *
   */
  public final IFunction lookup(String className)
  {
    IFunction function = (IFunction)functions.get(className);
    if (function == null)
      log.error("can't find function for class name \"" + className + "\"");

    return function;
  }
}
