/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datatype.Name;

import cgl.fthpis.datatype.service.BusinessService;
import cgl.fthpis.datatype.response.ServiceDetail;
import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.request.GetFTHPISServiceDetail;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class GetFTHPISServiceDetailFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(GetFTHPISServiceDetailFunction.class);
  Properties bootstrap = null;

  /**
   *
   */
  public GetFTHPISServiceDetailFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    GetFTHPISServiceDetail request = (GetFTHPISServiceDetail)regObject;
    String generic = request.getGeneric();
    Vector keyVector = request.getServiceKeyVector();

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();
    Vector serviceVector = new Vector();

    if (keyVector != null && keyVector.size() != 0) {
        try {
            dataStore.beginTrans();

            for (int i = 0; i < keyVector.size(); i++) {
                String serviceKey = (String) keyVector.elementAt(i);
                BusinessService service = (BusinessService) dataStore.
                                          fetchSessionService(serviceKey);
                service.setServiceKey(serviceKey);
                serviceVector.add(service);
            }
            dataStore.commit();
        } catch (InvalidKeyPassedException keyex) {
            try {
                dataStore.rollback();
            } catch (Exception e) {}
            log.info(keyex.getMessage());
            throw (RegistryException) keyex;
        } catch (RegistryException regex) {
            try {
                dataStore.rollback();
            } catch (Exception e) {}
            log.error(regex);
            throw (RegistryException) regex;
        } catch (Exception ex) {
            try {
                dataStore.rollback();
            } catch (Exception e) {}
            log.error(ex);
            throw new RegistryException(ex);
        } finally {
            if (dataStore != null)
                dataStore.release();
        }
    }
    // create a new ServiceDetail and stuff the new serviceVector into it.
    ServiceDetail detail = new ServiceDetail();
    detail.setGeneric(generic);
    detail.setOperator(Config.getOperator());
    detail.setBusinessServiceVector(serviceVector);
    return detail;
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
            Vector serviceKeyVector = new Vector();
            serviceKeyVector.add("BF7D8190-6C5E-11DA-8190-A62CFD90CF40");

            // create a request
            GetFTHPISServiceDetail request = new GetFTHPISServiceDetail();
            request.setServiceKeyVector(serviceKeyVector);

            // invoke the server
            ServiceDetail response = (ServiceDetail)(new
                    GetFTHPISServiceDetailFunction(reg).execute(request));
            //log.info("errno: " + response.toString());

            Vector businessServiceVector = response.getBusinessServiceVector();
            for (Iterator iter = businessServiceVector.iterator(); iter.hasNext(); ) {
                BusinessService service = (BusinessService) iter.next();
                log.info("SERVICE KEY = " + service.getServiceKey());

                Vector session_key_vector = service.getSessionKeyVector();
                for (Iterator iter2 = session_key_vector.iterator(); iter2.hasNext(); ) {
                    String session_key = (String) iter2.next();
                    log.info("SERVICE SESSION KEY = " + session_key);
                }

                  AttributeBag contextBag = service.getDynamicContextBag();
                  if (contextBag != null) {
                      Vector attributeVector = contextBag.getAttributeVector();
                      for (Iterator iter3 = attributeVector.iterator();
                                            iter3.hasNext(); ) {
                          Attribute context = (Attribute) iter3.next();
                          //log.info("context key = " + context.getAttributeKey());
                          GetContextDetail cntx_detail_request = new GetContextDetail();
                          Vector contextKeyVector = new Vector();
                          contextKeyVector.add(context.getAttributeKey());
                          cntx_detail_request.setContextKeyVector(contextKeyVector);
                          // invoke the server
                          AttributeDetail cntx_detail_response =
                                  (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                          Vector vector = cntx_detail_response.getContextVector();
                          for (Iterator iter1 = vector.iterator(); iter1.hasNext(); ) {
                              context = (Attribute) iter1.next();
                              log.info("context key => " + context.getAttributeKey());
                              log.info("context => session key = " + context.getSessionKey());
                              log.info("context => service key = " + context.getServiceKey());

                              Vector n_vector = context.getNameVector();
                              for (Iterator iterator = n_vector.iterator(); iterator.hasNext(); ) {
                                  Name name = (Name) iterator.next();
                                  log.info("context name = " +  name.getValue());
                              }

                              log.info("context value => " + context.getAttributeValue().getAttributeValue());
                              log.info("context version => " + context.getAttributeVersion());
                              Lease lease = context.getLease();
                              if (lease != null) {
                                  log.info("context lease start date = " +
                                                     lease.getLeaseStartDate().getLeaseStartDate());
                                  log.info("context lease end date = " +
                                                     lease.getLeaseExpirationDate().getLeaseExpirationDate());
                                }
                          }
                      }
                  }

            }

    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}

