/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.Description;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.ExtendedUDDIDataStoreFactory;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.datatype.service.BusinessService;
import cgl.fthpis.datatype.request.GetServiceDetail;
import cgl.fthpis.datatype.response.ServiceDetail;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.SessionKey;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class GetServiceDetailFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(GetServiceDetailFunction.class);
  Properties bootstrap = null;

  /**
   *
   */
  public GetServiceDetailFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    GetServiceDetail request = (GetServiceDetail)regObject;
    String generic = request.getGeneric();
    Vector keyVector = request.getServiceKeyVector();

    Hashtable detailtable = new Hashtable();

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = ExtendedUDDIDataStoreFactory.getDataStore();
    Vector serviceVector = new Vector();
    try
    {
      dataStore.beginTrans();

      for (int i=0; i<keyVector.size(); i++)
      {
        String serviceKey = (String)keyVector.elementAt(i);

        // If the a BusinessService doesn't exist hrow an InvalidKeyPassedException.
        if ((serviceKey == null) || (serviceKey.length() == 0) ||
            (!dataStore.isValidServiceKey(serviceKey)))
          throw new InvalidKeyPassedException(serviceKey);
      }

      for (int i=0; i<keyVector.size(); i++)
      {
        String serviceKey = (String)keyVector.elementAt(i);
        detailtable.put(serviceKey, dataStore.fetchService(serviceKey));
        //serviceVector.add(dataStore.fetchService(serviceKey));
      }
      dataStore.commit();
    }
    catch(InvalidKeyPassedException keyex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(keyex.getMessage());
      throw (RegistryException)keyex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(regex);
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(ex);
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
      	dataStore.release();
    }
    ////////////////////////////////////////////////////////////////////////////

    // aquire a FTHPIS datastore instance
    dataStore = WSContextDataStoreFactory.getDataStore();
    try
    {
      dataStore.beginTrans();

      for (int i=0; i<keyVector.size(); i++)
      {
        String serviceKey = (String)keyVector.elementAt(i);
        BusinessService semistatic = (BusinessService) detailtable.get(serviceKey);
        BusinessService dynamic = (BusinessService) dataStore.fetchSessionService(serviceKey);
        semistatic.setDynamicContextBag(dynamic.getDynamicContextBag());
        semistatic.setSessionKeyVector(dynamic.getSessionKeyVector());
        serviceVector.add(semistatic);
      }
      dataStore.commit();
    }
    catch(InvalidKeyPassedException keyex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.info(keyex.getMessage());
      throw (RegistryException)keyex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(regex);
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.error(ex);
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
        dataStore.release();
    }
    // create a new ServiceDetail and stuff the new serviceVector into it.
    ServiceDetail detail = new ServiceDetail();
    detail.setGeneric(generic);
    detail.setOperator(Config.getOperator());
    detail.setBusinessServiceVector(serviceVector);
    return detail;
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
//            // create & execute the GetAuthToken request
//            GetAuthToken authTokenRequest = new GetAuthToken("maktas",
//                    "changeIt");
//            AuthToken authToken = (AuthToken) (new GetAuthTokenFunction(reg).
//                                               execute(authTokenRequest));
//            log.info(" Auth Info: " + authToken.getAuthInfo());
//            AuthInfo authInfo = authToken.getAuthInfo();
//
//            GetAuthToken authTokenRequestForWSContext = new GetAuthToken(
//                    "maktas", "changeIt");
//            AuthToken authTokenForWSContext = (AuthToken) (new
//                    GetAuthTokenFunctionForWSContext(reg).execute(
//                    authTokenRequestForWSContext));
//            log.info(" Auth Info: " +
//                               authTokenForWSContext.getAuthInfo());
//            AuthInfo authInfoForWSContext = authTokenForWSContext.getAuthInfo();

            Vector serviceKeyVector = new Vector();
            serviceKeyVector.add("38A43770-5356-11DA-BD68-B154AF00895E");

            // create a request
            GetServiceDetail request = new GetServiceDetail();
            request.setServiceKeyVector(serviceKeyVector);

            // invoke the server
            ServiceDetail response = (ServiceDetail)(new
                    GetServiceDetailFunction(reg).execute(request));
            //log.info("errno: " + response.toString());

            Vector businessServiceVector = response.getBusinessServiceVector();
            for (Iterator iter = businessServiceVector.iterator(); iter.hasNext(); ) {
                BusinessService service = (BusinessService) iter.next();
                log.info("service key = " + service.getServiceKey());
                log.info("service business key = " + service.getBusinessKey());

                Vector session_key_vector = service.getSessionKeyVector();
                for (Iterator iter2 = session_key_vector.iterator(); iter2.hasNext(); ) {
                    SessionKey session_key = (SessionKey) iter2.next();
                    log.info("service session key = " + session_key.getValue());
                }

                Vector nameVector = service.getNameVector();
                for (Iterator iter1 = nameVector.iterator(); iter1.hasNext(); ) {
                    Name name = (Name) iter1.next();
                    log.info("service name = " + name.getValue());
                }

                Vector descVector = service.getDescriptionVector();
                for (Iterator iter2 = descVector.iterator(); iter2.hasNext(); ) {
                    Description desc = (Description) iter2.next();
                    log.info("service description = " + desc.getValue());
                  }

                  AttributeBag attributeBag = service.getAttributeBag();
                  if (attributeBag != null) {
                      Vector attributeVector = attributeBag.getAttributeVector();
                      for (Iterator iter3 = attributeVector.iterator();
                                            iter3.hasNext(); ) {
                          Attribute attribute = (Attribute) iter3.next();
                          log.info("attribute key = " +
                                             attribute.getAttributeKey());
//                          log.info("attribute name = " +
//                                             attribute.getAttributeName().
//                                             getAttributeName());
//                          log.info("attribute value = " +
//                                             attribute.getAttributeValue().
//                                             getAttributeValue());
                          Lease lease = attribute.getLease();
                          if (lease != null) {
                              log.info("attribute lease start date = " +
                                                 lease.getLeaseStartDate());
                              log.info("attribute lease end date = " +
                                                 lease.getLeaseExpirationDate());

                          }
                      }
                  }

                  AttributeBag contextBag = service.getDynamicContextBag();
                  if (contextBag != null) {
                      Vector attributeVector = contextBag.getAttributeVector();
                      for (Iterator iter3 = attributeVector.iterator();
                                            iter3.hasNext(); ) {
                          Attribute context = (Attribute) iter3.next();
                          //log.info("context key = " + context.getAttributeKey());
                          GetContextDetail cntx_detail_request = new GetContextDetail();
                          Vector contextKeyVector = new Vector();
                          contextKeyVector.add(context.getAttributeKey());
                          cntx_detail_request.setContextKeyVector(contextKeyVector);
                          // invoke the server
                          AttributeDetail cntx_detail_response =
                                  (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                          Vector vector = cntx_detail_response.getContextVector();
                          for (Iterator iter1 = vector.iterator(); iter1.hasNext(); ) {
                              context = (Attribute) iter1.next();
                              log.info("context key = " + context.getAttributeKey());
                              log.info("context => session key = " + context.getSessionKey());
                              log.info("context => service key = " + context.getServiceKey());
                              log.info("context name = " + context.getAttributeName().getAttributeName());
                              log.info("context value = " + context.getAttributeValue().getAttributeValue());
                              log.info("context version = " + context.getAttributeVersion());
                              Lease lease = context.getLease();
                              if (lease != null) {
                                  log.info("context lease start date = " +
                                                     lease.getLeaseStartDate());
                                  log.info("context lease end date = " +
                                                     lease.getLeaseExpirationDate());
                                }
                          }
                      }
                  }

                  Lease lease = service.getLease();
                  if (lease != null) {
                      log.info("service lease start date = " +
                                         lease.getLeaseStartDate());
                      log.info("service lease end date = " +
                                         lease.getLeaseExpirationDate());
                  }
            }

    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}

