/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.Name;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datatype.request.GetSessionDetail;
import cgl.fthpis.datatype.response.SessionDetail;
import cgl.fthpis.datatype.session.SessionEntity;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.response.AttributeDetail;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class GetSessionDetailFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(GetSessionDetailFunction.class);

  /**
   *
   */
  public GetSessionDetailFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    GetSessionDetail request = (GetSessionDetail)regObject;
    String generic = request.getGeneric();
    Vector keyVector = request.getSessionKeyVector();

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();

    try
    {
      dataStore.beginTrans();

      for (int i=0; i<keyVector.size(); i++)
      {
        // grab the next key from the vector
        String key = (String)keyVector.elementAt(i);

        // check that this session key really exists.
        // If not then throw an InvalidKeyPassedException.
        if ((key == null) || (key.length() == 0) ||
            (!dataStore.isValidSessionKey(key)))
          throw new InvalidKeyPassedException(key);
      }

      Vector sessionVector = new Vector();

      for (int i=0; i<keyVector.size(); i++)
      {
        String key = (String)keyVector.elementAt(i);
        sessionVector.add(dataStore.fetchSession(key));
      }

      dataStore.commit();

      // create a new SessionDetail and stuff the new sessionVector into it.
      SessionDetail detail = new SessionDetail();
      detail.setGeneric(generic);
      detail.setSessionEntityVector(sessionVector);
      detail.setOperator(Config.getOperator());
      return detail;
    }
    catch(InvalidKeyPassedException keyex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(keyex.getMessage());
      throw (RegistryException)keyex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(regex.toString());
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ex.toString());
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
      	dataStore.release();
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
        GetSessionDetail request = new GetSessionDetail();

        Vector sessionKeyVector = new Vector();
        sessionKeyVector.add("E67B6190-5593-11DA-A190-CD360C9F0C25");
        request.setSessionKeyVector(sessionKeyVector);
        // invoke the server
        SessionDetail response =
            (SessionDetail) (new GetSessionDetailFunction(reg).execute(request));

        Vector sessionEntityVector = response.getSessionEntityVector();
        for (Iterator iter = sessionEntityVector.iterator(); iter.hasNext(); ) {
            SessionEntity session = (SessionEntity) iter.next();
            log.info("session key = " + session.getSessionKey());

            Vector nameVector = session.getNameVector();
            for (Iterator iter1 = nameVector.iterator(); iter1.hasNext(); ) {
                Name name = (Name) iter1.next();
                log.info("name = " + name.getValue());
            }

            Vector descVector = session.getDescriptionVector();
            for (Iterator iter2 = descVector.iterator(); iter2.hasNext(); ) {
                Description desc = (Description) iter2.next();
                log.info("description = " + desc.getValue());
              }

              AttributeBag contextBag = session.getContextBag();
              if (contextBag != null) {
                  Vector attributeVector = contextBag.getAttributeVector();
                  for (Iterator iter3 = attributeVector.iterator();
                                        iter3.hasNext(); ) {
                      Attribute context = (Attribute) iter3.next();
                      //log.info("context key = " + context.getAttributeKey());
                      GetContextDetail cntx_detail_request = new GetContextDetail();
                      Vector contextKeyVector = new Vector();
                      contextKeyVector.add(context.getAttributeKey());
                      cntx_detail_request.setContextKeyVector(contextKeyVector);
                      // invoke the server
                      AttributeDetail cntx_detail_response =
                              (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                      Vector vector = cntx_detail_response.getContextVector();
                      for (Iterator iter1 = vector.iterator(); iter1.hasNext(); ) {
                          context = (Attribute) iter1.next();
                          log.info("context key = " + context.getAttributeKey());
                          log.info("context => session key = " + context.getSessionKey());
                          log.info("context => service key = " + context.getServiceKey());
                          log.info("context name = " + context.getAttributeName().getAttributeName());
                          log.info("context value = " + context.getAttributeValue().getAttributeValue());
                          log.info("context version = " + context.getAttributeVersion());
                          Lease lease = context.getLease();
                          if (lease != null) {
                              log.info("context lease start date = " +
                                                 lease.getLeaseStartDate());
                              log.info("context lease end date = " +
                                                 lease.getLeaseExpirationDate());
                            }
                          }
                  }
              }

              Lease lease = session.getLease();
              if (lease != null) {
                  log.info("session lease start date = " +
                                     lease.getLeaseStartDate());
                  log.info("session lease end date = " +
                                     lease.getLeaseExpirationDate());
              }
        }


    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}
