/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UserMismatchException;
import org.apache.juddi.util.Config;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.ExtendedUDDIDataStoreFactory;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.request.AuthInfo;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.function.GetAuthTokenFunction;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.request.SaveAttribute;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.request.GetAuthToken;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.request.GetAttributeDetail;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class SaveAttributeFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(SaveAttributeFunction.class);

  /**
   *
   */
  public SaveAttributeFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {
    log.info("SaveAttributeFunction.execute(RegistryObject regObject)");
    SaveAttribute request = (SaveAttribute)regObject;
    String generic = request.getGeneric();
    AuthInfo authInfo = request.getAuthInfo();
    Vector attributeVector = request.getAttributeVector();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = ExtendedUDDIDataStoreFactory.getDataStore();

    try
    {
      dataStore.beginTrans();

      // validate authentication parameters
      Publisher publisher = getPublisher(authInfo,dataStore);
      String publisherID = publisher.getPublisherID();

      log.info("publisherID = " + publisherID);

      // validate request parameters
      for (int i=0; i<attributeVector.size(); i++)
      {
        // move the BindingTemplate into a form we can work with easily
        Attribute attribute = (Attribute)attributeVector.elementAt(i);
        String attributeKey = attribute.getAttributeKey();
        String serviceKey = attribute.getServiceKey();

        log.info("attributeKey = " + attributeKey + " serviceKey = " + serviceKey);
        //String bindingKey = attribute.getBindingKey();

        // Confirm that the 'BusinessService' that this binding belongs to
        // really exists. If not then throw an InvalidKeyPassedException.
       if ((serviceKey == null) || (serviceKey.length() == 0) || (!dataStore.isValidServiceKey(serviceKey)))
          throw new InvalidKeyPassedException("ServiceKey: "+serviceKey);

        // Confirm that 'publisherID' controls the BusinessService that this
        // binding template belongs to.  If not then throw a UserMismatchException.
        if (!dataStore.isServicePublisher(serviceKey,publisherID))
          throw new UserMismatchException("ServiceKey: "+serviceKey);

//        // If a AttributeKey was specified then make sure it's a valid one.
//        if ((attributeKey != null) && (attributeKey.length() > 0) && (!dataStore.isValidAttributeKey(attributeKey)))
//          throw new InvalidKeyPassedException("AttributeKey: "+attributeKey);
      }

      for (int i=0; i<attributeVector.size(); i++)
      {
        // move the BindingTemplate data into a form we can work with easily
        Attribute attribute = (Attribute)attributeVector.elementAt(i);
        String attributeKey = attribute.getAttributeKey();

        // If the new BindingTemplate has a BindingKey then it must already
        // exists so delete the old one. It a BindingKey isn't specified then
        // this is a new BindingTemplate so create a new BindingKey for it.
        if ((attributeKey != null) && (attributeKey.length() > 0))
          dataStore.deleteAttribute(attributeKey);
        else
          attribute.setAttributeKey(uuidgen.uuidgen_with_timestamp());

        // everything checks out so let's save it.
        dataStore.saveAttribute(attribute);
      }

      dataStore.commit();

      AttributeDetail detail = new AttributeDetail();
      detail.setGeneric(generic);
      detail.setOperator(Config.getOperator());
      detail.setTruncated(false);
      detail.setAttributeVector(attributeVector);
      return detail;
    }
    catch(InvalidKeyPassedException ikpex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ikpex.toString());
      throw (RegistryException)ikpex;
    }
    catch(UserMismatchException umex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(umex.toString());
      throw (RegistryException)umex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(regex.toString());
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ex.toString());
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
      	dataStore.release();
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
            // create & execute the GetAuthToken request
            GetAuthToken authTokenRequest = new GetAuthToken("maktas","changeIt");
            AuthToken authToken = (AuthToken)(new GetAuthTokenFunction(reg).execute(authTokenRequest));
            log.info(" Auth Info: "+ authToken.getAuthInfo());
            AuthInfo authInfo = authToken.getAuthInfo();

        // generate a AttributeBag
        Attribute attribute = new Attribute();
        attribute.setServiceKey("0E8021D0-5594-11DA-BEEF-808188481FFF");

        Vector attributeVector = new Vector();
        attribute.setAttributeName(new AttributeName("cpu_number"));
        attribute.setAttributeValueType(new AttributeValueType("number"));
        attribute.setAttributeValue(new AttributeValue("3"));

        Lease context_lease = new Lease();
        context_lease.setIsInfinite("false");
        context_lease.setLeaseTimeOut("10000000000");
        attribute.setLease(context_lease);
        attributeVector.add(attribute);

        // create & execute the SaveSession request
        SaveAttribute request = new SaveAttribute();
        request.setAuthInfo(authInfo);
        request.setAttributeVector(attributeVector);

        AttributeDetail detail = (AttributeDetail)(new SaveAttributeFunction(reg).execute(request));

        if (detail != null) {
            Vector attribute_vector = detail.getAttributeVector();
            for (Iterator iter = attribute_vector.iterator(); iter.hasNext(); ) {
                Attribute item = (Attribute) iter.next();
                 log.info("attribute key = " + item.getAttributeKey());
                 GetAttributeDetail att_request = new GetAttributeDetail();

                 Vector attributeKeyVector = new Vector();
                 attributeKeyVector.add(item.getAttributeKey());
                 att_request.setAttributeKeyVector(attributeKeyVector);
                 // invoke the server
                 AttributeDetail response =
                     (AttributeDetail) (new GetAttributeDetailFunction(reg).execute(att_request));

                 Vector vector = response.getAttributeVector();
                 for (Iterator iter1 = vector.iterator(); iter1.hasNext(); ) {
                     attribute = (Attribute) iter1.next();
                     log.info("attribute key = " + attribute.getAttributeKey());
                     log.info("attribute => service key = " + attribute.getServiceKey());
                     log.info("attribute name = " + attribute.getAttributeName().getAttributeName());
                     log.info("attribute value = " + attribute.getAttributeValue().getAttributeValue());
                     log.info("attribute version = " + attribute.getAttributeVersion());
                     Lease lease = attribute.getLease();
                     if (lease != null) {
                         log.info("attribute lease start date = " +
                                            lease.getLeaseStartDate());
                         log.info("attribute lease end date = " +
                                            lease.getLeaseExpirationDate());
                       }
        }
            }
        }
    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}

