/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.fthpis.function;

import java.util.Vector;
import java.util.Vector;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UserMismatchException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datatype.Name;

import cgl.fthpis.datatype.request.AuthInfo;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.WSContextDataStoreFactory;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.request.SaveContext;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.function.GetAuthTokenForWSContextFunction;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.attribute.AttributeAssertion;
import cgl.fthpis.datatype.request.GetAuthTokenForWSContext;
import cgl.fthpis.datatype.attribute.AttributeAccessInfo;
import cgl.fthpis.datatype.attribute.AttributeAccessInfos;
import cgl.fthpis.datatype.publisher.PublisherID;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class SaveContextFunction extends AbstractFunction
{
  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(SaveContextFunction.class);

  /**
   *
   */
  public SaveContextFunction(FTHPISRegistryEngine registry)
  {
    super(registry);
  }

  /**
   *
   */
  public RegistryObject execute(RegistryObject regObject)
    throws RegistryException
  {

    log.info("SaveContextFunction.execute(RegistryObject regObject)");
    SaveContext request = (SaveContext)regObject;
    String generic = request.getGeneric();
    Vector contextVector = request.getContextVector();
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();
    log.info("SaveContextFunction I");

    AuthInfo authInfo = request.getAuthInfo();
    String publisher_id = authInfo.getPublisherID();
    PublisherID publisherID = new PublisherID(publisher_id);

    // aquire a FTHPIS datastore instance
    FTHPISDataStore dataStore = WSContextDataStoreFactory.getDataStore();

    try
    {
      dataStore.beginTrans();
           log.info("SaveContextFunction II");
          // validate request parameters
          for (int i = 0; i < contextVector.size(); i++) {
              // move the Context into a form we can work with easily
              Attribute context = (Attribute) contextVector.elementAt(i);
              String contextKey = context.getAttributeKey();
              context.setAttributeOwnerPublisherID(publisherID);
              log.info("SaveContextFunction III");

//              // If a ContextKey was specified then make sure it's a valid one.
//              if ((contextKey != null) && (contextKey.length() > 0) &&
//                  (!dataStore.isValidContextKey(contextKey)))
//                  throw new InvalidKeyPassedException("ContextKey: " + contextKey);

//              // THING_TO_DO: if the context is already exists then we must check
//              // if the AccessRightInfo for that context includes this particular
//              // publisher, so that he/she can modify it
//              if ((contextKey != null) && (contextKey.length() > 0) &&
//                  (!dataStore.isContextPublisher(contextKey, publisherID)))
//                  throw new UserMismatchException("ContextKey = " + contextKey);

              log.info("SAVE CONTEXT FUNCTION => contextKey = " + contextKey);
              // If the new Context has a ContextKey then it must already
              // exists so delete the old one. It a ContextKey isn't specified then
              // this is a new Context so create a new ContextKey for it.
              if ((contextKey != null) && (contextKey.length() > 0)) {
                  dataStore.deleteContext(contextKey);
              } else {
                  context.setAttributeKey("uuid:" + uuidgen.uuidgen_with_timestamp());
              }
              // everything checks out so let's save it.
              dataStore.saveContext(context);
          }
      dataStore.commit();

      AttributeDetail detail = new AttributeDetail();
      detail.setGeneric(generic);
      detail.setOperator(Config.getOperator());
      detail.setTruncated(false);
      detail.setContextVector(contextVector);
      return detail;
    }
    catch(InvalidKeyPassedException ikpex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ikpex.toString());
      throw (RegistryException)ikpex;
    }
    catch(UserMismatchException umex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(umex.toString());
      throw (RegistryException)umex;
    }
    catch(RegistryException regex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(regex.toString());
      throw (RegistryException)regex;
    }
    catch(Exception ex)
    {
      try { dataStore.rollback(); } catch(Exception e) { }
      log.warn(ex.toString());
      throw new RegistryException(ex);
    }
    finally
    {
      if (dataStore != null)
      	dataStore.release();
    }
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
  {
    // initialize the registry
    FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
    reg.init();

    try
    {
        GetAuthTokenForWSContext authTokenRequest = new GetAuthTokenForWSContext("maktas","changeIt");
        AuthToken authToken = (AuthToken)(new GetAuthTokenForWSContextFunction(reg).execute(authTokenRequest));
        log.info(" AuthInfo: "+ authToken.getAuthInfo());
        AuthInfo authInfo = authToken.getAuthInfo();

        // generate a AttributeBag
        Attribute context = new Attribute();
        Vector contextVector = new Vector();

        context.setAttributeName(new AttributeName("session_no"));
        context.setAttributeValueType(new AttributeValueType("number"));
        context.setAttributeValue(new AttributeValue("0006"));

        // generate a Name Vector
        Vector nameVector = new Vector();
        nameVector.add(new Name("GIS Session"));
        context.setNameVector(nameVector);

        Lease context_lease = new Lease();
        context_lease.setLeaseTimeOut("10000000");
        context_lease.setIsInfinite("false");
        context.setLease(context_lease);

        //insert access right info
        AttributeAccessInfos infos = new AttributeAccessInfos();

        AttributeAccessInfo group_info = new AttributeAccessInfo();
        ////////////////////////////////////////////////////////////////////////
        Vector publisherIDVector = new Vector();
        PublisherID publisherID = new PublisherID();
        publisherID.setValue("ali");
        publisherIDVector.add(publisherID);
        PublisherID publisher_ID = new PublisherID();
        publisher_ID.setValue("veli");
        publisherIDVector.add(publisher_ID);
        group_info.setPublisherIDVector(publisherIDVector);
        ////////////////////////////////////////////////////////////////////////
        group_info.setReadOK(true);
        group_info.setisReadWriteOK(false);
        infos.setGroupAttributeAccessInfo(group_info);

        AttributeAccessInfo others_info = new AttributeAccessInfo();
        others_info.setReadOK(true);
        others_info.setisReadWriteOK(true);
        infos.setOthersAttributeAccessInfo(others_info);

        context.setAttributeAccessRightInfos(infos);

        //insert some assertions
        AttributeAssertion context_assertion = new AttributeAssertion();
        context_assertion.setAssertionProperty("property_key");
        context_assertion.setAssertionObjectKey("some_id");
        Vector context_assertion_vector = new Vector();
        context_assertion_vector.add(context_assertion);
        context.setAttributeAssertionVector(context_assertion_vector);
        contextVector.add(context);

        // create & execute the SaveSession request
        SaveContext request = new SaveContext();
        request.setAuthInfo(authInfo);
        request.setGeneric("session");
        request.setServiceKey("A843EE10-4508-11DA-AB75-C1DB9A0EE37C");
        request.setSessionKey("656A4850-4508-11DA-BDFA-C3B7320D6066");
        request.setContextVector(contextVector);

        AttributeDetail detail = (AttributeDetail)reg.execute(request);

        if (detail != null) {
            Vector context_vector = detail.getContextVector();
            for (Iterator iter = context_vector.iterator(); iter.hasNext(); ) {
                context = (Attribute) iter.next();
                GetContextDetail cntx_detail_request = new GetContextDetail();
                Vector contextKeyVector = new Vector();
                contextKeyVector.add(context.getAttributeKey());
                cntx_detail_request.setContextKeyVector(contextKeyVector);
                // invoke the server
                AttributeDetail cntx_detail_response =
                        (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                Vector vector = cntx_detail_response.getContextVector();
                for (Iterator iter1 = vector.iterator(); iter1.hasNext(); ) {
                    context = (Attribute) iter1.next();
                    log.info("context key = " + context.getAttributeKey());
                    if (context.geAttributeOwnerPublisherID() != null) {
                        log.info("context owner publisher id = " +
                                 context.geAttributeOwnerPublisherID().getValue());
                    }
                    log.info("context => session key = " + context.getSessionKey());
                    log.info("context => service key = " + context.getServiceKey());
                    Vector name_vector = context.getNameVector();
                    if (name_vector != null && name_vector.size() != 0) {
                        Name context_name = (Name) name_vector.elementAt(0);
                        log.info("context name = " + context_name.getValue());
                    }
                    log.info("context value = " + context.getAttributeValue().getAttributeValue());
                    log.info("context version = " + context.getAttributeVersion());
                    Lease lease = context.getLease();
                    if (lease != null) {
                        log.info("context lease start date = " +
                                           lease.getLeaseStartDate());
                        log.info("context lease end date = " +
                                           lease.getLeaseExpirationDate());
                      }
                    }

            }
        }


    }
    catch (Exception ex)
    {
      // write execption to the console
      ex.printStackTrace();
    }
    finally
    {
      // destroy the registry
      reg.dispose();
    }
  }
}

