/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


package cgl.fthpis.function;

import java.util.Vector;
import java.util.Iterator;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.datatype.Description;
import org.apache.juddi.datatype.Name;
import org.apache.juddi.datatype.RegistryObject;
import org.apache.juddi.error.InvalidKeyPassedException;
import org.apache.juddi.error.RegistryException;
import org.apache.juddi.error.UserMismatchException;
import org.apache.juddi.util.Config;
import org.apache.juddi.datastore.DataStore;
import org.apache.juddi.datastore.DataStoreFactory;

import cgl.fthpis.datastore.FTHPISDataStore;
import cgl.fthpis.datastore.ExtendedUDDIDataStoreFactory;
import cgl.fthpis.datatype.publisher.Publisher;
import cgl.fthpis.datatype.request.AuthInfo;
import cgl.fthpis.datatype.request.GetAuthToken;
import cgl.fthpis.datatype.request.SaveService;
import cgl.fthpis.datatype.response.AuthToken;
import cgl.fthpis.datatype.response.ServiceDetail;
import cgl.fthpis.datatype.service.BusinessService;
import cgl.fthpis.registry.FTHPISRegistryEngine;
import cgl.fthpis.function.AbstractFunction;
import cgl.sequencer.uuidgen.UUIDGen;
import cgl.sequencer.uuidgen.UUIDGenFactory;
import cgl.fthpis.datatype.attribute.AttributeName;
import cgl.fthpis.datatype.attribute.Attribute;
import cgl.fthpis.datatype.attribute.AttributeValueType;
import cgl.fthpis.datatype.attribute.AttributeBag;
import cgl.fthpis.datatype.lease.Lease;
import cgl.fthpis.datatype.attribute.AttributeValue;
import cgl.fthpis.datatype.SessionKey;
import cgl.fthpis.function.GetAuthTokenFunction;
import cgl.fthpis.datatype.request.GetServiceDetail;
import cgl.fthpis.datatype.request.GetContextDetail;
import cgl.fthpis.datatype.response.AttributeDetail;
import cgl.fthpis.datatype.request.GetAttributeDetail;
import cgl.fthpis.datatype.response.AttributeDetail;

/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */
public class SaveServiceFunction extends AbstractFunction {
    // private reference to FTHPIS logger
    private static Log log = LogFactory.getLog(SaveServiceFunction.class);
    Properties bootstrap = null;

    /**
     *
     */
    public SaveServiceFunction(FTHPISRegistryEngine registry) {
        super(registry);
    }

    /**
     *
     */
    public RegistryObject execute(RegistryObject regObject) throws
            RegistryException {
        log.info(
                "SaveServiceFuntion.execute(RegistryObject regObject) function called.");
        SaveService request = (SaveService) regObject;
        //Let's use this generic variable to define what we are saving
        String generic = request.getGeneric();
        AuthInfo authInfo = request.getAuthInfo();
        Vector serviceVector = request.getBusinessServiceVector();
        UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();

        // aquire a FTHPIS datastore instance
        FTHPISDataStore datastore = ExtendedUDDIDataStoreFactory.getDataStore();

        DataStore juddiStore = DataStoreFactory.getDataStore();

        try {
            datastore.beginTrans();

            juddiStore.beginTrans();

            // validate authentication parameters
            Publisher publisher = getPublisher(authInfo, datastore);
            String publisherID = publisher.getPublisherID();
            log.info("SaveServiceFuntion.execute(RegistryObject regObject) I. publisherID = " + publisherID);
            // validate request parameters & execute
            for (int i = 0; i < serviceVector.size(); i++) {
                // move the BusinessService data into a form we can work with easily
                BusinessService service = (BusinessService) serviceVector. elementAt(i);
                String businessKey = service.getBusinessKey();
                String serviceKey = service.getServiceKey();
                log.info("businessKey = " + businessKey);
                log.info("serviceKey = " + serviceKey);
                if (businessKey != null) {
                    // If a BusinessKey wasn't included or it is an invalid BusinessKey then
                    // throw an InvalidKeyPassedException
                    if ((businessKey == null) || (businessKey.length() == 0) ||
                        (!juddiStore.isValidBusinessKey(businessKey))) {
                        throw new InvalidKeyPassedException("businessKey=" +
                                businessKey);
                    }

                    // Confirm that 'publisherID' controls the BusinessEntity that this
                    // BusinessService belongs to.  If not then throw a UserMismatchException.
                    if (!juddiStore.isBusinessPublisher(businessKey, publisherID)) {
                        throw new UserMismatchException("businessKey=" +
                                businessKey);
                    }
                    log.info("SaveServiceFuntion.execute(RegistryObject regObject) II.");
                }
                // If a ServiceKey was specified then make sure it's a valid one.
                if (((serviceKey != null) && (serviceKey.length() > 0))
                    &&
                    (!datastore.isValidServiceKey(serviceKey))) {
                    throw new InvalidKeyPassedException("serviceKey=" + serviceKey);
                }
            }
            log.info("SaveServiceFuntion.execute(RegistryObject regObject) III.");
            for (int i = 0; i < serviceVector.size(); i++) {
                // move the BusinessService data into a form we can work with easily
                BusinessService service = (BusinessService) serviceVector.elementAt(i);
                String serviceKey = service.getServiceKey();
                log.info("SaveServiceFuntion.execute(RegistryObject regObject) IV.");
                // If the new BusinessService has a ServiceKey then it must already
                // exists so delete the old one. It a ServiceKey isn't specified then
                // this is a new BusinessService so create a new ServiceKey for it.
                if ((serviceKey != null) && (serviceKey.length() > 0)) {
                    datastore.deleteService(serviceKey);
                } else {
                    serviceKey = uuidgen.uuidgen();
                    service.setServiceKey(serviceKey);
                }
                log.info("SaveServiceFuntion.execute(RegistryObject regObject) V.");
                // everything checks out so let's save it.
                datastore.saveService(service);
            }
            log.info("SaveServiceFuntion.execute(RegistryObject regObject) VI.");

            juddiStore.commit();

            datastore.commit();

            // create a new ServiceDetail and stuff the
            // original but 'updated' serviceVector in.

            ServiceDetail detail = new ServiceDetail();
            detail.setGeneric(generic);
            detail.setOperator(Config.getOperator());
            detail.setTruncated(false);
            detail.setBusinessServiceVector(serviceVector);
            return detail;
        } catch (InvalidKeyPassedException ikpex) {
            try {
                datastore.rollback();
            } catch (Exception e) {}
            log.warn(ikpex.toString());
            throw (RegistryException) ikpex;
        } catch (UserMismatchException umex) {
            try {
                datastore.rollback();
            } catch (Exception e) {}
            log.warn(umex.toString());
            throw (RegistryException) umex;
        } catch (RegistryException regex) {
            try {
                datastore.rollback();
            } catch (Exception e) {}
            log.warn(regex.toString());
            throw (RegistryException) regex;
        } catch (Exception ex) {
            try {
                datastore.rollback();
            } catch (Exception e) {
                e.printStackTrace();
            }
            log.warn(ex.toString());
            ex.printStackTrace();
            throw new RegistryException(ex);
        } finally {
            if (datastore != null) {
                datastore.release();
            }
            if (juddiStore != null) {
                juddiStore.release();
            }
        }
    }

    /***************************************************************************/
    /***************************** TEST DRIVER *********************************/
    /***************************************************************************/


    public static void main(String[] args) {
        // initialize the registry
        FTHPISRegistryEngine reg = new FTHPISRegistryEngine();
        reg.init();

        try {
            // create & execute the GetAuthToken request
            GetAuthToken authTokenRequest = new GetAuthToken("maktas","changeIt");
            AuthToken authToken = (AuthToken)(new GetAuthTokenFunction(reg).execute(authTokenRequest));
            log.info(" Auth Info: "+ authToken.getAuthInfo());
            AuthInfo authInfo = authToken.getAuthInfo();

//            GetAuthToken authTokenRequestForWSContext = new GetAuthToken("maktas","changeIt");
//            AuthToken authTokenForWSContext = (AuthToken)(new GetAuthTokenForWSContextFunction(reg).execute(authTokenRequestForWSContext));
//            log.info(" Auth Info: "+ authTokenForWSContext.getAuthInfo());
//            AuthInfo authInfoForWSContext = authTokenForWSContext.getAuthInfo();

            // generate a BusinessService
            BusinessService service = new BusinessService();

            service.addName(new Name("GIS Service"));
            service.addDescription(new Description("This is a test description"));
            service.setBusinessKey("A4A8F1E0-6616-11DA-BB77-C97A3D6DC484");
            Vector sessionKeyVector = new Vector();
            SessionKey sessionKey = new SessionKey();
            sessionKey.setValue("656A4850-4508-11DA-BDFA-C3B7320D6066");
            sessionKeyVector.add(sessionKey);
            service.setSessionKeyVector(sessionKeyVector);


            //CONSTRUCT BINDING TEMPLATES
            org.apache.juddi.datatype.binding.BindingTemplates
                bindingTemplates = new org.apache.juddi.datatype.binding.BindingTemplates();
            org.apache.juddi.datatype.binding.BindingTemplate bindingTemplate
                    = new org.apache.juddi.datatype.binding.BindingTemplate();
            org.apache.juddi.datatype.binding.AccessPoint accesspoint
                    = new org.apache.juddi.datatype.binding.AccessPoint();
            accesspoint.setURL("some_url");
            accesspoint.setURLType("http");
            bindingTemplate.setAccessPoint(accesspoint);
            Vector bindingVector = new Vector();
            bindingVector.add(bindingTemplate);
            bindingTemplates.setBindingTemplateVector(bindingVector);
            service.setBindingTemplates(bindingTemplates);

            // generate a AttributeBag
            AttributeBag dynamicContextBag = new AttributeBag();
            Attribute context = new Attribute();
            //context.setSessionKey("B25B1BA0-412D-11DA-9BA0-AD3374C97DA6");
            context.setAttributeName(new AttributeName("session_no"));
            context.setAttributeValueType(new AttributeValueType("number"));
            context.setAttributeValue(new AttributeValue("0006"));
            Lease context_lease = new Lease();
            context_lease.setIsInfinite("false");
            context_lease.setLeaseTimeOut("100000000000000");
            context.setLease(context_lease);
            dynamicContextBag.addAttribute(context);
            service.setDynamicContextBag(dynamicContextBag);

            // generate a AttributeBag
            AttributeBag semistaticContextBag = new AttributeBag();
            Attribute attribute = new Attribute();
            attribute.setAttributeName(new AttributeName("cpu_number"));
            attribute.setAttributeValueType(new AttributeValueType("number"));
            attribute.setAttributeValue(new AttributeValue("6"));
            Lease attribute_lease = new Lease();
            attribute_lease.setIsInfinite("false");
            attribute_lease.setLeaseTimeOut("100000000000000");
            attribute.setLease(context_lease);
            semistaticContextBag.addAttribute(attribute);
            service.setAttributeBag(semistaticContextBag);

            // generate a Lease
            Lease lease = new Lease();
            lease.setIsInfinite("false");
            lease.setLeaseTimeOut("100000000000000");
            service.setLease(lease);

            // generate a BusinessEntity Vector
            Vector servicesVector = new Vector();
            servicesVector.add(service);

            // create & execute the SaveBusiness request
            SaveService request = new SaveService();
            request.setAuthInfo(authInfo);
//            request.setAuthInfoForWSContext(authInfoForWSContext);
            request.setServiceVector(servicesVector);
            ServiceDetail detail = (ServiceDetail) reg.execute(request);
            String service_key = "";
            if (detail != null) {
                Vector vector = detail.getBusinessServiceVector();
                if (vector != null) {
                    for (Iterator iter = vector.iterator(); iter.hasNext(); ) {
                        BusinessService bservice = (BusinessService) iter.next();
                        //log.info("service key = " + bservice.getServiceKey());
                        service_key = bservice.getServiceKey();
                        Vector serviceKeyVector = new Vector();
                        serviceKeyVector.add(service_key);
                        // create a request
                        GetServiceDetail request_detail = new GetServiceDetail();
                        request_detail.setServiceKeyVector(serviceKeyVector);
                        // invoke the server
                        ServiceDetail response = (ServiceDetail)(new
                                GetServiceDetailFunction(reg).execute(request_detail));
                        //log.info("errno: " + response.toString());

                        Vector businessServiceVector = response.getBusinessServiceVector();
                        for (Iterator iter1 = businessServiceVector.iterator(); iter1.hasNext(); ) {
                            service = (BusinessService) iter1.next();
                            log.info("service key = " + service.getServiceKey());
                            log.info("service business key = " + service.getBusinessKey());

                            Vector session_key_vector = service.getSessionKeyVector();
                            for (Iterator iter2 = session_key_vector.iterator(); iter2.hasNext(); ) {
                                String session_key = (String) iter2.next();
                                log.info("service session key = " + session_key);
                            }

                            Vector nameVector = service.getNameVector();
                            for (Iterator iter2 = nameVector.iterator(); iter2.hasNext(); ) {
                                Name name = (Name) iter2.next();
                                log.info("service name = " + name.getValue());
                            }

                            Vector descVector = service.getDescriptionVector();
                            for (Iterator iter2 = descVector.iterator(); iter2.hasNext(); ) {
                                Description desc = (Description) iter2.next();
                                log.info("service description = " + desc.getValue());
                              }

                              AttributeBag attributeBag = service.getAttributeBag();
                              if (attributeBag != null) {
                                  Vector attributeVector = attributeBag.getAttributeVector();
                                  for (Iterator iter3 = attributeVector.iterator();
                                                        iter3.hasNext(); ) {
                                      attribute = (Attribute) iter3.next();
                                      //log.info("attribute key = " + attribute.getAttributeKey());
                                      GetAttributeDetail attribute_detail_request = new GetAttributeDetail();

                                     Vector attributeKeyVector = new Vector();
                                     attributeKeyVector.add(attribute.getAttributeKey());
                                     attribute_detail_request.setAttributeKeyVector(attributeKeyVector);
                                     // invoke the server
                                     AttributeDetail attribute_detail_response =
                                         (AttributeDetail) (new GetAttributeDetailFunction(reg).execute(attribute_detail_request));

                                     Vector vec = attribute_detail_response.getAttributeVector();
                                     for (Iterator iter2 = vec.iterator(); iter2.hasNext(); ) {
                                         attribute = (Attribute) iter2.next();
                                         log.info("attribute key = " + attribute.getAttributeKey());
                                         log.info("attribute => service key = " + attribute.getServiceKey());
                                         log.info("attribute name = " + attribute.getAttributeName().getAttributeName());
                                         log.info("attribute value = " + attribute.getAttributeValue().getAttributeValue());
                                         log.info("attribute version = " + attribute.getAttributeVersion());
                                         lease = attribute.getLease();
                                         if (lease != null) {
                                             log.info("attribute lease start date = " +
                                                                lease.getLeaseStartDate());
                                             log.info("attribute lease end date = " +
                                                                lease.getLeaseExpirationDate());
                                           }
                                     }
                                       }
                              }

                              AttributeBag contextBag = service.getDynamicContextBag();
                              if (contextBag != null) {
                                  Vector attributeVector = contextBag.getAttributeVector();
                                  for (Iterator iter3 = attributeVector.iterator();
                                                        iter3.hasNext(); ) {
                                      context = (Attribute) iter3.next();
                                      //log.info("context key = " + context.getAttributeKey());
                                      GetContextDetail cntx_detail_request = new GetContextDetail();
                                      Vector contextKeyVector = new Vector();
                                      contextKeyVector.add(context.getAttributeKey());
                                      cntx_detail_request.setContextKeyVector(contextKeyVector);
                                      // invoke the server
                                      AttributeDetail cntx_detail_response =
                                              (AttributeDetail) (new GetContextDetailFunction(reg).execute(cntx_detail_request));

                                      vector = cntx_detail_response.getContextVector();
                                      for (Iterator iter4 = vector.iterator(); iter4.hasNext(); ) {
                                          context = (Attribute) iter4.next();
                                          log.info("context key = " + context.getAttributeKey());
                                          log.info("context => session key = " + context.getSessionKey());
                                          log.info("context => service key = " + context.getServiceKey());
                                          log.info("context name = " + context.getAttributeName().getAttributeName());
                                          log.info("context value = " + context.getAttributeValue().getAttributeValue());
                                          log.info("context version = " + context.getAttributeVersion());
                                          lease = context.getLease();
                                          if (lease != null) {
                                              log.info("context lease start date = " +
                                                                 lease.getLeaseStartDate());
                                              log.info("context lease end date = " +
                                                                 lease.getLeaseExpirationDate());
                                            }
                    }
                                  }
                              }

                              lease = service.getLease();
                              if (lease != null) {
                                  log.info("service lease start date = " +
                                                     lease.getLeaseStartDate());
                                  log.info("service lease end date = " +
                                                     lease.getLeaseExpirationDate());
                              }
                        }

                    }
                }
            }






        } catch (Exception ex) {
            // write execption to the console
            ex.printStackTrace();
        } finally {
            // destroy the registry
            reg.dispose();
        }
    }
}
