/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.notifier;

import java.io.*;
import cgl.narada.event.NBEvent;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.ContextDetail;
import cgl.util.BootStrap;
import java.util.Properties;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.Context;
import cgl.axis.services.uddi.uddi_schema.Name;
import cgl.axis.services.WSCTX.uddi_wsctx_schema.ContextBag;
import cgl.narada.service.client.ClientService;
import cgl.narada.service.client.SessionService;
import cgl.narada.service.ServiceException;
import cgl.narada.service.client.EventProducer;
import cgl.narada.event.TemplateProfileAndSynopsisTypes;
import cgl.narada.service.client.EventConsumer;
import cgl.narada.service.client.NBEventListener;
import cgl.narada.matching.Profile;
import java.util.Random;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 */

public class ContextListener implements NBEventListener {

    private Object synchObj = new Object();
    private static Properties bootstrap = null;
    private String hostName = null;
    private String portNum = null;
    private Random generator = null;
    private static ContextListener nb_subscriber = null;
    private ClientService clientService = null;
    private int entityId;
    private String serviceConfigPath = null;
    private EventConsumer listener = null;
    Profile profile = null;
    private static Log log = LogFactory.getLog(ContextListener.class);

    public ContextListener() {
        try {
            this.bootstrap = BootStrap.loadProperties("properties.properties");
            this.hostName = bootstrap.getProperty("hostname");
            this.portNum = bootstrap.getProperty("portnum");
            generator = new Random(System.currentTimeMillis());
            entityId = generator.nextInt(1000000);
            String endpoint = "niotcp://" + this.hostName + ":" + this.portNum;
            log.info("endpoint ----> " + endpoint);
            log.info("entityId ----> " + entityId);
            log.info("serviceconfigpath ----> " + this.serviceConfigPath);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void initBroker() {
        try {
//            SessionService.setServiceConfigurationLocation("properties.properties"/*bootstrap.getProperty("serviceConfigPath")/*"E:/MyApps/LANL_Projects/narada/NaradaBrokering-1.0-RC1/config/ServiceConfiguration.txt"*/);
            SessionService.setServiceConfigurationLocation("properties.properties");
            clientService = SessionService.getClientService(entityId);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        try {
            clientService.initializeBrokerCommunications(bootstrap, "niotcp");
            // initializeTransmitter();
            // connector.initializeReceiver();
        } catch (ServiceException ex) {
            ex.printStackTrace();
       }
    }

    public void onEvent(NBEvent nbEvent) {
        log.info("NBEvent received:");
        synchronized (synchObj) {
            Context context = null;
            context = (Context)this.deserializeObject(nbEvent.getContentPayload());
            printContext(context);
        }
    }

    private void subscribe(String topic) {
        try {
            listener = clientService.createEventConsumer(this);
            profile = clientService.createProfile(TemplateProfileAndSynopsisTypes.STRING, topic);
            listener.subscribeTo(profile);
        } catch (ServiceException ex) {
        }

    }

    public void printContext(Context context) {
                    if (context != null) {
                        log.info(
                                "********* PRINT Context ***********");
                        log.info("saved context_key = " +
                                           context.getContextKey());
                        log.info("context operation type = " + context.getValueType());
                        Name[] returned_name_array = context.getName();
                        if (returned_name_array != null &&
                            returned_name_array.length != 0) {
                            log.info("context user identifer = " +
                                               returned_name_array[0].getValue());
                        }
                        log.info("context session key = " +
                                           context.getSessionKey());
                        log.info("context DATA = " + context.getValue());
                     }
    }

    public void printContextDetail(ContextDetail detail) {
        if (detail != null) {
            ContextBag contextBag = detail.getContextBag();
            if (contextBag != null &&
                contextBag.getContext() != null &&
                contextBag.getContext().length != 0) {
                Context[] context_arr = contextBag.getContext();
                for (int i = 0; i < context_arr.length; i++) {
                    Context context = context_arr[i];
                    if (context != null) {
                        log.info(
                                "********* PRINT Context ***********");
                        log.info("saved context_key = " +
                                           context.getContextKey());
                        Name[] returned_name_array = context.getName();
                        if (returned_name_array != null &&
                            returned_name_array.length != 0) {
                            log.info("context user identifer = " +
                                               returned_name_array[0].getValue());
                        }
                        log.info("context session key = " +
                                           context.getSessionKey());
                        log.info("context DATA = " + context.getValue());
                     }
                }
            }
        }
    }

    protected Object deserializeObject(byte[] bytes) {
        Object obj = null;
        try {
            ObjectInputStream in = new ObjectInputStream(new
                    ByteArrayInputStream(
                            bytes));
            obj = in.readObject();
            in.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return obj;
    }

    protected byte[] serializeObject(Object object) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            ObjectOutput out = new ObjectOutputStream(bos);
            out.writeObject(object);
            out.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } // Get the bytes of the serialized object
        return bos.toByteArray();
    }


    public static void main(String[] args) {
        ContextListener cp = new ContextListener();
        cp.initBroker();
        cp.subscribe("/uuid:43025660-6DF6-11DA-9660-F0A2CF132072-1134711541469");
    }
}
