/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.sequencer.uuidgen;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Open Source Apache jUDDI Project => http://ws.apache.org/juddi
 */

public final class Win32UUIDGen implements UUIDGen
{
  /**
   *
   */
  public String uuidgen()
  {
    String[] uuids = this.uuidgen(1);
    return uuids[0];
  }

  /**
   *
   */
  public String uuidgen_with_timestamp()
  {
    return null;
  }

  /**
   *
   */
  public String[] uuidgen(int nmbr)
  {
    String[] uuids = new String[nmbr];

    try
    {
      Runtime r = Runtime.getRuntime();
      Process p = r.exec("uuidgen -n" + nmbr);
      BufferedReader x = new BufferedReader(new InputStreamReader(p.getInputStream()));

      for (int i = 0; i < nmbr; ++i)
        uuids[i] = x.readLine();
    }
    catch (IOException ex)
    {
      ex.printStackTrace();
      throw new RuntimeException(ex.getMessage());
    }

    return uuids;
  }


  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String args[])
  {
    UUIDGen uuidgen = new Win32UUIDGen();

    long start = System.currentTimeMillis();

    // fast (3705 milliseconds)
    for (int i = 1; i <= 250; ++i)
      System.out.println( i + ":  " + uuidgen.uuidgen());

    long end = System.currentTimeMillis();

    System.out.println("Generation (and display) of 250 UUID's took "+(end-start)+" milliseconds.");


    start = System.currentTimeMillis();

    // much faster (90 milliseconds)
    String[] ids = uuidgen.uuidgen(250);
    for (int i = 1; i < 250; ++i)
      System.out.println( i + ":  " + ids[i]);

    end = System.currentTimeMillis();

    System.out.println("Generation (and display) of 250 UUID's took "+(end-start)+" milliseconds.");
  }
}
