/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.util;

import java.io.*;
import java.util.*;


/**
 * Holds methods for property file handling.
 *
 * <p> Community Grids Java Source Package</p>
 * <p> Provides supporting classess for portal environments.</p>
 * <p> Copyright (c) 2001-2002  Pervasive Technology Labs</p>
 * <p> $Id$</p>
 * @author <a href="mailto:obalsoy@indiana.edu">Ozgur Balsoy</a>
 * @version 0.0.1
 */
public class BootStrap {

  public BootStrap() {
  }

  /**
   * Returns the named file as a Property object.
   * <P>
   * @param fileName name of the file. Either full, or relative to the
   * current path, or must exist in the classpath or a JAR file.
   * @return properties
   * @throws NullPointerException if file name is null.
   * @throws Exception if file cannot be found or any other IO errors occur.
   */
  public static Properties loadProperties(String fileName) throws Exception {
    Properties p = null;


    if (fileName == null) {
      throw new NullPointerException("File name cannot be null.");
    }
    try {
      File file = new File(fileName);
      if (file.exists()) {
        p = new Properties();
        p.load(new FileInputStream(file));
        return p;
      }
    }
    catch (IOException ioe) {
      // do nothing
    }
    catch (Exception e) {
      throw e;
    }

    try {
      InputStream is = BootStrap.class.getResourceAsStream("/" + fileName);
      if (is != null) {
//              System.out.println("is "  + is.toString());
        p = new Properties();
        p.load(is);
      }
    }
    catch (Exception e) {
      System.out.println("Properties file not found in classpath.");
      throw e;
    }
    return p;
  }

  public static void main(String[] args) {

    try {
      Properties p = loadProperties("wsctx_bootsrap.properties");
      System.out.println(p);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }

  }
}

