package cgl.util;

import java.util.Date;
import java.util.TimeZone;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

/**
 *  Date and time parsing and formatting
 *  Utility methods to format date to one of the standard ISO 8601
 *  formats and to parse given string in ISO 8601 dat/time
 *  format and return a date object
 */

public class ISO_8601_UTC {

  // private reference to the FTHPIS logger
  private static Log log = LogFactory.getLog(ISO_8601_UTC.class);

  public static final int BASIC_FORMAT_MSEC = 0;
  public static final int BASIC_FORMAT_COMPACT_MSEC = 1;
  public static final int BASIC_FORMAT_SEC = 2;
  public static final int BASIC_FORMAT_COMPACT_SEC = 3;

  private static final int DEFAULT_FORMAT = BASIC_FORMAT_MSEC;
  private static final int NUM_FORMATS = 4;

  private static final DateFormat[] iso8601UTCFormatter;

  static
  {
    iso8601UTCFormatter = new SimpleDateFormat[NUM_FORMATS];

    iso8601UTCFormatter[BASIC_FORMAT_MSEC] =
      new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
    iso8601UTCFormatter[BASIC_FORMAT_COMPACT_MSEC] =
      new SimpleDateFormat("yyyyMMdd'T'HHmmss.SSS'Z'");
    iso8601UTCFormatter[BASIC_FORMAT_SEC] =
      new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
    iso8601UTCFormatter[BASIC_FORMAT_COMPACT_SEC] =
      new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");

    for(int formatId=0; formatId<NUM_FORMATS; formatId++){
      iso8601UTCFormatter[formatId].
        setTimeZone(TimeZone.getTimeZone("UTC"));
    }
  }

  public ISO_8601_UTC()
  {
  }

  public static String format(Date dateObj, int formatId)
  {
    return (iso8601UTCFormatter[formatId]).format(dateObj);
  }

  public static String format(Date dateObj)
  {
    return format(dateObj, DEFAULT_FORMAT);
  }


  public static Date parse(String dateString, int formatId)
    throws ParseException
  {
    Date parsedDate = null;
    try{
      parsedDate = (iso8601UTCFormatter[formatId]).parse(dateString);
    }
    catch(ParseException pe){
              throw new ParseException("String `"+ dateString +"'did not match " +
                                 "the ISO8601 UTC Basic date formats",
                                 pe.getErrorOffset());
    }
    return parsedDate;
  }

  public static Date parse(String dateString)
    throws ParseException
  {
    Date parsedDate = null;
    ParseException parseErr = null;

    for(int formatId=0; formatId<NUM_FORMATS; formatId++){
      try
      {
        parsedDate = parse(dateString, formatId);
        break;
      }
      catch(ParseException pe){
        parseErr = pe;
      }
    }
    if(parsedDate != null)
      return parsedDate;
    else
      throw new ParseException("String `"+ dateString +"'did not match " +
                               "the ISO8601 UTC Basic date formats",
                               parseErr.getErrorOffset());
  }

  public static java.sql.Date convert(long d) {
    java.sql.Date date = new java.sql.Date(d);
    return date;
  }

  public static void main(String args[]){
    int doWhat = 0;
    String dateArgStr = null;
    long dateArgLong = -1;
    int dateFormatId = -1;
    String errMsg = null;


    if(args.length == 0){
      doWhat = 0;
    }

    if(args.length == 1){
      dateArgStr = args[0];
      doWhat=3;
    }

    if(args.length == 2){
      try{
        dateArgStr = args[0];
        dateFormatId = new Integer(args[1]).intValue();
        if((dateFormatId < NUM_FORMATS) && (dateFormatId >= 0))
          doWhat=3;
        else{
          doWhat = -1;
          errMsg = "Error in DateFormat";
        }

      }
      catch(NumberFormatException nfe){
        doWhat = -1;
        errMsg = "Error in DateLong";
      }
    }

    if(args.length > 2){
      doWhat = -1;
      errMsg = "Extra arguments found";
    }


    if(doWhat>0){
      try{
        dateArgLong = new Long(dateArgStr).longValue();
        doWhat = 1;
      }
      catch(NumberFormatException nfe){
        doWhat = 2;
      }
    }

    switch(doWhat){
      case 0:
      {
       Date currentTime_1 = new Date();

       log.info("LONG   " +
                          currentTime_1.getTime());

       String temp = currentTime_1.getTime()+"";
       log.info("temp = " + temp);

       long temp2 = Long.parseLong(temp);
       //log.info(temp2);

       log.info("TEXT   " +
                          currentTime_1.toString());

        String dateString = "2006-05-21T05:00:00Z";//ISO_8601_UTC.format(currentTime_1, 2);
        log.info("UTC" + 2 + "   " + dateString);

        try {
          Date date = ISO_8601_UTC.parse(dateString, 2);
          log.info("date = " + date.toGMTString());
        }
        catch (Exception ex) {
          ex.printStackTrace();
        }

        for(int formatId=0; formatId<NUM_FORMATS; formatId++){
          log.info("UTC" + formatId + "   " +
                             ISO_8601_UTC.format(currentTime_1,
                                                 formatId));
        }
      }
      break;

      case 1:
      {
        Date currentTime_1 = new Date(dateArgLong);

        if(dateFormatId < 0){
          log.info("LONG   " +
                             currentTime_1.getTime());

          log.info("TEXT   " +
                           currentTime_1.toString());

          for(int formatId=0; formatId<NUM_FORMATS; formatId++){
            log.info("UTC" + formatId + "   " +
                               ISO_8601_UTC.format(currentTime_1,
                                                   formatId));
          }
        }
        else
          log.info("UTC" + dateFormatId + "   " +
                             ISO_8601_UTC.format(currentTime_1,
                                                 dateFormatId));
      }
      break;

      case 2:
      {
        Date currentTime_1;
        try{
          currentTime_1 = ISO_8601_UTC.parse(dateArgStr);

          if(dateFormatId < 0){
            log.info("LONG   " +
                               currentTime_1.getTime());

            log.info("TEXT   " +
                               currentTime_1.toString());

            for(int formatId=0; formatId<NUM_FORMATS; formatId++){
              log.info("UTC" + formatId + "   " +
                                 ISO_8601_UTC.format(currentTime_1,
                                                     formatId));
            }
          }
          else
            log.info("UTC" + dateFormatId + "   " +
                               ISO_8601_UTC.format(currentTime_1,
                                                   dateFormatId));
        }
        catch(ParseException pe){
          System.err.println("ParseException while parsing "+
                             dateArgStr+"\n"+
                             pe.getMessage());
          pe.printStackTrace();
        }
      }
      break;

      default:
      {
        if(errMsg != null){
          System.err.println("Error: " + errMsg);
        }

        System.err.println("Usage: ISO_8601_UTC " +
                           "[ DateString | DateLong [DateFormat]] \n"+
                           " DateString :" +
                           " eg. 2001-10-28T12:34:56.789Z \n" +
                           " DateLong   :" +
                           " eg. 1004272496789 \n" +
                           " DateFormat :" +
                           " [0.." + (NUM_FORMATS-1) + "]");
      }
    }
  }
}

