/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.util;

import java.sql.SQLException;
import java.io.*;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

/**
 * Wrapper class so that we can insert a Blob into the database from a byte array.
 */

public class MyByteBlob implements java.sql.Blob {

  // private reference to FTHPIS logger
  private static Log log = LogFactory.getLog(MyByteBlob.class);

    byte[] bytes = null;

    /**
     * Creates a ByteBlob using the specified byte array.
     */
    public MyByteBlob(byte[] b) {
        bytes = b;
    }

    // constructor for taking a Blob of input and returning as an  array
    public MyByteBlob(java.sql.Blob b) {
        java.io.InputStream is = null;
        try {
            is = b.getBinaryStream();
            bytes = new byte[(int) b.length()];
            is.read(bytes);
        } catch (java.sql.SQLException e) {
            bytes = null;
        } catch (java.io.IOException e) {
            bytes = null;
        } finally {
            try {
                if (is != null) {
                    is.close();
                }
            } catch (Exception e) {}
        }
    }

    public long length() throws java.sql.SQLException {
        return bytes.length;
    }

    // My own API call for simplicity
    public byte[] getBytes() {
        return bytes;
    }

    public byte[] getBytes(long pos, int length) throws java.sql.SQLException {
        if (pos == 0 && length == bytes.length) {
            return bytes;
        }

        try {
            byte[] newbytes = new byte[length];
            System.arraycopy(bytes, (int) pos, newbytes, 0, length);
            return newbytes;
        } catch (Exception e) {
            throw new java.sql.SQLException("Could not get subset of  array ");
        }
    }


    public java.io.InputStream getBinaryStream() throws java.sql.SQLException {
        return new java.io.ByteArrayInputStream(bytes);
    }


    public long position(byte[] pattern, long start) throws java.sql.
            SQLException {
        throw new java.sql.SQLException("Unsupported position() for blob");
    }

    public long position(java.sql.Blob pattern, long start) throws java.sql.
            SQLException {
        throw new java.sql.SQLException("Unsupported position() for blob");
    }


    public java.io.OutputStream setBinaryStream(long pos) throws SQLException{
        return null;
    }
    public int setBytes(long pos,byte[] bytes)  throws SQLException{
        return 1;
    }
    public int setBytes(long pos,byte[] bytes,int offset,int len)  throws SQLException{
        return 1;
    }


    public void truncate(long l) {

    }

    public static void main(String[] args) {
        try {
            byte[] bytearray = "Mehmet".getBytes(); //new byte[2];
            java.sql.Blob newblob = new cgl.util.MyByteBlob(
                    bytearray);
            log.info("length = " + newblob.length());

            java.io.InputStream is = null;
            byte [] bytes = null;
            try {
                is = newblob.getBinaryStream();
                bytes = new byte[(int) newblob.length()];
                is.read(bytes);
            } catch (java.sql.SQLException e) {
                bytes = null;
            } catch (java.io.IOException e) {
                bytes = null;
            } finally {
                try {
                    if (is != null) {
                        is.close();
                    }
                } catch (Exception e) {}
        }
        log.info("result = " + (new String(bytearray)).toString());

        } catch (SQLException ex) {
        }
    }

}
