/*
 * Indiana University Community Grid Computing Lab Software License,Version 1.1
 *
 * Copyright (c) 2002 Community Grid Computing Lab, Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *      "This product includes software developed by the Indiana University
 *       Community Grid Computing Lab (http://www.communitygrids.iu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Indiana Univeristy","Indiana Univeristy Pervasive Techonology
 *    Labs" and  "Indiana Univeristy Community Grid Computing Lab" must not be
 *    used to endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    http://www.communitygrids.iu.edu/.
 *
 * 5. Products derived from this software may not use "Indiana
 *    University" name nor may "Indiana University" appear in their name,
 *    without prior written permission of the Indiana University.
 *
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * INDIANA UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cgl.util.jdbc;

import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.pool.impl.GenericObjectPool;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import org.apache.juddi.registry.RegistryEngine;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.Loader;
import java.util.Properties;
import cgl.util.BootStrap;

import space.remote.MicroSpace;
import space.remote.SpaceAccessor;


/**
 * @author Mehmet S. Aktas (maktas@indiana.edu)
 * Enhanced code from an open source apache project jUDDI
 */

public class ConnectionManager {
    // private reference to the FTHPIS logger
    private static Log log = LogFactory.getLog(ConnectionManager.class);

    private static BootStrap bootstrap = new BootStrap();

    private static Properties props = new Properties();

    static {
        try {
            //String user_home = System.getProperty("user.home");
            //props.load(new java.io.FileInputStream(user_home + "/uddi_context/properties.prop"));
            props = bootstrap.loadProperties("properties.properties");
            Config.addProperties(props);
        } catch (Exception ex) {
            System.out.println("Error in loading Property file. " +
                               ex.getMessage());
            System.exit(1);
        }
    }

    // DBCP DataSource property keys and default values
    private static final String USE_CONNECTION_POOL_VALUE_KEY =
            "cgl.fthpis.useConnectionPool";
    private static final boolean DEFAULT_USE_CONNECTION_POOL_VALUE = Config.
            getBooleanProperty(USE_CONNECTION_POOL_VALUE_KEY, true);

    private static final String JDBC_DRIVER_KEY = "cgl.fthpis.jdbcDriver";
    private static final String DEFAULT_JDBC_DRIVER = Config.getStringProperty(
            JDBC_DRIVER_KEY);

    private static final String JDBC_URL_KEY = "cgl.fthpis.uddi.jdbcURL";
    private static final String DEFAULT_JDBC_URL = Config.getStringProperty(
            JDBC_URL_KEY);

    private static final String JDBC_WSContext_URL_KEY =
            "cgl.fthpis.wscontext.jdbcURL";
    private static final String DEFAULT_JDBC_WSContext_URL = Config.
            getStringProperty(JDBC_WSContext_URL_KEY);

    private static final String JDBC_USERNAME_KEY = "cgl.fthpis.jdbcUser";
    private static final String DEFAULT_JDBC_USERNAME = Config.
            getStringProperty(JDBC_USERNAME_KEY);

    private static final String JDBC_PASSWORD_KEY = "cgl.fthpis.jdbcPassword";
    private static final String DEFAULT_JDBC_PASSWORD = Config.
            getStringProperty(JDBC_PASSWORD_KEY);

    private static final String JDBC_MAX_ACTIVITY_KEY =
            "cgl.fthpis.jdbcMaxActive";
    private static final int DEFAULT_JDBC_MAX_ACTIVITY = Config.getIntProperty(
            JDBC_MAX_ACTIVITY_KEY, 10);

    private static final String JDBC_MAX_IDLE_KEY = "cgl.fthpis.jdbcMaxIdle";
    private static final int DEFAULT_JDBC_MAX_IDLE = Config.getIntProperty(
            JDBC_MAX_IDLE_KEY, 5);

    // Shared jUDDI DataSource
    private static DataSource dataSource = null;

    // Shared jUDDI DataSource
    private static DataSource UDDIDataSource = null;

    // Shared ContextSpace DataSource
    private static DataSource ContextDataSource = null;

    // private static RemoteSpace remoteSpace = null;
    private static MicroSpace remoteSpace = null;


    /**
     * All other functionalities are related with UDDI
     * So, I revised the data store to be UDDI by default.
     */
    public static Connection aquireConnection() throws SQLException {
        // Make sure we've got a DataSource first
        if (UDDIDataSource == null) {
            UDDIDataSource = getUDDIDataSource();
        }

        Connection conn = null;
        if (UDDIDataSource != null) {
            conn = UDDIDataSource.getConnection();
        }

        return conn;
    }


    /**
     *
     */
    public static Connection aquireConnection(String type) throws SQLException {
        Connection conn = null;
        if (type.equals("wscontext")) {
            // Make sure we've got a DataSource first
            if (dataSource == null) {
                dataSource = getDataSource();
            }

            //Connection conn = null;
            if (dataSource != null) {
                conn = dataSource.getConnection();
            }
            //return conn;
        } else if (type.equals("uddi")) {
            // Make sure we've got a DataSource first
            if (UDDIDataSource == null) {
                UDDIDataSource = getUDDIDataSource();
            }

            //Connection conn = null;
            if (UDDIDataSource != null) {
                conn = UDDIDataSource.getConnection();
            }
            //return conn;
        }
        return conn;
    }


    /**
     *
     */
    public static MicroSpace aquireContextSpaceConnection() throws SQLException {
        MicroSpace space = null;
        try {
            space = SpaceAccessor.getSpace("rmi://localhost/space");
        } catch (Exception ex) {
            System.err.println(ex);
        }
        return space;
    }


    /**
     *
     */
    public static Connection aquireUDDIConnection() throws SQLException {
        // Make sure we've got a DataSource first
        if (UDDIDataSource == null) {
            UDDIDataSource = getDataSource();
        }

        Connection conn = null;
        if (UDDIDataSource != null) {
            conn = UDDIDataSource.getConnection();
        }

        return conn;
    }


    /**
     *
     */
    private static synchronized DataSource getDataSource() throws SQLException {
        if (dataSource != null) {
            return dataSource;
        }

        boolean useInternalConnectionPool =
                Config.getBooleanProperty(USE_CONNECTION_POOL_VALUE_KEY,
                                          DEFAULT_USE_CONNECTION_POOL_VALUE);

        if (useInternalConnectionPool) {
            if (dataSource == null) {
                dataSource = createDataSource(); // use DBCP (jUDDI Managed Connection Pool)
            }
        } else {
            if (dataSource == null) {
                dataSource = lookupDataSource(); // use JNDI (Container Managed Connection Pool)
            }
        }

        return dataSource;
    }


    /**
     *
     */
    private static synchronized DataSource getUDDIDataSource() throws
            SQLException {
        if (UDDIDataSource != null) {
            return UDDIDataSource;
        }

        boolean useInternalConnectionPool =
                Config.getBooleanProperty(USE_CONNECTION_POOL_VALUE_KEY,
                                          DEFAULT_USE_CONNECTION_POOL_VALUE);

        if (useInternalConnectionPool) {
            if (UDDIDataSource == null) {
                UDDIDataSource = createUDDIDataSource(); // use DBCP (jUDDI Managed Connection Pool)
            }
        } else {
            if (UDDIDataSource == null) {
                UDDIDataSource = lookupUDDIDataSource(); // use JNDI (Container Managed Connection Pool)
            }
        }

        return UDDIDataSource;
    }

    /**
     *
     */
    private static DataSource createDataSource() throws SQLException {
        // make sure we still need to create a DataSource.
        if (dataSource != null) {
            return dataSource;
        }

        // grab the JDBC properties we'll need to setup the connection pool.
        String dbcpDriver = Config.getStringProperty(JDBC_DRIVER_KEY,
                DEFAULT_JDBC_DRIVER);
        String dbcpURL = Config.getStringProperty(JDBC_WSContext_URL_KEY,
                                                  DEFAULT_JDBC_WSContext_URL);
        String dbcpUser = Config.getStringProperty(JDBC_USERNAME_KEY,
                DEFAULT_JDBC_USERNAME);
        String dbcpPassword = Config.getStringProperty(JDBC_PASSWORD_KEY,
                DEFAULT_JDBC_PASSWORD);
        int dbcpMaxActive = Config.getIntProperty(JDBC_MAX_ACTIVITY_KEY,
                                                  DEFAULT_JDBC_MAX_ACTIVITY);
        int dbcpMaxIdle = Config.getIntProperty(JDBC_MAX_IDLE_KEY,
                                                DEFAULT_JDBC_MAX_IDLE);

        log.info("Using DBCP to create a JDBC DataSource using properties: \n" +
                 "\tDriver:    " + dbcpDriver + "\n" +
                 "\tURL:       " + dbcpURL + "\n" +
                 "\tUser:      " + dbcpUser + "\n" +
                 "\tPassword:  " + dbcpPassword + "\n" +
                 "\tMaxActive: " + dbcpMaxActive + "\n" +
                 "\tMaxIdle:   " + dbcpMaxIdle + "\n");

        // make sure the JDBC Driver is loaded
        try {
            Loader.getClassForName(dbcpDriver);
        } catch (ClassNotFoundException cnfex) {
            throw new SQLException("Could not locate JDBC Driver '" +
                                   dbcpDriver + "' in classpath: " +
                                   cnfex.getMessage());
        }

        // create an ObjectPool that serves as the actual
        // pool of connections. We'll use a GenericObjectPool
        // instance, although any ObjectPool implementation
        // will suffice.
        GenericObjectPool connPool = new GenericObjectPool(null);
        connPool.setMaxActive(10);
        connPool.setMaxIdle(10);

        // create a ConnectionFactory that the pool will use
        // to create Connections. We'll use the
        // DriverManagerConnectionFactory, using the connect
        // string passed in the command line arguments.
        ConnectionFactory connFact =
                new DriverManagerConnectionFactory(dbcpURL, dbcpUser,
                dbcpPassword);

        // create the PoolableConnectionFactory, which wraps
        // the "real" Connections created by the ConnectionFactory
        // with the classes that implement the pooling functionality.
        try {
            PoolableConnectionFactory pcf =
                    new PoolableConnectionFactory(connFact, connPool, null, null, false, true);
        } catch (Exception ex) {
            throw new SQLException(ex.getMessage());
        }

        // create the PoolingDriver itself, passing in the
        // object pool we created.
        PoolingDataSource dataSource = new PoolingDataSource(connPool);

        return dataSource;
    }

    /**
     *
     */
    private static DataSource createUDDIDataSource() throws SQLException {
        // make sure we still need to create a DataSource.
        if (UDDIDataSource != null) {
            return UDDIDataSource;
        }

        // grab the JDBC properties we'll need to setup the connection pool.
        String dbcpDriver = Config.getStringProperty(JDBC_DRIVER_KEY,
                DEFAULT_JDBC_DRIVER);
        String dbcpURL = Config.getStringProperty(JDBC_URL_KEY,
                                                  DEFAULT_JDBC_URL);
        String dbcpUser = Config.getStringProperty(JDBC_USERNAME_KEY,
                DEFAULT_JDBC_USERNAME);
        String dbcpPassword = Config.getStringProperty(JDBC_PASSWORD_KEY,
                DEFAULT_JDBC_PASSWORD);
        int dbcpMaxActive = Config.getIntProperty(JDBC_MAX_ACTIVITY_KEY,
                                                  DEFAULT_JDBC_MAX_ACTIVITY);
        int dbcpMaxIdle = Config.getIntProperty(JDBC_MAX_IDLE_KEY,
                                                DEFAULT_JDBC_MAX_IDLE);

        log.info("Using DBCP to create a JDBC DataSource using properties: \n" +
                 "\tDriver:    " + dbcpDriver + "\n" +
                 "\tURL:       " + dbcpURL + "\n" +
                 "\tUser:      " + dbcpUser + "\n" +
                 "\tPassword:  " + dbcpPassword + "\n" +
                 "\tMaxActive: " + dbcpMaxActive + "\n" +
                 "\tMaxIdle:   " + dbcpMaxIdle + "\n");

        // make sure the JDBC Driver is loaded
        try {
            Loader.getClassForName(dbcpDriver);
        } catch (ClassNotFoundException cnfex) {
            throw new SQLException("Could not locate JDBC Driver '" +
                                   dbcpDriver + "' in classpath: " +
                                   cnfex.getMessage());
        }

        // create an ObjectPool that serves as the actual
        // pool of connections. We'll use a GenericObjectPool
        // instance, although any ObjectPool implementation
        // will suffice.
        GenericObjectPool connPool = new GenericObjectPool(null);
        connPool.setMaxActive(10);
        connPool.setMaxIdle(10);

        // create a ConnectionFactory that the pool will use
        // to create Connections. We'll use the
        // DriverManagerConnectionFactory, using the connect
        // string passed in the command line arguments.
        ConnectionFactory connFact =
                new DriverManagerConnectionFactory(dbcpURL, dbcpUser,
                dbcpPassword);

        // create the PoolableConnectionFactory, which wraps
        // the "real" Connections created by the ConnectionFactory
        // with the classes that implement the pooling functionality.
        try {
            PoolableConnectionFactory pcf =
                    new PoolableConnectionFactory(connFact, connPool, null, null, false, true);
        } catch (Exception ex) {
            throw new SQLException(ex.getMessage());
        }

        // create the PoolingDriver itself, passing in the
        // object pool we created.
        PoolingDataSource UDDIDataSource = new PoolingDataSource(connPool);

        return UDDIDataSource;
    }


    /**
     *
     */
    private static DataSource lookupDataSource() throws SQLException {
        // make sure we still need to lookup the DataSource
        if (dataSource != null) {
            return dataSource;
        }

        // look it up.
        try {
            String dataSourceName =
                    Config.getStringProperty(RegistryEngine.
                                             PROPNAME_DATASOURCE_NAME,
                                             RegistryEngine.
                                             DEFAULT_DATASOURCE_NAME);

            log.info("Using JNDI to aquire a JDBC DataSource with " +
                     "name: " + dataSourceName);

            InitialContext initCtx = new InitialContext();
            dataSource = (DataSource) initCtx.lookup(dataSourceName);
        } catch (NamingException nex) {
            log.error("Exception occurred while attempting to acquire " +
                      "a JDBC DataSource from JNDI: " + nex.getMessage());
        }

        return dataSource;
    }


    /**
     *
     */
    private static DataSource lookupUDDIDataSource() throws SQLException {
        // make sure we still need to lookup the DataSource
        if (UDDIDataSource != null) {
            return UDDIDataSource;
        }

        // look it up.
        try {
            String dataSourceName =
                    Config.getStringProperty(RegistryEngine.
                                             PROPNAME_DATASOURCE_NAME,
                                             RegistryEngine.
                                             DEFAULT_DATASOURCE_NAME);

            log.info("Using JNDI to aquire a JDBC DataSource with " +
                     "name: " + dataSourceName);

            InitialContext initCtx = new InitialContext();
            UDDIDataSource = (DataSource) initCtx.lookup(dataSourceName);
        } catch (NamingException nex) {
            log.error("Exception occurred while attempting to acquire " +
                      "a JDBC DataSource from JNDI: " + nex.getMessage());
        }

        return UDDIDataSource;
    }


}
